@@ -100,7 +100,8 @@ include the pathname of the file and the position of the definition."
 (declaim (ftype (sfunction (t debug-info) debug-function) debug-info-debug-function))
 (defun debug-info-debug-function (function debug-info)
   (sb-di::compiled-debug-fun-from-pc debug-info
-                                     (sb-di::function-start-pc-offset function)))
+                                     (sb-di::function-start-pc-offset function)
+                                     t))
 
 (defun valid-function-name-p (name)
   "True if NAME denotes a valid function name, ie. one that can be passed to
@@ -819,7 +820,7 @@ Experimental: interface subject to change."
              (sb-sys:with-pinned-objects (object)
                (let ((space (sb-ext:heap-allocated-p object)))
                  (when space
-                   #+gencgc
+                   #+generational
                    (if (eq :dynamic space)
                        (symbol-macrolet ((page (sb-alien:deref sb-vm::page-table index)))
                          ;; No wonder #+big-endian failed introspection tests-
@@ -838,7 +839,7 @@ Experimental: interface subject to change."
                                  :large (logbitp 4 type)
                                  :page index)))
                        (list :space space))
-                   #-gencgc
+                   #-generational
                    (list :space space))))))
         (cond (plist
                (values :heap plist))
@@ -883,6 +884,7 @@ Experimental: interface subject to change."
                         (not (xset-member-p part seen)))
                  (add-to-xset part seen)
                  (funcall fun part))))
+      (declare (dynamic-extent #'call))
       (when ext
         (multiple-value-bind (value foundp)
             (let ((table sb-pcl::*eql-specializer-table*))
@@ -919,7 +921,7 @@ Experimental: interface subject to change."
                       (sb-thread:interrupt-thread-error ()))
                     ;; This is whacky - the other thread signals our condition var,
                     ;; *then* we call the funarg on objects that may no longer
-                    ;; satisfy VALID-LISP-POINTER-P.
+                    ;; satisfy VALID-TAGGED-POINTER-P.
                     ;; And incidentally, we miss any references from TLS indices
                     ;; that map onto the 'struct thread', which is just as well
                     ;; since they're either fixnums or dynamic-extent objects.
@@ -1035,17 +1037,17 @@ Experimental: interface subject to change."
                                    (= this-bin-size (+ prev-bin-size 2)))
                                this-bin-size))))))))
 
-(defun largest-objects (&key (threshold #+gencgc sb-vm:gencgc-page-bytes
-                                        #-gencgc sb-c:+backend-page-bytes+)
+(defun largest-objects (&key (threshold #+generational sb-vm:gencgc-page-bytes
+                                        #-generational sb-c:+backend-page-bytes+)
                              (sort :size))
   (declare (type (member :address :size) sort))
   (flet ((show-obj (obj)
-           #-gencgc
+           #-generational
            (format t "~10x ~7x ~s~%"
                      (get-lisp-obj-address obj)
                      (primitive-object-size obj)
                      (type-of obj))
-           #+gencgc
+           #+generational
            (let* ((gen (generation-of obj))
                   (page (sb-vm::find-page-index (sb-kernel:get-lisp-obj-address obj)))
                   (flags (if (>= page 0)
