@@ -32,6 +32,7 @@
          (asmstream (make-asmstream))
          (*asmstream* asmstream)
          (sb-vm::*adjustable-vectors* nil))
+    (declare (special sb-vm::*adjustable-vectors*))
     (unwind-protect
         (let ((sb-xc:*features* (cons :sb-assembling sb-xc:*features*))
               (*readtable* sb-cold:*xc-readtable*))
@@ -67,6 +68,12 @@
           (let ((segment (assemble-sections
                           asmstream nil
                           (make-segment :run-scheduler nil))))
+            (unless (= (length (remove-duplicates (mapcar 'car *entry-points*)))
+                       (length *entry-points*))
+              (error "Duplicate asm routine: ~S"
+                     (loop for (this . rest) on *entry-points*
+                           when (assoc (car this) rest)
+                           collect (car this))))
             (dump-assembler-routines segment
                                      (segment-buffer segment)
                                      (sb-assem::segment-fixup-notes segment)
@@ -197,7 +204,8 @@
          (results (remove :res vars :key #'reg-spec-kind :test #'neq))
          (return-style (or (cadr (assoc :return-style options)) :raw))
          (cost (or (cadr (assoc :cost options)) 247))
-         (vop (make-symbol "VOP")))
+         (vop (or (cadr (assoc :vop-var options))
+                  (make-symbol "VOP"))))
     ;; :full-call-no-return entails the call site behavior for :full-call
     ;; without automatic insertion of the return sequence. This avoids some confusion
     ;; on the part of the human reading the code, especially if the asm routine
@@ -244,10 +252,12 @@
          ;; any :SAVE-P specified in options supersedes one from call-temps.
          ;; It would be wiser to signal an error about duplicate options.
          ,@(remove-if (lambda (x)
-                        (member x '(:return-style :cost :call-temps)))
+                        (member x '(:return-style :cost :call-temps
+                                    :vop-var :vop-prefix)))
                       options
                       :key #'car)
          (:generator ,cost
+           ,@(cdr (assoc :vop-prefix options))
            ,@(mapcar (lambda (arg)
                        `(move ,(reg-spec-temp arg) ,(reg-spec-name arg)))
                      args)
