@@ -228,19 +228,29 @@
   (loadw rax-tn rax-tn fdefn-fun-slot other-pointer-lowtag)
   (inst jmp (object-slot-ea rax-tn closure-fun-slot fun-pointer-lowtag)))
 
-#-compact-instance-header
-(define-assembly-routine
-    (funcallable-instance-tramp (:return-style :none))
-    ()
-  (loadw rax-tn rax-tn funcallable-instance-function-slot fun-pointer-lowtag)
-  (inst jmp (object-slot-ea rax-tn closure-fun-slot fun-pointer-lowtag)))
-
 (define-assembly-routine (ensure-symbol-hash (:return-style :raw)) ()
   (with-registers-preserved (lisp)
     (inst mov rdx-tn (ea 16 rbp-tn)) ; arg
     (call-static-fun 'ensure-symbol-hash 1)
     (inst mov (ea 16 rbp-tn) rdx-tn))) ; result to arg passing loc
 
+#+debug-gc-barriers
+(define-assembly-routine (check-barrier (:return-style :none)) ()
+  (inst push rax-tn)
+  (inst mov rax-tn (ea 16 rsp-tn))
+  (inst not rax-tn)
+  (inst test :byte rax-tn 3)
+  (inst pop rax-tn)
+  (inst jmp :e check)
+  (inst ret 24)
+  check
+  (with-registers-preserved (c :except fp) ;; shouldn't have any fp operations
+    (call-c (make-fixup "check_barrier" :foreign)
+            (ea 16 rbp-tn)
+            (ea 24 rbp-tn)
+            (ea 32 rbp-tn)))
+  (inst ret 24))
+
 ;;; Perform a store to code, updating the GC card mark bit.
 ;;; This has two additional complications beyond the ordinary
 ;;; generational barrier:
