@@ -38,10 +38,10 @@
                   *the-class-standard-method*
                   *the-class-standard-reader-method*
                   *the-class-standard-writer-method*
-                  *the-class-standard-boundp-method*
                   *the-class-global-reader-method*
                   *the-class-global-writer-method*
                   *the-class-global-boundp-method*
+                  *the-class-global-makunbound-method*
                   *the-class-standard-generic-function*
                   *the-class-standard-direct-slot-definition*
                   *the-class-standard-effective-slot-definition*
@@ -68,35 +68,31 @@
 ;;;
 ;;; In each cons, the car is the symbol naming the layout, and the
 ;;; cdr is the layout itself.
-;;; If #+metaspace then the cdr is actually of type WRAPPER,
-;;; and if #-metaspace then the wrapper is a LAYOUT.
-(defvar *!initial-wrappers*)
+(defvar *!initial-layouts*)
 
 ;;; a table mapping class names to layouts for classes we have
 ;;; referenced but not yet loaded. This is initialized from an alist
 ;;; created by genesis describing the layouts that genesis created at
 ;;; cold-load time.
-(define-load-time-global *forward-referenced-wrappers*
+(define-load-time-global *forward-referenced-layouts*
     ;; FIXME: why is the test EQUAL and not EQ? Aren't the keys all symbols?
     (make-hash-table :test 'equal))
 #-sb-xc-host
 (!cold-init-forms
-  ;; *forward-referenced-wrappers* is protected by *WORLD-LOCK*
+  ;; *forward-referenced-layouts* is protected by *WORLD-LOCK*
   ;; so it does not need a :synchronized option.
- (setq *forward-referenced-wrappers* (make-hash-table :test 'equal))
- (dovector (x *!initial-wrappers*)
+ (setq *forward-referenced-layouts* (make-hash-table :test 'equal))
+ (dovector (x *!initial-layouts*)
    (let ((expected (hash-layout-name (car x)))
-         (actual (wrapper-clos-hash (cdr x))))
+         (actual (layout-clos-hash (cdr x))))
      (unless (= actual expected) (bug "XC layout hash calculation failed")))
-   (setf (gethash (car x) *forward-referenced-wrappers*) (cdr x))))
+   (setf (gethash (car x) *forward-referenced-layouts*) (cdr x))))
 
-;;; FIXME: This lock is only seized in the classoid/layout/class
-;;; system, and is now a misnomer.
 #-sb-xc-host
-(define-load-time-global **world-lock** nil)
+(define-load-time-global **world-lock** nil) ; the PCL world, that is
 #-sb-xc-host
 (!cold-init-forms
- (setq **world-lock** (sb-thread:make-mutex :name "World Lock")))
+ (setq **world-lock** (sb-thread:make-mutex :name "PCL global mutex")))
 
 (defmacro with-world-lock (() &body body)
   #+sb-xc-host `(progn ,@body)
@@ -107,68 +103,61 @@
 #+sb-xc-host
 (progn
 (defun make-layout (hash classoid &rest keys)
-  (macrolet ((make (&rest extra)
-               `(apply #'host-make-wrapper
-                       (cdr (assq (classoid-name classoid) *popular-structure-types*))
-                       hash classoid ,@extra :allow-other-keys t keys)))
-    #-metaspace (make)
-    #+metaspace (let* ((layout (%make-layout))
-                       (wrapper (make :friend layout)))
-                  (setf (layout-friend layout) wrapper)
-                  wrapper)))
-;; The target reconstructs wrappers using FOP-LAYOUT but the host uses MAKE-LOAD-FORM.
-(defmethod cl:make-load-form ((wrapper wrapper) &optional env)
+  (apply #'host-make-layout
+         (cdr (assq (classoid-name classoid) *popular-structure-types*))
+         hash classoid :allow-other-keys t keys))
+;; The target reconstructs layouts using FOP-LAYOUT but the host uses MAKE-LOAD-FORM.
+(defmethod cl:make-load-form ((layout layout) &optional env)
   (declare (ignore env))
-  (labels ((externalize (wrapper &aux (classoid (wrapper-classoid wrapper))
+  (labels ((externalize (layout &aux (classoid (layout-classoid layout))
                                       (name (classoid-name classoid)))
-             (when (or (wrapper-invalid wrapper)
+             (when (or (layout-invalid layout)
                        (not name)
                        (typep classoid 'undefined-classoid))
-               (sb-c:compiler-error "can't dump ~S" wrapper))
-             `(xc-load-wrapper ',name
-                               ,(wrapper-depthoid wrapper)
-                               (vector ,@(map 'list #'externalize (wrapper-inherits wrapper)))
-                               ,(wrapper-length wrapper)
-                               ,(wrapper-bitmap wrapper))))
-    (externalize wrapper)))
-(defun xc-load-wrapper (name depthoid inherits length bitmap)
+               (sb-c:compiler-error "can't dump ~S" layout))
+             `(xc-load-layout ',name
+                               ,(layout-depthoid layout)
+                               (vector ,@(map 'list #'externalize (layout-inherits layout)))
+                               ,(layout-length layout)
+                               ,(layout-bitmap layout))))
+    (externalize layout)))
+(defun xc-load-layout (name depthoid inherits length bitmap)
   (let ((classoid (find-classoid name)))
     (aver (and classoid (not (undefined-classoid-p classoid))))
-    (let ((wrapper (classoid-wrapper classoid)))
-      (unless (and (= (wrapper-depthoid wrapper) depthoid)
-                   (= (length (wrapper-inherits wrapper)) (length inherits))
-                   (every #'eq (wrapper-inherits wrapper) inherits)
-                   (= (wrapper-length wrapper) length)
-                   (= (wrapper-bitmap wrapper) bitmap))
+    (let ((layout (classoid-layout classoid)))
+      (unless (and (= (layout-depthoid layout) depthoid)
+                   (= (length (layout-inherits layout)) (length inherits))
+                   (every #'eq (layout-inherits layout) inherits)
+                   (= (layout-length layout) length)
+                   (= (layout-bitmap layout) bitmap))
         (error "XC can't reload layout for ~S with ~S vs ~A"
-               name (list depthoid inherits length bitmap) wrapper))
-      wrapper)))
+               name (list depthoid inherits length bitmap) layout))
+      layout)))
 ) ; end PROGN
 
-(defmethod print-object ((wrapper wrapper) stream)
-  (print-unreadable-object (wrapper stream :type t :identity t)
+(defmethod print-object ((layout layout) stream)
+  (print-unreadable-object (layout stream :type t :identity t)
     (format stream
             "~@[(ID=~d) ~]for ~S~@[, INVALID=~S~]"
-            (layout-id wrapper)
-            (wrapper-proper-name wrapper)
-            (wrapper-invalid wrapper))))
+            (layout-id layout)
+            (layout-proper-name layout)
+            (layout-invalid layout))))
 
 (eval-when (#-sb-xc :compile-toplevel :load-toplevel :execute)
-  (defun wrapper-proper-name (wrapper)
-    (classoid-proper-name (wrapper-classoid wrapper))))
+  (defun layout-proper-name (layout)
+    (classoid-proper-name (layout-classoid layout))))
 
 ;;; Return the layout currently installed in the classoid named NAME.
 ;;; If there is none, then make a layout referring for an undefined classoid.
-;;; NB: for #+metaspace this returns a WRAPPER, not a LAYOUT.
-(declaim (ftype (sfunction (symbol) wrapper) find-layout))
+(declaim (ftype (sfunction (symbol) layout) find-layout))
 (defun find-layout (name)
   (binding* ((classoid (find-classoid name nil) :exit-if-null) ; threadsafe
-             (wrapper (classoid-wrapper classoid) :exit-if-null))
-    (return-from find-layout wrapper))
-  (let ((table *forward-referenced-wrappers*))
+             (layout (classoid-layout classoid) :exit-if-null))
+    (return-from find-layout layout))
+  (let ((table *forward-referenced-layouts*))
     (with-world-lock ()
       (let ((classoid (find-classoid name nil)))
-        (or (and classoid (classoid-wrapper classoid))
+        (or (and classoid (classoid-layout classoid))
             (values (ensure-gethash name table
                                     (make-layout
                                      (hash-layout-name name)
@@ -178,7 +167,7 @@
 ;;; If LAYOUT's slot values differ from the specified slot values in
 ;;; any interesting way, then give a warning and return T.
 (declaim (ftype (function (simple-string
-                           wrapper
+                           layout
                            simple-string
                            index
                            simple-vector
@@ -187,17 +176,17 @@
                 warn-if-altered-layout))
 (defun warn-if-altered-layout (old-context old-layout context
                                length inherits depthoid bitmap)
-  (let ((name (wrapper-proper-name old-layout))
-        (old-inherits (wrapper-inherits old-layout)))
-    (or (when (mismatch old-inherits inherits :key #'wrapper-proper-name)
+  (let ((name (layout-proper-name old-layout))
+        (old-inherits (layout-inherits old-layout)))
+    (or (when (mismatch old-inherits inherits :key #'layout-proper-name)
           (warn "change in superclasses of class ~S:~%  ~
                        ~A superclasses: ~S~%  ~
                        ~A superclasses: ~S"
                       name
                       old-context
-                      (map 'list #'wrapper-proper-name old-inherits)
+                      (map 'list #'layout-proper-name old-inherits)
                       context
-                      (map 'list #'wrapper-proper-name inherits))
+                      (map 'list #'layout-proper-name inherits))
           t)
         (let ((diff (mismatch old-inherits inherits)))
           (when diff
@@ -206,10 +195,10 @@
                     ~A definition."
                    name
                    old-context
-                   (wrapper-proper-name (svref old-inherits diff))
+                   (layout-proper-name (svref old-inherits diff))
                    context)
             t))
-        (let ((old-length (wrapper-length old-layout)))
+        (let ((old-length (layout-length old-layout)))
           (unless (= old-length length)
             (warn "change in instance length of class ~S:~%  ~
                    ~A length: ~W~%  ~
@@ -226,7 +215,7 @@
 between the ~A definition and the ~A definition"
                   name old-context context)
             t))
-        (unless (= (wrapper-depthoid old-layout) depthoid)
+        (unless (= (layout-depthoid old-layout) depthoid)
           (warn "change in the inheritance structure of class ~S~%  ~
                  between the ~A definition and the ~A definition"
                 name old-context context)
@@ -242,25 +231,25 @@ between the ~A definition and the ~A definition"
 ;;; return it, after checking for compatibility. If incompatible, we
 ;;; allow the layout to be replaced, altered or left alone.
 (defun load-layout (name depthoid inherits length bitmap flags)
-  (let* ((table *forward-referenced-wrappers*)
+  (let* ((table *forward-referenced-layouts*)
          (classoid (find-classoid name nil))  ; thread safety
          (new-layout (make-layout (hash-layout-name name)
                                   (or classoid (make-undefined-classoid name))
                                   :depthoid depthoid :inherits inherits
                                   :length length :bitmap bitmap :flags flags))
          (existing-layout
-           (or (and classoid (classoid-wrapper classoid))
+           (or (and classoid (classoid-layout classoid))
                (with-world-lock ()
                  (let ((classoid (find-classoid name nil)))
                    (when classoid
-                     (setf (wrapper-classoid new-layout) classoid))
-                   (or (and classoid (classoid-wrapper classoid))
+                     (setf (layout-classoid new-layout) classoid))
+                   (or (and classoid (classoid-layout classoid))
                        (ensure-gethash name table new-layout))))))
          (classoid
-           (or (find-classoid name nil) (wrapper-classoid existing-layout))))
-    (cond ((or (eq (wrapper-invalid existing-layout) :uninitialized)
+           (or (find-classoid name nil) (layout-classoid existing-layout))))
+    (cond ((or (eq (layout-invalid existing-layout) :uninitialized)
                (not *type-system-initialized*))
-           (setf (wrapper-classoid existing-layout) classoid))
+           (setf (layout-classoid existing-layout) classoid))
           ;; There was an old layout already initialized with old
           ;; information, and we'll now check that old information
           ;; which was known with certainty is consistent with current
@@ -278,33 +267,38 @@ between the ~A definition and the ~A definition"
              (oldval (cas (classoid-%lock classoid) nil lock)))
         (if (eq oldval nil) lock oldval))))
 
-(defun add-subclassoid (super sub wrapper)
+(defun add-subclassoid (super sub layout)
+  (declare (sb-c::tlab :system))
   (with-system-mutex ((classoid-lock super))
     (let ((table (classoid-subclasses super))
           (count 0))
       (cond ((hash-table-p table)
-             (setf (gethash sub table) wrapper))
+             ;; If the table needs to resize, it'll stay on the heap
+             ;; even if we're using an arena.
+             (setf (gethash sub table) layout))
             ((dolist (cell table)
                (when (eq (car cell) sub)
-                 (return (setf (cdr cell) wrapper)))
+                 (return (setf (cdr cell) layout)))
                (incf (truly-the fixnum count))))
             ((<= count 7)
-             (setf (classoid-subclasses super) (acons sub wrapper table))
+             (setf (classoid-subclasses super) (acons sub layout table))
              ;; Is this barrier really necessary? mutex release is a release barrier.
              ;; I was probably struggling with crashes in the 'classoid-typep.impure'
              ;; test, and was just trying anything and everything.
              (sb-thread:barrier (:write))
-             wrapper)
+             layout)
             (t
              ;; Upgrade to a hash-table
              (let ((new #+sb-xc-host (make-hash-table :test 'eq)
-                        #-sb-xc-host (make-hash-table :hash-function #'type-hash-value
-                                                      :test 'eq)))
+                        #-sb-xc-host
+                        (sb-vm:without-arena "add-subclassoid"
+                            (make-hash-table :hash-function #'type-hash-value
+                                             :test 'eq))))
                (loop for (key . val) in table do (setf (gethash key new) val))
-               (setf (gethash sub new) wrapper)
+               (setf (gethash sub new) layout)
                (setf (classoid-subclasses super) new)
                (sb-thread:barrier (:write))
-               wrapper))))))
+               layout))))))
 
 ;;; Mnemonic device: the argument order is as GETHASH (1st = key, 2nd = table).
 ;;; But the 2nd arg is the superclassoid, *not* its subclassoid table,
@@ -333,9 +327,9 @@ between the ~A definition and the ~A definition"
                (remhash sub table))))))
   nil)
 
-(defmacro do-subclassoids (((classoid-var wrapper-var) super) &body body)
+(defmacro do-subclassoids (((classoid-var layout-var) super) &body body)
   (let ((f (make-symbol "FUNCTION")))
-    `(dx-flet ((,f (,classoid-var ,wrapper-var) ,@body))
+    `(dx-flet ((,f (,classoid-var ,layout-var) ,@body))
        (call-with-subclassoids #',f (the classoid ,super)))))
 
 (defun call-with-subclassoids (function super &aux (table (classoid-subclasses super)))
@@ -353,26 +347,20 @@ between the ~A definition and the ~A definition"
                table))
   nil)
 
-;;; Record WRAPPER as the layout for its class, adding it as a subtype
+;;; Record LAYOUT as the layout for its class, adding it as a subtype
 ;;; of all superclasses. This is the operation that "installs" a
 ;;; layout for a class in the type system, clobbering any old layout.
 ;;; However, this does not modify the class namespace; that is a
 ;;; separate operation (think anonymous classes.)
 ;;; -- If INVALIDATE, then all the layouts for any old definition
 ;;;    and subclasses are invalidated, and the SUBCLASSES slot is cleared.
-;;; -- If MODIFY is given, then it is some old layout, and is to be
-;;;    destructively altered to hold the same data as WRAPPER.
-(macrolet ((set-bitmap-from-layout (to-layout from-layout)
-             `(let ((to-index (bitmap-start ,to-layout))
-                    (from-index (bitmap-start ,from-layout)))
-                (dotimes (i (bitmap-nwords ,from-layout))
-                  (%raw-instance-set/word ,to-layout (+ to-index i)
-                        (%raw-instance-ref/word ,from-layout (+ from-index i)))))))
-(defun register-layout (wrapper &key (invalidate t) modify)
-  (declare (type wrapper wrapper) (type (or wrapper null) modify))
+;;; -- If DESTRUCT-LAYOUT is given, then it is some old layout, and is to be
+;;;    destructively altered to hold the same data as LAYOUT.
+(defun register-layout (layout &key (invalidate t) destruct-layout)
+  (declare (type layout layout) (type (or layout null) destruct-layout))
   (with-world-lock ()
-    (let* ((classoid (wrapper-classoid wrapper))
-           (classoid-wrapper (classoid-wrapper classoid)))
+    (let* ((classoid (layout-classoid layout))
+           (classoid-layout (classoid-layout classoid)))
 
       ;; Attempting to register ourselves with a temporary undefined
       ;; class placeholder is almost certainly a programmer error. (I
@@ -382,32 +370,31 @@ between the ~A definition and the ~A definition"
       ;; This assertion dates from classic CMU CL. The rationale is
       ;; probably that calling REGISTER-LAYOUT more than once for the
       ;; same LAYOUT is almost certainly a programmer error.
-      (aver (not (eq classoid-wrapper wrapper)))
+      (aver (not (eq classoid-layout layout)))
 
       ;; Figure out what classes are affected by the change, and issue
       ;; appropriate warnings and invalidations.
-      (when classoid-wrapper
+      (when classoid-layout
         (%modify-classoid classoid)
-        (do-subclassoids ((subclass subclass-wrapper) classoid) ; under WORLD-LOCK
-            (%modify-classoid subclass)
-            (when invalidate
-              (%invalidate-layout subclass-wrapper)))
+        (do-subclassoids ((subclass subclass-layout) classoid) ; under WORLD-LOCK
+          (%modify-classoid subclass)
+          (when invalidate
+            (%invalidate-layout subclass-layout)))
         (when invalidate
-          (%invalidate-layout classoid-wrapper)
+          (%invalidate-layout classoid-layout)
           (setf (classoid-subclasses classoid) nil)))
 
-      (if modify
+      (if destruct-layout
           #+sb-xc-host (error "Why mutate a layout in XC host?")
           #-sb-xc-host
           ;; Destructively modifying a layout is not threadsafe at all.
           ;; Use at your own risk (interactive use only).
-          (let ((inherits (wrapper-inherits wrapper))
-                (depthoid (wrapper-depthoid wrapper)) ; "new" depthoid
-                (extra-id-words ; "old" extra words
-                 (calculate-extra-id-words (wrapper-depthoid modify)))
-                (layout (wrapper-friend wrapper))
-                (id ; read my ID before screwing with the depthoid
-                 (layout-id modify)))
+          (let ((inherits (layout-inherits layout))
+                (depthoid (layout-depthoid layout)) ; "new" depthoid
+                (extra-id-words       ; "old" extra words
+                  (calculate-extra-id-words (layout-depthoid destruct-layout)))
+                (id   ; read my ID before screwing with the depthoid
+                  (layout-id destruct-layout)))
             (aver (logtest +structure-layout-flag+ (layout-flags layout)))
             (aver (= (length inherits) depthoid))
             ;; DEPTHOID implies the number of words of "extra" IDs preceding the bitmap.
@@ -416,10 +403,10 @@ between the ~A definition and the ~A definition"
             ;; again to be certain. Heap corruption is the greater evil versus a minor
             ;; inconvenience of not offering the RECKLESSLY-CONTINUE restart.
             (aver (= (calculate-extra-id-words depthoid) extra-id-words))
-            #-64-bit (setf (wrapper-depthoid modify) (wrapper-depthoid wrapper)
-                           (wrapper-length modify) (wrapper-length wrapper))
-            (setf (layout-flags (wrapper-friend modify)) (layout-flags layout)
-                  (wrapper-info modify) (wrapper-info wrapper))
+            #-64-bit (setf (layout-depthoid destruct-layout) (layout-depthoid layout)
+                           (layout-length destruct-layout) (layout-length layout))
+            (setf (layout-flags destruct-layout) (layout-flags layout)
+                  (layout-info destruct-layout) (layout-info layout))
             ;; Zero out the inherited ID values one word at a time.
             ;; This makes self-ID transiently disappear, but what else can we do?
             ;; It's may be in the wrong slot anyway, depending on whether depthoid changed.
@@ -427,28 +414,30 @@ between the ~A definition and the ~A definition"
             ;;   (/ (- (1+ layout-id-vector-fixed-capacity) 2) number-of-ids-per-word)
             ;; which is surely more confusing than spelling it as 3 or 6.
             (dotimes (i (+ extra-id-words #+64-bit 3 #-64-bit 6))
-              (%raw-instance-set/word (wrapper-friend modify)
-                                      (+ (get-dsd-index sb-vm:layout id-word0) i)
+              (%raw-instance-set/word destruct-layout
+                                      (+ (get-dsd-index layout id-word0) i)
                                       0))
-            (set-layout-inherits modify inherits t id)
-            (let ((dst (wrapper-friend modify))
-                  (src (wrapper-friend wrapper)))
-              (set-bitmap-from-layout dst src))
-            (setf (wrapper-invalid modify) nil
-                  (classoid-wrapper classoid) modify))
-          (setf (wrapper-invalid wrapper) nil
-                (classoid-wrapper classoid) wrapper))
+            (set-layout-inherits destruct-layout inherits t id)
+            (let ((to-index (bitmap-start destruct-layout))
+                  (from-index (bitmap-start layout)))
+              (dotimes (i (bitmap-nwords layout))
+                (%raw-instance-set/word destruct-layout (+ to-index i)
+                                        (%raw-instance-ref/word layout (+ from-index i)))))
+            (setf (layout-invalid destruct-layout) nil
+                  (classoid-layout classoid) destruct-layout))
+          (setf (layout-invalid layout) nil
+                (classoid-layout classoid) layout))
 
-      (dovector (super-wrapper (wrapper-inherits wrapper))
-        (let ((super (wrapper-classoid super-wrapper)))
+      (dovector (super-layout (layout-inherits layout))
+        (let ((super (layout-classoid super-layout)))
           (when (and (eq (classoid-state super) :sealed)
                      (not (get-subclassoid classoid super)))
             (warn "unsealing sealed class ~S in order to subclass it"
                   (classoid-name super))
             (setf (classoid-state super) :read-only))
-          (add-subclassoid super classoid (or modify wrapper))))))
+          (add-subclassoid super classoid (or destruct-layout layout))))))
 
-  (values)))
+  (values))
 
 ;;; Arrange the inherited layouts to appear at their expected depth,
 ;;; ensuring that hierarchical type tests succeed. Layouts with
@@ -474,10 +463,11 @@ between the ~A definition and the ~A definition"
 ;;; not be read as being in CPL order.
 (defun order-layout-inherits (layouts)
   (declare (simple-vector layouts))
+  (declare (sb-c::tlab :system))
   (let ((length (length layouts))
         (max-depth -1))
     (dotimes (i length)
-      (let ((depth (wrapper-depthoid (svref layouts i))))
+      (let ((depth (layout-depthoid (svref layouts i))))
         (when (> depth max-depth)
           (setf max-depth depth))))
     (let* ((new-length (max (1+ max-depth) length))
@@ -488,7 +478,7 @@ between the ~A definition and the ~A definition"
            (inherits (make-array new-length :initial-element 0)))
       (dotimes (i length)
         (let* ((layout (svref layouts i))
-               (depth (wrapper-depthoid layout)))
+               (depth (layout-depthoid layout)))
           (unless (eql depth -1)
             (let ((old-layout (svref inherits depth)))
               (unless (or (eql old-layout 0) (eq old-layout layout))
@@ -499,7 +489,7 @@ between the ~A definition and the ~A definition"
           ((>= i length))
         (declare (type index i j))
         (let* ((layout (svref layouts i))
-               (depth (wrapper-depthoid layout)))
+               (depth (layout-depthoid layout)))
           (when (eql depth -1)
             (loop (when (eql (svref inherits j) 0)
                     (return))
@@ -592,20 +582,19 @@ between the ~A definition and the ~A definition"
 ;;; the vector of layouts in the constant pool of the containing code.
 #-(or sb-xc-host (and compact-instance-header x86-64))
 (progn
-(declaim (inline wrapper-of))
-(defun wrapper-of (x)
+(declaim (inline layout-of))
+(defun layout-of (x)
   (declare (optimize (speed 3) (safety 0)))
-  (cond ((%instancep x) (%instance-wrapper x))
-        ((funcallable-instance-p x) (%fun-wrapper x))
+  (cond ((%instancep x) (%instance-layout x))
+        ((funcallable-instance-p x) (%fun-layout x))
         ;; Compiler can dump literal layouts, which handily sidesteps
         ;; the question of when cold-init runs L-T-V forms.
         ((null x) #.(find-layout 'null))
         (t
          ;; Note that WIDETAG-OF is slightly suboptimal here and could be
          ;; improved - we've already ruled out some of the lowtags.
-         (layout-friend
-          (svref (load-time-value **primitive-object-layouts** t)
-                 (widetag-of x)))))))
+         (svref (load-time-value **primitive-object-layouts** t)
+                (widetag-of x))))))
 
 #-sb-xc-host
 (progn
@@ -613,7 +602,7 @@ between the ~A definition and the ~A definition"
 (defun classoid-of (object)
   "Return the class of the supplied object, which may be any Lisp object, not
    just a CLOS STANDARD-OBJECT."
-  (wrapper-classoid (wrapper-of object))))
+  (layout-classoid (layout-of object))))
 
 
 ;;;; classoid namespace
@@ -666,7 +655,7 @@ between the ~A definition and the ~A definition"
              (clear-info :type :expander name)
              (clear-info :type :source-location name)))
 
-          (remhash name *forward-referenced-wrappers*)
+          (remhash name *forward-referenced-layouts*)
           (%note-type-defined name)
           ;; FIXME: I'm unconvinced of the need to handle either of these.
           ;; Package locks preclude the latter, and in the former case,
@@ -688,9 +677,9 @@ between the ~A definition and the ~A definition"
                  (setf (info :type :kind name) :instance)))
           (setf (classoid-cell-classoid cell) new-value)
           (unless (eq (info :type :compiler-layout name)
-                      (classoid-wrapper new-value))
+                      (classoid-layout new-value))
             (setf (info :type :compiler-layout name)
-                  (classoid-wrapper new-value)))))
+                  (classoid-layout new-value)))))
     new-value)
 
   (defun %clear-classoid (name cell)
@@ -750,16 +739,16 @@ between the ~A definition and the ~A definition"
 (defun insured-find-classoid (name predicate constructor)
   (declare (type function predicate)
            (type (or function symbol) constructor))
-  (let ((table *forward-referenced-wrappers*))
+  (let ((table *forward-referenced-layouts*))
     (with-system-mutex ((hash-table-lock table))
       (let* ((old (find-classoid name nil))
              (res (if (and old (funcall predicate old))
                       old
                       (funcall constructor :name name)))
              (found (or (gethash name table)
-                        (when old (classoid-wrapper old)))))
+                        (when old (classoid-layout old)))))
         (when found
-          (setf (wrapper-classoid found) res))
+          (setf (layout-classoid found) res))
         (values res found)))))
 
 ;;; If the classoid has a proper name, return the name, otherwise return
@@ -790,22 +779,38 @@ between the ~A definition and the ~A definition"
 (define-type-class classoid :enumerable #'classoid-enumerable-p
                     :might-contain-other-types nil)
 
+(defmacro classoid-bits (x)
+  ;; CLASSOIDs have a deterministic hash based on the symbol naming the classoid,
+  ;; but HASH-LAYOUT-NAME will pick a pseudo-random hash if NAME is NIL.
+  `(logior ,(ctype-class-bits 'classoid)
+           (logand (hash-layout-name ,x) +ctype-hash-mask+)))
+;;; Now that the type-class has an ID, the various constructors can be defined.
+(macrolet ((def-make (name args &aux (allocator (symbolicate "!ALLOC-" name)))
+             `(defun ,(symbolicate "MAKE-" name) ,args
+                (declare (inline ,allocator))
+                (,allocator (classoid-bits name) ,@(remove '&key args)))))
+  (def-make undefined-classoid (name))
+  (def-make condition-classoid (&key name))
+  (def-make structure-classoid (&key name))
+  (def-make standard-classoid (&key name pcl-class))
+  (def-make static-classoid (&key name)))
+
 (defun classoid-inherits-from (sub super-or-name)
   (declare (type classoid sub)
            (type (or symbol classoid) super-or-name))
   (let ((super (if (symbolp super-or-name)
                    (find-classoid super-or-name)
                    super-or-name)))
-    (find (classoid-wrapper super)
-          (wrapper-inherits (classoid-wrapper sub)))))
+    (find (classoid-layout super)
+          (layout-inherits (classoid-layout sub)))))
 
 ;;; We might be passed classoids with invalid layouts; in any pairwise
 ;;; class comparison, we must ensure that both are valid before
 ;;; proceeding.
 (defun %ensure-classoid-valid (classoid layout error-context)
   (declare (ignorable error-context)) ; not used on host
-  (aver (eq classoid (wrapper-classoid layout)))
-  (or (not (wrapper-invalid layout))
+  (aver (eq classoid (layout-classoid layout)))
+  (or (not (layout-invalid layout))
       ;; Avoid accidentally reaching code that can't work.
       #+sb-xc-host (bug "(TYPEP x 'STANDARD-CLASSOID) can't be tested")
       #-sb-xc-host
@@ -831,26 +836,20 @@ between the ~A definition and the ~A definition"
                classoid (or error-context 'subtypep)))))
 
 (defun %ensure-both-classoids-valid (class1 class2 &optional errorp)
-  (do ((layout1 (classoid-wrapper class1) (classoid-wrapper class1))
-       (layout2 (classoid-wrapper class2) (classoid-wrapper class2))
+  (do ((layout1 (classoid-layout class1) (classoid-layout class1))
+       (layout2 (classoid-layout class2) (classoid-layout class2))
        (i 0 (+ i 1)))
-      ((and (not (wrapper-invalid layout1)) (not (wrapper-invalid layout2)))
+      ((and (not (layout-invalid layout1)) (not (layout-invalid layout2)))
        t)
     (aver (< i 2))
     (unless (and (%ensure-classoid-valid class1 layout1 errorp)
                  (%ensure-classoid-valid class2 layout2 errorp))
       (return-from %ensure-both-classoids-valid nil))))
 
-;;; Simple methods for TYPE= and SUBTYPEP should never be called when
-;;; the two classes are equal, since there are EQ checks in those
-;;; operations.
-(define-type-method (classoid :simple-=) (type1 type2)
-  (aver (not (eq type1 type2)))
-  (values nil t))
-
 (define-type-method (classoid :simple-subtypep) (class1 class2)
+  ;; Simple method should never be called on EQ args since there is an EQ check higher up
   (aver (not (eq class1 class2)))
-  (with-world-lock () ; FIXME: why such coarse lock granularity here?
+  (with-world-lock ()
     (if (%ensure-both-classoids-valid class1 class2)
         (let ()
           (if (get-subclassoid class1 class2)
@@ -879,8 +878,8 @@ between the ~A definition and the ~A definition"
         ;; FIXME: should we put more locking here?
         ;; [contrast with define-type-method (classoid :simple-subtypep)]
         (collect ((res *empty-type* type-union))
-          (do-subclassoids ((subclass wrapper) sealed)
-            (declare (ignore wrapper))
+          (do-subclassoids ((subclass layout) sealed)
+            (declare (ignore layout))
             (when (get-subclassoid subclass other)
               (res (specifier-type subclass))))
           (res))
@@ -954,7 +953,7 @@ between the ~A definition and the ~A definition"
 
 (define-type-method (classoid :negate) (type) (make-negation-type type))
 
-(define-type-method (classoid :unparse) (type)
+(define-type-method (classoid :unparse) (flags type)
   (classoid-proper-name type))
 
 ;;;; built-in classes
@@ -1044,7 +1043,7 @@ between the ~A definition and the ~A definition"
               :prototype-form sb-pcl:+slot-unbound+)
          (fdefn :codes (,sb-vm:fdefn-widetag)
                 :predicate fdefn-p
-                :prototype-form (find-or-create-fdefn 'sb-mop:class-prototype))
+                :prototype-form (find-or-create-fdefn '(setf car)))
          (random-class             ; used for unknown type codes
           ;; Make the PROTOTYPE slot unbound.
           :prototype-form sb-pcl:+slot-unbound+)
@@ -1119,7 +1118,7 @@ between the ~A definition and the ~A definition"
          (fixnum
           :translation (integer ,most-negative-fixnum ,most-positive-fixnum)
           :inherits (integer rational real number)
-          :codes ,(mapcar #'symbol-value sb-vm::fixnum-lowtags)
+          :codes ,sb-vm::fixnum-lowtags
           :prototype-form 42)
          (bignum
           :translation (and integer (not fixnum))
@@ -1252,7 +1251,7 @@ between the ~A definition and the ~A definition"
          ;;
          ;; Essentially the hardwiring corresponds to the indices of the
          ;; respective types in the inherits vector for FD-STREAM.
-         ;;  * (wrapper-inherits (find-layout 'fd-stream))
+         ;;  * (layout-inherits (find-layout 'fd-stream))
          ;;  #(#<LAYOUT for T {50300003}>
          ;;    #<LAYOUT for STRUCTURE-OBJECT {50300103}>
          ;;    #<LAYOUT for STREAM {50301003}>
@@ -1355,7 +1354,7 @@ between the ~A definition and the ~A definition"
                       (setf (classoid-cell-classoid
                              (find-classoid-cell name :create t))
                             (!make-built-in-classoid
-                             :%bits (pack-ctype-bits classoid name)
+                             :%bits (classoid-bits name)
                              :name name
                              :translation #+sb-xc-host (if trans-p :initializing nil)
                                           #-sb-xc-host translation
@@ -1374,8 +1373,8 @@ between the ~A definition and the ~A definition"
                 (map 'simple-vector
                      (lambda (x)
                        (let ((super-layout
-                               (classoid-wrapper (find-classoid x))))
-                         (when (minusp (wrapper-depthoid super-layout))
+                               (classoid-layout (find-classoid x))))
+                         (when (minusp (layout-depthoid super-layout))
                            (setf hierarchical-p nil))
                          super-layout))
                      inherits-list))
@@ -1416,24 +1415,23 @@ between the ~A definition and the ~A definition"
 ;;; Remove class from all superclasses
 ;;; too (might not be registered, so might not be in subclasses of the
 ;;; nominal superclasses.)  We set the layout-clos-hash slots to 0 to
-;;; invalidate the wrappers for specialized dispatch functions, which
+;;; invalidate the layouts for specialized dispatch functions, which
 ;;; use those slots as indexes into tables.
-(defun %invalidate-layout (wrapper)
-  (declare (type wrapper wrapper))
-  #+sb-xc-host (error "Can't invalidate layout ~S" wrapper)
+(defun %invalidate-layout (layout)
+  (declare (type layout layout))
+  #+sb-xc-host (error "Can't invalidate layout ~S" layout)
   #-sb-xc-host
   (progn
-    (setf (wrapper-invalid wrapper) t)
+    (setf (layout-invalid layout) t)
     ;; Ensure that the INVALID slot conveying ancillary data describing the
     ;; invalidity reason is published before causing the invalid layout trap.
     (sb-thread:barrier (:write))
-    #+metaspace (setf (layout-clos-hash (wrapper-friend wrapper)) 0)
-    (setf (wrapper-clos-hash wrapper) 0)
-    (let ((inherits (wrapper-inherits wrapper))
-          (classoid (wrapper-classoid wrapper)))
+    (setf (layout-clos-hash layout) 0)
+    (let ((inherits (layout-inherits layout))
+          (classoid (layout-classoid layout)))
       (%modify-classoid classoid)
       (dovector (super inherits)
-        (remove-subclassoid classoid (wrapper-classoid super)))))
+        (remove-subclassoid classoid (layout-classoid super)))))
   (values))
 
 ;;;; cold loading initializations
@@ -1444,14 +1442,14 @@ between the ~A definition and the ~A definition"
 ;;; late in the build-order.lisp-expr sequence, and be put in
 ;;; !COLD-INIT-FORMS there?
 (defun !class-finalize ()
-  (dohash ((name wrapper) *forward-referenced-wrappers*)
+  (dohash ((name layout) *forward-referenced-layouts*)
     (let ((class (find-classoid name nil)))
       (cond ((not class)
              (error "How is there no classoid for ~S ?" name))
-            ((eq (classoid-wrapper class) wrapper)
-             (remhash name *forward-referenced-wrappers*))
+            ((eq (classoid-layout class) layout)
+             (remhash name *forward-referenced-layouts*))
             (t
              (error "Something strange with forward layout for ~S:~%  ~S"
-                    name wrapper))))))
+                    name layout))))))
 
 (!defun-from-collected-cold-init-forms !classes-cold-init)
