@@ -11,12 +11,16 @@
 
 (in-package "SB-IMPL")
 
-;;;; We often use a source-transform to do macro-like rewriting of an
-;;;; ordinary function call. Source-transforms seem to pre-date the ANSI
-;;;; specification and are redundant with compiler-macros.
-;;;; In the interest of not multiplying entities needlessly, it should
-;;;; be feasible to get rid of source-transforms.
-;;;; A problem is namespace clobbering: these must not affect the host Lisp.
+;;;; Source transforms are used by the compiler to make code more
+;;;; canonical so that the compiler can compile it futher; they are
+;;;; not optional. Compiler macros are an optional source rewriting
+;;;; mechanism mainly for compile-time syntax checking and
+;;;; optimizations that can be declined for any reason, but especially
+;;;; through the use of NOTINLINE. Perhaps the actual mechanism
+;;;; outside the decision to do rewriting could be reunified. We also
+;;;; must pay special attention when writing compiler macros for the
+;;;; purposes of cross-compiling. A problem is namespace clobbering:
+;;;; these must not affect the host Lisp.
 
 ;;; A sanity-checker for an extremely common programmer error.
 (define-compiler-macro format (&whole form destination control &rest args)
@@ -26,6 +30,7 @@
   form)
 
 (eval-when (:compile-toplevel :load-toplevel :execute)
+(declaim (inline maybe-note-read-from-string-signature-issue))
 (defun maybe-note-read-from-string-signature-issue (eof-error-p)
   ;; The interface is so unintuitive that we explicitly check for the common
   ;; error.
@@ -36,7 +41,8 @@
                 eof-error-p 'read-from-string)
     t)))
 
-(define-compiler-macro read-from-string (&whole form string &rest args)
+(define-compiler-macro read-from-string (&whole form string &rest args
+                                         &environment env)
   ;; Check this at compile-time, and rewrite it so we're silent at runtime.
   (destructuring-bind (&optional (eof-error-p t) eof-value &rest keys) args
     (if (maybe-note-read-from-string-signature-issue eof-error-p)
@@ -51,12 +57,15 @@
                  form ; Odd number of keys, punt.
                  (let ((positionals (list (copy-symbol 'string)
                                           (copy-symbol 'eof-error-p)
-                                          (copy-symbol 'eof-value))))
+                                          (copy-symbol 'eof-value)))
+                       (fun-name (if (sb-c:policy env (= safety 3))
+                                     '%read-from-string/safe
+                                     '%read-from-string)))
                    `(let (,@(mapcar #'list positionals
                                     (list string eof-error-p eof-value))
                           ,@(nreverse bind))
                       ,@(when ignore `((declare (ignore ,@ignore))))
-                      (%read-from-string ,@positionals ,@list)))))
+                      (,fun-name ,@positionals ,@list)))))
           (let* ((key (pop keys))
                  (index (case key
                           (:start 0)
@@ -64,9 +73,11 @@
                           (:preserve-whitespace 2)
                           (otherwise (return-from read-from-string form))))
                  (var (if (logbitp index seen)
-                          (let ((x (sb-xc:gensym "IGNORE")))
+                          (let ((x (gensym "IGNORE")))
                             (push x ignore)
                             x)
                           (setf seen (logior (ash 1 index) seen)
                                 (nth index list) (copy-symbol key)))))
             (push (list var (pop keys)) bind))))))
+
+(defmacro def!struct (&rest args) `(defstruct ,@args))
