@@ -149,16 +149,17 @@
   ;; debugging.
   (show-and-call !format-cold-init)
   (unless (!c-runtime-noinform-p)
-    ;; I'd like FORMAT to remain working in cold-init, where it does work,
-    ;; hence the conditional.
-    #+(or x86 x86-64) (format t "COLD-INIT... ")
-    #-(or x86 x86-64) (write-string "COLD-INIT... "))
+    (write-string "COLD-INIT... "))
 
   ;; Anyone might call RANDOM to initialize a hash value or something;
   ;; and there's nothing which needs to be initialized in order for
   ;; this to be initialized, so we initialize it right away.
   (show-and-call !random-cold-init)
 
+  ;; We can't do such much as a simple PROCLAIM without this global
+  ;; hash-table (because of WARN-IF-INLINE-FAILED/PROCLAIM)
+  (setf sb-c::*emitted-full-calls* (make-hash-table :test 'equal :synchronized t))
+
   ;; All sorts of things need INFO and/or (SETF INFO).
   (/show0 "about to SHOW-AND-CALL !GLOBALDB-COLD-INIT")
   (show-and-call !globaldb-cold-init)
@@ -215,8 +216,7 @@
   (show-and-call sb-kernel::!set-up-structure-object-class)
 
   (unless (!c-runtime-noinform-p)
-    #+(or x86 x86-64) (format t "[Length(TLFs)=~D]" (length *!cold-toplevels*))
-    #-(or x86 x86-64) (write `("Length(TLFs)=" ,(length *!cold-toplevels*)) :escape nil))
+    (write `("Length(TLFs)=" ,(length *!cold-toplevels*)) :escape nil))
 
   (setq sb-pcl::*!docstrings* nil) ; needed before any documentation is set
   (setq sb-c::*queued-proclaims* nil) ; needed before any proclaims are run
@@ -414,6 +414,7 @@ process to continue normally."
     (setf (extern-alien "internal_errors_enabled" int) 1)
     (float-cold-init-or-reinit))
   (gc-reinit)
+  (finalizers-reinit)
   (foreign-reinit)
   #+win32 (reinit-internal-real-time)
   ;; If the debugger was disabled in the saved core, we need to
