@@ -185,7 +185,7 @@
            (exp-marker (if (and marker-pos
                                 (char-not-equal (char string marker-pos) #\E))
                            (char-upcase (char string marker-pos))
-                           (ecase *read-default-float-format*
+                           (ecase cl:*read-default-float-format*
                             ((cl:single-float cl:short-float) #\F)
                             ((cl:double-float cl:long-float)  #\D))))
            (significand (if marker-pos (subseq string 0 marker-pos) string))
@@ -287,9 +287,9 @@
                                             (rest values)
                                             values)
                                         table))))
+          (setf (cdr cache) (hash-table-count table))
           (when cl:*compile-verbose*
-            (format t "~&; Float-ops cache prefill: ~D entries~%"
-                    (setf (cdr cache) (hash-table-count table)))))))
+            (format t "~&; Float-ops cache prefill: ~D entries~%" (cdr cache))))))
     table))
 
 (defun record-math-op (key &rest values)
@@ -381,7 +381,7 @@
                  ((null value) ; must not be one of the exceptional symbols
                   (setf (flonum-%value x) (calculate-flonum-value x)))
                  (t ; infinity or minus-zero
-                  (error "~S has no portable value" x)))))
+                  (error "~S (~D) has no portable value" value x)))))
         (t (error "Got host float"))))
 
 (defun realnumify* (args) (mapcar #'realnumify args))
@@ -475,15 +475,19 @@
     (return-from coerce object))
   (with-memoized-math-op (coerce (list object type))
     (if (realp object)
-        (let ((actual-type (if (member type '(double-float long-float))
-                               'double-float
-                               'single-float))
-              (source-value (realnumify object)))
-          (make-flonum (cl:coerce source-value
-                                  (ecase actual-type
-                                    (single-float 'cl:single-float)
-                                    (double-float 'cl:double-float)))
-                       actual-type))
+        (if (and (floatp object)
+                 (eq (flonum-%value object) :minus-zero)
+                 (member type '(double-float single-float)))
+            (make-flonum :minus-zero type)
+            (let ((actual-type (if (member type '(double-float long-float))
+                                   'double-float
+                                   'single-float))
+                  (source-value (realnumify object)))
+              (make-flonum (cl:coerce source-value
+                                      (ecase actual-type
+                                        (single-float 'cl:single-float)
+                                        (double-float 'cl:double-float)))
+                           actual-type)))
         (error "Can't COERCE ~S ~S" object type))))
 
 (macrolet ((define (name)
@@ -714,28 +718,28 @@
 ;;; These use "#." so that they are dumped as literals rather than having to
 ;;; call read-from-string at load-time (and failing) due to the reader intercept.
 ;;      #define __FLT_MAX__ 3.40282347e+38F
-(defconstant sb-xc:most-positive-single-float
+(defconstant most-positive-single-float
   #.(make-flonum (read-from-string "+3.40282347F38") 'single-float))
-(defconstant sb-xc:most-negative-single-float
+(defconstant most-negative-single-float
   #.(make-flonum (read-from-string "-3.40282347F38") 'single-float))
 
 ;;      #define __DBL_MAX__ 1.7976931348623157e+308
-(defconstant sb-xc:most-positive-double-float
+(defconstant most-positive-double-float
   #.(make-flonum (read-from-string "+1.7976931348623157D308") 'double-float))
-(defconstant sb-xc:most-negative-double-float
+(defconstant most-negative-double-float
   #.(make-flonum (read-from-string "-1.7976931348623157D308") 'double-float))
 
 ;;; PI is needed in order to build the cross-compiler mainly so that vm-fndb
 ;;; can define bounds on irrational functions.
-(defconstant sb-xc:pi
+(defconstant pi
   #.(make-flonum (read-from-string "3.14159265358979323846264338327950288419716939937511L0")
                  'double-float))
 
 (eval-when (:compile-toplevel :execute) (setq sb-cold::*choke-on-host-irrationals* t))
 
-;;; These two constants are used in 'late-type'
-(defconstant sb-xc:most-positive-long-float sb-xc:most-positive-double-float)
-(defconstant sb-xc:most-negative-long-float sb-xc:most-negative-double-float)
+;;; These two constants are used in 'type'
+(defconstant most-positive-long-float most-positive-double-float)
+(defconstant most-negative-long-float most-negative-double-float)
 
 (defun substitute-minus-zero (list)
   (substitute $0.0d0
@@ -758,7 +762,17 @@
              `(let ((x (car args)) (y (cadr args)))
                 (and (floatp x) (floatp y) (eql (flonum-%bits x) (flonum-%bits y)))))
            (two-zeros-p ()
-             `(and (eql nargs 2) (zerop (car args)) (zerop (cadr args)))))
+             `(and (eql nargs 2) (zerop (car args)) (zerop (cadr args))))
+           (same-sign-infinities-p ()
+             `(and (eql nargs 2)
+                   (floatp (car args))
+                   (floatp (cadr args))
+                   (member (flonum-%value (car args)) '(:-infinity :+infinity))
+                   (eq (flonum-%value (cadr args)) (flonum-%value (car args)))))
+           (infinity-p (n kind)
+             `(let ((arg (nth ,n args)))
+                (and (floatp arg)
+                     (eq (flonum-%value arg) ,kind)))))
 
   ;; Simple case of using the interceptor to return a boolean.  If
   ;; infinity leaks in, the host will choke since those are
@@ -768,10 +782,24 @@
 
   ;; Simple case of using the interceptor to return a float.
   ;; As above, no funky values allowed.
-  (intercept (max min + sqrt acos acosh asin asinh atan atanh conjugate cos cosh fceiling ffloor fround ftruncate phase sin sinh tan tanh) (&rest args)
+  (intercept (max min + acos acosh asin asinh atan atanh conjugate cos cosh phase sin sinh tan tanh) (&rest args)
     (dispatch :me
      (make-flonum (apply #':me (realnumify* args))
                   (apply #'pick-result-format args))))
+  (intercept (fceiling ffloor fround ftruncate) (&rest args)
+             (dispatch :me
+                       (multiple-value-bind (a b) (apply #':me (realnumify* args))
+                         (let ((format (apply #'pick-result-format args)))
+                          (values (make-flonum a format)
+                                  (make-flonum b format))))))
+
+  (intercept (sqrt) (&rest args)
+    (destructuring-bind (x) args
+      (if (zerop x)
+          x
+          (dispatch :me
+                    (make-flonum (funcall #':me (realnumify x))
+                                 (pick-result-format x))))))
 
   (intercept (*) (&rest args)
     (dispatch :me
@@ -833,6 +861,19 @@
                          (eql b $2.0d0))))
              nil)
             ((two-zeros-p) t) ; signed zeros are equal
+            ((same-sign-infinities-p) t) ; infinities are =
+            ((and (eql nargs 2)
+                  (infinity-p 0 :+infinity))
+             t)
+            ((and (eql nargs 2)
+                  (infinity-p 1 :+infinity))
+             nil)
+            ((and (eql nargs 2)
+                  (infinity-p 0 :-infinity))
+             nil)
+            ((and (eql nargs 2)
+                  (infinity-p 1 :-infinity))
+             t)
             ((and (eql nargs 2) (zerop (cadr args)))
              ;; Need this case if the first arg is represented as bits
              (if (rationalp (car args))
@@ -851,16 +892,29 @@
                (if (and (eq (flonum-format a) (flonum-format c))
                         (or (eq (flonum-format b) (flonum-format a))
                             (eq (flonum-format b) 'single-float)))
-                   (cond ((and (eql a sb-xc:most-negative-single-float)
-                               (eql c sb-xc:most-positive-single-float))
+                   (cond ((and (eql a most-negative-single-float)
+                               (eql c most-positive-single-float))
                           (not (float-infinity-p b)))
-                         ((and (eql a sb-xc:most-negative-double-float)
-                               (eql c sb-xc:most-positive-double-float))
+                         ((and (eql a most-negative-double-float)
+                               (eql c most-positive-double-float))
                           (not (float-infinity-p b)))
                          (t
                           (error "Unhandled")))
                    (error "Unhandled"))))
             ((two-zeros-p) t) ; signed zeros are equal
+            ((same-sign-infinities-p) t) ; infinities are =
+            ((and (eql nargs 2)
+                  (infinity-p 0 :+infinity))
+             nil)
+            ((and (eql nargs 2)
+                  (infinity-p 1 :+infinity))
+             t)
+            ((and (eql nargs 2)
+                  (infinity-p 0 :-infinity))
+             t)
+            ((and (eql nargs 2)
+                  (infinity-p 1 :-infinity))
+             nil)
             ((and (eql nargs 2) (zerop (cadr args)))
              ;; Need this case if the first arg is represented as bits
              (if (floatp (car args))
@@ -881,7 +935,8 @@
 (defun sb-c::maybe-exact-reciprocal (x)
   (cond ((eql x $2.0d0) $.5d0)
         ((eql x $10.0d0) nil)
-        ((eql x #.(make-flonum #x3FE62E42FEFA39EF 'double-float))
+        ((or (eql x #.(make-flonum #x3FE62E42FEFA39EF 'double-float))
+             (eql x #.(make-flonum #x3FF71547652B82FE 'double-float)))
          nil)
         ((eql x #.(make-flonum #x49742400 'single-float))
          nil)
@@ -992,6 +1047,7 @@
   (assert (eq (make-flonum :+infinity format) (sb-xc:- (make-flonum :-infinity format))))
   (assert (eq (make-flonum :-infinity format) (sb-xc:- (make-flonum :+infinity format))))
   (assert (eq (sb-xc:- (coerce 0 format)) (make-flonum :minus-zero format)))
+  (let ((*break-on-signals* nil))
   (flet ((assert-not-number (x)
            (handler-case (realnumify x)
              (:no-error (x) (error "Expected an error, got ~S" x))
@@ -1001,7 +1057,7 @@
       (assert-not-number nan)
       (assert (float-nan-p nan)))
     (dolist (symbol '(:+infinity :-infinity :minus-zero))
-      (assert-not-number (make-flonum symbol format)))))
+      (assert-not-number (make-flonum symbol format))))))
 
 #+host-quirks-sbcl ; Cross-check some more things if we can
 (loop for (our-symbol host-single host-double)
