@@ -45,14 +45,17 @@
 
 (in-package "SB-IMPL")
 
+(sb-c::unless-vop-existsp (:named sb-vm::%set-funinstance-info)
 (declaim (inline (setf %funcallable-instance-info)))
 ;;; Funcallable instances are just like closures, but there's another slot or two
 ;;; depending on whether the layout pointer is in a slot or in the header word.
+;;; In retrospect, backends may want to emit different code for funcallable-instance
+;;; so it may not have been wise to reduce to the closure setter.
 (defun (setf %funcallable-instance-info) (newval fin index)
   (%closure-index-set fin (+ index (- sb-vm:funcallable-instance-info-offset
                                       sb-vm:closure-info-offset))
                       newval)
-  newval)
+  newval))
 ;;; This is just to keep the DEFSTRUCT logic consistent with %INSTANCE-SET,
 ;;; but the canonical setter is the function named (setf %funcallable-instance-info)
 (declaim (inline %set-funcallable-instance-info))
@@ -171,9 +174,6 @@
 (defsetf subseq (sequence start &optional end) (v)
   `(progn (replace ,sequence ,v :start1 ,start :end1 ,end) ,v))
 
-;;; from fdefinition.lisp
-(defsetf fdefinition %set-fdefinition)
-
 ;;; from kernel.lisp
 #-darwin-jit
 (progn
@@ -361,3 +361,21 @@ with bits from the corresponding position in the new value."
 (defun 1-arg-t   (a) (declare (ignore a)) t)
 (defun 2-arg-nil (a b) (declare (ignore a b)) nil)
 (defun 3-arg-nil (a b c) (declare (ignore a b c)) nil)
+
+(in-package "SB-VM")
+
+(defun blt-copier-for-widetag (x)
+  (declare ((mod 256) x))
+  (aref (load-time-value
+         (map-into (make-array 32)
+                   (lambda (x) (and x
+                                    (symbol-function x)))
+                   '#.(let ((a (make-array 32 :initial-element nil)))
+                        (dovector (saetp *specialized-array-element-type-properties* a)
+                          (when (and (not (member (saetp-specifier saetp) '(t nil)))
+                                     (<= (saetp-n-bits saetp) n-word-bits))
+                            (setf (svref a (logand #x1F (ash (saetp-typecode saetp) -2)))
+                                  (intern (format nil "UB~D-BASH-COPY" (saetp-n-bits saetp))
+                                          "SB-KERNEL"))))))
+         t)
+        (logand #x1F (ash x -2))))
