@@ -135,6 +135,7 @@
                             name
                             (classoid-state class))))))
              #-sb-xc-host (:pure nil))
+  ;;; KLUDGE: Keep synchronized with hardcoded slot order in 'instance.inc'
   ;; the value to be returned by CLASSOID-NAME.
   (name nil :type symbol)
   ;; the current LAYOUT for this class, or NIL if none assigned yet
@@ -162,13 +163,6 @@
   ;; we don't just call it the CLASS slot) object for this class, or
   ;; NIL if none assigned yet
   (pcl-class nil))
-;;; For low-level C code if it wants to display a classoid
-(defconstant classoid-name-wordindex
-  (+ 2 #-compact-instance-header 1))  ; <Header, Layout (if needed), %BITS, NAME>
-#-sb-xc-host
-(eval-when (:compile-toplevel)
-  (assert (= (+ sb-vm:instance-slots-offset (get-dsd-index classoid name))
-             classoid-name-wordindex)))
 
 ;;; The LAYOUT structure is pointed to by the first cell of instance
 ;;; (or structure) objects. It represents what we need to know for
@@ -266,6 +260,9 @@
   ;; Could be the generalized function, or a type-specific one
   ;; if the defstruct was compiled in a policy of SPEED 3.
   (equalp-impl #'equalp-err :type (sfunction (t t) boolean) :read-only t)
+  ;; Information for the quicker variant of SLOT-VALUE on STRUCTURE-OBJECT
+  #+64-bit
+  (struct-slot-map nil)
   ;; Information about slots in the class to PCL: this provides fast
   ;; access to slot-definitions and locations by name, etc.
   ;; See MAKE-SLOT-TABLE in pcl/slots-boot.lisp for further details.
@@ -344,6 +341,10 @@
            ;; Give T a 0 bitmap. It's arbitrary, but when we need some layout
            ;; that has this bitmap we can use the layout of T.
            ((eq layout (find-layout t)) 0)
+           ;; KLUDGE: PROMISE-COMPILE stuffs the layout of FUNCTION into the
+           ;; funcallable-instance it produces.  gencgc verifies that funcallable-instances
+           ;; have a bitmap with 0 bits where the trampoline word and layout are.
+           #-compact-instance-header ((eq layout (find-layout 'function)) -4)
            (t
             +layout-all-tagged+)))
   (defun %layout-bitmap (layout) (layout-bitmap layout))
@@ -387,24 +388,6 @@
   (aver (not (consp (layout-%info layout))))
   (setf (layout-%info layout) newval))
 
-;;; See the pictures above DD-BITMAP in src/code/defstruct for the details.
-(defconstant standard-gf-primitive-obj-layout-bitmap
-  ;; layout has no bitmap bit. The other slots all look tagged even though
-  ;; it's an accident that the raw instruction words resemble fixnums.
-  ;; I should probably find a more principled way to express this,
-  ;; but the old - technically correct - way based on a bitmap really made
-  ;; the logic look like crap. But now that the variable-length part of a
-  ;; funcallable-instance follows its embedded trampoline, it would be simple
-  ;; to alter the GC to skip a fixed number of slots.
-  ;; I think that is preferable to using a bitmap.
-  #+compact-instance-header -1
-  ;; the layout is considered untagged so that we don't doubly process it
-  ;; if walking tagged slots. The assumption is that layout is always
-  ;; handled separately, so therefore we don't want to process it again
-  ;; in a bitmap-based scan. And the entry point is considered untagged
-  ;; because it points to an assembler routine, thus ignorable.
-  #-compact-instance-header -4)
-
 #-sb-xc-host
 (progn
 (declaim (inline bitmap-start bitmap-nwords bitmap-all-taggedp))
@@ -422,44 +405,9 @@
           +layout-all-tagged+)
        ;; Then check that there are no additional words.
        (= (bitmap-nwords layout) 1)))
-
-;;; Extract the bitmap from 1 or more words of bits that have the same format
-;;; as a BIGNUM - least-significant word first, native bit order within each word,
-;;; all but the last are unsigned, and the last is signed.
-(defun %layout-bitmap (layout)
-  (declare (type layout layout))
-  (do ((accumulator 0)
-       (shift 0)
-       (index (bitmap-start layout) (1+ index))
-       (last (1- (%instance-length layout))))
-      ((> index last) accumulator)
-    (setf accumulator (logior accumulator
-                              (ash (if (= index last)
-                                       (%raw-instance-ref/signed-word layout index)
-                                       (%raw-instance-ref/word layout index))
-                                   shift)))
-    (incf shift sb-vm:n-word-bits)))
-
-(defun layout-bitmap (layout)
-  (declare (type layout layout))
-  ;; Whenever we call LAYOUT-BITMAP on a structure-object subtype,
-  ;; it's supposed to have the INFO slot populated, linking it to a defstruct-description.
-  (acond ((layout-info layout) (dd-bitmap it))
-         ;; Instances lacking DD-INFO are CLOS objects, which can't generally have
-         ;; raw slots, except that funcallable-instances may have 2 raw slots -
-         ;; the trampoline and the layout. The trampoline can have a tag, depending
-         ;; on the platform, and the layout is tagged but a special case.
-         ;; In any event, the bitmap is always 1 word, and there are no "extra ID"
-         ;; words preceding it.
-         (t
-          (aver (not (logtest +structure-layout-flag+ (layout-flags layout))))
-          (the fixnum
-               (%raw-instance-ref/signed-word layout
-                                              (type-dd-length layout))))))
 #+64-bit
 (defmacro layout-length (layout) ; SETFable
   `(ldb (byte 16 16) (layout-flags ,layout)))
-
 ) ; end PROGN #-sb-xc-host
 
 ;;; True of STANDARD-OBJECT, which include generic functions.
