@@ -108,17 +108,16 @@
   ;; +MAGIC-HASH-VECTOR-VALUE+ represents address-based hashing on the
   ;; respective key.
   (hash-vector nil :type (or null (simple-array hash-table-index (*))))
-  ;; flags: WEAKNESS | KIND | WEAKP | FINALIZERSP | USERFUNP | SYNCHRONIZEDP
+  ;; flags: WEAKNESS | KIND | WEAKP | {notused} | USERFUNP | SYNCHRONIZED
   ;; WEAKNESS is 2 bits, KIND is 2 bits, the rest are 1 bit each
   ;;   - WEAKNESS     : {K-and-V, K, V, K-or-V}, irrelevant unless WEAKP
   ;;   - KIND         : {EQ, EQL, EQUAL, EQUALP}, irrelevant if USERFUNP
   ;;   - WEAKP        : table is weak
-  ;;   - FINALIZERSP  : table is the global finalizer store
   ;;   - USERFUNP     : table has a nonstandard hash function
-  ;;   - SYCHRONIZEDP : all operations are automatically guarded by a mutex
-  ;; If you change these, be sure to check the definition of hash_table_weakp()
-  ;; in 'gc-private.h'
-  (flags 0 :type (unsigned-byte 8) :read-only t)
+  ;;   - SYNCHRONIZED : all operations are automatically guarded by a mutex
+  ;; If you change these, be sure to check the definitions of hash_table_weakp()
+  ;; and other autogenerated C code (see WRITE-HASH-TABLE-FLAG-EXTRACTORS)
+  (flags 0 :type sb-vm:word :read-only t)
   ;; Used for locking GETHASH/(SETF GETHASH)/REMHASH
   ;; The lock is always created for synchronized tables, or created just-in-time
   ;; with nonsynchronized tables that are guarded by WITH-LOCKED-HASH-TABLE
@@ -169,10 +168,7 @@
   ;; this counter is incremented if the fast-read-lock (implicit in the
   ;; 'stamp' field) implies that there was an inconsistent view of the table
   #+hash-table-metrics (n-stamp-change 0 :type word)
-
-  ;; only for debugging system bootstrap when hash-tables are completely
-  ;; broken (which seems to be quite often as I optimize them)
-  #+hash-table-simulate (%alist))
+  )
 
 (sb-xc:defstruct (general-hash-table (:copier nil)
                              (:conc-name hash-table-)
@@ -201,21 +197,59 @@
   (smashed-cells nil)
   ;; This slot is used to link weak hash tables during GC. When the GC
   ;; isn't running it is always NIL.
-  (next-weak-hash-table nil :type null)
-  ;; List of values (i.e. the second half of the k/v pair) culled out during
-  ;; GC, used only by the finalizer hash-table. This informs Lisp of the IDs
-  ;; (small fixnums) of the finalizers that need to run.
-  (culled-values nil :type list))
+  (next-weak-hash-table nil :type null))
+
+(defconstant hash-table-weak-flag         8)
+;;; USERFUN-FLAG implies a nonstandard hash function. Such tables may also have
+;;; a custom comparator. But you can't have a custom comparator without a custom
+;;; hash, because there's no way in general to produce a compatible hash.
+(defconstant hash-table-userfun-flag      2)
+(defconstant hash-table-synchronized-flag 1)
+
+;;; Keep in sync with weak_ht_alivep_funs[] in gc-common
+(defconstant +ht-weak-key-AND-value+ 0)
+(defconstant +ht-weak-key+           1)
+(defconstant +ht-weak-value+         2)
+(defconstant +ht-weak-key-OR-value+  3)
 
 (sb-xc:defmacro hash-table-lock (table)
   `(let ((ht ,table)) (or (hash-table-%lock ht) (install-hash-table-lock ht))))
 
-(sb-xc:defmacro pack-ht-flags-weakness (x) `(logior (ash ,x 6) hash-table-weak-flag))
-(sb-xc:defmacro ht-flags-weakness (flags) `(ldb (byte 2 6) ,flags))
+(defmacro pack-ht-flags-weakness (x) `(logior (ash ,x 6) hash-table-weak-flag))
+(defmacro ht-flags-weakness (flags) `(ldb (byte 2 6) ,flags))
 ;;; KIND corresponds directly to the HASH-TABLE-TEST for the 4 standard tests,
 ;;; but is not meaningful with a user-provided test or hash function.
 (sb-xc:defmacro pack-ht-flags-kind (x) `(ash ,x 4))
-(sb-xc:defmacro ht-flags-kind (flags) `(ldb (byte 2 4) ,flags))
+(defmacro ht-flags-kind (flags) `(ldb (byte 2 4) ,flags))
+
+(eval-when (:compile-toplevel :load-toplevel :execute)
+  (defconstant default-rehash-size $1.5)
+  ;; Don't raise this number to 8 - if you do it'll increase the memory
+  ;; consumption of a default MAKE-HASH-TABLE call by 7% just due to
+  ;; padding slots.  This is a "perfect" minimal size.
+  (defconstant +min-hash-table-size+ 7))
+
+(sb-xc:defmacro make-system-hash-table (&key test synchronized weakness)
+  (multiple-value-bind (kind args)
+      (cond ((equal test '(quote eq))  (values 0 '('eq  #'eq  #'eq-hash)))
+            ((equal test '(quote eql)) (values 1 '('eql #'eql #'eql-hash)))
+            (t
+             (bug "Incomplete implementation of MAKE-SYSTEM-HASH-TABLE")
+             0))
+    `(%make-hash-table
+      (logior ,(ecase weakness
+                (:key   (pack-ht-flags-weakness +ht-weak-key+))
+                (:value (pack-ht-flags-weakness +ht-weak-value+)))
+               (pack-ht-flags-kind ,kind)
+               ,(if synchronized 'hash-table-synchronized-flag 0))
+      ,@args
+      ;; Splicing these constant values at expansion time avoids a problem invoking
+      ;; MAKE-SYSTEM-HASH-TABLE before the constants are known to make-host-2, as happens
+      ;; when compiling type-class. hash-table.lisp can't be moved earlier in build-order
+      ;; (without pain) so the expanded code can't use the values.
+      ,+min-hash-table-size+
+      ,default-rehash-size
+      $1.0)))
 
 ;; Our hash-tables store precomputed hashes to speed rehash and to guard
 ;; the call of the general comparator.
