@@ -25,6 +25,7 @@
 
 ;;; Return T if any pointers were replaced in a code object.
 (defun apply-forwarding-map (map print &aux any-change)
+  (declare (optimize (sb-c::aref-trapping 0)))
   (when print
     (let ((*print-pretty* nil))
       (dohash ((k v) map)
@@ -47,38 +48,32 @@
      (lambda (object widetag size &aux touchedp)
        (declare (ignore size))
        (macrolet ((rewrite (place &aux (accessor (if (listp place) (car place))))
-                    ;; These two slots have no setters, but nor are
-                    ;; they possibly affected by code folding.
-                    (unless (member accessor '(symbol-name symbol-package))
+                    ;; SYMBOL-NAME, SYMBOL-PACKAGE, SYMBOL-%INFO have no setters,
+                    ;; but nor are they possibly affected by code folding.
+                    ;; {%INSTANCE,%FUN}-LAYOUT have setters but they are not spelled
+                    ;; SETF, nor are they affected by code folding.
+                    (unless (member accessor '(symbol-name symbol-package symbol-%info
+                                               %fun-layout %instance-layout))
                       `(let* ((oldval ,place) (newval (forward oldval)))
                          (unless (eq newval oldval)
                            ,(case accessor
                               (data-vector-ref
                                `(setf (svref ,@(cdr place)) newval touchedp t))
                               (value-cell-ref
-                               ;; pinned already because we're iterating over the heap
-                               ;; which disables GC, but maybe some day it won't.
-                               `(with-pinned-objects (object)
-                                  (setf (sap-ref-lispobj (int-sap (get-lisp-obj-address object))
-                                                         (- (ash value-cell-value-slot word-shift)
-                                                            other-pointer-lowtag))
-                                        newval)))
+                               `(%primitive value-cell-set object newval))
                               (weak-pointer-value
                                ;; Preserve gencgc invariant that a weak pointer
                                ;; can't point to an object younger than itself.
-                               `(cond #+gencgc
-                                      ((let ((newval-gen (generation-of newval)))
+                               `(cond ((let ((newval-gen (generation-of newval)))
                                          (and (fixnump newval-gen)
                                               (< newval-gen (generation-of object))))
                                        #+nil
                                        (warn "Can't update weak pointer ~s" object))
                                       (t
-                                       (with-pinned-objects (object)
-                                         (setf (sap-ref-lispobj
-                                                (int-sap (get-lisp-obj-address object))
-                                                (- (ash weak-pointer-value-slot word-shift)
-                                                   other-pointer-lowtag))
-                                               newval)))))
+                                       (%primitive set-slot object newval
+                                                   '(setf weak-pointer-value)
+                                                   weak-pointer-value-slot
+                                                   other-pointer-lowtag))))
                               (%primitive
                                (ecase (cadr place)
                                  (fast-symbol-global-value
@@ -89,8 +84,9 @@
          (do-referenced-object (object rewrite)
            (simple-vector
             :extend
-            (when (and (logtest (get-header-data object) vector-addr-hashing-subtype)
-                       touchedp)
+            (when (and touchedp
+                       (test-header-data-bit object (ash vector-addr-hashing-flag
+                                                         array-flags-data-position)))
               (setf (svref object 1) 1)))
            (code-component
             :override
@@ -114,6 +110,7 @@
             (let* ((oldval (%closure-fun object))
                    (newval (forward oldval)))
               (unless (eq newval oldval)
+                #+nil ; FIXME: gotta figure out GC marking situation here
                 (with-pinned-objects (object newval)
                   (setf (sap-ref-sap (int-sap (- (get-lisp-obj-address object)
                                                  fun-pointer-lowtag))
@@ -123,13 +120,10 @@
               (let* ((oldval (%closure-index-ref object i))
                      (newval (forward oldval)))
                 (unless (eq newval oldval)
-                  (with-pinned-objects (object)
-                    (setf (sap-ref-lispobj (int-sap (get-lisp-obj-address object))
-                                           (- (ash (+ i closure-info-offset) word-shift)
-                                              fun-pointer-lowtag))
-                          newval))))))
+                  (%closure-index-set object i newval)))))
            (ratio :override)
            ((complex rational) :override)
+           (weak-pointer :override)
            (t
             :extend
             (case widetag
@@ -280,14 +274,27 @@
                        sb-c::deftransform
                        :source-transform))))))
 
+;;; Using mark-region-gc, map-allocated-objects can miss objects because the 'allocated'
+;;; bits are not materialized until a garbage collection occurs. This can cause problems
+;;; for identical-code-folding as follows:
+;;; (1) after "Pass 1: count code objects" there could be an overrun of the code-objects
+;;;     array because "Pass 2: collect them" sees more objects then pass 1 saw.
+;;; (2) if, through extraordinarily bad luck, among the missed objects is #'MAKE-HASH-TABLE
+;;;     then the map-allocated-objects in apply-forwarding-map could fail to fix constants
+;;;     in that code header, which is among the worst possible objects to miss,
+;;;     as it cares very much about the identity of #'EQL. If #'EQL becomes #'%EQL
+;;;     or vice-versa, then whichever is bound to (symbol-function 'eql) had better be
+;;;     the same in the header constants of make-hash-table, or you're totally screwed.
 (defun fold-identical-code (&key aggressive preserve-docstrings (print nil))
   (loop
-    #+gencgc (gc :gen 7)
+    (gc :gen 7)
     ;; Pass 1: count code objects.  I'd like to enhance MAP-ALLOCATED-OBJECTS
     ;; to have a mode that scans only GC pages with that can hold code
     ;; (or any subset of page types). This is fine though.
     (let ((code-objects
-           (let ((count 0))
+           ;; arbitrary fudge factor because mark-region-gc can enumerate _more_
+           ;; objects on the second pass than on the first pass.
+           (let ((count 100))
              (map-allocated-objects
               (lambda (obj widetag size)
                 (declare (ignore size))
@@ -347,6 +354,8 @@
             ;; recurse into constants during the binning step.
             (constants (make-hash-table :test 'eq))
             (n 0))
+        ;; KLUDGE and FIXME: never mess with #'EQL for the time being
+        (setq code-objects (delete (fun-code-header #'eql) code-objects))
         (dovector (x code-objects)
           (when (or (not (gethash x referenced-objects))
                     (default-allow-icf-p x))
@@ -385,6 +394,7 @@
                                               (name (and entry
                                                          (sb-kernel:%fun-name entry))))
                                          (and (symbolp name)
+                                              (symbol-package name)
                                               (eq (nth-value 1 (find-symbol (symbol-name name)
                                                                             (symbol-package name)))
                                                   :external))))
