@@ -631,22 +631,22 @@
 (defun control-stack-exhausted-error ()
   (let ((sb-debug:*stack-top-hint* nil))
     (infinite-error-protect
-     (format *error-output*
-             "Control stack guard page temporarily disabled: proceed with caution~%")
+     (write-line "Control stack guard page temporarily disabled: proceed with caution"
+                 *error-output*)
      (error 'control-stack-exhausted))))
 
 (defun binding-stack-exhausted-error ()
   (let ((sb-debug:*stack-top-hint* nil))
     (infinite-error-protect
-     (format *error-output*
-             "Binding stack guard page temporarily disabled: proceed with caution~%")
+     (write-line "Binding stack guard page temporarily disabled: proceed with caution"
+                 *error-output*)
      (error 'binding-stack-exhausted))))
 
 (defun alien-stack-exhausted-error ()
   (let ((sb-debug:*stack-top-hint* nil))
     (infinite-error-protect
-     (format *error-output*
-             "Alien stack guard page temporarily disabled: proceed with caution~%")
+     (write-line "Alien stack guard page temporarily disabled: proceed with caution"
+                 *error-output*)
      (error 'alien-stack-exhausted))))
 
 ;;; KLUDGE: we keep a single HEAP-EXHAUSTED-ERROR object around, so
