@@ -139,6 +139,8 @@
 ;;; Lisp assembler routines are named by Lisp symbols, not strings,
 ;;; and so can be compared by EQ.
 (define-load-time-global *assembler-routines* nil)
+;;; word vector in static space of addresses of asm routines
+#+immobile-space (define-load-time-global *asm-routine-vector* nil)
 #-sb-xc-host (declaim (code-component *assembler-routines*))
 
 ;;; Output the current number of semicolons after a fresh-line.
@@ -321,10 +323,11 @@
 
 (defun nuke-fop-vector (vector)
   (declare (simple-vector vector)
-           #-gencgc (ignore vector)
+           (ignorable vector)
            (optimize speed))
   ;; Make sure we don't keep any garbage.
-  #+gencgc
+  ;; NOTE: for the work-in-progress concurrent GC, it is better *NOT* to 0-fill
+  ;; if the the deletion barrier is enabled.
   (fill vector 0))
 
 
@@ -678,6 +681,10 @@
                       nil
                       nil)))))))
 
+(defstruct fasl-group
+  (fun-names))
+
+(defvar *current-fasl-group*)
 ;;;
 ;;; a helper function for LOAD-AS-FASL
 ;;;
@@ -689,33 +696,36 @@
         (trace *show-fops-p*))
     (unless (check-fasl-header stream)
       (return-from load-fasl-group))
-    (catch 'fasl-group-end
-      (setf (svref (%fasl-input-table fasl-input) 0) 0)
-      (loop
-       (binding* ((pos (when trace (file-position stream)))
-                  ((byte function pushp n-operands arg1 arg2 arg3)
-                   (decode-fop fasl-input))
-                  (result
-                   (if (functionp function)
-                       (case n-operands
-                         (0 (funcall function fasl-input))
-                         (1 (funcall function fasl-input arg1))
-                         (2 (funcall function fasl-input arg1 arg2))
-                         (3 (funcall function fasl-input arg1 arg2 arg3)))
-                       (error "corrupt fasl file: FOP code #x~x" byte))))
-         (when pushp
-           (push-fop-stack result fasl-input))
-         (when trace
-           ;; show file pos prior to decoding the fop,
-           ;; table and stack ptrs *after* executing it
-           (format *trace-output* "~&~6x : [~D,~D] ~2,'0x~v@{ ~x~}"
-                   pos
-                   (svref (%fasl-input-table fasl-input) 0) ; table pointer
-                   (svref (%fasl-input-stack fasl-input) 0) ; stack pointer
-                   byte
-                   n-operands arg1 arg2 arg3)
-           (when (functionp function)
-             (format *trace-output* " ~35t~(~a~)" (%fun-name function)))))))))
+    (let ((*current-fasl-group* (make-fasl-group)))
+      (catch 'fasl-group-end
+        (setf (svref (%fasl-input-table fasl-input) 0) 0)
+        (loop
+         (binding* ((pos (when trace (file-position stream)))
+                    ((byte function pushp n-operands arg1 arg2 arg3)
+                     (decode-fop fasl-input))
+                    (result
+                     (if (functionp function)
+                         (case n-operands
+                           (0 (funcall function fasl-input))
+                           (1 (funcall function fasl-input arg1))
+                           (2 (funcall function fasl-input arg1 arg2))
+                           (3 (funcall function fasl-input arg1 arg2 arg3)))
+                         (error "corrupt fasl file: FOP code #x~x" byte))))
+           (when pushp
+             (push-fop-stack result fasl-input))
+           (when trace
+             ;; show file pos prior to decoding the fop,
+             ;; table and stack ptrs *after* executing it
+             (format *trace-output* "~&~6x : [~D,~D] ~2,'0x~v@{ ~x~}"
+                     pos
+                     (svref (%fasl-input-table fasl-input) 0) ; table pointer
+                     (svref (%fasl-input-stack fasl-input) 0) ; stack pointer
+                     byte
+                     n-operands arg1 arg2 arg3)
+             (when (functionp function)
+               (format *trace-output* " ~35t~(~a~)" (%fun-name function))
+               (when (eql (%fun-name function) 'fop-push)
+                 (format *trace-output* " ~(~A~)" (ref-fop-table fasl-input arg1)))))))))))
 
 (defun load-as-fasl (stream verbose print)
   (when (zerop (file-length stream))
@@ -765,17 +775,13 @@
         (n-data-words (- size sb-vm:instance-data-start)))
     (with-fop-stack ((stack (operand-stack)) ptr n-data-words)
       (declare (type index ptr))
-      ;; FIXME: this is basically DO-LAYOUT-BITMAP, but probably not as efficient.
-      (let ((bitmap (sb-kernel::%layout-bitmap layout)))
-        ;; Values on the stack are in the same order as in the structure itself.
-        (do ((i sb-vm:instance-data-start (1+ i)))
-            ((>= i size))
-          (declare (type index i))
-          (let ((val (fop-stack-ref ptr)))
-            (if (logbitp i bitmap)
-                (%instance-set res i val)
-                (%raw-instance-set/word res i val))
-            (incf ptr)))))
+      ;; Values on the stack are in the same order as in the structure itself.
+      (do-layout-bitmap (i taggedp layout size)
+        (let ((val (fop-stack-ref ptr)))
+          (if taggedp
+              (%instance-set res i val)
+              (%raw-instance-set/word res i val)))
+        (incf ptr)))
     res))
 
 ;;; Symbol-like entities
@@ -806,15 +812,12 @@
             (setf (slot-value instance slot-name) value))))
     instance))
 
-(define-fop 64 (fop-end-group () nil)
-  (throw 'fasl-group-end t))
-
-(define-fop 62 (fop-verify-table-size ((:operands expected-index)) nil)
-  (unless (= (svref (%fasl-input-table (fasl-input)) 0) expected-index)
-    (bug "fasl table of improper size")))
-(define-fop 63 (fop-verify-empty-stack () nil)
+(define-fop 64 (fop-end-group ((:operands table-size)) nil)
+  (unless (= (svref (%fasl-input-table (fasl-input)) 0) table-size)
+    (bug "fasl table of improper size"))
   (unless (fop-stack-empty-p (operand-stack))
-    (bug "fasl stack not empty when it should be")))
+    (bug "fasl stack not empty when it should be"))
+  (throw 'fasl-group-end t))
 
 ;;;; fops for loading symbols
 
@@ -1139,22 +1142,55 @@
     `(with-pinned-objects (,code-var) ,@body)))
 
 (define-load-time-global *show-new-code* nil)
+#+sb-xc-host
+(defun possibly-log-new-code (object reason)
+  (declare (ignore reason))
+  object)
+#-sb-xc-host
+(defun possibly-log-new-code (object reason &aux (show *show-new-code*))
+  (when show
+    (let ((size (code-object-size object))
+          (fmt "~&New code(~Db,~A): ~A~%")
+          (file "jit-code.txt")
+          (*print-pretty* nil))
+      ;; DISASSEMBLE is for limited debugging only.
+      ;; It may write garbled output if multiple threads
+      ;; I tried WITH-OPEN-STREAM during cold-init and got:
+      ;;   "vicious metacircle:  The computation of an effective method of
+      ;;    #<STREAM-FUNCTION COMMON-LISP:CLOSE (2)> for arguments of types
+      ;;    (#<STRUCTURE-CLASS SB-SYS:FD-STREAM>) uses the effective method
+      ;; being computed."
+      ;; so just leave the stream open. Or we could call the fd-stream misc routine.
+      (if (or (eq show 'disassemble) (streamp show))
+          (let ((f (if (streamp show)
+                       show
+                       (prog1
+                           (setq *show-new-code*
+                                 (open file :direction :output
+                                       :if-exists :append :if-does-not-exist :create))
+                         (format t "~&; Logging code allocation to ~S~%" file)))))
+            (format f fmt size reason object)
+            (disassemble object :stream f)
+            (terpri f)
+            (force-output f))
+          (format *trace-output* fmt (code-object-size object) reason object))))
+  object)
+
 (define-fop 17 :not-host (fop-load-code ((:operands header n-code-bytes n-fixup-elts)))
   ;; The stack looks like:
   ;; ... | constant0 constant1 ... constantN | DEBUG-INFO | FIXUPS-ITEMS ....   ||
   ;;     | <--------- n-constants ---------> |            | <-- n-fixup-elts -> ||
   (let* ((n-simple-funs (read-unsigned-byte-32-arg (fasl-input-stream)))
-         (n-named-calls (read-unsigned-byte-32-arg (fasl-input-stream)))
+         (n-fdefns (read-unsigned-byte-32-arg (fasl-input-stream)))
          (n-boxed-words (ash header -1))
          (n-constants (- n-boxed-words sb-vm:code-constants-offset))
          (stack-elts-consumed (+ n-constants 1 n-fixup-elts)))
     (with-fop-stack ((stack (operand-stack)) ptr stack-elts-consumed)
       ;; We've already ensured that all FDEFNs the code uses exist.
       ;; This happened by virtue of calling fop-fdefn for each.
-      (let ((stack-index (+ ptr (* n-simple-funs sb-vm:code-slots-per-simple-fun))))
-        (dotimes (i n-named-calls)
-          (aver (typep (svref stack stack-index) 'fdefn))
-          (incf stack-index)))
+      (loop for stack-index from (+ ptr (* n-simple-funs sb-vm:code-slots-per-simple-fun))
+            repeat n-fdefns
+            do (aver (typep (svref stack stack-index) 'fdefn)))
       (binding* (((code total-nwords)
                   (sb-c:allocate-code-object
                    (if (oddp header) :immobile :dynamic)
@@ -1163,42 +1199,37 @@
                  (real-code code)
                  (debug-info (svref stack (+ ptr n-constants))))
         (with-writable-code-instructions
-            (code total-nwords debug-info n-named-calls n-simple-funs)
+            (code total-nwords debug-info n-fdefns n-simple-funs)
           :copy (read-n-bytes (fasl-input-stream) (code-instructions code) 0 n-code-bytes)
           :fixup (sb-c::apply-fasl-fixups code stack (+ ptr (1+ n-constants)) n-fixup-elts real-code))
         ;; Don't need the code pinned from here on
-        (progn
-          (setf (sb-c::debug-info-source (%code-debug-info code))
-                (%fasl-input-partial-source-info (fasl-input)))
-          ;; Boxed constants can be assigned only after figuring out where the range
-          ;; of implicitly tagged words is, which requires knowing how many functions
-          ;; are in the code component, which requires reading the code trailer.
-          #+darwin-jit (sb-c::assign-code-constants
-                        code (subseq stack ptr (+ ptr n-constants)))
-          #-darwin-jit
-          (let* ((header-index sb-vm:code-constants-offset)
-                 (stack-index ptr))
+        (setf (sb-c::debug-info-source (%code-debug-info code))
+              (%fasl-input-partial-source-info (fasl-input)))
+        ;; Boxed constants can be assigned only after figuring out where the range
+        ;; of implicitly tagged words is, which requires knowing how many functions
+        ;; are in the code component, which requires reading the code trailer.
+        #+darwin-jit (sb-c::assign-code-constants code (subseq stack ptr (+ ptr n-constants)))
+        #-darwin-jit
+        (let* ((header-index sb-vm:code-constants-offset)
+               (stack-index ptr))
             (declare (type index header-index stack-index))
             (dotimes (n (* n-simple-funs sb-vm:code-slots-per-simple-fun))
               (setf (code-header-ref code header-index) (svref stack stack-index))
               (incf header-index)
               (incf stack-index))
-            (dotimes (i n-named-calls)
+            (dotimes (i n-fdefns)
               (sb-c::set-code-fdefn code header-index (svref stack stack-index))
               (incf header-index)
               (incf stack-index))
             (do () ((>= header-index n-boxed-words))
               (setf (code-header-ref code header-index) (svref stack stack-index))
               (incf header-index)
-              (incf stack-index))))
-        (when *show-new-code*
-          (let ((*print-pretty* nil))
-            (format t "~&New code(~Db,load): ~A~%" (code-object-size code) code)))
+              (incf stack-index)))
         (when (typep (code-header-ref code (1- n-boxed-words))
                      '(cons (eql sb-c::coverage-map)))
           ;; Record this in the global list of coverage-instrumented code.
           (atomic-push (make-weak-pointer code) (cdr *code-coverage-info*)))
-        code))))
+        (possibly-log-new-code code "load")))))
 
 ;; this gets you an #<fdefn> object, not the result of (FDEFINITION x)
 ;; cold-loader uses COLD-FDEFINITION-OBJECT instead.
@@ -1213,6 +1244,10 @@
 (define-fop 19 :not-host (fop-known-fun (name))
   (%coerce-name-to-fun name))
 
+#+arm64
+(define-fop 23 :not-host (fop-tls-index (name))
+  (ash (ensure-symbol-tls-index name) (- sb-vm:n-fixnum-tag-bits)))
+
 ;;; This FOP is only encountered in cross-compiled FASLs for cold load,
 ;;; and is a no-op except in cold load. A developer may want to load a
 ;;; cross-compiled FASL into a running system, and this FOP doesn't
@@ -1272,6 +1307,10 @@
                                  :plist plist))
   (values))
 
+(define-fop 125 :not-host (fop-note-full-calls (alist) nil)
+  (sb-c::accumulate-full-calls alist)
+  (values))
+
 ;;;; fops for code coverage
 
 (define-fop 120 :not-host (fop-record-code-coverage (namestring cc) nil)
