@@ -94,7 +94,8 @@
     ((head-var tail-var &optional user-head-var) &body body)
   (let ((l (and user-head-var (list (list user-head-var nil)))))
     `(let* ((,head-var (list nil)) (,tail-var ,head-var) ,@l)
-       (declare (truly-dynamic-extent ,head-var))
+       (declare (dynamic-extent ,head-var)
+                ,@(and user-head-var `((list ,user-head-var))))
        ,@body)))
 
 (sb-xc:defmacro loop-collect-rplacd
@@ -140,13 +141,13 @@
         (when user-head-var
           (setq answer
                 `(progn ,answer
-                        (setq ,user-head-var (cdr ,head-var)))))
+                        (setq ,user-head-var (sb-ext:truly-the list (cdr ,head-var))))))
         answer))))
 
 (sb-xc:defmacro loop-collect-answer (head-var
-                                                   &optional user-head-var)
-  (or user-head-var
-      `(cdr ,head-var)))
+                                     &optional user-head-var)
+  `(sb-ext:truly-the list ,(or user-head-var
+                               `(cdr ,head-var))))
 
 ;;;; maximization technology
 
@@ -289,10 +290,10 @@ code to be loaded.
              (:constructor !make-loop-universe)
              (:copier nil)
              (:predicate nil))
-  keywords             ; hash table, value = (fn-name . extra-data)
-  iteration-keywords   ; hash table, value = (fn-name . extra-data)
-  for-keywords         ; hash table, value = (fn-name . extra-data)
-  path-keywords)       ; hash table, value = (fn-name . extra-data)
+  (keywords nil :read-only t)             ; hash table, value = (fn-name . extra-data)
+  (iteration-keywords nil :read-only t)   ; hash table, value = (fn-name . extra-data)
+  (for-keywords nil :read-only t)         ; hash table, value = (fn-name . extra-data)
+  (path-keywords nil :read-only t))       ; hash table, value = (fn-name . extra-data)
 (declaim (sb-ext:freeze-type loop-universe))
 (defmethod print-object ((u loop-universe) stream)
   (print-unreadable-object (u stream :type t :identity t)))
@@ -561,8 +562,7 @@ code to be loaded.
   (do ((l (source-context loop) (cdr l)) (new nil (cons (car l) new)))
       ((eq l (cdr (source-code loop))) (nreverse new))))
 
-(defun loop-error (format-string &rest format-args)
-  #-sb-xc-host(declare (optimize allow-non-returning-tail-call))
+(define-error-wrapper loop-error (format-string &rest format-args)
   (%program-error "~?~%current LOOP context:~{ ~S~}."
                   format-string format-args (loop-context)))
 
@@ -769,6 +769,9 @@ code to be loaded.
                   "")))
            #+sb-unicode
            ((csubtypep ctype (specifier-type 'extended-char))
+            #+sb-xc-host
+            (error "Unimplemented on cross-compiler.")
+            #-sb-xc-host
             (code-char base-char-code-limit))
            ((csubtypep ctype (specifier-type 'character))
             #\x)
@@ -905,6 +908,12 @@ code to be loaded.
          (check-var-name name)
          (loop-declare-var name dtype :step-var-p step-var-p
                                       :initialization initialization)
+         ;; IGNORABLEize every variable because neither binding nor assignment constitutes
+         ;; a "use". Unfortunately there is no syntax for declaring what to ignore in LOOP.
+         ;; The idiom is to use NIL as a variable, but sometimes users don't, because they
+         ;; want variables names as information within a destructuring operation,
+         ;;  e.g. (for (this . that-not-used) in stuff do (frob this))
+         (push `(ignorable ,name) (declarations loop))
          ;; We use ASSOC on this list to check for duplications (above),
          ;; so don't optimize out this list:
          (push (list name (or initialization (loop-typed-init dtype step-var-p)))
@@ -1063,11 +1072,11 @@ code to be loaded.
 (defstruct (loop-collector
             (:copier nil)
             (:predicate nil))
-  name
-  class
+  (name nil :read-only t)
+  (class nil :read-only t)
   (history nil)
   (tempvars nil)
-  specified-type
+  (specified-type nil :read-only t)
   dtype
   (data nil)) ;collector-specific data
 (declaim (sb-ext:freeze-type loop-collector))
@@ -1399,15 +1408,17 @@ code to be loaded.
       (loop-constant-fold-if-possible val)
     (let ((listvar var))
       (cond ((and var (symbolp var))
-             (loop-make-var var list data-type))
+             (loop-make-var var
+                            ;; Don't want to assert the type, as ENDP will do that
+                            `(the* (list :use-annotations t :source-form ,list) ,list)
+                            data-type))
             (t
              (loop-make-var (setq listvar (gensym)) list 't)
              (loop-make-var var nil data-type)))
       (let ((list-step (loop-list-step listvar)))
         (let* ((first-endtest
-                ;; mysterious comment from original CMU CL sources:
-                ;;   the following should use `atom' instead of `endp',
-                ;;   per [bug2428]
+                ;; the following should use `atom' instead of `endp',
+                ;; per 6.1.2.1.3
                 `(atom ,listvar))
                (other-endtest first-endtest))
           (when (and constantp (listp list-value))
@@ -1428,7 +1439,10 @@ code to be loaded.
       (loop-constant-fold-if-possible val)
     (let ((listvar (gensym "LOOP-LIST-")))
       (loop-make-var var nil data-type)
-      (loop-make-var listvar list t)
+      (loop-make-var listvar
+                     ;; Don't want to assert the type, as ENDP will do that
+                     `(the* (list :use-annotations t :source-form ,list) ,list)
+                     t)
       (let ((list-step (loop-list-step listvar)))
         (let* ((first-endtest `(endp ,listvar))
                (other-endtest first-endtest)
