@@ -14,7 +14,6 @@
 ;;; Declare each of the free space pointers (except dynamic) as an alien var
 ;;; with darwin-jit, READ-ONLY-SPACE-START is a constant from genesis
 ;;; Maybe this whole file should go in sb-kernel to avoid sb-kernel::
-#-darwin-jit
 (define-alien-variable ("READ_ONLY_SPACE_START" sb-vm:read-only-space-start) sb-kernel::os-vm-size-t)
 (define-alien-variable ("read_only_space_free_pointer"
                         sb-vm:*read-only-space-free-pointer*)
@@ -38,8 +37,7 @@
 (declaim (inline read-only-space-obj-p dynamic-space-obj-p))
 (defun read-only-space-obj-p (x)
   (let ((a (get-lisp-obj-address x))
-        (rospace-start #-darwin-jit (extern-alien "READ_ONLY_SPACE_START" unsigned)
-                       #+darwin-jit sb-vm:read-only-space-start)
+        (rospace-start sb-vm:read-only-space-start)
         (rospace-end (extern-alien "read_only_space_free_pointer" unsigned)))
     (and (< rospace-start a rospace-end))))
 
@@ -51,11 +49,11 @@
     (let ((addr (get-lisp-obj-address x)))
       (< sb-vm:dynamic-space-start addr (sap-int (dynamic-space-free-pointer))))))
 
+(define-alien-variable ("TEXT_SPACE_START" sb-vm:text-space-start) unsigned-long)
+
 #+immobile-space
-(progn
 (define-symbol-macro sb-vm:alien-linkage-table-space-start
     (extern-alien "ALIEN_LINKAGE_TABLE_SPACE_START" unsigned))
-(define-alien-variable ("TEXT_SPACE_START" sb-vm:text-space-start) unsigned-long))
 
 #+darwin-jit
 (define-alien-variable ("static_code_space_free_pointer" sb-vm:*static-code-space-free-pointer*)
@@ -85,6 +83,10 @@
 (setf (documentation 'get-errno 'function)
       "Return the value of the C library pseudo-variable named \"errno\".")
 
+(define-alien-routine ("os_set_errno" set-errno) void (new-errno int))
+(setf (documentation 'set-errno 'function)
+      "Set the C library pseudo-variable named \"errno\", for obscure syscalls.")
+
 ;;; Decode errno into a string.
 #-win32
 (defun strerror (&optional (errno (get-errno)))
