@@ -9,19 +9,22 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-;;; FIXME: The latin9 stuff is currently #!+sb-unicode, because I
+;;; FIXME: The latin9 stuff is currently #+sb-unicode, because I
 ;;; don't like the idea of trying to do CODE-CHAR #x<big>.  Is that a
 ;;; justified fear?  Can we arrange that it's caught and converted to
 ;;; a decoding error error?  Or should we just give up on non-Unicode
 ;;; builds?
 
-(in-package "SB!IMPL")
+(in-package "SB-IMPL")
 
 
 ;;;; conditions
 
 ;;; encoding condition
 
+(eval-when (:compile-toplevel :load-toplevel :execute)
+  (defparameter *safety-0* '(safety 0)))
+
 (define-condition octets-encoding-error (character-encoding-error)
   ((string :initarg :string :reader octets-encoding-error-string)
    (position :initarg :position :reader octets-encoding-error-position)
@@ -33,31 +36,35 @@
                                       (octets-encoding-error-position c)))
                      (octets-encoding-error-external-format c)))))
 
-(declaim (ftype (sfunction (t t t) (simple-array (unsigned-byte 8) 1))
+(defun encoding-replacement-octetify (thing external-format)
+  (etypecase thing
+    ;; TODO: make sure EXTERNAL-FORMAT here does not have a REPLACEMENT, and handle
+    ;; encoding errors explicitly
+    (character
+     (string-to-octets (string thing) :external-format external-format))
+    (string
+     (string-to-octets thing :external-format external-format))
+    ((unsigned-byte 8) (make-array 1 :element-type '(unsigned-byte 8) :initial-element thing))
+    (sequence (coerce thing '(simple-array (unsigned-byte 8) 1)))))
+
+(declaim (ftype (sfunction (t t t t) (simple-array (unsigned-byte 8) 1))
                 encoding-error))
-(defun encoding-error (external-format string pos)
-  (restart-case
-      (error 'octets-encoding-error
-             :external-format external-format
-             :string string
-             :position pos)
-    (use-value (replacement)
-      :report "Supply a set of bytes to use in place of the invalid one."
-      :interactive
-      (lambda ()
-        (read-evaluated-form
-         "Replacement byte, bytes, character, or string (evaluated): "))
-      (typecase replacement
-        ((unsigned-byte 8)
-         (make-array 1 :element-type '(unsigned-byte 8) :initial-element replacement))
-        (character
-         (string-to-octets (string replacement)
-                           :external-format external-format))
-        (string
-         (string-to-octets replacement
-                           :external-format external-format))
-        (t
-         (coerce replacement '(simple-array (unsigned-byte 8) (*))))))))
+(defun encoding-error (external-format replacement string pos)
+  (flet ((replacement (replacement) (encoding-replacement-octetify replacement external-format)))
+    (if replacement
+        (replacement replacement)
+        (restart-case
+            (error 'octets-encoding-error
+                   :external-format external-format
+                   :string string
+                   :position pos)
+          (use-value (replacement)
+            :report "Supply a set of bytes to use in place of the invalid one."
+            :interactive
+            (lambda ()
+              (read-evaluated-form
+               "Replacement byte, bytes, character, or string (evaluated): "))
+            (replacement replacement))))))
 
 ;;; decoding condition
 
@@ -95,23 +102,37 @@
 (define-condition invalid-utf8-continuation-byte (octet-decoding-error) ())
 (define-condition overlong-utf8-sequence (octet-decoding-error) ())
 
-(define-condition malformed-ascii (octet-decoding-error) ())
+(defun decoding-replacement-stringify (thing external-format)
+  (etypecase thing
+    (character (string thing))
+    (string thing)
+    ;; TODO: make sure EXTERNAL-FORMAT here does not have a REPLACEMENT, and handle
+    ;; decoding errors explicitly
+    ((unsigned-byte 8)
+     (let ((octets (make-array 1 :element-type '(unsigned-byte 8) :initial-element thing)))
+       (octets-to-string octets :external-format external-format)))
+    (sequence
+     (let ((octets (coerce thing '(simple-array (unsigned-byte 8) 1))))
+       (octets-to-string octets :external-format external-format)))))
 
-(defun decoding-error (array start end external-format reason pos)
-  (restart-case
-      (error reason
-             :external-format external-format
-             :array array
-             :start start
-             :end end
-             :pos pos)
-    (use-value (s)
-      :report "Supply a replacement string designator."
-      :interactive
-      (lambda ()
-        (read-evaluated-form
-         "Enter a replacement string designator (evaluated): "))
-      (string s))))
+(defun decoding-error (array start end external-format replacement reason pos)
+  (flet ((replacement (thing) (decoding-replacement-stringify thing external-format)))
+    (if replacement
+        (replacement replacement)
+        (restart-case
+            (error reason
+                   :external-format external-format
+                   :array array
+                   :start start
+                   :end end
+                   :pos pos)
+          (use-value (replacement)
+            :report "Supply a replacement string."
+            :interactive
+            (lambda ()
+              (read-evaluated-form
+               "Replacement byte, bytes, character, or string (evaluated): "))
+            (replacement replacement))))))
 
 ;;; Utilities used in both to-string and to-octet conversions
 
@@ -122,7 +143,7 @@
 
 (eval-when (:compile-toplevel :load-toplevel :execute)
   (defun make-od-name (sym1 sym2)
-    (package-symbolicate (symbol-package sym1) sym1 "-" sym2)))
+    (package-symbolicate (cl:symbol-package sym1) sym1 "-" sym2)))
 
 ;;;; to-octets conversions
 
@@ -174,24 +195,14 @@
          ;; Create the lookup table.
          (sorted-lookup-table
           (reduce #'append sorted-pairs :from-end t :initial-value nil)))
-    (flet ((pick-type (vector &optional missing-points-p)
-             (if missing-points-p
-                 (let ((max (reduce #'max (remove nil vector))))
-                   (cond ((<= max #x7F) '(signed-byte 8))
-                         ((<= max #x7FFF) '(signed-byte 16))
-                         (t '(signed-byte 32))))
-                 (let ((max (reduce #'max vector)))
-                   (cond ((<= max #xFF) '(unsigned-byte 8))
-                         ((<= max #xFFFF) '(unsigned-byte 16))
-                         (t '(unsigned-byte 32)))))))
-      `(progn
+    `(progn
          ;; We *could* inline this, but it's not obviously the right thing,
          ;; because each use of the inlined function in a different file
          ;; would be forced to dump the large-ish array. To do things like
          ;; this, you generally want a load-time ref to a global constant.
        ;(declaim (inline ,byte-char-name))
          (defun ,byte-char-name (byte)
-           (declare (optimize speed (safety 0))
+           (declare (optimize speed #.*safety-0*)
                     (type (unsigned-byte 8) byte))
            ,(let ((byte-to-code
                    (loop for byte below 256
@@ -205,25 +216,20 @@
                   ;; We could use a single otherwise-unused point to mean NIL,
                   ;; but it would be confusing if in one table #xFFFF represents
                   ;; NIL and another #xF00D represents NIL.
-                  `(let ((code (aref ,(!make-specialized-array
-                                       256 (pick-type byte-to-code t)
+                  `(let ((code (aref ,(sb-c::coerce-to-smallest-eltype
                                        (substitute -1 nil byte-to-code))
                                      byte)))
                      (if (>= code 0) code))
                   ;; Every byte has a translation
-                  `(aref ,(!make-specialized-array
-                           256 (pick-type byte-to-code) byte-to-code)
+                  `(aref ,(sb-c::coerce-to-smallest-eltype byte-to-code)
                          byte))))
          (defun ,code-byte-name (code)
-           (declare (optimize speed (safety 0))
-                    (type char-code code))
+           (declare (optimize speed #.*safety-0*)
+                    (%char-code code))
            (if (< code ,lowest-non-equivalent-code)
                code
                (loop with code-to-byte-table =
-                    ,(!make-specialized-array
-                      (length sorted-lookup-table)
-                      (pick-type sorted-lookup-table)
-                      sorted-lookup-table)
+                    ,(sb-c::coerce-to-smallest-eltype sorted-lookup-table)
                   with low = 0
                   with high = (- (length code-to-byte-table) 2)
                   while (< low high)
@@ -233,20 +239,20 @@
                            (setf low mid)))
                   finally (return (if (eql code (aref code-to-byte-table low))
                                       (aref code-to-byte-table (1+ low))
-                                      nil)))))))))
+                                      nil))))))))
 
 (declaim (inline get-latin-bytes))
-(defun get-latin-bytes (mapper external-format string pos)
+(defun get-latin-bytes (mapper external-format replacement string pos)
   (let ((code (funcall mapper (char-code (char string pos)))))
-    (declare (type (or null char-code) code))
+    (declare (type (or null %char-code) code))
     (values (cond
               ((and code (< code 256)) code)
               (t
-               (encoding-error external-format string pos)))
+               (encoding-error external-format replacement string pos)))
             1)))
 
 (declaim (inline string->latin%))
-(defun string->latin% (string sstart send get-bytes null-padding)
+(defun string->latin% (string sstart send get-bytes null-padding replacement)
   (declare (optimize speed)
            (type simple-string string)
            (type index sstart send)
@@ -265,10 +271,10 @@
     (tagbody
      :no-error
        (loop for pos of-type index from sstart below send
-          do (let ((byte (funcall get-bytes string pos)))
+          do (let ((byte (funcall get-bytes string pos replacement)))
                (typecase byte
                  ((unsigned-byte 8)
-                  (locally (declare (optimize (sb!c::insert-array-bounds-checks 0)))
+                  (locally (declare (optimize (sb-c:insert-array-bounds-checks 0)))
                     (setf (aref octets index) byte)))
                  ((simple-array (unsigned-byte 8) (*))
                   ;; KLUDGE: We ran into encoding errors.  Bail and do
@@ -294,7 +300,7 @@
                       (vector-push-extend (aref thing i) new-octets))))))
            (extend error-replacement)
            (loop for pos of-type index from (1+ error-position) below send
-                 do (extend (funcall get-bytes string pos))
+                 do (extend (funcall get-bytes string pos replacement))
                  finally (return-from string->latin%
                            (progn
                              (unless (zerop null-padding)
@@ -310,7 +316,7 @@
     `(progn
       (declaim (inline ,name))
       (defun ,name (string sstart send array astart aend mapper)
-        (declare (optimize speed (safety 0))
+        (declare (optimize speed #.*safety-0*)
                  (type simple-string string)
                  (type ,type array)
                  (type array-range sstart send astart aend)
@@ -327,7 +333,7 @@
     `(progn
       (declaim (inline ,name))
       (defun ,name (array astart aend mapper)
-        (declare (optimize speed (safety 0))
+        (declare (optimize speed #.*safety-0*)
                  (type ,type array)
                  (type array-range astart aend)
                  (type function mapper))
@@ -339,47 +345,12 @@
 
 ;;;; external formats
 
-(defvar *default-external-format* nil)
+(defvar *default-external-format* :utf-8)
 
 (defun default-external-format ()
-  (or *default-external-format*
-      ;; On non-unicode, use iso-8859-1 instead of detecting it from
-      ;; the locale settings. Defaulting to an external-format which
-      ;; can represent characters that the CHARACTER type can't
-      ;; doesn't seem very sensible.
-      #!-sb-unicode
-      (setf *default-external-format* :latin-1)
-      (let ((external-format #!-win32 (intern (or #!-android
-                                                  (alien-funcall
-                                                   (extern-alien
-                                                    "nl_langinfo"
-                                                    (function (c-string :external-format :latin-1)
-                                                              int))
-                                                   sb!unix:codeset)
-                                                  "LATIN-1")
-                                              "KEYWORD")
-                             #!+win32 (sb!win32::ansi-codepage)))
-        (let ((entry (get-external-format external-format)))
-          (cond
-            (entry
-             (/show0 "matched"))
-            (t
-             ;; FIXME! This WARN would try to do printing
-             ;; before the streams have been initialized,
-             ;; causing an infinite erroring loop. We should
-             ;; either print it by calling to C, or delay the
-             ;; warning until later. Since we're in freeze
-             ;; right now, and the warning isn't really
-             ;; essential, I'm doing what's least likely to
-             ;; cause damage, and commenting it out. This
-             ;; should be revisited after 0.9.17. -- JES,
-             ;; 2006-09-21
-             #+nil
-             (warn "Invalid external-format ~A; using LATIN-1"
-                   external-format)
-             (setf external-format :latin-1))))
-        (/show0 "/default external format ok")
-        (setf *default-external-format* external-format))))
+  (/show0 "/getting default external format")
+  *default-external-format*)
+
 
 ;;;; public interface
 
@@ -388,12 +359,20 @@
                                    (default-external-format)
                                    external-format)))
 
-(declaim (ftype (sfunction ((vector (unsigned-byte 8)) &key (:external-format t)
-                                   (:start index)
-                                   (:end sequence-end))
+(declaim (inline %octets-to-string))
+(declaim (ftype (sfunction (function (vector (unsigned-byte 8)) index sequence-end t)
                            (or (simple-array character (*))
                                (simple-array base-char (*))))
-                octets-to-string))
+                %octets-to-string))
+(defun %octets-to-string (fun vector start end replacement)
+  (declare (explicit-check start end :result))
+  (with-array-data ((vector vector)
+                    (start start)
+                    (end end)
+                    :check-fill-pointer t)
+    (declare (type (simple-array (unsigned-byte 8) (*)) vector))
+    (funcall fun vector start end replacement)))
+(declaim (notinline %octets-to-string))
 (defun octets-to-string (vector &key (external-format :default) (start 0) end)
   "Return a string obtained by decoding VECTOR according to EXTERNAL-FORMAT.
 
@@ -409,21 +388,24 @@ SB-INT:CHARACTER-DECODING-ERROR is signaled.
 Note that for some values of EXTERNAL-FORMAT the length of the
 returned string may be different from the length of VECTOR (or the
 subsequence bounded by START and END)."
+  (let* ((ef (maybe-defaulted-external-format external-format))
+         (replacement (ef-replacement ef)))
+    (declare (inline %octets-to-string))
+    (%octets-to-string (ef-octets-to-string-fun ef) vector start end replacement)))
+
+(declaim (inline %string-to-octets))
+(declaim (ftype (sfunction (function string index sequence-end t t)
+                           (simple-array (unsigned-byte 8) (*)))
+                %string-to-octets))
+(defun %string-to-octets (fun string start end null-terminate replacement)
   (declare (explicit-check start end :result))
-  (with-array-data ((vector vector)
+  (with-array-data ((string string)
                     (start start)
                     (end end)
                     :check-fill-pointer t)
-    (declare (type (simple-array (unsigned-byte 8) (*)) vector))
-    (let ((ef (maybe-defaulted-external-format external-format)))
-      (funcall (ef-octets-to-string-fun ef) vector start end))))
-
-(declaim (ftype (sfunction (string &key (:external-format t)
-                                   (:start index)
-                                   (:end sequence-end)
-                                   (:null-terminate t))
-                           (simple-array (unsigned-byte 8) (*)))
-                string-to-octets))
+    (declare (type simple-string string))
+    (funcall fun string start end null-terminate replacement)))
+(declaim (notinline %string-to-octets))
 (defun string-to-octets (string &key (external-format :default)
                          (start 0) end null-terminate)
   "Return an octet vector that is STRING encoded according to EXTERNAL-FORMAT.
@@ -444,72 +426,110 @@ Note that for some values of EXTERNAL-FORMAT and NULL-TERMINATE the
 length of the returned vector may be different from the length of
 STRING (or the subsequence bounded by START and END)."
   (declare (explicit-check start end :result))
-  (with-array-data ((string string)
-                    (start start)
-                    (end end)
-                    :check-fill-pointer t)
-    (declare (type simple-string string))
-    (let ((ef (maybe-defaulted-external-format external-format)))
-      (funcall (ef-string-to-octets-fun ef) string start end
-               (if null-terminate 1 0)))))
+  (let* ((ef (maybe-defaulted-external-format external-format))
+         (replacement (ef-replacement ef)))
+    (declare (inline %string-to-octets))
+    (%string-to-octets (ef-string-to-octets-fun ef) string start end
+                       (if null-terminate 1 0) replacement)))
 
-#!+sb-unicode
-(defvar +unicode-replacement-character+ (string (code-char #xfffd)))
-#!+sb-unicode
-(defun use-unicode-replacement-char (condition)
-  (use-value +unicode-replacement-character+ condition))
+;;; Vector of all available EXTERNAL-FORMAT instances. Each format is named
+;;; by one or more keyword symbols. The mapping from symbol to index into this
+;;; vector is memoized into the symbol's :EXTERNAL-FORMAT property.
+(define-load-time-global *external-formats* (make-array 60 :initial-element nil))
 
-;;; Utilities that maybe should be exported
-
-#!+sb-unicode
-(defmacro with-standard-replacement-character (&body body)
-  `(handler-bind ((octet-encoding-error #'use-unicode-replacement-char))
-    ,@body))
-
-(defmacro with-default-decoding-replacement ((c) &body body)
-  (let ((cname (gensym)))
-  `(let ((,cname ,c))
-    (handler-bind
-        ((octet-decoding-error (lambda (c)
-                                 (use-value ,cname c))))
-      ,@body))))
+(defun register-external-format (names &rest args)
+  ;; TODO: compare-and-swap the entry if NAME already has an index
+  ;; specifying to demand-load this format from a fasl.
+  ;; All synonyms of that name will also references the loaded format.
+  (let* ((table *external-formats*)
+         (newline-variant (getf args :newline-variant))
+         (index (get (car names) :external-format))
+         (current-entry (and index (aref table index)))
+         (canonical-ef (if (consp current-entry) (car current-entry) current-entry))
+         (names (if canonical-ef (ef-names canonical-ef) names))
+         (ef (apply #'%make-external-format :names names args))
+         (free-index (or index (position nil table))))
+    (unless (eql newline-variant :lf)
+      (aver index)
+      (aver current-entry)
+      (unless (consp current-entry)
+        (setf current-entry (list current-entry)))
+      (setf (aref table index)
+            (cons (car current-entry)
+                  (acons (list newline-variant) ef (cdr current-entry))))
+      (return-from register-external-format))
+    (dolist (name names)
+      (setf (get name :external-format) free-index))
+    (setf (aref table free-index) ef)))
 
 ;;; This function was moved from 'fd-stream' because it depends on
 ;;; the various error classes, two of which are defined just above.
+;;; XXX: Why does this get called with :DEFAULT and NIL when neither is
+;;; the name of any format? Shouldn't those be handled higher up,
+;;; or else this should return the actual default?
 (defun get-external-format (external-format)
-  (flet ((keyword-external-format (keyword)
-           (declare (type keyword keyword))
-           (gethash keyword *external-formats*))
-         (replacement-handlerify (entry replacement)
-           (when entry
-             (wrap-external-format-functions
-              entry
-              (lambda (fun)
-                (and fun
-                     (lambda (&rest rest)
-                       (declare (dynamic-extent rest))
-                       (handler-bind
-                           ((stream-decoding-error
-                             (lambda (c)
-                               (declare (ignore c))
-                               (invoke-restart 'input-replacement replacement)))
-                            (stream-encoding-error
-                             (lambda (c)
-                               (declare (ignore c))
-                               (invoke-restart 'output-replacement replacement)))
-                            (octets-encoding-error
-                             (lambda (c) (use-value replacement c)))
-                            (octet-decoding-error
-                             (lambda (c) (use-value replacement c))))
-                         (apply fun rest)))))))))
-    (typecase external-format
-      (keyword (keyword-external-format external-format))
-      ((cons keyword)
-       (let ((entry (keyword-external-format (car external-format)))
-             (replacement (getf (cdr external-format) :replacement)))
-         (if replacement
-             (replacement-handlerify entry replacement)
-             entry))))))
+  (let* ((external-format (ensure-list external-format))
+         (options (cdr external-format)))
+    (unless (symbolp (car external-format))
+      (return-from get-external-format nil))
+    (loop for (option value) on options by 'cddr
+          unless (or (eql option :newline) (eql option :replacement))
+          do (return-from get-external-format nil))
+    (binding*
+        (((format-name newline replacement)
+          (values (car external-format)
+                  (getf options :newline :lf)
+                  (getf options :replacement)))
+         (table-index (get format-name :external-format) :exit-if-null)
+         (formats *external-formats*)
+         (table-entry
+          ;; The table entry can be one of:
+          ;;   1. #<external-format>
+          ;;   2. (#<external-format> ((:crlf #\char) . #<modified-ef>) ...)
+          ;;      for a list of modified formats that alter the
+          ;;      newline encoding and choice of replacement character.
+          ;;   3. "namestring" - to autoload from a fasl containing
+          ;;      the named format.
+          (let ((ef (svref formats table-index)))
+            (etypecase ef
+              ((or instance list) ef)
+              #+nil
+              (string
+               ;; Theoretically allow demand-loading the external-format
+               ;; from a fasl of this name. (Not done yet)
+               ;; (module-provide-contrib ef)
+               (let ((ef (svref formats table-index)))
+                 (aver (external-format-p ef))
+                 ef)))))
+         ((base-format variations)
+          (if (listp table-entry)
+              (values (car table-entry) (cdr table-entry))
+              (values table-entry nil)))
+         (newline-base-format
+          (if (eql newline :lf)
+              base-format
+              (cdr (assoc (list newline) variations :test #'equal)))
+          :exit-if-null))
+      (unless (typep replacement '(or null character string (unsigned-byte 8) (simple-array (unsigned-byte 8) 1)))
+        (return-from get-external-format nil))
+      (when (or (not newline-base-format) (not replacement))
+        (return-from get-external-format newline-base-format))
+      (loop
+        (let ((key (cons newline replacement)))
+          (awhen (assoc key variations :test #'equal)
+            (return (cdr it)))
+          (let* ((new-ef (let ((copy (copy-structure newline-base-format)))
+                           (setf (ef-replacement copy) replacement)
+                           copy))
+                 (new-table-entry
+                  (cons base-format (acons key new-ef variations)))
+                 (old (cas (svref formats table-index) table-entry new-table-entry)))
+            (when (eq old table-entry)
+              (return new-ef))
+            ;; CAS failure -> some other thread added an entry. It's probably
+            ;; for the same replacement char which is usually #\ufffd.
+            ;; So try again. At worst this conses some more garbage.
+            (setq table-entry old)))))))
 
 (push
   `("SB-IMPL"
