@@ -16,14 +16,13 @@
 ;;; to CALL-WITH-FOO less ugly.
 (defmacro dx-flet (functions &body forms)
   `(flet ,functions
-     (declare (truly-dynamic-extent ,@(mapcar (lambda (func) `#',(car func))
-                                              functions)))
+     (declare (dynamic-extent ,@(mapcar (lambda (func) `#',(car func)) functions)))
      ,@forms))
 
 ;;; Another similar one.
 (defmacro dx-let (bindings &body forms)
   `(let ,bindings
-     (declare (truly-dynamic-extent
+     (declare (dynamic-extent
                ,@(mapcar (lambda (bind) (if (listp bind) (car bind) bind))
                          bindings)))
      ,@forms))
@@ -61,7 +60,7 @@
                           (stem (if (every #'alpha-char-p symbol-name)
                                     symbol-name
                                     (string (gensymify* symbol-name "-")))))
-                     `(,symbol (sb-xc:gensym ,stem))))
+                     `(,symbol (gensym ,stem))))
                  symbols)
      ,@body))
 
@@ -69,7 +68,7 @@
 ;;; macros and other code-manipulating code.)
 (defun make-gensym-list (n &optional name)
   (let ((arg (if name (string name) "G")))
-    (loop repeat n collect (sb-xc:gensym arg))))
+    (loop repeat n collect (gensym arg))))
 
 ;;;; miscellany
 
@@ -92,8 +91,8 @@
 
 (defun gensymify (x)
   (if (symbolp x)
-      (sb-xc:gensym (symbol-name x))
-      (sb-xc:gensym)))
+      (gensym (symbol-name x))
+      (gensym)))
 
 (eval-when (:load-toplevel :execute #+sb-xc-host :compile-toplevel)
 (labels ((symbol-concat (package &rest things)
@@ -284,7 +283,6 @@
              (values (setf (gethash ,n-key ,n-hash-table) ,default) nil))))))
 
 (defvar *!removable-symbols* nil)
-(push '("SB-INT" check-designator) *!removable-symbols*)
 
 (defun %defconstant-eqx-value (symbol expr eqx)
   (declare (type function eqx))
@@ -303,16 +301,27 @@
 ;;;
 #+sb-xc-host
 (defmacro defconstant-eqx (symbol expr eqx &optional doc)
-  ;; CLISP needs the EVAL-WHEN here, or else the symbol defined is unavailable
-  ;; for later uses within the same file. For instance, in x86-64/vm, defining
-  ;; TEMP-REG-TN as R11-TN would get an error that R11-TN is unbound.
-  ;; We don't want that junk in our expansion, especially as our requirement
-  ;; is to separate the compile-time and load-time effect.
   `(eval-when (:compile-toplevel :load-toplevel :execute)
-     (defconstant ,symbol (%defconstant-eqx-value ',symbol ,expr ,eqx)
+     ;; We use DEFVAR rather than DEFCONSTANT as a host effect in
+     ;; order to avoid differences in host decisions about inlining
+     ;; the value of the constant, with knock-on effects on EQLity
+     ;; of references to internal parts of the constant.
+     (defvar ,symbol (%defconstant-eqx-value ',symbol ,expr ,eqx)
        ,@(when doc (list doc)))))
 
 #-sb-xc-host
 (defmacro defconstant-eqx (symbol expr eqx &optional doc)
   `(defconstant ,symbol (%defconstant-eqx-value ',symbol ,expr ,eqx)
      ,@(when doc (list doc))))
+
+;;; These are shorthand.
+;;; If in some policy we decide to open-code MEMBER, there should be
+;;; no discernable difference between MEMQ and (MEMBER ... :test 'EQ).
+(declaim (inline memq assq))
+(defun memq (item list)
+  "Return tail of LIST beginning with first element EQ to ITEM."
+  (member item list :test #'eq))
+
+(defun assq (item alist)
+  "Return the first pair of alist where item is EQ to the key of pair."
+  (assoc item alist :test #'eq))
