@@ -68,7 +68,7 @@
                              :element-type 'bit :initial-element fill)))
       (do ((i 0 (1+ i)))
           ((= i input-len) bvec)
-        (declare (index i) (optimize (sb-c::insert-array-bounds-checks 0)))
+        (declare (index i) (optimize (sb-c:insert-array-bounds-checks 0)))
         (let ((char (char bstring i)))
           (setf (elt bvec i)
                 (case char
@@ -150,7 +150,7 @@
                              (car body)))
       (let ((default-constructor
              (dd-default-constructor
-              (sb-kernel::wrapper-%info (classoid-wrapper classoid)))))
+              (sb-kernel::layout-%info (classoid-layout classoid)))))
         (unless default-constructor
           (simple-reader-error
            stream
@@ -296,7 +296,7 @@
                       (i sb-vm:instance-data-start (1+ i)))
                      ((>= i len))
                    (process (%instance-ref tree i)))
-                 (let ((dd (wrapper-dd (%instance-wrapper tree))))
+                 (let ((dd (layout-dd (%instance-layout tree))))
                    (dolist (dsd (dd-slots dd))
                      (when (eq (dsd-raw-type dsd) t)
                        (process (%instance-ref tree (dsd-index dsd))))))))
