@@ -64,7 +64,7 @@
   ;; It would probably be simple to eliminate this as a special case
   ;; by ensuring that instances of LAYOUT commence life with a trailing
   ;; hash slot and the SB-VM:HASH-SLOT-PRESENT-FLAG set.
-  (when (typep instance 'sb-vm:layout)
+  (when (typep instance 'layout)
     ;; This might be wrong if the clos-hash was clobbered to 0
     (return-from %instance-sxhash (layout-clos-hash instance)))
   ;; Non-simple cases: no hash slot, and either unhashed or hashed-not-moved.
@@ -100,7 +100,7 @@
 (declaim (inline fsc-instance-hash))
 (defun fsc-instance-hash (fin)
   (truly-the hash-code
-   #+compact-instance-header
+   #+executable-funinstances
    (with-pinned-objects (fin)
      (let ((hash (sb-vm::compact-fsc-instance-hash
                   (truly-the sb-pcl::standard-funcallable-instance fin))))
@@ -109,7 +109,7 @@
        ;; Some uses of the hash might expect the high bits to have randomness in them.
        ;; This returns a positive fixnum to conform with the requirement on SXHASH.
        (murmur-hash-word/+fixnum hash)))
-   #-compact-instance-header
+   #-executable-funinstances
    (sb-pcl::standard-funcallable-instance-hash-code
     (truly-the sb-pcl::standard-funcallable-instance fin))))
 
@@ -399,80 +399,74 @@
                           do (mixf result (number-psxhash (funcall getter data i)))))))))
              result))
        (structure-object-psxhash (key depthoid)
-       ;; Compute a PSXHASH for KEY. Salient points:
-       ;; * It's not enough to use the bitmap to figure out how to mix in raw slots.
-       ;;   The floating-point types all need special treatment. And we want to avoid
-       ;;   consing, so we can't very well call PSXHASH.
-       ;; * Even though PSXHASH requires that numerically equal numbers have the same
-       ;;   hash e.g. 12 and 12d0 and #c(12d0 0d0) all hash the same, structures can
-       ;;   weaken that restriction: instances are EQUAL only if they are of the same
-       ;;   type and slot-for-slot EQUAL. So a float in a raw slot can't be EQUAL
-       ;;   to a word in a different raw slot. In fact we don't even require that
-       ;;   SINGLE- and DOUBLE-float hash the same for a given numerical value,
-       ;;   because a raw slot can't hold either/or. But -0 and +0 must hash the same.
-       (declare (type structure-object key))
-       (declare (type (integer 0 #.+max-hash-depthoid+) depthoid))
-       (macrolet ((rsd-index+1 (dsd)
-                    ;; Return 0 if the DSD is not raw, otherwise 1+ the index into
-                    ;; *RAW-SLOT-DATA*. This is exactly the low 3 bits of DSD-BITS.
-                    `(truly-the (mod ,(1+ (length sb-kernel::*raw-slot-data*)))
-                                (ldb (byte 3 0) (sb-kernel::dsd-bits ,dsd))))
-                  (raw-cases ()
-                    (flet ((1+index-of (type)
-                             (1+ (position type sb-kernel::*raw-slot-data*
-                                           :key #'sb-kernel::raw-slot-data-raw-type)))
-                           (mix-float (val zero)
-                             `(let ((x ,val))
-                                (mixf result (sxhash (if (= x ,zero) ,zero x))))))
-                      ;; This compiles to a jump table if supported
-                      `(case rsd-index+1
-                        ((,(1+index-of 'word) ,(1+index-of 'sb-vm:signed-word))
-                         ;; Access as unsigned. +X and -X hash differently because
-                         ;; of 2's complement, so disregarding the sign bit is fine.
-                         (mixf result (logand (%raw-instance-ref/word key i)
-                                              most-positive-fixnum)))
-                        (,(1+index-of 'single-float)
-                         ,(mix-float '(%raw-instance-ref/single key i) $0f0))
-                        (,(1+index-of 'double-float)
-                         ,(mix-float '(%raw-instance-ref/double key i) $0d0))
-                        (,(1+index-of 'sb-kernel:complex-single-float)
-                         (let ((cplx (%raw-instance-ref/complex-single key i)))
-                           ,(mix-float '(realpart cplx) $0f0)
-                           ,(mix-float '(imagpart cplx) $0f0)))
-                        (,(1+index-of 'sb-kernel:complex-double-float)
-                         (let ((cplx (%raw-instance-ref/complex-double key i)))
-                           ,(mix-float '(realpart cplx) $0d0)
-                           ,(mix-float '(imagpart cplx) $0d0)))))))
-         (let* ((wrapper (%instance-wrapper key))
-                (result (wrapper-clos-hash wrapper)))
-           (declare (type fixnum result))
-           (when (plusp depthoid)
-             (let ((max-iterations depthoid)
-                   (depthoid (1- depthoid))
-                   (dd (wrapper-dd wrapper)))
-               (declare (index max-iterations))
-               (if (/= (sb-kernel::dd-bitmap dd) +layout-all-tagged+)
-                   (let ((slots (dd-slots dd)))
-                     (loop (unless slots (return))
-                           (let* ((slot (pop slots))
-                                  (rsd-index+1 (rsd-index+1 slot))
-                                  (i (dsd-index slot)))
-                             (cond ((= rsd-index+1 0) ; non-raw
-                                    (mixf result (%psxhash (%instance-ref key i) depthoid))
-                                    (if (zerop (decf max-iterations)) (return)))
-                                   (t
-                                    ;; Don't decrement MAX-ITERATIONS.
-                                    ;; These can't cause unbounded work.
-                                    (raw-cases))))))
-                   (let ((len (%instance-length key))
-                         ;; Don't mix in LAYOUT (if it takes a slot) because it was the seed value.
-                         (i sb-vm:instance-data-start))
-                     (declare (index i))
-                     (loop (when (>= i len) (return))
-                           (mixf result (%psxhash (%instance-ref key i) depthoid))
-                           (incf i)
-                           (if (zerop (decf max-iterations)) (return)))))))
-           result)))
+         ;; Compute a PSXHASH for KEY. Salient points:
+         ;; * It's not enough to use the bitmap to figure out how to mix in raw slots.
+         ;;   The floating-point types all need special treatment. And we want to avoid
+         ;;   consing, so we can't very well call PSXHASH.
+         ;; * Even though PSXHASH requires that numerically equal numbers have the same
+         ;;   hash e.g. 12 and 12d0 and #c(12d0 0d0) all hash the same, structures can
+         ;;   weaken that restriction: instances are EQUAL only if they are of the same
+         ;;   type and slot-for-slot EQUAL. So a float in a raw slot can't be EQUAL
+         ;;   to a word in a different raw slot. In fact we don't even require that
+         ;;   SINGLE- and DOUBLE-float hash the same for a given numerical value,
+         ;;   because a raw slot can't hold either/or. But -0 and +0 must hash the same.
+         (declare (type structure-object key))
+         (declare (type (integer 0 #.+max-hash-depthoid+) depthoid))
+         (macrolet ((rsd-index+1 (dsd)
+                      ;; Return 0 if the DSD is not raw, otherwise 1+ the index into
+                      ;; *RAW-SLOT-DATA*. This is exactly the low 3 bits of DSD-BITS.
+                      `(truly-the (mod ,(1+ (length sb-kernel::*raw-slot-data*)))
+                         (ldb (byte 3 0) (sb-kernel::dsd-bits ,dsd))))
+                    (raw-cases ()
+                      (flet ((1+index-of (type)
+                               (1+ (position type sb-kernel::*raw-slot-data*
+                                             :key #'sb-kernel::raw-slot-data-raw-type)))
+                             (mix-float (val zero)
+                               `(let ((x ,val))
+                                  (mixf result (sxhash (if (= x ,zero) ,zero x))))))
+                        ;; This compiles to a jump table if supported
+                        `(case rsd-index+1
+                           ((,(1+index-of 'word) ,(1+index-of 'sb-vm:signed-word))
+                            ;; Access as unsigned. +X and -X hash differently because
+                            ;; of 2's complement, so disregarding the sign bit is fine.
+                            (mixf result (logand (%raw-instance-ref/word key i)
+                                                 most-positive-fixnum)))
+                           (,(1+index-of 'single-float)
+                            ,(mix-float '(%raw-instance-ref/single key i) $0f0))
+                           (,(1+index-of 'double-float)
+                            ,(mix-float '(%raw-instance-ref/double key i) $0d0))
+                           (,(1+index-of 'sb-kernel:complex-single-float)
+                            (let ((cplx (%raw-instance-ref/complex-single key i)))
+                              ,(mix-float '(realpart cplx) $0f0)
+                              ,(mix-float '(imagpart cplx) $0f0)))
+                           (,(1+index-of 'sb-kernel:complex-double-float)
+                            (let ((cplx (%raw-instance-ref/complex-double key i)))
+                              ,(mix-float '(realpart cplx) $0d0)
+                              ,(mix-float '(imagpart cplx) $0d0)))))))
+           (let* ((layout (%instance-layout key))
+                  (result (layout-clos-hash layout)))
+             (declare (type fixnum result))
+             (when (plusp depthoid)
+               (let ((depthoid (1- depthoid))
+                     (dd (layout-dd layout)))
+                 (if (/= (sb-kernel::dd-bitmap dd) +layout-all-tagged+)
+                     (let ((slots (dd-slots dd)))
+                       (loop (unless slots (return))
+                             (let* ((slot (pop slots))
+                                    (rsd-index+1 (rsd-index+1 slot))
+                                    (i (dsd-index slot)))
+                               (cond ((= rsd-index+1 0) ; non-raw
+                                      (mixf result (%psxhash (%instance-ref key i) depthoid)))
+                                     (t
+                                      (raw-cases))))))
+                     (let ((len (%instance-length key))
+                           ;; Don't mix in LAYOUT (if it takes a slot) because it was the seed value.
+                           (i sb-vm:instance-data-start))
+                       (declare (index i))
+                       (loop (when (>= i len) (return))
+                             (mixf result (%psxhash (%instance-ref key i) depthoid))
+                             (incf i))))))
+             result)))
        (sfloat-psxhash (key)
          (declare (single-float key))
          (hash-float single-float key))
@@ -573,19 +567,3 @@
 ;;; Not needed post-build
 (clear-info :function :inlining-data '%sxhash-simple-substring)
 
-(defun show-hashed-instances ()
-  (flet ((foo (legend pred)
-           (format t "~&Instances in ~a state:~%" legend)
-           (sb-vm:map-allocated-objects pred :all)))
-    (foo "HASHED+MOVED"
-         (lambda (obj type size)
-           (declare (ignore size))
-           (when (and (= type sb-vm:instance-widetag)
-                      (logbitp 9 (instance-header-word obj)))
-             (format t "~x ~s~%" (get-lisp-obj-address obj) obj))))
-    (foo "HASHED (unmoved)"
-         (lambda (obj type size)
-           (declare (ignore size))
-           (when (and (= type sb-vm:instance-widetag)
-                      (= (ldb (byte 2 8) (instance-header-word obj)) 1))
-             (format t "~x ~s~%" (get-lisp-obj-address obj) obj))))))
