@@ -17,10 +17,10 @@
 (eval-when (:compile-toplevel :load-toplevel :execute)
   (defparameter *proplist-properties*
     (mapcar (lambda (x) (cons (car x) (sb-xc:coerce (cdr x) '(vector (unsigned-byte 32)))))
-            '#.(plist-to-alist (sb-cold:read-from-file "output/misc-properties.lisp-expr")))))
+            '#.(plist-to-alist (sb-cold:read-from-file "output/ucd/misc-properties.lisp-expr")))))
 
 (sb-ext:define-load-time-global **confusables**
-    (let ((data '#.(sb-cold:read-from-file "output/confusables.lisp-expr")))
+    (let ((data '#.(sb-cold:read-from-file "output/ucd/confusables.lisp-expr")))
       (sb-impl::%stuff-hash-table
         (make-hash-table :test #'eq #+sb-unicode :size #+sb-unicode (length data))
         (loop for (source . target) in data
@@ -40,7 +40,7 @@
         t)))
 
 (sb-ext:define-load-time-global **bidi-mirroring-glyphs**
-    (let ((list '#.(sb-cold:read-from-file "output/bidi-mirrors.lisp-expr")))
+    (let ((list '#.(sb-cold:read-from-file "output/ucd/bidi-mirrors.lisp-expr")))
       (sb-impl::%stuff-hash-table
        (make-hash-table :test #'eq :size (length list))
        (loop for (k v) in list collect (cons k v))
@@ -49,7 +49,7 @@
 ;;; Unicode property access
 (defun ordered-ranges-member (item vector)
   (declare (type (simple-array (unsigned-byte 32) 1) vector)
-           (type char-code item)
+           (%char-code item)
            (optimize speed))
   (labels ((recurse (start end)
              (declare (type index start end)
@@ -169,7 +169,7 @@ Numeric value is the most general of the Unicode numeric properties.
 The only constraint on the numeric value is that it be a rational number."
   (or (gethash character
                (load-time-value
-                (let ((list '#.(sb-cold:read-from-file "output/numerics.lisp-expr")))
+                (let ((list '#.(sb-cold:read-from-file "output/ucd/numerics.lisp-expr")))
                   (sb-impl::%stuff-hash-table
                    (make-hash-table :test #'eq :size (length list))
                    (loop for (k . v) in list
@@ -213,10 +213,10 @@ If CHARACTER does not have a known block, returns :NO-BLOCK"
   (let* ((code (char-code character))
          (block-index (ordered-ranges-position
                        code
-                       #.(coerce (sb-cold:read-from-file "output/block-ranges.lisp-expr")
+                       #.(coerce (sb-cold:read-from-file "output/ucd/block-ranges.lisp-expr")
                                        '(vector (unsigned-byte 32))))))
     (if block-index
-        (aref #.(sb-cold:read-from-file "output/block-names.lisp-expr") block-index)
+        (aref #.(sb-cold:read-from-file "output/ucd/block-names.lisp-expr") block-index)
         :no-block)))
 
 (defun unicode-1-name (character)
@@ -453,6 +453,20 @@ disappears when accents are placed on top of it. and NIL otherwise"
       (nreverse chars))))
 
 (defun primary-composition (char1 char2)
+  #-sb-unicode
+  (declare (ignore char1 char2))
+  #-sb-unicode
+  #.(let* ((data (sb-cold:read-from-file "output/ucd/comp.lisp-expr"))
+           (entries (loop for pair across data
+                          for key = (car pair)
+                          for c1 = (ldb (byte 21 21) key)
+                          for c2 = (ldb (byte 21 0) key)
+                          when (and (< c1 sb-xc:char-code-limit)
+                                    (< c2 sb-xc:char-code-limit))
+                          collect pair)))
+      (aver (null entries))
+      nil)
+  #+sb-unicode
   (flet ((composition-hangul-syllable-type (cp)
            (cond
              ((and (<= #x1100 cp) (<= cp #x1112)) :L)
@@ -466,11 +480,10 @@ disappears when accents are placed on top of it. and NIL otherwise"
        (cond
          ((gethash (dpb c1 (byte 21 21) c2)
                    (load-time-value
-                    (let ((data '#.(sb-cold:read-from-file "output/comp.lisp-expr")))
+                    (let ((data '#.(sb-cold:read-from-file "output/ucd/comp.lisp-expr")))
                       (sb-impl::%stuff-hash-table
                        (make-hash-table :size (length data) #+64-bit :test #+64-bit #'eq)
                        (loop for pair across data
-                             when (< (cdr pair) char-code-limit) ; Why is this even defined if #-sb-unicode?
                              collect (cons (car pair) (code-char (cdr pair))))
                        t))
                     t)))
@@ -522,7 +535,7 @@ disappears when accents are placed on top of it. and NIL otherwise"
 
 (defun normalize-string (string &optional (form :nfd)
                                           filter)
-  "Normalize STRING to the Unicode normalization form form.
+  "Normalize STRING to the Unicode normalization form FORM.
 Acceptable values for form are :NFD, :NFC, :NFKD, and :NFKC.
 If FILTER is a function it is called on each decomposed character and
 only characters for which it returns T are collected."
@@ -579,10 +592,10 @@ only characters for which it returns T are collected."
 ;;; Unicode case algorithms
 ;; FIXME: Make these parts less redundant (macro?)
 (sb-ext:define-load-time-global **special-titlecases**
-  '#.(sb-cold:read-from-file "output/titlecases.lisp-expr"))
+  '#.(sb-cold:read-from-file "output/ucd/titlecases.lisp-expr"))
 
 (sb-ext:define-load-time-global **special-casefolds**
-  '#.(sb-cold:read-from-file "output/foldcases.lisp-expr"))
+  '#.(sb-cold:read-from-file "output/ucd/foldcases.lisp-expr"))
 
 (defun has-case-p (char)
   ;; Bit 6 is the Unicode case flag, as opposed to the Common Lisp one
@@ -851,7 +864,7 @@ The result is not guaranteed to have the same length as the input."
                      ((>= end length)
                       (if (= end length) (progn (funcall function string start end) nil)))
                   (flet ((brk () (funcall function string start end) (setf start end)))
-                    (declare (truly-dynamic-extent #'brk))
+                    (declare (dynamic-extent #'brk))
                     (shiftf char1 char2 (char string end))
                     (shiftf c1 c2 (grapheme-break-class char2))
                     (if (eql c2 :regional-indicator) (incf nri) (setf nri 0))
@@ -890,7 +903,7 @@ The result is not guaranteed to have the same length as the input."
                   (flet ((fun (string start end)
                            (declare (type string string))
                            (funcall function (adjust-array array (- end start) :displaced-to string :displaced-index-offset start))))
-                    (declare (truly-dynamic-extent #'fun))
+                    (declare (dynamic-extent #'fun))
                     (,mapper #'fun string))))))
   (def map-legacy-graphemes map-legacy-grapheme-boundaries)
   (def map-graphemes map-grapheme-boundaries))
@@ -1409,7 +1422,7 @@ it defaults to 80 characters"
 
 ;;; Collation
 (defconstant +maximum-variable-primary-element+
-  #.(sb-cold:read-from-file "output/other-collation-info.lisp-expr"))
+  #.(sb-cold:read-from-file "output/ucd/other-collation-info.lisp-expr"))
 
 (defun unpack-collation-key (key)
   (flet ((unpack (value)
@@ -1426,7 +1439,7 @@ it defaults to 80 characters"
 
 (macrolet ((collations-hash-table ()
              (let ((data (let ((*read-base* 16))
-                           (sb-cold:read-from-file "output/collation.lisp-expr"))))
+                           (sb-cold:read-from-file "output/ucd/collation.lisp-expr"))))
                #+64-bit (dovector (item data) (aver (fixnump (car item))))
                `(load-time-value
                  (sb-impl::%stuff-hash-table
