@@ -381,7 +381,13 @@ any non-negative real number."
                    (*debug-io* (make-two-way-stream *stdin* *stderr*))
                    (*standard-input* *stdin*)
                    (*standard-output* *stdout*)
-                   (*error-output* *stderr*))
+                   (*error-output* *stderr*)
+                   ;; Compile/load messages from the language implementation
+                   ;; are not really what one would expect from a script. And
+                   ;; these messages are printed on stdout interfere when
+                   ;; scripts are used as part as a UNIX pipeline.
+                   (*compile-verbose* nil)
+                   (*load-verbose* nil))
                (load stream :verbose nil :print nil)))))
     (handling-end-of-the-world
       (if (eq t script)
@@ -402,6 +408,14 @@ any non-negative real number."
           args)
   (exit :code 1))
 
+
+(defconstant-eqx +runtime-options+
+  #("--noinform" "--core" "--help" "--version" "--dynamic-space-size"
+    "--control-stack-size" "--tls-limit"
+    "--debug-environment" "--disable-ldb" "--lose-on-corruption"
+    "--end-runtime-options" "--merge-core-pages" "--no-merge-core-pages")
+  #'equalp)
+
 ;;; the default system top level function
 (defun toplevel-init ()
   (/show0 "entering TOPLEVEL-INIT")
@@ -497,6 +511,9 @@ any non-negative real number."
                    ((string= option "--end-toplevel-options")
                     (pop-option)
                     (return))
+                   ((find option +runtime-options+ :test #'string=)
+                    (startup-error "C runtime option ~a in the middle of Lisp options."
+                                   option))
                    (t
                     ;; Anything we don't recognize as a toplevel
                     ;; option must be the start of user-level
