@@ -286,29 +286,34 @@
              ;; Bind temporarily so that TARGET-FEATUREP and TARGET-PLATFORM-KEYWORD
              ;; can see the tentative list.
              (sb-xc:*features* (funcall customizer default-features))
-             (gc (find-if (lambda (x) (member x '(:cheneygc :gencgc)))
-                          sb-xc:*features*))
+             (gc (intersection '(:cheneygc :gencgc :mark-region-gc)
+                               sb-xc:*features*))
              (arch (target-platform-keyword)))
+        (when (member :mark-region-gc sb-xc:*features*)
+          (setf sb-xc:*features* (remove :gencgc sb-xc:*features*)
+                gc (remove :gencgc gc)))
+        (unless (and gc (not (cdr gc)))
+          (error "Exactly 1 GC implementation needs to be selected"))
+        (setq gc (car gc))
+        ;; all our GCs are generational
+        (when (member gc '(:gencgc :mark-region-gc))
+          (pushnew :generational sb-xc:*features*))
+        (when (eq gc :mark-region-gc)
+          (setq sb-xc:*features* (remove :immobile-space sb-xc:*features*)))
         ;; Win32 conditionally adds :sb-futex in grovel-features.sh
-        ;; Futexes aren't available in all macos versions, but they are available in all versions that support arm, so always enable them there
+        ;; Futexes aren't available in all macos versions, but they are available in
+        ;; all versions that support arm, so always enable them there
         (when (target-featurep '(:and :sb-thread (:or :linux :freebsd :openbsd (:and :darwin :arm64))))
           (pushnew :sb-futex sb-xc:*features*))
-        ;; If may not be the best idea to put clock_gettime calls around every
-        ;; futex_wait if the OS/libc you're building for does not have a vdso entry point.
-        ;; So leave this out unless enabled explicitly and/or I gather more data on
-        ;; its performance impact.
-        #+nil
-        (when (target-featurep '(:and :sb-futex :x86-64 :linux))
-          (pushnew :futex-wait-metric sb-xc:*features*))
+        (when (target-featurep '(:and :sb-thread :x86-64))
+          (pushnew :system-tlabs sb-xc:*features*))
         (when (target-featurep :immobile-space)
           (when (target-featurep :x86-64)
-            (when (member :sb-thread sb-xc:*features*)
-              (pushnew :system-tlabs sb-xc:*features*))
             (pushnew :compact-instance-header sb-xc:*features*))
           (pushnew :immobile-code sb-xc:*features*))
         (when (target-featurep :64-bit)
           (push :compact-symbol sb-xc:*features*))
-        (when (target-featurep '(:and :sb-thread (:or :darwin :openbsd)))
+        (when (target-featurep '(:and :sb-thread (:or (:and :darwin (:not (:or :ppc :x86))) :openbsd)))
           (push :os-thread-stack sb-xc:*features*))
         (when (target-featurep '(:and :x86 :int4-breakpoints))
           ;; 0xCE is a perfectly good 32-bit instruction,
@@ -359,17 +364,15 @@
 ;;; combinations now, and provide a description of what the actual
 ;;; failure is (not always obvious from when the build fails).
 (let ((feature-compatibility-tests
-       '(("(and sb-thread (not gencgc))"
-          ":SB-THREAD requires :GENCGC")
-         ("(and sb-safepoint (not sb-thread))" ":SB-SAFEPOINT requires :SB-THREAD")
+       '(("(and sb-safepoint (not sb-thread))" ":SB-SAFEPOINT requires :SB-THREAD")
          ("(and sb-thread (not (or riscv ppc ppc64 x86 x86-64 arm64)))"
           ":SB-THREAD not supported on selected architecture")
+         ("(and mark-region-gc (not (or x86-64 arm64)))"
+          "mark-region is not supported on selected architecture")
          ("(and (not sb-thread) (or arm64 ppc64))"
           "The selected architecture requires :SB-THREAD")
          ("(and gencgc cheneygc)"
           ":GENCGC and :CHENEYGC are incompatible")
-         ("(not (or gencgc cheneygc))"
-          "One of :GENCGC or :CHENEYGC must be enabled")
          ("(and sb-safepoint (not (and (or arm64 x86 x86-64) (or darwin linux win32))))"
           ":SB-SAFEPOINT not supported on selected arch/OS")
          ("(not (or elf mach-o win32))"
