@@ -28,13 +28,8 @@
   (:info name offset lowtag)
   (:results)
   (:vop-var vop)
-  (:node-var node)
   (:generator 1
-    (unless (or
-             ;; gencgc does not need to emit the barrier for constructors
-             (eq name :allocator)
-             (sb-c::set-slot-old-p node))
-      (emit-gengc-barrier object nil tmp-tn (vop-nth-arg 1 vop) value))
+    (emit-gengc-barrier object nil tmp-tn (vop-nth-arg 1 vop) value name)
     (storew value object offset lowtag)))
 
 (define-vop (compare-and-swap-slot)
@@ -354,12 +349,10 @@
 
 (define-vop (set-fdefn-fun)
   (:policy :fast-safe)
-  (:translate (setf fdefn-fun))
-  (:args (function :scs (descriptor-reg) :target result)
+  (:args (function :scs (descriptor-reg))
          (fdefn :scs (descriptor-reg)))
   (:temporary (:scs (non-descriptor-reg)) lip)
   (:temporary (:scs (non-descriptor-reg)) type)
-  (:results (result :scs (descriptor-reg)))
   (:generator 38
     (emit-gengc-barrier fdefn nil lip)
     (inst add-sub lip function (- (* simple-fun-insts-offset n-word-bytes)
@@ -370,8 +363,7 @@
     (load-asm-routine lip 'closure-tramp)
     SIMPLE-FUN
     (storew lip fdefn fdefn-raw-addr-slot other-pointer-lowtag)
-    (storew function fdefn fdefn-fun-slot other-pointer-lowtag)
-    (move result function)))
+    (storew function fdefn fdefn-fun-slot other-pointer-lowtag)))
 
 (define-vop (fdefn-makunbound)
   (:policy :fast-safe)
