@@ -354,22 +354,16 @@
 
 (define-vop (float-compare)
   (:args (x) (y))
-  (:variant-vars format is-=)
+  (:variant-vars is-=)
   (:policy :fast-safe)
   (:note "inline float comparison")
   (:vop-var vop)
   (:save-p :compute-only)
   (:generator 3
     (note-this-location vop :internal-error)
-    (ecase format
-      (:single
-       (if is-=
-           (inst fcmp x y)
-           (inst fcmpe x y)))
-      (:double
-       (if is-=
-           (inst fcmp x y)
-           (inst fcmpe x y))))))
+    (if is-=
+        (inst fcmp x y)
+        (inst fcmpe x y))))
 
 (macrolet ((frob (name sc ptype)
              `(define-vop (,name float-compare)
@@ -384,14 +378,14 @@
                 (define-vop (,sname single-float-compare)
                   (:translate ,translate)
                   (:conditional ,cond)
-                  (:variant :single ,is-=))
+                  (:variant ,is-=))
                 (define-vop (,dname double-float-compare)
                   (:translate ,translate)
                   (:conditional ,cond)
-                  (:variant :double  ,is-=)))))
+                  (:variant ,is-=)))))
   (frob < :mi </single-float </double-float nil)
   (frob > :gt >/single-float >/double-float nil)
-  (frob <= :le <=/single-float <=/double-float nil)
+  (frob <= :ls <=/single-float <=/double-float nil)
   (frob >= :ge >=/single-float >=/double-float nil)
   (frob = :eq =/single-float =/double-float t))
 
@@ -431,7 +425,7 @@
                   (:variant ,is-=)))))
   (frob < :mi </single-float-zero </double-float-zero nil)
   (frob > :gt >/single-float-zero >/double-float-zero nil)
-  (frob <= :le <=/single-float-zero <=/double-float-zero nil)
+  (frob <= :ls <=/single-float-zero <=/double-float-zero nil)
   (frob >= :ge >=/single-float-zero >=/double-float-zero nil)
   (frob = :eq eql/single-float-zero eql/double-float-zero t))
 
@@ -481,7 +475,9 @@
     single-reg single-float double-reg double-float))
 
 (macrolet ((frob (trans from-sc from-type inst)
-             `(define-vop (,(symbolicate trans "/" from-type))
+             `(define-vop (,(if (find #\/ (string trans))
+                                trans
+                                (symbolicate trans "/" from-type)))
                 (:args (x :scs (,from-sc)))
                 (:results (y :scs (signed-reg)))
                 (:arg-types ,from-type)
@@ -497,7 +493,11 @@
   (frob %unary-truncate/single-float single-reg single-float fcvtzs)
   (frob %unary-truncate/double-float double-reg double-float fcvtzs)
   (frob %unary-round single-reg single-float fcvtns)
-  (frob %unary-round double-reg double-float fcvtns))
+  (frob %unary-round double-reg double-float fcvtns)
+  (frob %unary-ceiling single-reg single-float fcvtps)
+  (frob %unary-ceiling double-reg double-float fcvtps)
+  (frob %unary-floor single-reg single-float fcvtms)
+  (frob %unary-floor double-reg double-float fcvtms))
 
 (define-vop (make-single-float)
   (:args (bits :scs (signed-reg) :target res
@@ -546,6 +546,22 @@
       (double-stack
        (storew temp (current-nfp-tn vop) (tn-offset res))))))
 
+(define-vop (%make-double-float)
+  (:args (bits :scs (signed-reg)))
+  (:results (res :scs (double-reg)
+                 :load-if (not (sc-is res double-stack))))
+  (:arg-types signed-num)
+  (:result-types double-float)
+  (:translate %make-double-float)
+  (:policy :fast-safe)
+  (:vop-var vop)
+  (:generator 2
+    (sc-case res
+      (double-reg
+       (inst fmov res bits))
+      (double-stack
+       (storew bits (current-nfp-tn vop) (tn-offset res))))))
+
 (define-vop (single-float-bits)
   (:args (float :scs (single-reg descriptor-reg)
                 :load-if (not (sc-is float single-stack))))
@@ -675,7 +691,9 @@
   (:translate floating-point-modes)
   (:policy :fast-safe)
   (:generator 3
-    (inst mrs res :fpsr)))
+    (inst mrs res :fpsr)
+    (inst mrs tmp-tn :fpcr)
+    (inst orr res res tmp-tn)))
 
 (define-vop (set-floating-point-modes)
   (:args (new :scs (unsigned-reg) :target res))
@@ -686,6 +704,7 @@
   (:policy :fast-safe)
   (:generator 3
     (inst msr :fpsr new)
+    (inst msr :fpcr new)
     (move res new)))
 
 ;;;; Complex float VOPs
