@@ -90,22 +90,20 @@
                          #+big-endian `(+ ,n-offset (1- n-word-bytes))))
       `(inst ldrb ,n-target (@ ,n-source ,target-offset)))))
 
-(defmacro lisp-jump (lip)
-  `(let ((lip ,lip))
-     (aver (sc-is lip interior-reg))
-     (inst add lip lip 4)
-     (inst br lip)))
+(defmacro lisp-jump (lr)
+  `(progn
+     (inst add ,lr ,lr 4)
+     (inst br ,lr)))
 
-(defmacro lisp-return (lip return-style)
+(defmacro lisp-return (lr return-style)
   "Return to RETURN-PC."
-  `(let* ((lip ,lip))
-     (aver (sc-is lip interior-reg))
+  `(progn
      ;; Indicate a single-valued return by clearing the Z flag
      ,@(ecase return-style
          (:single-value '((inst cmp null-tn 0)))
          (:multiple-values '((inst cmp zr-tn zr-tn)))
          (:known))
-     (inst ret lip)))
+     (inst ret ,lr)))
 
 ;;;; Stack TN's
 
@@ -136,7 +134,7 @@
   (once-only ((n-reg reg)
               (n-stack reg-or-stack))
     `(sc-case ,n-reg
-       ((any-reg descriptor-reg interior-reg)
+       ((any-reg descriptor-reg non-descriptor-reg)
         (sc-case ,n-stack
           ((any-reg descriptor-reg)
            (move ,n-reg ,n-stack))
@@ -160,14 +158,13 @@
 ;;; surround a call to ALLOCATION anyway), and to indicate that the
 ;;; P-A FLAG-TN is also acceptable here.
 
-#+gencgc
-(defun allocation-tramp (type alloc-tn size back-label lip)
+#+generational
+(defun allocation-tramp (type alloc-tn size back-label)
   (if (integerp size)
       (load-immediate-word tmp-tn size)
       (inst mov tmp-tn size))
   (let ((asm-routine (if (eq type 'list) 'list-alloc-tramp 'alloc-tramp)))
-    (load-inline-constant alloc-tn `(:fixup ,asm-routine :assembly-routine) lip))
-  (inst blr alloc-tn)
+    (invoke-asm-routine asm-routine alloc-tn))
   (inst b back-label))
 
 ;;; Leaves the untagged pointer in TMP-TN,
@@ -175,9 +172,8 @@
 (defun allocation (type size lowtag result-tn
                    &key flag-tn
                         stack-allocate-p
-                        (lip (if stack-allocate-p nil (missing-arg)))
                         overflow)
-  (declare (ignorable type lip))
+  (declare (ignorable type))
   ;; Normal allocation to the heap.
   (if stack-allocate-p
       (assemble ()
@@ -185,29 +181,22 @@
         (inst and tmp-tn tmp-tn (lognot lowtag-mask))
         (inst add csp-tn tmp-tn (add-sub-immediate size result-tn))
         (inst add result-tn tmp-tn lowtag))
-      #-gencgc
-      (progn
-        (load-symbol-value flag-tn *allocation-pointer*)
-        (inst add result-tn flag-tn lowtag)
-        (inst add flag-tn flag-tn (add-sub-immediate size))
-        (store-symbol-value flag-tn *allocation-pointer*))
-      #+gencgc
       (let ((alloc (gen-label))
             #+sb-thread (tlab (if (eq type 'list) thread-cons-tlab-slot thread-mixed-tlab-slot))
-            #-sb-thread (region (if (eq type 'list) cons-region mixed-region))
+            #-sb-thread (region-offset (if (eq type 'list)
+                                           cons-region-offset
+                                           mixed-region-offset))
             (back-from-alloc (gen-label)))
         #-sb-thread
         (progn
-          ;; load-pair can't base off null-tn because the displacement
-          ;; has to be a multiple of 8
-          (load-immediate-word flag-tn region)
-          (inst ldp result-tn flag-tn (@ flag-tn 0)))
+          (loadw result-tn null-tn 0 (- nil-value-offset region-offset))
+          (loadw flag-tn null-tn 1 (- nil-value-offset region-offset)))
         #+sb-thread
         (inst ldp tmp-tn flag-tn (@ thread-tn (* n-word-bytes tlab)))
         (inst add result-tn tmp-tn (add-sub-immediate size result-tn))
         (inst cmp result-tn flag-tn)
         (inst b :hi ALLOC)
-        #-sb-thread (inst str result-tn (@ null-tn (load-store-offset (- region nil-value))))
+        #-sb-thread (inst str result-tn (@ null-tn (load-store-offset (- region-offset nil-value-offset))))
         #+sb-thread (storew result-tn thread-tn tlab)
 
         (emit-label BACK-FROM-ALLOC)
@@ -219,13 +208,11 @@
               (allocation-tramp type
                                 result-tn
                                 size
-                                BACK-FROM-ALLOC
-                                lip))))))
+                                BACK-FROM-ALLOC))))))
 
 (defmacro with-fixed-allocation ((result-tn flag-tn type-code size
                                             &key (lowtag other-pointer-lowtag)
                                                  stack-allocate-p
-                                                 (lip (missing-arg))
                                                  (store-type-code t))
                                  &body body)
   "Do stuff to allocate an other-pointer object of fixed Size with a single
@@ -235,17 +222,16 @@
   initializes the object."
   (once-only ((result-tn result-tn) (flag-tn flag-tn)
               (type-code type-code) (size size) (lowtag lowtag)
-              (stack-allocate-p stack-allocate-p)
-              (lip lip))
+              (stack-allocate-p stack-allocate-p))
     `(pseudo-atomic (,flag-tn :sync ,type-code
                      :elide-if ,stack-allocate-p)
        (allocation nil (pad-data-block ,size) ,lowtag ,result-tn
                    :flag-tn ,flag-tn
-                   :stack-allocate-p ,stack-allocate-p
-                   :lip ,lip)
-       (load-immediate-word ,flag-tn (compute-object-header ,size ,type-code))
-       ,@(and store-type-code
-              `((storew ,flag-tn ,result-tn 0 ,lowtag)))
+                   :stack-allocate-p ,stack-allocate-p)
+       (when ,type-code
+         (load-immediate-word ,flag-tn (compute-object-header ,size ,type-code))
+         ,@(and store-type-code
+                `((storew ,flag-tn ,result-tn 0 ,lowtag))))
        ,@body)))
 
 ;;;; Error Code
@@ -269,9 +255,14 @@
           (t
            (let ((first-value (car values)))
              (inst brk (dpb (cond ((and (tn-p first-value)
-                                        (sc-is first-value descriptor-reg any-reg))
+                                        (sc-is first-value descriptor-reg any-reg unsigned-reg signed-reg))
                                    (pop values)
-                                   (tn-offset first-value))
+                                   (dpb (sc-case first-value
+                                          (unsigned-reg 1)
+                                          (signed-reg 2)
+                                          (t 0))
+                                        (byte 2 5)
+                                        (tn-offset first-value)))
                                   (t
                                    zr-offset))
                             (byte 8 8)
@@ -284,11 +275,16 @@
     (emit-alignment 2)))
 
 (defun generate-error-code (vop error-code &rest values)
+  (apply #'generate-error-code+ nil vop error-code values))
+
+(defun generate-error-code+ (preamble-emitter vop error-code &rest values)
   "Generate-Error-Code Error-code Value*
   Emit code for an error with the specified Error-Code and context Values."
   (assemble (:elsewhere)
     (let ((start-lab (gen-label)))
       (emit-label start-lab)
+      (when preamble-emitter
+        (funcall preamble-emitter))
       (emit-error-break vop
                         (if (eq error-code 'invalid-arg-count-error)
                             invalid-arg-count-trap
@@ -301,14 +297,14 @@
 #+sb-safepoint
 (defun emit-safepoint ()
   (inst ldr zr-tn (@ null-tn
-                     (- (+ gc-safepoint-trap-offset n-word-bytes
-                           other-pointer-lowtag)))))
+                     (load-store-offset
+                      (- (+ nil-value-offset gc-safepoint-trap-offset))))))
 
 ;;; handy macro for making sequences look atomic
 (defmacro pseudo-atomic ((flag-tn &key elide-if (sync t)) &body forms)
   (declare (ignorable sync))
   #+sb-safepoint
-  `(progn ,@forms (emit-safepoint))
+  `(progn ,flag-tn (assemble () ,@forms) (unless ,elide-if (emit-safepoint)))
   #-sb-safepoint
   `(progn
      (unless ,elide-if
@@ -330,7 +326,7 @@
          #+sb-thread
          (progn
            (when ,sync
-            (inst dmb))
+            (inst dmb :ishst))
            (inst str (32-bit-reg zr-tn)
                  (@ thread-tn
                     (* n-word-bytes thread-pseudo-atomic-bits-slot)))
@@ -348,12 +344,11 @@
                               &optional translate)
   `(define-vop (,name)
      ,@(when translate
-             `((:translate ,translate)))
+         `((:translate ,translate)))
      (:policy :fast-safe)
      (:args (object :scs (descriptor-reg))
-            (index :scs (any-reg immediate)))
+            (index :scs (any-reg unsigned-reg signed-reg immediate)))
      (:arg-types ,type tagged-num)
-     (:temporary (:scs (interior-reg)) lip)
      (:results (value :scs ,scs))
      (:result-types ,el-type)
      (:generator 5
@@ -363,42 +358,82 @@
                                      (- (ash (+ ,offset (tn-value index)) word-shift)
                                         ,lowtag)))))
          (t
-          (inst add lip object (lsl index (- word-shift n-fixnum-tag-bits)))
-          (loadw value lip ,offset ,lowtag))))))
+          (inst add tmp-tn object (lsl index (- word-shift (if (sc-is index any-reg)
+                                                               n-fixnum-tag-bits
+                                                               0))))
+          (loadw value tmp-tn ,offset ,lowtag))))))
 
 (defmacro define-full-setter (name type offset lowtag scs el-type
-                              &optional translate)
-  `(define-vop (,name)
-     ,@(when translate
-             `((:translate ,translate)))
-     (:policy :fast-safe)
-     (:args (object :scs (descriptor-reg))
-            (index :scs (any-reg immediate))
-            (value :scs (,@scs zero)))
-     (:arg-types ,type tagged-num ,el-type)
-     (:temporary (:scs (interior-reg)) lip)
-     (:generator 2
-       (sc-case index
-         (immediate
-          (inst str value (@ object (load-store-offset
-                                     (- (ash (+ ,offset (tn-value index)) word-shift)
-                                        ,lowtag)))))
-         (t
-          (inst add lip object (lsl index (- word-shift n-fixnum-tag-bits)))
-          (storew value lip ,offset ,lowtag))))))
+                              &optional translate
+                              &aux (barrierp (member 'descriptor-reg scs)))
+  (case name
+    ((data-vector-set/simple-vector %weakvec-set)
+     `(define-vop (,name)
+        (:translate ,translate)
+        (:policy :fast-safe)
+        (:args (object :scs (descriptor-reg))
+               (index :scs (any-reg unsigned-reg signed-reg immediate))
+               (value :scs (,@scs zero)))
+        (:arg-types ,type tagged-num ,el-type)
+        (:temporary (:sc non-descriptor-reg) ea)
+        (:vop-var vop)
+        (:generator 2
+          (cond ((require-gengc-barrier-p object (vop-nth-arg 2 vop) value)
+                 (sc-case index
+                   (immediate
+                    (inst add ea object (load-store-offset
+                                         (- (ash (+ ,offset (tn-value index)) word-shift)
+                                            ,lowtag))))
+                   (t
+                    ;; Scale the index
+                    (let ((unshift (if (sc-is index any-reg) n-fixnum-tag-bits 0)))
+                      (inst add ea object (lsl index (- word-shift unshift))))
+                    ;; Calculate the exact cell address to ensure the right card is marked
+                    (inst add ea ea (- (ash vector-data-offset word-shift) other-pointer-lowtag))))
+                 (emit-gengc-barrier object ea tmp-tn t)
+                 (storew value ea 0 0))
+                (t
+                 (sc-case index
+                   (immediate
+                    (inst str value (@ object (load-store-offset
+                                               (- (ash (+ ,offset (tn-value index)) word-shift)
+                                                  ,lowtag)))))
+                   (t
+                    (let ((unshift (if (sc-is index any-reg) n-fixnum-tag-bits 0)))
+                      (inst add tmp-tn object (lsl index (- word-shift unshift))))
+                    (storew value tmp-tn ,offset ,lowtag))))))))
+    (t
+     `(define-vop (,name)
+        (:translate ,translate)
+        (:policy :fast-safe)
+        (:args (object :scs (descriptor-reg))
+               (index :scs (any-reg unsigned-reg signed-reg immediate))
+               (value :scs (,@scs zero)))
+        (:arg-types ,type tagged-num ,el-type)
+        (:vop-var vop)
+        (:generator 2
+          ,@(if barrierp '((emit-gengc-barrier object nil tmp-tn (vop-nth-arg 2 vop) value)))
+          (sc-case index
+            (immediate
+             (inst str value (@ object (load-store-offset
+                                        (- (ash (+ ,offset (tn-value index)) word-shift)
+                                           ,lowtag)))))
+            (t
+             (let ((unshift (if (sc-is index any-reg) n-fixnum-tag-bits 0)))
+               (inst add tmp-tn object (lsl index (- word-shift unshift))))
+             (storew value tmp-tn ,offset ,lowtag))))))))
 
 (defmacro define-partial-reffer (name type size signed offset lowtag scs
                                  el-type &optional translate)
   `(define-vop (,name)
      ,@(when translate
-             `((:translate ,translate)))
+         `((:translate ,translate)))
      (:policy :fast-safe)
      (:args (object :scs (descriptor-reg))
-            (index :scs (any-reg unsigned-reg immediate)))
+            (index :scs (any-reg unsigned-reg signed-reg immediate)))
      (:arg-types ,type tagged-num)
      (:results (value :scs ,scs))
      (:result-types ,el-type)
-     (:temporary (:scs (interior-reg)) lip)
      (:generator 5
        ,@(multiple-value-bind (op shift)
              (ecase size
@@ -407,7 +442,9 @@
                (:short
                 (values (if signed 'ldrsh 'ldrh) 1))
                (:word
-                (values (if signed 'ldrsw 'ldr) 2)))
+                (values (if signed 'ldrsw 'ldr) 2))
+               (:single-float
+                (values 'ldr 2)))
            (let ((value (if (and (eq size :word)
                                  (not signed))
                             '(32-bit-reg value)
@@ -420,64 +457,68 @@
                                                (- (* ,offset n-word-bytes) ,lowtag))))))
                  (t
                   (let ((shift ,shift))
-                    (sc-case index
-                      (any-reg
-                       (decf shift n-fixnum-tag-bits)))
-                    (inst add lip object (if (minusp shift)
-                                             (asr index (- shift))
-                                             (lsl index shift)))
+                    (when (sc-is index any-reg)
+                      (decf shift n-fixnum-tag-bits))
+                    (inst add tmp-tn object (if (minusp shift)
+                                                (asr index (- shift))
+                                                (lsl index shift)))
                     (inst ,op
-                          ,value (@ lip (- (* ,offset n-word-bytes) ,lowtag))))))))))))
+                          ,value (@ tmp-tn (- (* ,offset n-word-bytes) ,lowtag))))))))))))
 
 (defmacro define-partial-setter (name type size offset lowtag scs el-type
                                  &optional translate)
-  `(define-vop (,name)
-     ,@(when translate
-             `((:translate ,translate)))
-     (:policy :fast-safe)
-     (:args (object :scs (descriptor-reg))
-            (index :scs (any-reg unsigned-reg immediate))
-            (value :scs ,scs
-                   :load-if (not (and (sc-is value immediate)
-                                      (eql (tn-value value) 0)))))
-     (:arg-types ,type tagged-num ,el-type)
-     (:temporary (:scs (interior-reg)) lip)
-     (:generator 5
-       (when (sc-is value immediate)
-         (setf value zr-tn))
-       ,@(multiple-value-bind (op shift)
-             (ecase size
-               (:byte
-                (values 'strb 0))
-               (:short
-                (values 'strh 1))
-               (:word
-                (values 'str 2)))
-           (let ((value (if (eq size :word)
-                            '(32-bit-reg value)
-                            'value)))
-             `((sc-case index
-                 (immediate
-                  (inst ,op ,value (@ object (load-store-offset
-                                              (+
-                                               (ash (tn-value index) ,shift)
-                                               (- (* ,offset n-word-bytes) ,lowtag))))))
-                 (t
-                  (let ((shift ,shift))
-                    (sc-case index
-                      (any-reg
-                       (decf shift n-fixnum-tag-bits)))
-                    (inst add lip object (if (minusp shift)
-                                             (asr index (- shift))
-                                             (lsl index shift)))
-                    (inst ,op
-                          ,value (@ lip (- (* ,offset n-word-bytes) ,lowtag))))))))))))
+  (let ((value `((value :scs ,scs
+                             :load-if (not (and (sc-is value immediate)
+                                                (eql (tn-value value) 0))))))
+        (setf-p (typep translate '(cons (eql setf)))))
+   `(define-vop (,name)
+      ,@(when translate
+          `((:translate ,translate)))
+      (:policy :fast-safe)
+      (:args ,@(when setf-p
+                 value)
+             (object :scs (descriptor-reg))
+             (index :scs (any-reg unsigned-reg signed-reg immediate))
+             ,@(unless setf-p
+                 value))
+      (:arg-types ,@(when setf-p
+                      `(,el-type))
+                  ,type
+                  tagged-num
+                  ,@(unless setf-p
+                      `(,el-type)))
+      (:generator 5
+        (when (sc-is value immediate)
+          (setf value zr-tn))
+        ,@(multiple-value-bind (op shift)
+              (ecase size
+                (:byte
+                 (values 'strb 0))
+                (:short
+                 (values 'strh 1))
+                ((:word :single-float)
+                 (values 'str 2)))
+            (let ((value (if (eq size :word)
+                             '(32-bit-reg value)
+                             'value)))
+              `((sc-case index
+                  (immediate
+                   (inst ,op ,value (@ object (load-store-offset
+                                               (+
+                                                (ash (tn-value index) ,shift)
+                                                (- (* ,offset n-word-bytes) ,lowtag))))))
+                  (t
+                   (let ((shift ,shift))
+                     (when (sc-is index any-reg)
+                       (decf shift n-fixnum-tag-bits))
+                     (inst add tmp-tn object (if (minusp shift)
+                                                 (asr index (- shift))
+                                                 (lsl index shift)))
+                     (inst ,op
+                           ,value (@ tmp-tn (- (* ,offset n-word-bytes) ,lowtag)))))))))))))
 
-(defun load-inline-constant (dst value &optional lip)
-  (destructuring-bind (size . label) (register-inline-constant value)
-    (ecase size
-      (:qword
-       (inst load-from-label dst label lip)))))
+(defun load-inline-constant (dst &rest constant-descriptor)
+  (inst load-from-label dst (cdr (apply #'register-inline-constant constant-descriptor))))
 
 ;;;
 
