@@ -20,54 +20,95 @@
         (inst tbz* value 0 target))))
 
 (defun %test-fixnum-immediate-and-headers (value temp target not-p immediate
-                                           headers &key value-tn-ref)
+                                           headers &key value-tn-ref immediate-tested)
   (let ((drop-through (gen-label)))
     #.(assert (= fixnum-tag-mask 1))
-    (inst tbz* value 0 (if not-p drop-through target))
+    (when (types-equal-or-intersect (tn-ref-type value-tn-ref)
+                                    (specifier-type 'fixnum))
+      (inst tbz* value 0 (if not-p drop-through target)))
     (%test-immediate-and-headers value temp target not-p immediate headers
                                  :drop-through drop-through
-                                 :value-tn-ref value-tn-ref)))
+                                 :value-tn-ref value-tn-ref
+                                 :immediate-tested immediate-tested)))
 
 (defun %test-immediate-and-headers (value temp target not-p immediate headers
                                     &key (drop-through (gen-label))
-                                         value-tn-ref)
-  (cond ((= immediate single-float-widetag)
-         (inst cmp (32-bit-reg value) single-float-widetag))
-        (t
-         (inst mov temp immediate)
-         (inst cmp temp (extend value :uxtb))))
-  (inst b :eq (if not-p drop-through target))
+                                         value-tn-ref
+                                         immediate-tested)
+  (multiple-value-bind (bit set) (tn-ref-lowtag-bit immediate value-tn-ref)
+    (case set
+      (1
+       (inst tbnz* value bit (if not-p drop-through target)))
+      (0
+       (inst tbz* value bit (if not-p drop-through target)))
+      (t
+       (cond ((= immediate single-float-widetag)
+              (when (types-equal-or-intersect (tn-ref-type value-tn-ref)
+                                              (specifier-type 'single-float))
+                (inst cmp (32-bit-reg value) single-float-widetag)
+                (inst b :eq (if not-p drop-through target))))
+             (t
+              (inst mov temp immediate)
+              (inst cmp temp (extend value :uxtb))
+              (inst b :eq (if not-p drop-through target)))))))
   (%test-headers value temp target not-p nil headers
                  :drop-through drop-through
-                 :value-tn-ref value-tn-ref))
+                 :value-tn-ref value-tn-ref
+                 :immediate-tested immediate-tested))
 
 (defun %test-fixnum-and-headers (value temp target not-p headers
-                                 &key value-tn-ref)
+                                 &key value-tn-ref
+                                      immediate-tested)
   (let ((drop-through (gen-label)))
     (assemble ()
       #.(assert (= fixnum-tag-mask 1))
-      (inst tbz* value 0 (if not-p drop-through target)))
+      (when (types-equal-or-intersect (tn-ref-type value-tn-ref)
+                                      (specifier-type 'fixnum))
+        (inst tbz* value 0 (if not-p drop-through target))))
     (%test-headers value temp target not-p nil headers
                    :drop-through drop-through
-                   :value-tn-ref value-tn-ref)))
+                   :value-tn-ref value-tn-ref
+                   :immediate-tested immediate-tested)))
 
-(defun %test-immediate (value temp target not-p immediate)
-  (cond ((= immediate unbound-marker-widetag)
-         (inst cmp value immediate))
-        (t
-         (inst and temp value widetag-mask)
-         (inst cmp temp immediate)))
-  (inst b (if not-p :ne :eq) target))
+(defun %test-immediate (value temp target not-p immediate &key value-tn-ref)
+  (multiple-value-bind (bit set) (tn-ref-lowtag-bit immediate value-tn-ref)
+    (case set
+      (1
+       (if not-p
+           (inst tbz* value bit target)
+           (inst tbnz* value bit target)))
+      (0
+       (if not-p
+           (inst tbnz* value bit target)
+           (inst tbz* value bit target)))
+      (t
+       (cond ((= immediate unbound-marker-widetag)
+              (inst cmp value immediate))
+             (t
+              (inst and temp value widetag-mask)
+              (inst cmp temp immediate)))
+       (inst b (if not-p :ne :eq) target)))))
 
-(defun %test-lowtag (value temp target not-p lowtag)
-  (assemble ()
-    (inst and temp value lowtag-mask)
-    (inst cmp temp lowtag)
-    (inst b (if not-p :ne :eq) target)))
+(defun %test-lowtag (value temp target not-p lowtag &key value-tn-ref)
+  (multiple-value-bind (bit set) (tn-ref-lowtag-bit lowtag value-tn-ref)
+    (case set
+      (1
+       (if not-p
+           (inst tbz* value bit target)
+           (inst tbnz* value bit target)))
+      (0
+       (if not-p
+           (inst tbnz* value bit target)
+           (inst tbz* value bit target)))
+      (t
+       (inst and temp value lowtag-mask)
+       (inst cmp temp lowtag)
+       (inst b (if not-p :ne :eq) target)))))
 
 (defun %test-headers (value widetag target not-p function-p headers
                       &key (drop-through (gen-label))
                            value-tn-ref
+                           immediate-tested
                            (nil-in-other-pointers t))
   (let ((lowtag (if function-p fun-pointer-lowtag other-pointer-lowtag))
         (temp widetag))
@@ -89,9 +130,17 @@
         (assemble ()
           (cond (widetag
                  (unless (and value-tn-ref
-                              (eq lowtag other-pointer-lowtag)
-                              (other-pointer-tn-ref-p value-tn-ref nil-in-other-pointers))
-                   (%test-lowtag value widetag when-false t lowtag))
+                              (= lowtag other-pointer-lowtag)
+                              (other-pointer-tn-ref-p value-tn-ref nil-in-other-pointers immediate-tested))
+                   (multiple-value-bind (bit set) (tn-ref-lowtag-bit lowtag value-tn-ref
+                                                                       nil-in-other-pointers immediate-tested)
+                     (case set
+                       (1
+                        (inst tbz* value bit when-false))
+                       (0
+                        (inst tbnz* value bit when-false))
+                       (t
+                        (%test-lowtag value widetag when-false t lowtag)))))
                  (load-type widetag value (- lowtag)))
                 (t
                  (setf widetag value
@@ -180,6 +229,28 @@
           (inst b (if not-p :ne :eq) target))))
     not-target))
 
+(define-vop (signed-byte-64-p-move-to-word signed-byte-64-p)
+  (:results (r :scs (signed-reg)))
+  (:result-types signed-num)
+  (:translate)
+  (:generator 10
+    (let ((fixnum-p (types-equal-or-intersect (tn-ref-type args) (specifier-type 'fixnum))))
+      (multiple-value-bind (yep nope)
+          (if not-p
+              (values not-target target)
+              (values target not-target))
+        (assemble ()
+          (inst asr r value n-fixnum-tag-bits)
+          (when fixnum-p
+            (inst tbz* value 0 yep))
+          (unless (fixnum-or-other-pointer-tn-ref-p args t)
+            (test-type value temp nope t (other-pointer-lowtag)))
+          (loadw temp value 0 other-pointer-lowtag)
+          (inst cmp temp (+ (ash 1 n-widetag-bits) bignum-widetag))
+          (loadw r value bignum-digits-offset other-pointer-lowtag)
+          (inst b (if not-p :ne :eq) target))))
+    not-target))
+
 (define-vop (signed-byte-64-p/unsigned)
   (:args (value :scs (unsigned-reg)))
   (:arg-types unsigned-num)
@@ -195,8 +266,11 @@
 (define-vop (unsigned-byte-64-p type-predicate)
   (:translate unsigned-byte-64-p)
   (:generator 10
-    (let ((fixnum-p (types-equal-or-intersect (tn-ref-type args) (specifier-type 'fixnum)))
-          (other-pointer-p (fixnum-or-other-pointer-tn-ref-p args t)))
+    (let* ((fixnum-p (types-equal-or-intersect (tn-ref-type args) (specifier-type 'fixnum)))
+           (other-pointer-p (fixnum-or-other-pointer-tn-ref-p args t))
+           (not-signed-byte-64-p (not (types-equal-or-intersect (tn-ref-type args) (specifier-type 'signed-word))))
+           (unsigned-p (or not-signed-byte-64-p
+                           (not (types-equal-or-intersect (tn-ref-type args) (specifier-type '(integer * -1)))))))
       (multiple-value-bind (yep nope)
           (if not-p
               (values not-target target)
@@ -210,36 +284,156 @@
                 (fixnum-p
                  (move temp value)))
           (when fixnum-p
-            (%test-fixnum temp nil fixnum nil))
+            (%test-fixnum temp nil (if unsigned-p
+                                       yep
+                                       fixnum) nil))
           (unless other-pointer-p
             (inst cmp (32-bit-reg temp) other-pointer-lowtag)
             (inst b :ne nope))
           ;; Get the header.
           (loadw temp value 0 other-pointer-lowtag)
-          ;; Is it one?
-          (inst cmp temp (+ (ash 1 n-widetag-bits) bignum-widetag))
-          (inst b :eq single-word)
+          (unless not-signed-byte-64-p
+            ;; Is it one?
+            (inst cmp temp (+ (ash 1 n-widetag-bits) bignum-widetag))
+            (inst b :eq (if unsigned-p
+                            yep
+                            single-word)))
           ;; If it's other than two, it can't be an (unsigned-byte 64)
           (inst cmp temp (+ (ash 2 n-widetag-bits) bignum-widetag))
           (inst b :ne nope)
           ;; Get the second digit.
           (loadw temp value (1+ bignum-digits-offset) other-pointer-lowtag)
           ;; All zeros, it's an (unsigned-byte 64).
-          (inst cbz temp yep)
-          (inst b nope)
+          (cond (unsigned-p
+                 (if not-p
+                     (inst cbnz temp target)
+                     (inst cbz temp target)))
+                (t
+                 (inst cbz temp yep)
+                 (inst b nope)))
 
           single-word
-          ;; Get the single digit.
-          (loadw temp value bignum-digits-offset other-pointer-lowtag)
+          (unless unsigned-p
+            ;; Get the single digit.
+            (loadw temp value bignum-digits-offset other-pointer-lowtag))
 
           ;; positive implies (unsigned-byte 64).
           fixnum
-          (if not-p
-              (inst tbnz* temp (1- n-word-bits) target)
-              (inst tbz* temp (1- n-word-bits) target))))
+          (unless unsigned-p
+            (if not-p
+                (inst tbnz* temp (1- n-word-bits) target)
+                (inst tbz* temp (1- n-word-bits) target)))))
       (values))
     NOT-TARGET))
 
+(define-vop (unsigned-byte-64-p-move-to-word unsigned-byte-64-p)
+  (:results (r :scs (unsigned-reg)))
+  (:result-types unsigned-num)
+  (:translate)
+  (:generator 10
+    (let* ((fixnum-p (types-equal-or-intersect (tn-ref-type args) (specifier-type 'fixnum)))
+           (other-pointer-p (fixnum-or-other-pointer-tn-ref-p args t))
+           (not-signed-byte-64-p (not (types-equal-or-intersect (tn-ref-type args) (specifier-type 'signed-word))))
+           (unsigned-p (or not-signed-byte-64-p
+                           (not (types-equal-or-intersect (tn-ref-type args) (specifier-type '(integer * -1)))))))
+      (multiple-value-bind (yep nope)
+          (if not-p
+              (values not-target target)
+              (values target not-target))
+        (assemble ()
+          (cond ((not other-pointer-p)
+                 ;; Move to a temporary and mask off the lowtag,
+                 ;; but leave the sign bit for testing for positive fixnums.
+                 ;; When using 32-bit registers that bit will not be visible.
+                 (inst and temp value (logior (ash 1 (1- n-word-bits)) lowtag-mask)))
+                (fixnum-p
+                 (move temp value)))
+          (when fixnum-p
+            (inst asr r value n-fixnum-tag-bits)
+            (%test-fixnum temp nil (if unsigned-p
+                                       yep
+                                       fixnum) nil))
+          (unless other-pointer-p
+            (inst cmp (32-bit-reg temp) other-pointer-lowtag)
+            (inst b :ne nope))
+          ;; Get the header.
+          (loadw temp value 0 other-pointer-lowtag)
+          (loadw r value bignum-digits-offset other-pointer-lowtag)
+          (unless not-signed-byte-64-p
+            ;; Is it one?
+            (inst cmp temp (+ (ash 1 n-widetag-bits) bignum-widetag))
+            (inst b :eq (if unsigned-p
+                            yep
+                            single-word)))
+          ;; If it's other than two, it can't be an (unsigned-byte 64)
+          (inst cmp temp (+ (ash 2 n-widetag-bits) bignum-widetag))
+          (inst b :ne nope)
+          ;; Get the second digit.
+          (loadw temp value (1+ bignum-digits-offset) other-pointer-lowtag)
+          ;; All zeros, it's an (unsigned-byte 64).
+          (cond (unsigned-p
+                 (if not-p
+                     (inst cbnz temp target)
+                     (inst cbz temp target)))
+                (t
+                 (inst cbz temp yep)
+                 (inst b nope)))
+
+          single-word
+
+          ;; positive implies (unsigned-byte 64).
+          fixnum
+          (unless unsigned-p
+            (if not-p
+                (inst tbnz* r (1- n-word-bits) target)
+                (inst tbz* r (1- n-word-bits) target)))))
+      (values))
+    NOT-TARGET))
+
+(define-vop (un/signed-byte-64-p-move-to-word signed-byte-64-p)
+  ;; Ideally, this would a single register but the rest of the stuff
+  ;; depends on their storage class.
+  (:results (rs :scs (signed-reg))
+            (ru :scs (unsigned-reg)))
+  (:info target not-p target-unsigned not-p-unsigned unsigned-fall-through)
+  (:result-types signed-num unsigned-num)
+  (:translate)
+  (:generator 10
+    (let ((fixnum-p (types-equal-or-intersect (tn-ref-type args) (specifier-type 'fixnum))))
+      (assemble ()
+        (when fixnum-p
+          (inst asr rs value n-fixnum-tag-bits)
+          (inst tbz* value 0 (if not-p
+                                 not-target
+                                 target)))
+        (unless (fixnum-or-other-pointer-tn-ref-p args t)
+          (test-type value temp
+              (cond (not-p-unsigned
+                     unsigned-fall-through)
+                    (t
+                     not-target))
+              t (other-pointer-lowtag)))
+        (loadw temp value 0 other-pointer-lowtag)
+        (inst cmp temp (+ (ash 1 n-widetag-bits) bignum-widetag))
+        (loadw rs value bignum-digits-offset other-pointer-lowtag)
+        ;; Single digit bignum is always signed-byte-64
+        (inst b :eq (if not-p
+                        not-target
+                        target))
+        (inst cmp temp (+ (ash 2 n-widetag-bits) bignum-widetag))
+        (inst b :ne (cond (not-p-unsigned
+                           unsigned-fall-through)
+                          (t
+                           not-target)))
+        (move ru rs)
+        ;; If the second digit is zero then this is an unsigned-byte-64
+        (loadw temp value (1+ bignum-digits-offset) other-pointer-lowtag)
+        (cond ((and (not not-p) not-p-unsigned)
+               (inst cbnz temp target-unsigned))
+              (t
+               (inst cbz temp target-unsigned)))))
+    not-target))
+
 (define-vop (fixnump/unsigned)
   (:policy :fast-safe)
   (:args (value :scs (unsigned-reg)))
@@ -585,11 +779,21 @@
 
 (define-vop (single-float-p)
   (:args (value :scs (any-reg descriptor-reg)))
+  (:arg-refs value-ref)
   (:conditional :eq)
   (:policy :fast-safe)
   (:translate single-float-p)
+  (:vop-var vop)
   (:generator 7
-    (inst cmp (32-bit-reg value) single-float-widetag)))
+    (multiple-value-bind (bit set) (tn-ref-lowtag-bit single-float-widetag value-ref)
+      (case set
+        (1
+         (change-vop-flags vop '(:ne))
+         (inst tst value (ash 1 bit)))
+        (0
+         (inst tst value (ash 1 bit)))
+        (t
+         (inst cmp (32-bit-reg value) single-float-widetag))))))
 
 (define-vop (load-other-pointer-widetag)
   (:args (value :scs (any-reg descriptor-reg)))
@@ -598,7 +802,7 @@
   (:results (r :scs (unsigned-reg)))
   (:result-types unsigned-num)
   (:generator 1
-    (unless (other-pointer-tn-ref-p args)
+    (unless (other-pointer-tn-ref-p args (not null-label))
       (when null-label
         (inst cmp value null-tn)
         (inst b :eq null-label))
@@ -728,7 +932,6 @@
   (:args (layout :scs (descriptor-reg)))
   (:arg-types * (:constant t))
   (:policy :fast-safe)
-  (:conditional)
   (:info target not-p test-layout)
   (:temporary (:sc unsigned-reg
                :unused-if
