@@ -58,11 +58,13 @@
 
 ;;; Signal a warning if appropriate and set *FOO-DETECTED*.
 (defun note-lossage (format-string &rest format-args)
+  (declare (explicit-check))
   (setq *lossage-detected* t)
   (when *lossage-fun*
     (apply *lossage-fun* format-string format-args))
   (values))
 (defun note-unwinnage (format-string &rest format-args)
+  (declare (explicit-check))
   (setq *unwinnage-detected* t)
   (when *unwinnage-fun*
     (apply *unwinnage-fun* format-string format-args))
@@ -906,9 +908,10 @@ and no value was provided for it." name))))))))))
 ;;; Call FUN with (arg-lvar arg-type lvars &optional annotation)
 (defun map-combination-args-and-types (fun call &optional info
                                                           unknown-keys-fun
-                                                          declared-only)
+                                                          defined-here
+                                                          asserted-type)
   (declare (type function fun) (type combination call))
-  (binding* ((type (lvar-fun-type (combination-fun call) declared-only declared-only))
+  (binding* ((type (lvar-fun-type (combination-fun call) defined-here asserted-type))
              (nil (fun-type-p type) :exit-if-null)
              (annotation (and info
                               (fun-info-annotation info)))
@@ -975,7 +978,7 @@ and no value was provided for it." name))))))))))
         (use-lvar cast internal-lvar)
         t))))
 
-(defun apply-type-annotation (fun-name arg type lvars policy &optional annotation)
+(defun apply-type-annotation (fun-name arg type lvars policy &optional annotation context)
   (case (car annotation)
     (function-designator
      (assert-function-designator fun-name lvars arg (cdr annotation) policy)
@@ -999,7 +1002,7 @@ and no value was provided for it." name))))))))))
         (add-annotation arg
                         (make-lvar-proper-sequence-annotation
                          :kind (car annotation)))))
-     (assert-lvar-type arg type policy))))
+     (assert-lvar-type arg type policy context))))
 
 ;;; Assert that CALL is to a function of the specified TYPE. It is
 ;;; assumed that the call is legal and has only constants in the
@@ -1023,7 +1026,7 @@ and no value was provided for it." name))))))))))
                      (not (return-p (lvar-dest lvar)))
                      (not (mv-combination-p (lvar-dest lvar)))
                      (lvar-has-single-use-p lvar))
-            (when (assert-lvar-type lvar returns policy)
+            (when (assert-lvar-type lvar returns policy 'ftype-context)
               (reoptimize-lvar lvar)))))
     (let* ((name (lvar-fun-name (combination-fun call) t))
            (info (and name
@@ -1035,7 +1038,9 @@ and no value was provided for it." name))))))))))
            (lambda (arg type lvars &optional annotation)
              (when (and
                     (apply-type-annotation name arg type
-                                           lvars policy annotation)
+                                           lvars policy annotation
+                                           (and (not trusted)
+                                                'ftype-context))
                     (not trusted))
                (reoptimize-lvar arg)))
            call
