@@ -17,7 +17,8 @@
 (declaim (type byte-buffer *byte-buffer*))
 (defvar *contexts*)
 (declaim (type (vector t) *contexts*))
-
+(defvar *local-call-context*)
+(defvar *location-context* nil)
 
 ;;;; debug blocks
 
@@ -43,6 +44,7 @@
             (:copier nil))
   (label nil :type (or null label))
   (tn nil :type (or null tn) :read-only t))
+(!set-load-form-method restart-location (:xc :target) :ignore-it)
 
 ;;; This is called during code generation in places where there is an
 ;;; "interesting" location: someplace where we are likely to end up
@@ -77,10 +79,10 @@
                          (- posn (segment-header-skew segment))))))
   (values))
 
-#-sb-fluid (declaim (inline ir2-block-physenv))
-(defun ir2-block-physenv (2block)
+(declaim (inline ir2-block-environment))
+(defun ir2-block-environment (2block)
   (declare (type ir2-block 2block))
-  (block-physenv (ir2-block-block 2block)))
+  (block-environment (ir2-block-block 2block)))
 
 (defun make-lexenv-var-cache (lexenv)
   (or (lexenv-var-cache lexenv)
@@ -104,15 +106,47 @@
 (defun leaf-visible-to-debugger-p (leaf node)
   (gethash leaf (make-lexenv-var-cache (node-lexenv node))))
 
+(defun optional-leaf-p (leaf)
+  (let ((home (lambda-var-home leaf)))
+    (case (functional-kind home)
+      (:external
+       (let ((entry (lambda-entry-fun home)))
+         (when (optional-dispatch-p entry)
+           (let ((pos (position leaf (lambda-vars home))))
+             (and pos
+                  (>= (1- pos) (optional-dispatch-min-args entry))))))))))
+
+;;; Type checks of the arguments in an external function happen before
+;;; all the &optionals are initialized. PROPAGATE-TO-ARGS marks such
+;;; locations with the entry point, which allows to deduce if an
+;;; optional is alive at that point.
+(defun optional-processed (leaf)
+  (let ((home (lambda-var-home leaf)))
+    (case (functional-kind home)
+      (:external
+       (let ((entry (lambda-entry-fun home)))
+         (if (and (optional-dispatch-p entry)
+                  *local-call-context*)
+             (let ((pos (position leaf (lambda-vars home)))
+                   (entry-pos (position *local-call-context*
+                                        (optional-dispatch-entry-points entry)
+                                        :key #'force)))
+               (not (and pos
+                         entry-pos
+                         (>= (1- pos) (+ (optional-dispatch-min-args entry) entry-pos)))))
+             t)))
+      (t t))))
+
 ;;; Given a local conflicts vector and an IR2 block to represent the
 ;;; set of live TNs, and the VAR-LOCS hash-table representing the
 ;;; variables dumped, compute a bit-vector representing the set of
 ;;; live variables. If the TN is environment-live, we only mark it as
 ;;; live when it is in scope at NODE.
-(declaim (ftype (sfunction (local-tn-bit-vector node ir2-block hash-table (or null vop))
-                           simple-bit-vector)
-                compute-live-vars))
 (defun compute-live-vars (live node block var-locs vop)
+  (declare (type ir2-block block) (type local-tn-bit-vector live)
+           (type hash-table var-locs) (type node node)
+           (type (or vop null) vop)
+           #-sb-xc-host (values simple-bit-vector))
   (let ((res (make-array (logandc2 (+ (hash-table-count var-locs) 7) 7)
                          :element-type 'bit
                          :initial-element 0))
@@ -126,9 +160,11 @@
                                     '(:environment :debug-environment)))
                        (leaf-visible-to-debugger-p leaf node))
                    (or (null spilled)
-                       (not (member tn spilled))))
+                       (not (member tn spilled)))
+                   (optional-processed leaf))
           (let ((num (gethash leaf var-locs)))
             (when num
+
               (setf (sbit res num) 1))))))
     res))
 
@@ -163,16 +199,22 @@
 ;;; the code/source map and live info. If true, VOP is the VOP
 ;;; associated with this location, for use in determining whether TNs
 ;;; are spilled.
-(defun dump-1-location (node block kind label live var-locs vop
+(defun dump-1-location (node block kind tlf-num label live var-locs vop
                         &optional context)
   (declare (type node node) (type ir2-block block)
            (type (or null local-tn-bit-vector) live)
            (type (or label index) label)
-           (type location-kind kind)
+           (type location-kind kind) (type (or index null) tlf-num)
            (type hash-table var-locs) (type (or vop null) vop))
   (let* ((byte-buffer *byte-buffer*)
          (stepping (and (combination-p node)
                         (combination-step-info node)))
+         (*local-call-context*
+           (if (local-call-context-p context)
+               (local-call-context-fun context)))
+         (context (if (local-call-context-p context)
+                      (local-call-context-var context)
+                      context))
          (live (and live
                     (compute-live-vars live node block var-locs vop)))
          (anything-alive (and live
@@ -215,6 +257,8 @@
     (write-var-integer (- loc *previous-location*) byte-buffer)
     (setq *previous-location* loc)
 
+    (unless tlf-num
+      (write-var-integer (source-path-tlf-number path) byte-buffer))
     (unless (zerop form-number)
       (setf *previous-form-number* form-number)
       (write-var-integer form-number byte-buffer))
@@ -234,13 +278,14 @@
 
 ;;; Extract context info from a Location-Info structure and use it to
 ;;; dump a compiled code-location.
-(defun dump-location-from-info (loc var-locs)
-  (declare (type location-info loc)
+(defun dump-location-from-info (loc tlf-num var-locs)
+  (declare (type location-info loc) (type (or index null) tlf-num)
            (type hash-table var-locs))
   (let ((vop (location-info-vop loc)))
     (dump-1-location (vop-node vop)
                      (vop-block vop)
                      (location-info-kind loc)
+                     tlf-num
                      (location-info-label loc)
                      (vop-save-set vop)
                      var-locs
@@ -248,21 +293,44 @@
                      (location-info-context loc)))
   (values))
 
+;;; Scan all the blocks, determining if all locations are in the same
+;;; TLF, and returning it or NIL.
+(defun find-tlf-number (fun)
+  (declare (type clambda fun))
+  (let ((res (source-path-tlf-number (node-source-path (lambda-bind fun)))))
+    (declare (type (or index null) res))
+    (do-environment-ir2-blocks (2block (lambda-environment fun))
+      (let ((block (ir2-block-block 2block)))
+        (when (eq (block-info block) 2block)
+          (unless (eql (source-path-tlf-number
+                        (node-source-path
+                         (block-start-node block)))
+                       res)
+            (setq res nil)))
+
+        (dolist (loc (ir2-block-locations 2block))
+          (unless (eql (source-path-tlf-number
+                        (node-source-path
+                         (vop-node (location-info-vop loc))))
+                       res)
+            (setq res nil)))))
+    res))
+
 ;;; Dump out the number of locations and the locations for Block.
-(defun dump-block-locations (block locations var-locs)
+(defun dump-block-locations (block locations tlf-num var-locs)
   (declare (type cblock block) (list locations))
   (unless (and locations
                (eq (location-info-kind (first locations))
                    :non-local-entry))
       (let ((2block (block-info block)))
         (dump-1-location (block-start-node block)
-                         2block :block-start
+                         2block :block-start tlf-num
                          (ir2-block-%label 2block)
                          (ir2-block-live-out 2block)
                          var-locs
                          nil)))
   (dolist (loc locations)
-    (dump-location-from-info loc var-locs))
+    (dump-location-from-info loc tlf-num var-locs))
   (values))
 
 ;;; Return a vector and an integer (or null) suitable for use as the
@@ -272,18 +340,19 @@
   (let ((*previous-location* 0)
         *previous-live*
         *previous-form-number*
-        (physenv (lambda-physenv fun))
+        (tlf-num (find-tlf-number fun))
+        (env (lambda-environment fun))
         (byte-buffer *byte-buffer*)
         prev-block
         locations
         elsewhere-locations)
     (setf (fill-pointer byte-buffer) 0)
-    (do-physenv-ir2-blocks (2block physenv)
+    (do-environment-ir2-blocks (2block env)
       (let ((block (ir2-block-block 2block)))
         (when (eq (block-info block) 2block)
           (when prev-block
             (dump-block-locations prev-block (nreverse (shiftf locations nil))
-                                  var-locs))
+                                  tlf-num var-locs))
           (setf prev-block block)))
       (dolist (loc (ir2-block-locations 2block))
         (if (label-elsewhere-p (location-info-label loc)
@@ -291,13 +360,15 @@
             (push loc elsewhere-locations)
             (push loc locations))))
 
-    (dump-block-locations prev-block (nreverse locations) var-locs)
+    (dump-block-locations prev-block (nreverse locations)
+                          tlf-num var-locs)
 
     (when elsewhere-locations
       (dolist (loc (nreverse elsewhere-locations))
-        (dump-location-from-info loc var-locs)))
+        (dump-location-from-info loc tlf-num var-locs)))
     ;; lz-compress accept any array of octets and returns a simple-array
-    (logically-readonlyize (lz-compress byte-buffer))))
+    (values (logically-readonlyize (lz-compress byte-buffer))
+            tlf-num)))
 
 ;;; Return DEBUG-SOURCE structure containing information derived from
 ;;; INFO.
@@ -305,19 +376,21 @@
   (declare (type source-info info))
   (let ((file-info (get-toplevelish-file-info info)))
     (multiple-value-call
-        (if function #'sb-c::make-core-debug-source #'make-debug-source)
+        (if function 'sb-di::make-core-debug-source 'make-debug-source)
      :namestring (or *source-namestring*
                      (make-file-info-namestring
                       (let ((pathname
                              (case *name-context-file-path-selector*
-                               (pathname (file-info-untruename file-info))
-                               (truename (file-info-name file-info)))))
+                               (pathname (file-info-pathname file-info))
+                               (truename (file-info-truename file-info)))))
                         (if (pathnamep pathname) pathname))
                       file-info))
-     :created (file-info-write-date file-info)
+      :created (file-info-write-date file-info)
+      :start-positions (coerce-to-smallest-eltype
+                        (file-info-positions file-info))
      (if function
          (values :form (let ((direct-file-info (source-info-file-info info)))
-                         (when (eq :lisp (file-info-name direct-file-info))
+                         (when (eq :lisp (file-info-truename direct-file-info))
                            (elt (file-info-forms direct-file-info) 0)))
                  :function function)
          (values)))))
@@ -365,12 +438,12 @@
       (if (zerop length)
           #()
           (logically-readonlyize
-           (sb-xc:coerce seq
-                         `(simple-array
-                            ,(smallest-element-type (max max-positive
-                                                         (1- max-negative))
-                                                    (plusp max-negative))
-                            1)))))))
+           (coerce seq
+                   `(simple-array
+                     ,(smallest-element-type (max max-positive
+                                                  (1- max-negative))
+                                             (plusp max-negative))
+                     1)))))))
 
 (defun compact-vector (sequence)
   (cond ((and (= (length sequence) 1)
@@ -405,12 +478,12 @@
          (info (lambda-var-arg-info var))
          (indirect (and (lambda-var-indirect var)
                         (not (lambda-var-explicit-value-cell var))
-                        (neq (lambda-physenv fun)
-                             (lambda-physenv (lambda-var-home var)))))
+                        (neq (lambda-environment fun)
+                             (lambda-environment (lambda-var-home var)))))
          ;; Keep this condition in sync with PARSE-COMPILED-DEBUG-VARS
-         (large-fixnums (>= (integer-length sb-xc:most-positive-fixnum) 62))
+         (large-fixnums (>= (integer-length most-positive-fixnum) 62))
          more)
-    (declare (type index flags))
+    (declare (type (and sb-xc:fixnum unsigned-byte) flags))
     (when minimal
       (setq flags (logior flags compiled-debug-var-minimal-p))
       (unless (and tn (tn-offset tn))
@@ -420,8 +493,9 @@
                         (null (basic-var-sets var))))
                (not (gethash tn (ir2-component-spilled-tns
                                  (component-info *component-being-compiled*))))
-               (lexenv-contains-lambda fun
-                                       (lambda-lexenv (lambda-var-home var))))
+               (or (eq (lambda-var-home var) fun)
+                   (member var (environment-closure (lambda-environment fun))))
+               (not (optional-leaf-p var))) ;; not always initialized
       (setq flags (logior flags compiled-debug-var-environment-live)))
     (when save-tn
       (setq flags (logior flags compiled-debug-var-save-loc-p)))
@@ -509,7 +583,7 @@
                  (frob-leaf leaf (leaf-info leaf) gensym-p))))
       (frob-lambda fun t)
       (when (>= level 1)
-        (dolist (x (ir2-physenv-closure (physenv-info (lambda-physenv fun))))
+        (dolist (x (ir2-environment-closure (environment-info (lambda-environment fun))))
           (let ((thing (car x)))
             (when (lambda-var-p thing)
               (frob-leaf thing (cdr x) (>= level 2)))))
@@ -619,7 +693,7 @@
 ;;; Return a C-D-F structure with all the mandatory slots filled in.
 (defun dfun-from-fun (fun)
   (declare (type clambda fun))
-  (let* ((2env (physenv-info (lambda-physenv fun)))
+  (let* ((2env (environment-info (lambda-environment fun)))
          (dispatch (lambda-optional-dispatch fun))
          (main-p (and dispatch
                       (eq fun (optional-dispatch-main-entry dispatch))))
@@ -642,26 +716,26 @@
     (funcall (compiled-debug-fun-ctor kind)
              :name name
              #-fp-and-pc-standard-save :return-pc
-             #-fp-and-pc-standard-save (tn-sc+offset (ir2-physenv-return-pc 2env))
+             #-fp-and-pc-standard-save (tn-sc+offset (ir2-environment-return-pc 2env))
              #-fp-and-pc-standard-save :return-pc-pass
-             #-fp-and-pc-standard-save (tn-sc+offset (ir2-physenv-return-pc-pass 2env))
+             #-fp-and-pc-standard-save (tn-sc+offset (ir2-environment-return-pc-pass 2env))
              #-fp-and-pc-standard-save :old-fp
-             #-fp-and-pc-standard-save (tn-sc+offset (ir2-physenv-old-fp 2env))
+             #-fp-and-pc-standard-save (tn-sc+offset (ir2-environment-old-fp 2env))
              :encoded-locs
              (cdf-encode-locs
-              (label-position (ir2-physenv-environment-start 2env))
-              (label-position (ir2-physenv-elsewhere-start 2env))
+              (label-position (ir2-environment-environment-start 2env))
+              (label-position (ir2-environment-elsewhere-start 2env))
               (source-path-form-number (node-source-path (lambda-bind fun)))
               (label-position (block-label (lambda-block fun)))
-              (when (ir2-physenv-closure-save-tn 2env)
-                (tn-sc+offset (ir2-physenv-closure-save-tn 2env)))
+              (when (ir2-environment-closure-save-tn 2env)
+                (tn-sc+offset (ir2-environment-closure-save-tn 2env)))
               #+unwind-to-frame-and-call-vop
-              (when (ir2-physenv-bsp-save-tn 2env)
-                (tn-sc+offset (ir2-physenv-bsp-save-tn 2env)))
+              (when (ir2-environment-bsp-save-tn 2env)
+                (tn-sc+offset (ir2-environment-bsp-save-tn 2env)))
               #-fp-and-pc-standard-save
-              (label-position (ir2-physenv-lra-saved-pc 2env))
+              (label-position (ir2-environment-lra-saved-pc 2env))
               #-fp-and-pc-standard-save
-              (label-position (ir2-physenv-cfp-saved-pc 2env))))))
+              (label-position (ir2-environment-cfp-saved-pc 2env))))))
 
 ;;; Return a complete C-D-F structure for FUN. This involves
 ;;; determining the DEBUG-INFO level and filling in optional slots as
@@ -687,9 +761,13 @@
                  (compute-vars fun level var-locs))
            (setf (compiled-debug-fun-arguments dfun)
                  (compute-args fun var-locs))))
-    (when (>= level 1)
-      (setf (compiled-debug-fun-blocks dfun)
-            (compute-debug-blocks fun var-locs)))
+
+    (if (>= level 1)
+        (multiple-value-bind (blocks tlf-num)
+            (compute-debug-blocks fun var-locs)
+          (setf (compiled-debug-fun-blocks dfun) blocks
+                (compiled-debug-fun-tlf-number dfun) tlf-num))
+        (setf (compiled-debug-fun-tlf-number dfun) (find-tlf-number fun)))
     (if (xep-p fun)
         (setf (compiled-debug-fun-returns dfun) :standard)
         (let ((info (tail-set-info (lambda-tail-set fun))))
@@ -697,6 +775,7 @@
             (cond ((eq (return-info-kind info) :unknown)
                    (setf (compiled-debug-fun-returns dfun)
                          :standard))
+                  ((eq (return-info-kind info) :unboxed))
                   ((/= level 0)
                    (setf (compiled-debug-fun-returns dfun)
                          (compute-debug-returns fun)))))))
@@ -720,8 +799,8 @@
      (eq start (ir2-block-last-vop 2block))
      (eq (vop-name start) 'note-environment-start)
      next
-     (neq (ir2-block-physenv 2block)
-          (ir2-block-physenv next)))))
+     (neq (ir2-block-environment 2block)
+          (ir2-block-environment next)))))
 
 ;;; Return a DEBUG-INFO structure describing COMPONENT. This has to be
 ;;; called after assembly so that source map information is available.
@@ -735,18 +814,12 @@
                                    :adjustable t))
         (*contexts* (make-array 10
                                 :fill-pointer 0
-                                :adjustable t))
-        component-tlf-num)
+                                :adjustable t)))
     (dolist (lambda (component-lambdas component))
       (unless (empty-fun-p lambda)
-       (clrhash var-locs)
-       (let ((tlf-num (source-path-tlf-number
-                       (node-source-path (lambda-bind lambda)))))
-         (if component-tlf-num
-             (aver (or (block-compile *compilation*)
-                       (= component-tlf-num tlf-num)))
-             (setf component-tlf-num tlf-num))
-         (push (compute-1-debug-fun lambda var-locs) dfuns))))
+        (clrhash var-locs)
+        (push (compute-1-debug-fun lambda var-locs)
+              dfuns)))
     (let* ((sorted (sort dfuns #'< :key #'compiled-debug-fun-offset))
            (fun-map (compute-debug-fun-map sorted)))
       (make-compiled-debug-info
@@ -760,12 +833,6 @@
                         (sb-c::entry-info-name (car entries)))
                    (component-name component)))
        :fun-map fun-map
-       :tlf-num+offset (pack-tlf-num+offset
-                        component-tlf-num
-                        (and component-tlf-num
-                             (aref (file-info-positions
-                                    (source-info-file-info *source-info*))
-                                   component-tlf-num)))
        :contexts (compact-vector *contexts*)))))
 
 ;;; Write BITS out to BYTE-BUFFER in backend byte order. The length of
