@@ -255,18 +255,12 @@
 ;; Atomic update will be important for making the fasloader threadsafe
 ;; using a predominantly lock-free design, and other nice things.
 (defmacro atomic-set-info-value (category kind name lambda)
-  (with-unique-names (info-number proc)
+  (with-unique-names (info-number)
     `(let ((,info-number
             ,(if (and (keywordp category) (keywordp kind))
                  (meta-info-number (meta-info category kind))
                  `(meta-info-number (meta-info ,category ,kind)))))
-       ,(if (and (listp lambda) (eq (car lambda) 'lambda))
-            ;; rewrite as FLET because the compiler is unable to dxify
-            ;;   (DX-LET ((x (LAMBDA <whatever>))) (F x))
-            (destructuring-bind (lambda-list . body) (cdr lambda)
-              `(dx-flet ((,proc ,lambda-list ,@body))
-                 (%atomic-set-info-value ,name ,info-number #',proc)))
-            `(%atomic-set-info-value ,name ,info-number ,lambda)))))
+       (%atomic-set-info-value ,name ,info-number ,lambda))))
 
 ;; Perform the approximate equivalent operations of retrieving
 ;; (INFO :CATEGORY :KIND NAME), but if no info is found, invoke CREATION-FORM
@@ -279,13 +273,11 @@
 ;; A mutex-guarded table would probably be more appropriate in such cases.
 ;;
 (defmacro get-info-value-initializing (category kind name creation-form)
-  (let ((proc (make-symbol "THUNK")))
-    `(dx-flet ((,proc () ,creation-form))
-       (%get-info-value-initializing
-        ,(if (and (keywordp category) (keywordp kind))
-             (meta-info-number (meta-info category kind))
-             `(meta-info-number (meta-info ,category ,kind)))
-        ,name #',proc))))
+  `(%get-info-value-initializing
+    ,(if (and (keywordp category) (keywordp kind))
+         (meta-info-number (meta-info category kind))
+         `(meta-info-number (meta-info ,category ,kind)))
+    ,name (lambda () ,creation-form)))
 
 #+sb-xc-host
 (progn
