@@ -83,7 +83,7 @@
                 packagep functionp compiled-function-p not)
   (t) boolean (movable foldable flushable))
 
-(defknown (eq eql %eql/integer) (t t) boolean
+(defknown (eq eql) (t t) boolean
   (movable foldable flushable commutative))
 (defknown (equal equalp) (t t) boolean (foldable flushable recursive))
 
@@ -407,7 +407,7 @@
   (movable foldable unsafely-flushable))
 (defknown scale-float (float integer) float
   (movable foldable unsafely-flushable))
-(defknown float-radix (float) float-radix
+(defknown float-radix (float) sb-kernel::%float-radix
   (movable foldable unsafely-flushable))
 ;;; This says "unsafely flushable" as if to imply that there is a possibility
 ;;; of signaling a condition in safe code, but in practice we can never trap
@@ -429,10 +429,10 @@
                        (t
                         (specifier-type 'float)))))
 
-(defknown (float-digits float-precision) (float) float-digits
+(defknown (float-digits float-precision) (float) sb-kernel::%float-digits
   (movable foldable unsafely-flushable))
 (defknown integer-decode-float (float)
-    (values integer float-int-exponent (member -1 1))
+    (values double-float-significand float-int-exponent (member -1 1))
     (movable foldable unsafely-flushable))
 
 (defknown single-float-sign (single-float) single-float (movable foldable flushable))
@@ -533,17 +533,17 @@
   (movable foldable flushable no-verify-arg-count))
 
 (defknown character (t) character (movable foldable unsafely-flushable))
-(defknown char-code (character) char-code (movable foldable flushable))
+(defknown char-code (character) %char-code (movable foldable flushable))
 (defknown (char-upcase char-downcase) (character) character
   (movable foldable flushable))
 (defknown digit-char (unsigned-byte &optional (integer 2 36))
   (or character null) (movable foldable flushable))
-(defknown char-int (character) char-code (movable foldable flushable))
+(defknown char-int (character) %char-code (movable foldable flushable))
 (defknown char-name (character) (or simple-string null)
   (movable foldable flushable))
 (defknown name-char (string-designator) (or character null)
   (movable foldable flushable))
-(defknown code-char (char-code) character
+(defknown code-char (%char-code) character
   ;; By suppressing constant folding on CODE-CHAR when the
   ;; cross-compiler is running in the cross-compilation host vanilla
   ;; ANSI Common Lisp, we can use CODE-CHAR expressions to delay until
@@ -560,6 +560,9 @@
 (defknown subseq (proper-sequence index &optional sequence-end) consed-sequence
   (flushable))
 
+(defknown vector-subseq* (vector index sequence-end) (simple-array * (*))
+  (flushable))
+
 (defknown copy-seq (proper-sequence) consed-sequence (flushable)
   :derive-type (sequence-result-nth-arg 0 :preserve-dimensions t))
 
@@ -594,11 +597,14 @@
     vector
   (flushable no-verify-arg-count))
 
+(defknown (possibly-base-stringize possibly-base-stringize-to-heap) (string) simple-string
+  (flushable no-verify-arg-count))
+
 (defknown map (type-specifier (function-designator ((nth-arg 2 :sequence t)
                                                     (rest-args :sequence t))
                                                    (nth-arg 0 :sequence-type t))
                               proper-sequence &rest proper-sequence)
-  consed-sequence (call))
+    consed-sequence (call))
 (defknown %map (type-specifier function-designator &rest sequence) consed-sequence
   (call no-verify-arg-count))
 (defknown %map-for-effect-arity-1 (function-designator sequence) null
@@ -633,8 +639,8 @@
 (defknown (every notany notevery) (function-designator proper-sequence &rest proper-sequence) boolean
   (foldable unsafely-flushable call))
 
-(defknown reduce ((function-designator ((nth-arg 1 :sequence t :key :key :value :initial-value)
-                                        (nth-arg 1 :sequence t :key :key)))
+(defknown reduce ((function-designator ((nth-arg 1 :sequence t :key :key :value (:initial-value :from-end nil))
+                                        (nth-arg 1 :sequence t :key :key :value (:initial-value :from-end t))))
                   proper-sequence &rest t &key (:from-end t)
                   (:start (inhibit-flushing index 0))
                   (:end (inhibit-flushing sequence-end nil))
@@ -1207,19 +1213,19 @@
 
 (defknown array-element-type (array) (or list symbol)
   (foldable flushable))
-(defknown array-rank (array) array-rank (foldable flushable))
+(defknown array-rank (array) %array-rank (foldable flushable))
 ;; FIXME: there's a fencepost bug, but for all practical purposes our
 ;; ARRAY-RANK-LIMIT is infinite, thus masking the bug. e.g. if the
 ;; exclusive limit on rank were 8, then your dimension numbers can
 ;; be in the range 0 through 6, not 0 through 7.
-(defknown array-dimension (array array-rank) index (foldable flushable))
+(defknown array-dimension (array %array-rank) index (foldable flushable))
 (defknown array-dimensions (array) list (foldable flushable))
 (defknown array-in-bounds-p (array &rest integer) boolean (foldable flushable)
   :call-type-deriver #'array-call-type-deriver)
-(defknown array-row-major-index (array &rest index) array-total-size
+(defknown array-row-major-index (array &rest index) sb-kernel::%array-total-size
   (foldable flushable)
   :call-type-deriver #'array-call-type-deriver)
-(defknown array-total-size (array) array-total-size (foldable flushable))
+(defknown array-total-size (array) sb-kernel::%array-total-size (foldable flushable))
 (defknown adjustable-array-p (array) boolean (movable foldable flushable))
 
 (defknown svref (simple-vector index) t (foldable flushable))
@@ -1311,7 +1317,7 @@
    (:start (inhibit-flushing index 0))
    (:end (inhibit-flushing sequence-end nil)))
   simple-string (flushable)
-  :derive-type (sequence-result-nth-arg 0 :preserve-dimensions t))
+  :derive-type (sequence-result-nth-arg 0 :preserve-dimensions t :string-designator t))
 
 (defknown (nstring-upcase nstring-downcase nstring-capitalize)
   ((modifying string) &key (:start index) (:end sequence-end))
@@ -1632,39 +1638,39 @@
   (recursive))
 
 (defknown merge-pathnames
-  (pathname-designator &optional pathname-designator pathname-version)
+  (pathname-designator &optional pathname-designator sb-impl::%pathname-version)
   pathname
   ())
 
 (defknown make-pathname
  (&key (:defaults pathname-designator)
-       (:host (or string pathname-host))
-       (:device (or string pathname-device))
-       (:directory (or pathname-directory string (member :wild)))
-       (:name (or pathname-name string (member :wild)))
-       (:type (or pathname-type string (member :wild)))
-       (:version pathname-version) (:case pathname-component-case))
+       (:host (or string sb-impl::%pathname-host))
+       (:device (or string sb-impl::%pathname-device))
+       (:directory (or sb-impl::%pathname-directory string (member :wild)))
+       (:name (or sb-impl::%pathname-name string (member :wild)))
+       (:type (or sb-impl::%pathname-type string (member :wild)))
+       (:version sb-impl::%pathname-version) (:case pathname-component-case))
   pathname (unsafely-flushable))
 
 (defknown pathnamep (t) boolean (movable flushable))
 
 (defknown pathname-host (pathname-designator
                          &key (:case pathname-component-case))
-  pathname-host (flushable))
+  sb-impl::%pathname-host (flushable))
 (defknown pathname-device (pathname-designator
                            &key (:case pathname-component-case))
-  pathname-device (flushable))
+  sb-impl::%pathname-device (flushable))
 (defknown pathname-directory (pathname-designator
                               &key (:case pathname-component-case))
-  pathname-directory (flushable))
+  sb-impl::%pathname-directory (flushable))
 (defknown pathname-name (pathname-designator
                          &key (:case pathname-component-case))
-  pathname-name (flushable))
+  sb-impl::%pathname-name (flushable))
 (defknown pathname-type (pathname-designator
                          &key (:case pathname-component-case))
-  pathname-type (flushable))
+  sb-impl::%pathname-type (flushable))
 (defknown pathname-version (pathname-designator)
-  pathname-version (flushable))
+  sb-impl::%pathname-version (flushable))
 
 (defknown pathname= (pathname pathname) boolean (movable foldable flushable))
 
@@ -1723,6 +1729,8 @@
 
 (defknown (signal warn) (condition-designator-head &rest t) null)
 (defknown error (condition-designator-head &rest t) nil)
+(defknown %simple-error (condition-designator-head t &rest t) nil)
+(defknown %simple-type-error (condition-designator-head t t t &rest t) nil)
 (defknown cerror (format-control condition-designator-head &rest t) null)
 (defknown invalid-method-error (t format-control &rest t) *) ; FIXME: first arg is METHOD
 (defknown method-combination-error (format-control &rest t) *)
@@ -1859,6 +1867,23 @@
 
 (defknown array-storage-vector (array) (simple-array * (*))
     (any))
+
+(defknown octets-to-string ((vector (unsigned-byte 8))
+                            &key
+                            (:external-format t)
+                            (:start index)
+                            (:end sequence-end))
+  (or (simple-array character (*)) (simple-array base-char (*)))
+  (flushable))
+(defknown string-to-octets (string
+                            &key
+                            (:external-format t)
+                            (:start index)
+                            (:end sequence-end)
+                            (:null-terminate t))
+  (simple-array (unsigned-byte 8) (*))
+  (flushable))
+
 
 ;;;; magical compiler frobs
 
@@ -1891,15 +1916,12 @@
   :derive-type #'result-type-first-arg)
 
 (defknown %cleanup-point (&rest t) t (reoptimize-when-unlinking))
+(defknown %dynamic-extent-start () t)
 (defknown %special-bind (t t) t)
 (defknown %special-unbind (&rest symbol) t)
 (defknown %listify-rest-args (t index) list (flushable))
 (defknown %more-arg-context (t t) (values t index) (flushable))
 (defknown %more-arg (t index) t (flushable))
-(defknown %more-keyword-pair (t fixnum) (values t t) (flushable))
-#+stack-grows-downward-not-upward
-;;; FIXME: The second argument here should really be NEGATIVE-INDEX, but doing that
-;;; breaks the build, and I cannot seem to figure out why. --NS 2006-06-29
 (defknown %more-kw-arg (t fixnum) (values t t))
 (defknown %more-arg-values (t index index) * (flushable))
 
@@ -1918,7 +1940,6 @@
 (defknown %%primitive (t &rest t) *)
 (defknown %pop-values (t) t)
 (defknown %nip-values (t t &rest t) (values))
-(defknown %dummy-dx-alloc (t t) t)
 (defknown %type-check-error (t t t) nil)
 (defknown %type-check-error/c (t t t) nil)
 
@@ -1936,7 +1957,9 @@
   (movable foldable flushable no-verify-arg-count))
 (defknown %negate (number) number
   (movable foldable flushable no-verify-arg-count))
-(defknown (%check-bound check-bound) (array index t) index
+(defknown check-bound (array index t) index
+  (dx-safe))
+(defknown %check-bound (array index t) (values)
   (dx-safe))
 (defknown data-vector-ref (simple-array index) t
   (foldable flushable always-translatable))
@@ -1953,6 +1976,12 @@
 (defknown hairy-data-vector-set (array index t) t (no-verify-arg-count))
 (defknown hairy-data-vector-ref/check-bounds (array index) t (foldable no-verify-arg-count))
 (defknown hairy-data-vector-set/check-bounds (array index t) t (no-verify-arg-count))
+
+(defknown vector-hairy-data-vector-ref (vector index) t (foldable flushable no-verify-arg-count))
+(defknown vector-hairy-data-vector-set (vector index t) t (no-verify-arg-count))
+(defknown vector-hairy-data-vector-ref/check-bounds (vector index) t (foldable no-verify-arg-count))
+(defknown vector-hairy-data-vector-set/check-bounds (vector index t) t (no-verify-arg-count))
+
 (defknown %caller-frame () t (flushable))
 (defknown %caller-pc () system-area-pointer (flushable))
 (defknown %with-array-data (array index (or index null))
@@ -1964,7 +1993,7 @@
 (defknown %set-symbol-package (symbol t) t ())
 (defknown (%coerce-callable-to-fun %coerce-callable-for-call)
     (function-designator)
-    function (flushable))
+    function (flushable no-verify-arg-count))
 (defknown array-bounding-indices-bad-error (t t t) nil)
 (defknown sequence-bounding-indices-bad-error (t t t) nil)
 (defknown %find-position
@@ -2108,7 +2137,7 @@
 (defknown %program-error (&optional t &rest t) nil ())
 (defknown compiler-error (t &rest t) nil ())
 (defknown (compiler-warn compiler-style-warn) (t &rest t) (values) ())
-(defknown (compiler-mumble note-lossage note-unwinnage) (string &rest t) (values) ())
+(defknown (compiler-mumble note-lossage note-unwinnage) (format-control &rest t) (values) ())
 (defknown (compiler-notify maybe-compiler-notify) ((or format-control symbol) &rest t)
   (values)
   ())
@@ -2240,7 +2269,13 @@
   (defknown round-single (single-float #1#) single-float
       (foldable flushable movable always-translatable)))
 
-(defknown (overflow* overflow+ overflow-)
+(defknown (overflow* overflow+ overflow-
+           overflow-ash)
   (integer integer t)
   integer
   (movable always-translatable))
+
+(defknown overflow-negate
+  (integer t)
+  integer
+  (movable always-translatable))
