@@ -22,14 +22,15 @@
     (setf (info :variable :wired-tls (slot-special slot))
           (ash (slot-offset slot) word-shift))))
 
-#+gencgc
 (progn
 ;;; don't change allocation granularity
 (assert (= gencgc-alloc-granularity 0))
 ;;; cards are not larger than pages
 (assert (<= gencgc-page-bytes +backend-page-bytes+))
 ;;; largeness does not depend on the hardware page size
-(defconstant large-object-size (* 4 gencgc-page-bytes)))
+(defconstant large-object-size #-mark-region-gc (* 4 gencgc-page-bytes)
+                               #+mark-region-gc (* 3/4 gencgc-page-bytes))
+(assert (integerp large-object-size)))
 
 ;;; Keep this (mostly) lined up with 'early-objdef' for sanity's sake!
 ;;; The "transport" function is used only if the object is an OTHER-POINTER
@@ -144,10 +145,6 @@
                       (ldb (byte 32 0) bits) (ldb (byte 32 32) bits))
     (format stream "~%}~%"))
 
-  (format stream "extern unsigned char widetag_lowtag[256];
-static inline lispobj compute_lispobj(lispobj* base_addr) {
-  return make_lispobj(base_addr, LOWTAG_FOR_WIDETAG(*base_addr & WIDETAG_MASK));~%}~%")
-
   (format stream "~%#ifdef WANT_SCAV_TRANS_SIZE_TABLES~%")
   (let ((lowtag-tbl (make-array 256 :initial-element 0)))
     ;; Build a table translating from the from low byte of first word of any
