@@ -64,9 +64,9 @@
                ;; except in forcing discontiguous addresses for testing.
                ;; And of course, don't use them if unsupported.
                ((:fixedobj-space-start fixedobj-space-start*))
-               ((:fixedobj-space-size  fixedobj-space-size*) (* 24 1024 1024))
+               ((:fixedobj-space-size  fixedobj-space-size*) (* 48 1024 1024))
                ((:text-space-start text-space-start*))
-               ((:text-space-size  text-space-size*) (* 104 1024 1024))
+               ((:text-space-size  text-space-size*) (* 160 1024 1024))
                (small-space-size #x100000)
                ((:read-only-space-size ro-space-size)
                 #+darwin-jit small-space-size
@@ -86,7 +86,9 @@
                            #+(and win32 x86-64)
                            (seh-data ,(symbol-value '+backend-page-bytes+) win64-seh-data-addr)
                            #-immobile-space (alien-linkage-table ,alien-linkage-table-space-size)
-                           #+sb-safepoint
+                           ;; safepoint on 64-bit uses a relocatable trap page just below the card mark
+                           ;; table, which works nicely assuming a register is wired to the card table
+                           #+(and sb-safepoint (not x86-64))
                            ;; Must be just before NIL.
                            (safepoint ,(symbol-value '+backend-page-bytes+) gc-safepoint-page-addr)
                            (static ,small-space-size)
@@ -101,10 +103,9 @@
            (loop for (space size var-name) in spaces
                  appending
                  (let* ((relocatable
-                          ;; READONLY is usually movable now.
                           (member space '(fixedobj text #+relocatable-static-space static
                                           #+immobile-space alien-linkage-table
-                                          #-darwin-jit read-only)))
+                                          read-only)))
                         (start ptr)
                         (end (+ ptr size)))
                    (setf ptr end)
@@ -139,7 +140,7 @@
                  (32 (expt 2 29))
                  (64 (expt 2 30)))))
          ;; an arbitrary value to avoid kludging genesis
-         #+(and sb-xc-host (not darwin-jit))
+         #+sb-xc-host
          (defparameter read-only-space-end read-only-space-start)
          #-soft-card-marks (defconstant cards-per-page 1)
          (defconstant gencgc-card-bytes (/ gencgc-page-bytes cards-per-page))
@@ -200,7 +201,7 @@
     #+sb-thread *stop-for-gc-pending*
     sb-impl::*unweakened-vectors*
     *pinned-objects*
-    #+gencgc (*gc-pin-code-pages* 0)
+    (*gc-pin-code-pages* 0)
     ;; things needed for non-local-exit
     (*current-catch-block* 0)
     (*current-unwind-protect-block* 0)
@@ -365,7 +366,6 @@
   #+(and (not x86-64) immobile-space) 14 ; the safepoint trap page is at word index -15
   #+(and (not x86-64) (not immobile-space)) 0)
 
-#+gencgc
 (progn
   (defconstant +highest-normal-generation+ 5)
   (defconstant +pseudo-static-generation+ 6))
@@ -402,17 +402,16 @@
 (defconstant single-float-hidden-bit (ash 1 23))
 
 (defconstant double-float-bias 1022)
-(defconstant-eqx double-float-exponent-byte (byte 11 20) #'equalp)
-(defconstant-eqx double-float-significand-byte (byte 20 0) #'equalp)
+(defconstant-eqx double-float-exponent-byte (byte 11 52) #'equalp)
+(defconstant-eqx double-float-significand-byte (byte 52 0) #'equalp)
+(defconstant-eqx double-float-hi-exponent-byte (byte 11 20) #'equalp)
+(defconstant-eqx double-float-hi-significand-byte (byte 20 0) #'equalp)
 (defconstant double-float-normal-exponent-min 1)
 (defconstant double-float-normal-exponent-max #x7FE)
-(defconstant double-float-hidden-bit (ash 1 20))
+(defconstant double-float-hidden-bit (ash 1 52))
 
-(defconstant single-float-digits
-  (+ (byte-size single-float-significand-byte) 1))
-
-(defconstant double-float-digits
-  (+ (byte-size double-float-significand-byte) 32 1))
+(defconstant single-float-digits 24)
+(defconstant double-float-digits 53)
 )
 
 (push '("SB-VM" +c-callable-fdefns+ +common-static-symbols+)
