@@ -137,7 +137,9 @@
           #-c-stack-is-control-stack *non-descriptor-args*))
 
 (defun fixed-call-arg-location (type state)
-  (let* ((primtype (primitive-type type))
+  (let* ((primtype (if (typep type 'primitive-type)
+                       type
+                       (primitive-type type)))
          (sc (find descriptor-reg-sc-number (sb-c::primitive-type-scs primtype) :test-not #'eql)))
     (case (primitive-type-name primtype)
       ((double-float single-float)
@@ -177,7 +179,7 @@
 
 (defun make-number-stack-pointer-tn ()
   #+c-stack-is-control-stack
-  (make-restricted-tn *fixnum-primitive-type* any-reg-sc-number)
+  (sb-c::make-unused-tn)
   #-c-stack-is-control-stack
   (make-normal-tn *fixnum-primitive-type*))
 
@@ -206,13 +208,6 @@
   "Cause a continuable error.  ERROR-CODE is the error to cause."
   (emit-error-break vop cerror-trap (error-number-or-lose error-code) values))
 
-#+sb-safepoint
-(define-vop (insert-safepoint)
-  (:policy :fast-safe)
-  (:translate sb-kernel::gc-safepoint)
-  (:generator 0
-    (emit-safepoint)))
-
 ;;; Does the TN definitely hold *any* of the 4 pointer types
 (defun pointer-tn-ref-p (tn-ref)
   (and (sc-is (tn-ref-tn tn-ref) descriptor-reg)
@@ -233,39 +228,75 @@
 ;;; then NIL is ok as the input. Indicate this by specifying PERMIT-NIL.
 ;;; With rare exception it should always be permitted, though not on ppc64
 ;;; where it would never be. The safe default is NIL.
-(defun other-pointer-tn-ref-p (tn-ref &optional permit-nil)
+(defun other-pointer-tn-ref-p (tn-ref &optional permit-nil
+                                                immediates-tested)
   (and (sc-is (tn-ref-tn tn-ref) descriptor-reg)
        (not (types-equal-or-intersect
              (tn-ref-type tn-ref)
              (if permit-nil
-                 (specifier-type '(or fixnum
-                                   #+64-bit single-float
-                                   function
-                                   cons
-                                   instance
-                                   character))
-                 (specifier-type '(or fixnum
-                                   #+64-bit single-float
-                                   function
-                                   list
-                                   instance
-                                   character)))))))
+                 (specifier-type '(or function cons instance character))
+                 (specifier-type '(or function list instance character)))))
+       (or (memq 'fixnum immediates-tested)
+           (not (types-equal-or-intersect (tn-ref-type tn-ref) (specifier-type 'fixnum))))
+       #+64-bit
+       (or (memq single-float-widetag immediates-tested)
+           (not (types-equal-or-intersect (tn-ref-type tn-ref) (specifier-type 'single-float))))))
 
 (defun fixnum-or-other-pointer-tn-ref-p (tn-ref &optional permit-nil)
   (and (sc-is (tn-ref-tn tn-ref) descriptor-reg)
        (not (types-equal-or-intersect
              (tn-ref-type tn-ref)
-             (specifier-type (if permit-nil
-                                 '(or #+64-bit single-float
-                                   function
-                                   cons
-                                   instance
-                                   character)
-                                 '(or #+64-bit single-float
-                                   function
-                                   list
-                                   instance
-                                   character)))))))
+             (if permit-nil
+                 (specifier-type '(or cons . #1=(#+64-bit single-float function cons instance character)))
+                 (specifier-type '(or list . #1#)))))))
+
+;;; Can LOWTAG be distinguished from other tn lowtags by testing a single bit?
+(defun tn-ref-lowtag-bit (lowtag tn-ref &optional permit-nil immediates-tested)
+  (declare (fixnum lowtag))
+  (when tn-ref
+    (let ((type (tn-ref-type tn-ref))
+          (set 0)
+          (clear 0))
+      (when (eq type *universal-type*)
+        (return-from tn-ref-lowtag-bit))
+      (unless (memq 'fixnum immediates-tested)
+        (when (types-equal-or-intersect type (specifier-type 'fixnum))
+          (setf set (logandc2 lowtag-mask fixnum-tag-mask)
+                clear lowtag-mask)))
+      (macrolet ((s (lowtag type)
+                   `(when (and (/= lowtag ,lowtag)
+                               (types-equal-or-intersect type ,(if (symbolp type)
+                                                                   `(specifier-type ',type)
+                                                                   type)))
+                      (setf set (logior set (logand lowtag-mask ,lowtag))
+                            clear (logior clear (logandc2 lowtag-mask ,lowtag))))))
+        #+64-bit
+        (unless (memq single-float-widetag immediates-tested)
+          (s single-float-widetag single-float))
+        (if permit-nil
+            (s list-pointer-lowtag cons)
+            (s list-pointer-lowtag list))
+        (s fun-pointer-lowtag function)
+        (s instance-pointer-lowtag instance)
+        (s character-widetag character)
+        (s other-pointer-lowtag
+           (if permit-nil
+               (specifier-type '(not (or cons . #1=(#+64-bit fixnum single-float function cons instance character))))
+               (specifier-type '(not (or list . #1#)))))
+        (let ((set-bit (logand lowtag-mask (logandc2 lowtag set)))
+              (clear-bit (logandc2 lowtag-mask (logior lowtag clear))))
+          (cond ((plusp set-bit)
+                 (values (sb-kernel::first-bit-set set-bit) 1))
+                ((plusp clear-bit)
+                 (values (sb-kernel::first-bit-set clear-bit) 0))))))))
+
+(defun fun-or-other-pointer-tn-ref-p (tn-ref &optional permit-nil)
+  (and (sc-is (tn-ref-tn tn-ref) descriptor-reg)
+       (not (types-equal-or-intersect
+             (tn-ref-type tn-ref)
+             (if permit-nil
+                 (specifier-type '(or cons instance character fixnum #+64-bit single-float))
+                 (specifier-type '(or cons instance character fixnum #+64-bit single-float)))))))
 
 (defun not-nil-tn-ref-p (tn-ref)
   (not (types-equal-or-intersect (tn-ref-type tn-ref)
@@ -314,11 +345,11 @@
 (define-load-time-global *store-barriers-potentially-emitted* 0)
 (define-load-time-global *store-barriers-emitted* 0)
 
-(defun require-gc-store-barrier-p (object value-tn-ref value-tn)
+(defun require-gengc-barrier-p (object value-tn-ref value-tn &optional allocator)
   (incf *store-barriers-potentially-emitted*)
   ;; If OBJECT is stack-allocated, elide the barrier
   (when (stack-consed-p object)
-    (return-from require-gc-store-barrier-p nil))
+    (return-from require-gengc-barrier-p nil))
   (flet ((potential-heap-pointer-p (tn tn-ref)
            (when (sc-is tn any-reg) ; must be fixnum
              (return-from potential-heap-pointer-p nil))
@@ -328,22 +359,38 @@
            ;; If immediate non-pointer, elide the barrier
            (when (sc-is tn immediate)
              (let ((value (tn-value tn)))
-               (when (sb-xc:typep value '(or character sb-xc:fixnum boolean
-                                             #+64-bit single-float))
+               (when (sb-xc:typep value '(or character sb-xc:fixnum
+                                          #+64-bit single-float
+                                          boolean))
                  (return-from potential-heap-pointer-p nil))))
            (when (sb-c::unbound-marker-tn-p tn)
              (return-from potential-heap-pointer-p nil))
            ;; And elide for things like (OR FIXNUM NULL)
            (let ((type (tn-ref-type tn-ref)))
-             (when (csubtypep type (specifier-type '(or character sb-xc:fixnum boolean
-                                                        #+64-bit single-float)))
+             (when (or (csubtypep type #1=(specifier-type '(or character sb-xc:fixnum boolean
+                                                         #+64-bit single-float)))
+                       (let ((diff (type-difference type #1#)))
+                         (and (member-type-p diff)
+                              #-sb-xc-host
+                              (loop for member in (member-type-members diff)
+                                    always
+                                    (and (eql (generation-of member) +pseudo-static-generation+)
+                                         (or (not (sb-c::producing-fasl-file))
+                                             (and (symbolp member)
+                                                  (logtest +symbol-initial-core+ (get-header-data member)))))))))
                (return-from potential-heap-pointer-p nil)))
-           t))
+           t)
+         (boxed-tn-p (value-tn)
+           (let* ((prim-type (sb-c::tn-primitive-type value-tn))
+                  (scs (and prim-type
+                            (sb-c::primitive-type-scs prim-type))))
+             (or (singleton-p scs)
+                 (not (member descriptor-reg-sc-number scs))))))
     (cond (value-tn
            (unless (eq (tn-ref-tn value-tn-ref) value-tn)
              (aver (eq (tn-ref-load-tn value-tn-ref) value-tn)))
            (unless (potential-heap-pointer-p value-tn value-tn-ref)
-             (return-from require-gc-store-barrier-p nil)))
+             (return-from require-gengc-barrier-p nil)))
           (value-tn-ref ; a list of refs linked through TN-REF-ACROSS
            ;; (presumably from INSTANCE-SET-MULTIPLE)
            (let ((any-pointer
@@ -352,14 +399,35 @@
                     (when (potential-heap-pointer-p (tn-ref-tn ref) ref)
                       (return t)))))
              (unless any-pointer
-               (return-from require-gc-store-barrier-p nil))))))
-  (incf *store-barriers-emitted*)
-  t)
+               (return-from require-gengc-barrier-p nil)))))
+    (let (why)
+      (cond ((and value-tn
+                  value-tn-ref
+                  ;; Can this TN be boxed after the allocator?
+                  (boxed-tn-p value-tn)
+                  (or (eq allocator :allocator)
+                      (setf why
+                            (sb-c::set-slot-old-p (sb-c::vop-node (tn-ref-vop value-tn-ref))
+                                                  (vop-arg-position value-tn-ref (tn-ref-vop value-tn-ref))))))
+             (values nil why))
+            (t
+             (incf *store-barriers-emitted*)
+             t)))))
 
 (defun vop-nth-arg (n vop)
   (let ((ref (vop-args vop)))
     (dotimes (i n ref) (setq ref (tn-ref-across ref)))))
 
+(defun vop-arg-position (tn-ref vop)
+  (let ((types (sb-c::vop-info-arg-types (sb-c::vop-info vop))))
+    (loop with i = -1
+          for ref = (vop-args vop) then (tn-ref-across ref)
+          do
+          (loop do (pop types)
+                   (incf i)
+                while (typep (car types) '(cons (eql :constant))))
+          when (eq ref tn-ref) return i)))
+
 (defun length-field-shift (widetag)
   (if (= widetag instance-widetag)
       instance-length-shift
@@ -465,3 +533,31 @@
                (when (and (eq (first data) :declare)
                           (eq (second data) 'sb-c::tlab))
                  (return (eq (third data) :system))))))))
+
+(defun call-out-pseudo-atomic-p (vop)
+  (declare (ignorable vop))
+  ;; If #+sb-safepoint, the decision to poll for a safepoint
+  ;; occurs at the end. In that case, we can not prevent stop-for-GC
+  ;; from occurring in the C code, because foreign code is allowed
+  ;; to run during GC; it just can't go back into Lisp until GC is over.
+  #-sb-safepoint
+  (loop for e = (sb-c::node-lexenv (sb-c::vop-node vop))
+        then (sb-c::lexenv-parent e)
+        while e
+        thereis (sb-c::lexenv-find 'sb-vm::.pseudo-atomic-call-out.
+                                   vars :lexenv e)))
+
+;;; The SET vop should alway get a symbol that is known at compile-time
+;;; even though the compiler might have already done you a "favor" of
+;;; producing a load-TN for a random constant. This tries to undo that.
+(defun known-symbol-use-p (vop symbol)
+  (cond ((sc-is symbol constant immediate)
+         (tn-value symbol))
+        (t
+         ;; Given a DESCRIPTOR-REG you can refer back to the vop args
+         ;; to figure out what was loaded.
+         (let ((type (tn-ref-type (vop-args vop))))
+           ;; I'm pretty sure this _must_ be a MEMBER type.
+           (when (and (member-type-p type)
+                      (= (member-type-size type) 1))
+             (the symbol (first (member-type-members type))))))))
