@@ -14,7 +14,6 @@
 ;;; We need to define these predicates, since the TYPEP source
 ;;; transform picks whichever predicate was defined last when there
 ;;; are multiple predicates for equivalent types.
-(define-source-transform short-float-p (x) `(single-float-p ,x))
 #-long-float
 (define-source-transform long-float-p (x) `(double-float-p ,x))
 
@@ -121,7 +120,7 @@
 
                  ;; Definitely do this if copying to stack
                  ;; (Allocation has to be inlined, otherwise there's no way to DX it)
-                 (or (lvar-dynamic-extent result)
+                 (or (node-stack-allocate-p node)
                      (and dd-copier
                           (eq (sb-int:info :function :inlinep dd-copier) 'inline))
                      ;; Or if it's a small fixed number of words
@@ -543,49 +542,6 @@
           (if (zerop (lvar-value item)) '(- length count) 'count)
           '(if (zerop item) (- length count) count))))
 
-;;; This transform does not require that ITEM be derived as BIT,
-;;; but at runtime it has to be.
-(deftransform fill ((sequence item) (simple-bit-vector t) *
-                    :policy (>= speed space))
-  `(let ((value (logand (- (the bit item)) most-positive-word)))
-     ;; Unlike for the SIMPLE-BASE-STRING case, we are allowed to touch
-     ;; bits beyond LENGTH with impunity.
-     (dotimes (index (ceiling (vector-length sequence) sb-vm:n-word-bits))
-       (declare (optimize (speed 3) (safety 0))
-                (type index index))
-       (setf (%vector-raw-bits sequence index) value))
-     sequence))
-
-(deftransform fill ((sequence item) (simple-base-string t) *
-                                    :policy (>= speed space))
-  (let ((multiplier (logand #x0101010101010101 most-positive-word)))
-    `(let* ((value ,(if (and (constant-lvar-p item)
-                             (typep (lvar-value item) 'base-char))
-                        (* multiplier (char-code (lvar-value item)))
-                        ;; Use multiplication if it's known to be cheap
-                        #+(or x86 x86-64)
-                        `(* ,multiplier (char-code (the base-char item)))
-                        #-(or x86 x86-64)
-                        '(let ((code (char-code (the base-char item))))
-                          (setf code (dpb code (byte 8 8) code))
-                          (setf code (dpb code (byte 16 16) code))
-                          #+64-bit (dpb code (byte 32 32) code))))
-            (len (vector-length sequence))
-            (words (truncate len sb-vm:n-word-bytes)))
-       (dotimes (index words)
-         (declare (optimize (speed 3) (safety 0))
-                  (type index index))
-         (setf (%vector-raw-bits sequence index) value))
-       ;; For 64-bit:
-       ;;  if 1 more byte should be written, then shift-towards-start 56
-       ;;  if 2 more bytes ...               then shift-towards-start 48
-       ;;  etc
-       ;; This correctly rewrites the trailing null in its proper place.
-       (let ((bits (ash (mod len sb-vm:n-word-bytes) 3)))
-         (when (plusp bits)
-           (setf (%vector-raw-bits sequence words)
-                 (shift-towards-start value (- bits)))))
-       sequence)))
 
 ;;;; %BYTE-BLT
 
@@ -596,7 +552,7 @@
 ;;; currently (ca. sbcl-0.6.12.30) the main interface for code in
 ;;; SB-KERNEL and SB-SYS (e.g. i/o code). It's not clear that it's the
 ;;; ideal interface, though, and it probably deserves some thought.
-(deftransform %byte-blt ((src src-start dst dst-start dst-end)
+(deftransform %byte-blt ((src src-start dst dst-start nbytes)
                          ((or (simple-unboxed-array (*)) system-area-pointer)
                           index
                           (or (simple-unboxed-array (*)) system-area-pointer)
@@ -630,7 +586,7 @@
               (sap-ref-8 dst (1- dst-end)) (sap-ref-8 dst (1- dst-end))))
       (memmove (sap+ (sapify dst) dst-start)
                (sap+ (sapify src) src-start)
-               (- dst-end dst-start)))
+               nbytes))
      (values)))
 
 ;;;; transforms for EQL of floating point values
@@ -666,7 +622,7 @@
                      (push `(define-good-modular-fun ,fun :untagged t) result)
                      (push `(define-good-modular-fun ,fun :tagged t) result))))))
   (define-good-signed-modular-funs
-      logand logandc1 logandc2 logeqv logior lognand lognor lognot
+      logand logandc2 logeqv logior lognand lognor lognot
       logorc1 logorc2 logxor))
 
 ;;;; word-wise logical operations
@@ -702,7 +658,7 @@
   `(logand (logorc2 x y) ,most-positive-word))
 
 (deftransform word-logical-andc1 ((x y))
-  `(logand (logandc1 x y) ,most-positive-word))
+  `(logand (logandc2 y x) ,most-positive-word))
 
 (deftransform word-logical-andc2 ((x y))
   `(logand (logandc2 x y) ,most-positive-word))
