@@ -223,7 +223,7 @@
                  (return-from mark-2block))
                (setf (gethash 2block live-2blocks) t)
                (map nil #'mark-2block (cdr (gethash 2block *2block-info*)))))
-      (declare (truly-dynamic-extent #'mark-2block))
+      (declare (dynamic-extent #'mark-2block))
       (mark-2block (block-info (component-head component))))
 
     (flet ((delete-2block (2block)
@@ -1024,34 +1024,32 @@
                                           tail-call tail-call-named
                                           static-tail-call-named))
                   (delete-vop vop)
-                  ;; Delete the VOP that saves the stack pointer too.
+                  ;; Delete the VOPs that save the stack pointer too.
                   (let ((tn (tn-ref-tn (vop-args vop))))
                     (unless (tn-reads tn)
-                      (aver (eq (vop-name (tn-ref-vop (tn-writes tn)))
-                                'current-stack-pointer))
-                      (delete-vop (tn-ref-vop (tn-writes tn)))))
+                      (do ((ref (tn-writes tn) (tn-ref-next ref)))
+                          ((null ref))
+                        (aver (eq (vop-name (tn-ref-vop ref))
+                                  'current-stack-pointer))
+                        (delete-vop (tn-ref-vop ref)))))
                   (return))
                  (t
                   (return)))))
 
-;;; stack-analyze may have avoided creating a cleanup
-;;; leaving this unused
-(defoptimizer (vop-optimize current-stack-pointer) (vop)
-  (let ((tn (tn-ref-tn (vop-results vop))))
-    (when (not (tn-reads tn))
-      (delete-vop vop))))
-
 ;;; Load the WIDETAG once for a series of type tests.
 (when-vop-existsp (:named sb-vm::load-other-pointer-widetag)
   (defoptimizer (vop-optimize %other-pointer-subtype-p) (vop)
     (let (vops
           stop
           null
-          (value (tn-ref-tn (vop-args vop))))
+          (value (tn-ref-tn (vop-args vop)))
+          zero-extend)
       (labels ((good-vop-p (vop)
                  (and (singleton-p (ir2block-predecessors (vop-block vop)))
                       (or (getf sb-vm::*other-pointer-type-vops* (vop-name vop))
-                          (eq (vop-name vop) '%other-pointer-subtype-p))
+                          (eq (vop-name vop) '%other-pointer-subtype-p)
+                          (and (eq (vop-name vop) '%other-pointer-widetag)
+                               (setf zero-extend t)))
                       (eq (tn-ref-tn (vop-args vop)) value)))
                (chain (vop &optional (collect t))
                  (let ((next (branch-destination vop nil)))
@@ -1090,7 +1088,8 @@
                                  vop
                                  (list (ir2-block-label stop)
                                        (and null
-                                            (ir2-block-label (vop-block null)))))
+                                            (ir2-block-label (vop-block null)))
+                                       #+x86-64 zero-extend))
             (update-block-succ block
                                (cons stop
                                      (ir2block-successors block)))
@@ -1102,18 +1101,25 @@
                                    (third info)
                                    (getf sb-vm::*other-pointer-type-vops* (vop-name vop)))
                     do
-                    (let ((next (vop-next vop)))
-                      (when (and next
-                                 (eq (vop-name next) 'branch-if))
-                        (setf info (vop-codegen-info next))
-                        (delete-vop next))
-                      (emit-and-insert-vop (vop-node vop)
-                                           (vop-block vop)
-                                           test-vop
-                                           (reference-tn widetag nil)
-                                           nil
-                                           vop
-                                           (list (first info) (second info) tags)))
+                    (if (eq (vop-name vop) '%other-pointer-widetag)
+                        (emit-and-insert-vop (vop-node vop)
+                                             (vop-block vop)
+                                             (template-or-lose 'move)
+                                             (reference-tn widetag nil)
+                                             (reference-tn (tn-ref-tn (vop-results vop )) t)
+                                             vop)
+                        (let ((next (vop-next vop)))
+                          (when (and next
+                                     (eq (vop-name next) 'branch-if))
+                            (setf info (vop-codegen-info next))
+                            (delete-vop next))
+                          (emit-and-insert-vop (vop-node vop)
+                                               (vop-block vop)
+                                               test-vop
+                                               (reference-tn widetag nil)
+                                               nil
+                                               vop
+                                               (list (first info) (second info) tags))))
                     (delete-vop vop)))))))
     nil)
 
@@ -1226,6 +1232,116 @@
               (delete-vop to)
               nil)))))))
 
+(when-vop-existsp (:named sb-vm::return-values-list)
+  (defoptimizer (vop-optimize values-list)
+      (vop)
+    (let ((return (next-vop-is vop '(return-multiple))))
+      (when return
+        (vop-bind (list) () vop
+          (emit-and-insert-vop (vop-node return)
+                               (vop-block return)
+                               (template-or-lose 'sb-vm::return-values-list)
+                               (reference-tn list nil)
+                               nil
+                               return)
+          (delete-vop vop)
+          (delete-vop return))
+        nil))))
+
+;; Try to combine consecutive uses of %INSTANCE-SET. This can't be
+;; done prior to selecting representations because
+;; SELECT-REPRESENTATIONS might insert some things like
+;; MOVE-FROM-DOUBLE which makes the "consecutive" vops no longer
+;; consecutive.
+;; It seems like this should also supplant the #+arm64 hack in GENERATE-CODE.
+(when-vop-existsp (:named sb-vm::instance-set-multiple)
+  (defoptimizer (vop-optimize sb-vm::instance-index-set select-representations) (vop)
+    (let ((instance (tn-ref-tn (vop-args vop)))
+          (this vop)
+          (pairs))
+      (loop
+       (let ((index (tn-ref-tn (tn-ref-across (vop-args this)))))
+         (unless (constant-tn-p index) (return))
+         (push (cons (tn-value index) (tn-ref-tn (sb-vm::vop-nth-arg 2 this)))
+               pairs))
+       (let ((next (vop-next this)))
+         (unless (and next
+                      (eq (vop-name next) 'sb-vm::instance-index-set)
+                      (eq (tn-ref-tn (vop-args next)) instance))
+           (return))
+         (setq this next)))
+      (when (cdr pairs)                 ; if at least 2
+        (setq pairs (nreverse pairs))
+        (let ((new (emit-and-insert-vop
+                    (vop-node vop) (vop-block vop)
+                    (template-or-lose 'sb-vm::instance-set-multiple)
+                    (reference-tn-list (cons instance (mapcar #'cdr pairs)) nil)
+                    nil vop (list (mapcar #'car pairs)))))
+          (loop (let ((next (vop-next vop)))
+                  (delete-vop vop)
+                  (pop pairs)
+                  (setq vop next))
+                (unless pairs (return)))
+          new)))))
+
+(defun vop-label (vop)
+  (let ((block (vop-block vop)))
+    (or (ir2-block-%label block)
+        (setf (ir2-block-%label block) (gen-label)))))
+
+(when-vop-existsp (:named sb-vm::signed-byte-64-p-move-to-word)
+  (flet ((opt (vop new-vop &optional not-vop)
+           (let ((dest (branch-destination vop))
+                 (vop2 (branch-destination vop nil)))
+             (vop-bind (in) () vop
+               (when (and dest
+                          (singleton-p (ir2block-predecessors (vop-block dest)))
+                          (eq (vop-name dest) 'sb-vm::move-to-word/integer))
+                 (vop-bind (in2) () dest
+                   (when (eq in in2)
+                     (cond ((and vop2
+                                 (eq (vop-name vop2) not-vop)
+                                 (singleton-p (ir2block-predecessors (vop-block dest)))
+                                 (let ((dest2 (branch-destination vop2)))
+                                   (when (and dest2
+                                              (singleton-p (ir2block-predecessors (vop-block dest2)))
+                                              (eq (vop-name dest2) 'sb-vm::move-to-word/integer))
+                                     (vop-bind (in21) () vop2
+                                       (vop-bind (in22) () dest2
+                                         (when (and (eq in21 in)
+                                                    (eq in22 in))
+                                           (emit-and-insert-vop
+                                            (vop-node vop) (vop-block vop)
+                                            (template-or-lose 'sb-vm::un/signed-byte-64-p-move-to-word)
+                                            (reference-tn-refs (vop-args vop) nil)
+                                            (reference-tn-ref-list (list (vop-results dest)
+                                                                         (vop-results dest2))
+                                                                   t)
+                                            vop
+                                            (append (vop-codegen-info vop)
+                                                    (vop-codegen-info vop2)
+                                                    (list (vop-label (branch-destination vop2 nil)))))
+                                           (delete-vop vop)
+                                           (delete-vop vop2)
+                                           (delete-vop dest)
+                                           (delete-vop dest2)
+                                           t)))))))
+                           (t
+                            (emit-and-insert-vop
+                             (vop-node vop) (vop-block vop)
+                             (template-or-lose new-vop)
+                             (reference-tn-refs (vop-args vop) nil)
+                             (reference-tn-refs (vop-results dest) t)
+                             vop (vop-codegen-info vop))
+                            (delete-vop vop)
+                            (delete-vop dest))))))))
+           nil))
+    (defoptimizer (vop-optimize signed-byte-64-p select-representations) (vop)
+      (opt vop 'sb-vm::signed-byte-64-p-move-to-word
+           'unsigned-byte-64-p))
+    (defoptimizer (vop-optimize unsigned-byte-64-p select-representations) (vop)
+      (opt vop 'sb-vm::unsigned-byte-64-p-move-to-word))))
+
 (defun very-temporary-p (tn)
   (let ((writes (tn-writes tn))
         (reads (tn-reads tn)))
@@ -1294,51 +1410,201 @@
                                      (funcall it vop))))
                           (vop-next vop)))))))
 
-(defun merge-instance-set-vops (vop)
-  (let ((instance (tn-ref-tn (vop-args vop)))
-        (this vop)
-        (pairs))
-    (loop
-       (let ((index (tn-ref-tn (tn-ref-across (vop-args this)))))
-         (unless (constant-tn-p index) (return))
-         (push (cons (tn-value index) (tn-ref-tn (sb-vm::vop-nth-arg 2 this)))
-               pairs))
-       (let ((next (vop-next this)))
-         (unless (and next
-                      (eq (vop-name next) 'sb-vm::instance-index-set)
-                      (eq (tn-ref-tn (vop-args next)) instance))
-           (return))
-         (setq this next)))
-    (unless (cdr pairs) ; if at least 2
-      (return-from merge-instance-set-vops nil))
-    (setq pairs (nreverse pairs))
-    (let ((new (emit-and-insert-vop
-                (vop-node vop) (vop-block vop)
-                (template-or-lose 'sb-vm::instance-set-multiple)
-                (reference-tn-list (cons instance (mapcar #'cdr pairs)) nil)
-                nil vop (list (mapcar #'car pairs)))))
-      (loop (let ((next (vop-next vop)))
-              (delete-vop vop)
-              (pop pairs)
-              (setq vop next))
-            (unless pairs (return)))
-      new)))
+;;; These are the acceptable vops in a sequence that can be brought together
+;;; under one pseudo-atomic invocation. In general any vop that does not
+;;; allocate is OK, but I'd rather be restrictive than permissive here.
+(defglobal *vops-allowed-within-pseudo-atomic*
+    '(set-slot %raw-instance-set/word %raw-instance-set/signed-word %raw-instance-set/single
+      %raw-instance-set/double %raw-instance-set/complex-single %raw-instance-set/complex-double
+      move move-operand make-unbound-marker make-funcallable-instance-tramp
+      sb-vm::move-from-word/fixnum sb-vm::move-to-word/fixnum
+      sb-vm::move-from-fixnum+1 sb-vm::move-from-fixnum-1))
 
-(defun ir2-optimize-stores (component)
-  ;; This runs after representation selection. It's the same as RUN-VOP-OPTIMIZERS,
-  ;; but with hardcoded vop names and function to call.
-  ;; It seems like this should also supplant the #+arm64 hack in GENERATE-CODE.
+;;; Return list of vops between and including VOP and LAST
+;;; without regard to IR2 block boundaries (as long as there is
+;;; no branching control flow in the specified range)
+(defun collect-vops-between (vop last)
+  (collect ((result))
+    (loop (result vop)
+          (if (eq vop last) (return))
+          (setq vop
+                (or (vop-next vop)
+                    ;; IR2 blocks were split. Assert that the flow is straight-line
+                    (let ((successors (ir2block-successors (vop-block vop))))
+                      (aver (singleton-p successors))
+                      (let* ((successor (car successors))
+                             (predecessors (ir2block-predecessors successor)))
+                        (aver (eq (car predecessors) (vop-block vop)))
+                        (aver (not (cdr predecessors)))
+                        (ir2-block-start-vop successor))))))
+    (result)))
+
+;;; This should be among the final IR2 optimizer passes so that no new vops get
+;;; inserted that would change the decision about whether to extend
+;;; the scope of pseudo-atomic to cover them.
+(defun attempt-pseudo-atomic-store-bunching (component)
+  (labels
+      ((terminate-inits (last)
+         ;; The allocator has to recognize :PSEUDO-ATOMIC in its codegen info
+         ;; but the slot setters don't all have to be updated to understand how to
+         ;; terminate the pseudo-atomic sequence. It's a separate vop to do that.
+         (emit-and-insert-vop (vop-node last) (vop-block last)
+                              (template-or-lose 'end-pseudo-atomic)
+                              nil nil (vop-next last)))
+       (process-closure-inits (vop)
+         (let* ((result-ref (vop-results vop))
+                (closure (tn-ref-tn result-ref))
+                (last-init))
+           (do ((init (vop-next vop) (vop-next init)))
+               ((or (not init) (neq (vop-name init) 'closure-init)))
+             ;; IR2-CONVERT-ENCLOSE can output more than one MAKE-CLOSURE
+             ;; and then some CLOSURE-INITs.  This happens with mutually-referential
+             ;; closures. Sadly we can't optimize that to move the inits underneath
+             ;; the allocator's pseudo-atomic.So just beware of the pattern
+             ;;   MAKE => c1 / INIT c1 / MAKE => c2 / INIT c2 / INIT c1
+             (unless (eq closure (tn-ref-tn (vop-args init)))
+               (return))
+             (setf (vop-codegen-info init) (append (vop-codegen-info init) '(:pseudo-atomic))
+                   last-init init))
+           (when last-init
+             (setf (vop-codegen-info vop)
+                   (append (vop-codegen-info vop) '(:pseudo-atomic)))
+             (terminate-inits last-init))))
+       (process-general-inits (first last &aux last-init)
+         (aver (neq first last))
+         (dolist (init (cdr (collect-vops-between first last)))
+           (cond
+             ((eq (vop-name init) 'set-slot)
+              (setf (vop-codegen-info init) (append (vop-codegen-info init) '(:pseudo-atomic))
+                    last-init init))
+             ((not (member (vop-name init) *vops-allowed-within-pseudo-atomic*))
+              (return))))
+         (when last-init
+           (setf (vop-codegen-info first)
+                 (append (butlast (vop-codegen-info first)) '(:pseudo-atomic)))
+           (terminate-inits last-init))))
   (do-ir2-blocks (block component)
     (let ((vop (ir2-block-start-vop block)))
+      ;; This needs to avoid processing an allocator more than once.
+      ;; Here's how it could happen:
+      ;; ir2 block 1 | whatever
+      ;;             | allocate \
+      ;;             | set-slot | -- to be bunched
+      ;;             | set-slot |
+      ;; ir2 block 2 | set-slot /
+      ;;             | whatever
+      ;;             | ..
+      ;;             | allocate
+      ;;             | set-slot
+      ;;
+      ;; Depending on where the loop continues iterating after performing the bunching
+      ;; operation, we might see the second ALLOCATE twice. Consider if we pick up
+      ;; at the "whatever" vop after the third SET-SLOT. The we process the next
+      ;; allocate and set-slot.  When that's done, the inner loop finishes and we start
+      ;; on the outer loop (in DO-IR2-BLOCKS) which takes BLOCK-NEXT of block 1 as the
+      ;; starting point. So then we see block 2 again, and the 2nd allocate again.
       (loop (unless vop (return))
-            (let ((optimizer
-                   (case (vop-name vop)
-                     (sb-vm::instance-index-set
-                      (when (gethash 'sb-vm::instance-set-multiple
-                                     *backend-parsed-vops*)
-                        'merge-instance-set-vops)))))
-              (setq vop (or (awhen optimizer (funcall it vop))
-                            (vop-next vop))))))))
+            (case (vop-name vop)
+              (make-closure
+               (let ((dx (third (vop-codegen-info vop)))
+                     (already-done (eq (fourth (vop-codegen-info vop)) :pseudo-atomic)))
+                 (unless (or dx already-done)
+                   (process-closure-inits vop))))
+              ((fixed-alloc var-alloc)
+               (let ((last (car (last (vop-codegen-info vop)))))
+                 (when (vop-p last)
+                   (process-general-inits vop last)))))
+            ;; Probably could skip over some vops if any were already processed
+            ;; but it's clearer to just do the naive one-at-a-time skip
+            ;; which avoids confusion when IR2 blocks were split
+            ;; and the processing either did or didn't modify anything.
+            ;; As long as we don't re-process an allocation vop, all it well.
+            (setq vop (vop-next vop)))))))
+
+;;; If a constant is already loaded into a register use that register.
+(defun optimize-constant-loads (component)
+  (let* ((register-sb (sb-or-lose 'sb-vm::registers))
+         (loaded-constants
+           (make-array (sb-size register-sb)
+                       :initial-element nil)))
+    (do-ir2-blocks (block component)
+      (fill loaded-constants nil)
+      (do ((vop (ir2-block-start-vop block) (vop-next vop)))
+          ((null vop))
+        (labels ((register-p (tn)
+                   (and (tn-p tn)
+                        (not (eq (tn-kind tn) :unused))
+                        (eq (sc-sb (tn-sc tn)) register-sb)))
+                 (constant-eql-p (a b)
+                   (or (eq a b)
+                       (and (eq (sc-name (tn-sc a)) 'constant)
+                            (eq (tn-sc a) (tn-sc b))
+                            (eql (tn-offset a) (tn-offset b)))))
+                 (remove-constant (tn)
+                   (when (register-p tn)
+                     (setf (svref loaded-constants (tn-offset tn)) nil)))
+                 (remove-written-tns ()
+                   (cond ((memq (vop-info-save-p (vop-info vop))
+                                '(t :force-to-stack))
+                          (fill loaded-constants nil))
+                         (t
+                          (do ((ref (vop-results vop) (tn-ref-across ref)))
+                              ((null ref))
+                            (remove-constant (tn-ref-tn ref))
+                            (remove-constant (tn-ref-load-tn ref)))
+                          (do ((ref (vop-temps vop) (tn-ref-across ref)))
+                              ((null ref))
+                            (remove-constant (tn-ref-tn ref)))
+                          (do ((ref (vop-args vop) (tn-ref-across ref)))
+                              ((null ref))
+                            (remove-constant (tn-ref-load-tn ref))))))
+                 (compatible-scs-p (a b)
+                   (or (eql a b)
+                       (and (eq (sc-name a) 'sb-vm::control-stack)
+                            (eq (sc-name b) 'sb-vm::descriptor-reg))
+                       (and (eq (sc-name b) 'sb-vm::control-stack)
+                            (eq (sc-name a) 'sb-vm::descriptor-reg))))
+                 (find-constant-tn (constant sc)
+                   (loop for (saved-constant . tn) across loaded-constants
+                         when (and saved-constant
+                                   (constant-eql-p saved-constant constant)
+                                   (compatible-scs-p (tn-sc tn) sc))
+                         return tn)))
+          (case (vop-name vop)
+            ((move sb-vm::move-arg)
+             (let* ((args (vop-args vop))
+                    (results (vop-results vop))
+                    (x (tn-ref-tn args))
+                    (x-load-tn (tn-ref-load-tn args))
+                    (y (tn-ref-tn results))
+                    constant)
+               (cond ((or (eq (sc-name (tn-sc x)) 'null)
+                          (not (eq (tn-kind x) :constant)))
+                      (remove-written-tns))
+                     ((setf constant (find-constant-tn x (tn-sc y)))
+                      (when (register-p y)
+                        (setf (svref loaded-constants (tn-offset y))
+                              (cons x y)))
+                      ;; XOR is more compact on x86oids and many
+                      ;; RISCs have a zero register
+                      (unless (and (constant-p (tn-leaf x))
+                                   (eql (tn-value x) 0)
+                                   (register-p y))
+                        (change-tn-ref-tn args constant)
+                        (setf (tn-ref-load-tn args) nil)))
+                     ((register-p y)
+                      (setf (svref loaded-constants (tn-offset y))
+                            (cons x y)))
+                     ((and x-load-tn
+                           (or (not (tn-ref-load-tn results))
+                               (location= (tn-ref-load-tn results)
+                                          x-load-tn)))
+                      (setf (svref loaded-constants (tn-offset x-load-tn))
+                            (cons x x-load-tn)))
+                     (t
+                      (remove-written-tns)))))
+            (t
+             (remove-written-tns))))))))
 
 (defun ir2-optimize (component &optional stage)
   (let ((*2block-info* (make-hash-table :test #'eq)))
@@ -1348,18 +1614,13 @@
        (run-vop-optimizers component stage)
        (delete-no-op-vops component)
        (ir2-optimize-jumps component)
+       #+x86-64 (attempt-pseudo-atomic-store-bunching component)
        (optimize-constant-loads component))
       (select-representations
        ;; Give the optimizers a second opportunity to alter newly inserted vops
        ;; by looking for patterns that have a shorter expression as a single vop.
        (run-vop-optimizers component stage t)
-       (delete-unused-ir2-blocks component)
-       ;; Try to combine consecutive uses of %INSTANCE-SET.
-       ;; This can't be done prior to selecting representations
-       ;; because SELECT-REPRESENTATIONS might insert some
-       ;; things like MOVE-FROM-DOUBLE which makes the
-       ;; "consecutive" vops no longer consecutive.
-       (ir2-optimize-stores component))
+       (delete-unused-ir2-blocks component))
       (t
        (when (and *compiler-trace-output*
                   (member :pre-ir2-optimize *compile-trace-targets*))
