@@ -16,47 +16,50 @@
 
 ;;; FIND-DOMINATORS  --  Internal
 ;;;
-;;; Find the set of blocks that dominates each block in COMPONENT.  We
-;;; assume that the DOMINATORS for each block is initially NIL, which
-;;; serves to represent the set of all blocks.  If a block is not
-;;; reachable from an entry point, then its dominators will still be
-;;; NIL when we are done.
+;;; Find the immediate dominator of each block in COMPONENT.  If a
+;;; block is not reachable from an entry point, then its immediate
+;;; dominator will still be NIL when we are done.
 (defun find-dominators (component)
   (let ((head (loop-head (component-outer-loop component)))
         changed)
-    (let ((set (make-sset)))
-      (sset-adjoin head set)
-      (setf (block-dominators head) set))
-    (loop
-     (setq changed nil)
-     (do-blocks (block component :tail)
-       (let ((dom (block-dominators block)))
-         (when dom (sset-delete block dom))
-         (dolist (pred (block-pred block))
-           (let ((pdom (block-dominators pred)))
-             (when pdom
-               (if dom
-                   (when (sset-intersection dom pdom)
-                     (setq changed t))
-                   (setq dom (copy-sset pdom) changed t)))))
-         (setf (block-dominators block) dom)
-         (when dom (sset-adjoin block dom))))
-     (unless changed (return)))
+    (setf (block-dominator head) head)
+    (unless (dfo-as-needed component)
+      (number-blocks component))
+    (labels ((intersect (block1 block2)
+               (cond ((eq block1 block2) block1)
+                     ((< (block-number block1) (block-number block2))
+                      (intersect (block-dominator block1) block2))
+                     (t
+                      (intersect block1 (block-dominator block2))))))
+      (loop
+       (setq changed nil)
+       (do-blocks (block component :tail)
+         (let ((dom))
+           (dolist (pred (block-pred block))
+             (unless (null (block-dominator pred))
+               (setq dom (if dom
+                             (intersect pred dom)
+                             pred))))
+           (unless (eq (block-dominator block) dom)
+             (setf (block-dominator block) dom)
+             (setq changed t))))
+       (unless changed (return))))
+    (setf (block-dominator head) nil)
     (setf (component-dominators-computed component) t)))
 
 (defun clear-dominators (component)
   (do-blocks (block component)
-    (setf (block-dominators block) nil))
+    (setf (block-dominator block) nil))
   (setf (component-dominators-computed component) nil))
 
 ;;; DOMINATES-P  --  Internal
 ;;;
 ;;;    Return true if BLOCK1 dominates BLOCK2, false otherwise.
 (defun dominates-p (block1 block2)
-  (let ((set (block-dominators block2)))
-    (if set
-        (sset-member block1 set)
-        t)))
+  (cond ((null block2) nil)
+        ((eq block1 block2) t)
+        (t
+         (dominates-p block1 (block-dominator block2)))))
 
 ;;; LOOP-ANALYZE  --  Interface
 ;;;
@@ -92,7 +95,7 @@
                 (find-strange-loop-segments block component))
               (find-loop-blocks loop)
               ;; Loops with no exits are unreachable by predecessor walk and
-              ;; by definition belong to the component toplevel loop.
+              ;; by definition belong to the component outer loop.
               (when (eq (loop-kind loop) :natural)
                 (unless (or (loop-exits loop)
                             (eq outer-loop loop))
@@ -101,15 +104,6 @@
             (return)))))
     ;; Remaining blocks belong to the outer loop.
     (find-loop-blocks outer-loop)
-    ;; Special case: Code deletion may cause a graph of only a head
-    ;; and a tail, where the two are disconnected. Assign the
-    ;; component head a loop block for this degenerate case.
-    (let ((head (component-head component)))
-      (unless (block-loop head)
-        (setf (block-loop head) outer-loop)
-        (shiftf (block-loop-next head)
-                (loop-blocks outer-loop)
-                head)))
     (labels ((assign-depth (loop depth)
                (setf (loop-depth loop) depth)
                (dolist (inferior (loop-inferiors loop))
@@ -118,7 +112,7 @@
 
 ;;; FIND-LOOP-BLOCKS  --  Internal
 ;;;
-;;; This function initializes the block lists and inferiors of LOO.
+;;; This function initializes the block lists and inferiors of LOOP.
 ;;; When we are done, we scan the blocks looking for exits.  An exit
 ;;; is always a block that has a successor which doesn't have a LOOP
 ;;; assigned yet, since the target of the exit must be in a superior
@@ -129,41 +123,14 @@
 ;;; loop heads are necessary because the walks from the tails
 ;;; terminate when they encounter a block in an inferior loop.
 (defun find-loop-blocks (loop)
-  (labels ((find-blocks-from-here (block loop)
-             (when (dominates-p (loop-head loop) block)
-               (cond ((eq (block-loop block) loop))
-                     ((block-loop block)
-                      (let* ((inner (block-loop block))
-                             (inner-superior (loop-superior inner)))
-                        (cond ((not inner-superior)
-                               ;; We've hit a block from an inner loop
-                               ;; that doesn't have a superior loop yet,
-                               ;; so it is a direct inferior.
-                               (setf (loop-superior inner) loop)
-                               (push inner (loop-inferiors loop))
-                               ;; Keep looking for more blocks in the
-                               ;; loop from the inferior head.
-                               (dolist (pred (block-pred (loop-head inner)))
-                                 (find-blocks-from-here pred loop)))
-                              ((not (eq inner-superior loop))
-                               ;; We've hit a block from an inner loop
-                               ;; that has a different superior
-                               ;; loop. Traverse from the superior
-                               ;; loop's head to discover more blocks.
-                               (find-blocks-from-here (loop-head inner-superior) loop)))))
-                     (t
-                      (setf (block-loop block) loop)
-                      (shiftf (block-loop-next block) (loop-blocks loop) block)
-                      (dolist (pred (block-pred block))
-                        (find-blocks-from-here pred loop)))))))
-    (dolist (tail (loop-tail loop))
-      (find-blocks-from-here tail loop))
-    ;; For the outermost loop, new blocks can still be discovered by
-    ;; walking back from natural loops with no exits.
-    (when (eq (loop-kind loop) :outer)
-      (dolist (sub-loop (loop-inferiors loop))
-        (dolist (pred (block-pred (loop-head sub-loop)))
-          (find-blocks-from-here pred loop)))))
+  (dolist (tail (loop-tail loop))
+    (find-blocks-from-here tail loop))
+  ;; For the outermost loop, new blocks can still be discovered by
+  ;; walking back from natural loops with no exits.
+  (when (eq (loop-kind loop) :outer)
+    (dolist (sub-loop (loop-inferiors loop))
+      (dolist (pred (block-pred (loop-head sub-loop)))
+        (find-blocks-from-here pred loop))))
   (collect ((exits))
     (dolist (sub-loop (loop-inferiors loop))
       (dolist (exit (loop-exits sub-loop))
@@ -181,6 +148,38 @@
     (setf (loop-exits loop) (exits))))
 
 
+;;; FIND-BLOCKS-FROM-HERE  --  Internal
+;;;
+;;; This function does a graph walk to find the blocks directly within
+;;; LOOP that can be reached by a backward walk from BLOCK.  If BLOCK
+;;; is already in LOOP or is not dominated by the LOOP-HEAD, then we
+;;; return.  If another loop is already assigned to BLOCK, it must be
+;;; an inferior loop.  If this loop doesn't have a superior yet, we
+;;; record that it must be a direct inferior of LOOP, and recurse on
+;;; the head of this loop's predecessor.  But if BLOCK's loop already
+;;; has a superior, then we can directly recurse on its existing
+;;; superior's head, since all predecessors of the head of BLOCK's
+;;; loop are contained in its superior already.  Otherwise, we add the
+;;; block to the BLOCKS for LOOP and recurse on its predecessors.
+(defun find-blocks-from-here (block loop)
+  (when (and (not (eq (block-loop block) loop))
+             (dominates-p (loop-head loop) block))
+    (cond ((block-loop block)
+           (let* ((inner (block-loop block))
+                  (inner-superior (loop-superior inner)))
+             (cond ((not inner-superior)
+                    (setf (loop-superior inner) loop)
+                    (push inner (loop-inferiors loop))
+                    (dolist (pred (block-pred (loop-head inner)))
+                      (find-blocks-from-here pred loop)))
+                   ((not (eq inner-superior loop))
+                    (find-blocks-from-here (loop-head inner-superior) loop)))))
+          (t
+           (setf (block-loop block) loop)
+           (shiftf (block-loop-next block) (loop-blocks loop) block)
+           (dolist (pred (block-pred block))
+             (find-blocks-from-here pred loop))))))
+
 ;;; NOTE-LOOP-HEAD  --  Internal
 ;;;
 ;;; Create a loop structure to describe the loop headed by the block
