@@ -447,12 +447,12 @@ necessary, since type inference may take arbitrarily long to converge.")
   (when (component-reanalyze component)
     (maybe-mumble "DFO")
     (loop
-      (find-dfo component)
+     (find-dfo component)
       (unless (component-reanalyze component)
         (maybe-mumble " ")
         (return))
-      (maybe-mumble ".")))
-  (values))
+      (maybe-mumble "."))
+    t))
 
 (defparameter *reoptimize-limit* 10)
 
@@ -469,6 +469,7 @@ necessary, since type inference may take arbitrarily long to converge.")
                (maybe-mumble "Locall ")
                (locall-analyze-component component))
               ((and (>= loop-count 1)
+                    (not (component-reanalyze component))
                     (not reoptimized))
                ;; Constraint propagation did something but that
                ;; information didn't lead to any new optimizations.
@@ -536,6 +537,7 @@ necessary, since type inference may take arbitrarily long to converge.")
 (declaim (type (member :immobile :dynamic :auto) *compile-to-memory-space*)
          (type (member :immobile :dynamic) *compile-file-to-memory-space*))
 (defvar *compile-to-memory-space* :immobile) ; BUILD-TIME default
+(export '*compile-file-to-memory-space*) ; silly user code looks at, even if no immobile-space
 (defvar *compile-file-to-memory-space* :immobile) ; BUILD-TIME default
 
 #-immobile-code
