@@ -150,6 +150,18 @@
 (define-binop logand 1 3 and (unsigned-byte 14) (unsigned-byte 16))
 (define-binop logxor 1 3 xor (unsigned-byte 14) (unsigned-byte 16))
 
+(define-vop (fast-logand/signed-unsigned=>unsigned fast-logand/unsigned=>unsigned)
+  (:args (x :scs (signed-reg) :target r)
+         (y :scs (unsigned-reg) :target r))
+  (:arg-types signed-num unsigned-num))
+
+(define-vop (fast-logand-c/signed-unsigned=>unsigned fast-logand-c/unsigned=>unsigned)
+  (:args (x :scs (signed-reg) :target r))
+  (:arg-types signed-num (:constant (eql #.most-positive-word)))
+  (:ignore y)
+  (:generator 1
+    (move r x)))
+
 ;;; No -C/ VOPs for LOGNOR because the NOR instruction doesn't take
 ;;; immediate args.  -- CSR, 2003-09-11
 (define-vop (fast-lognor/fixnum=>fixnum fast-fixnum-binop)
@@ -191,7 +203,7 @@
   (:generator 3
     (inst bgez amount positive)
     (inst subu ndesc zero-tn amount)
-    (inst slt temp ndesc 32)
+    (inst slt temp ndesc 32) ; FIXME: should be sltu here ?
     (inst bne temp done)
     (inst srl result number ndesc)
     (inst b done)
@@ -217,7 +229,7 @@
   (:generator 3
     (inst bgez amount positive)
     (inst subu ndesc zero-tn amount)
-    (inst slt temp ndesc 31)
+    (inst sltu temp ndesc 31)
     (inst bne temp done)
     (inst sra result number ndesc)
     (inst b done)
@@ -380,7 +392,7 @@
   (:vop-var vop)
   (:save-p :compute-only)
   (:generator 11
-    (let ((zero (generate-error-code vop 'division-by-zero-error x y)))
+    (let ((zero (generate-error-code vop 'division-by-zero-error x)))
       (inst beq y zero))
     (inst nop)
     (inst div x y)
@@ -396,7 +408,7 @@
   (:vop-var vop)
   (:save-p :compute-only)
   (:generator 12
-    (let ((zero (generate-error-code vop 'division-by-zero-error x y)))
+    (let ((zero (generate-error-code vop 'division-by-zero-error x)))
       (inst beq y zero))
     (inst nop)
     (inst divu x y)
@@ -411,7 +423,7 @@
   (:vop-var vop)
   (:save-p :compute-only)
   (:generator 12
-    (let ((zero (generate-error-code vop 'division-by-zero-error x y)))
+    (let ((zero (generate-error-code vop 'division-by-zero-error x)))
       (inst beq y zero))
     (inst nop)
     (inst div x y)
@@ -640,8 +652,6 @@
   (if (oddp (length args))
       `(logxor ,@args)
       `(lognot (logxor ,@args))))
-(define-source-transform logandc1 (x y)
-  `(logand (lognot ,x) ,y))
 (define-source-transform logandc2 (x y)
   `(logand ,x (lognot ,y)))
 (define-source-transform logorc1 (x y)
@@ -794,19 +804,6 @@
     (inst mflo lo)
     (inst mfhi hi)))
 
-(define-vop (bignum-lognot lognot-mod32/unsigned=>unsigned)
-  (:translate sb-bignum:%lognot))
-
-(define-vop (fixnum-to-digit)
-  (:translate sb-bignum:%fixnum-to-digit)
-  (:policy :fast-safe)
-  (:args (fixnum :scs (any-reg)))
-  (:arg-types tagged-num)
-  (:results (digit :scs (unsigned-reg)))
-  (:result-types unsigned-num)
-  (:generator 1
-    (inst sra digit fixnum n-fixnum-tag-bits)))
-
 (define-vop (bignum-floor)
   (:translate sb-bignum:%bigfloor)
   (:policy :fast-safe)
