@@ -18,7 +18,7 @@
   (:args (type :scs (any-reg))
          (rank :scs (any-reg)))
   (:arg-types positive-fixnum positive-fixnum)
-  (:temporary (:scs (non-descriptor-reg)) bytes header)
+  (:temporary (:scs (non-descriptor-reg)) bytes header temp)
   (:temporary (:sc non-descriptor-reg :offset nl4-offset) pa-flag)
   (:results (result :scs (descriptor-reg)))
   (:generator 13
@@ -35,9 +35,8 @@
     ;; were fixnums
     (inst srl header n-fixnum-tag-bits)
     (pseudo-atomic (pa-flag)
-      (inst or result alloc-tn other-pointer-lowtag)
-      (storew header result 0 other-pointer-lowtag)
-      (inst addu alloc-tn bytes))))
+      (allocation type bytes result other-pointer-lowtag `(,pa-flag ,temp))
+      (storew header result 0 other-pointer-lowtag))))
 
 ;;;; Additional accessors and setters for the array header.
 (define-full-reffer %array-dimension *
@@ -55,9 +54,10 @@
   (:results (res :scs (unsigned-reg)))
   (:result-types positive-fixnum)
   (:generator 6
-    ;; ASSUMPTION: n-widetag-bits = 8
-    (inst lbu res x #+little-endian (- 2 other-pointer-lowtag)
-                    #+big-endian    (- 1 other-pointer-lowtag))
+    ;; convert ARRAY-RANK-POSITION to byte index and compensate for endianness
+    ;; ASSUMPTION: n-widetag-bits = 8 and rank is adjacent to widetag
+    (inst lbu res x #+little-endian (- 1 other-pointer-lowtag)
+                    #+big-endian    (- 2 other-pointer-lowtag))
     (inst nop)
     (inst addu res 1)
     (inst and res array-rank-mask)))
@@ -86,27 +86,58 @@
 ;;; elements are represented in integer registers and are built out of
 ;;; 8, 16, or 32 bit elements.
 (macrolet ((def-full-data-vector-frobs (type element-type &rest scs)
-  `(progn
-     (define-full-reffer ,(symbolicate "DATA-VECTOR-REF/" type) ,type
-       vector-data-offset other-pointer-lowtag
-       ,(remove-if #'(lambda (x) (member x '(null zero))) scs)
-       ,element-type
-       data-vector-ref)
-     (define-full-setter ,(symbolicate "DATA-VECTOR-SET/" type) ,type
-       vector-data-offset other-pointer-lowtag ,scs ,element-type
-       data-vector-set)))
-
+             `(progn (define-full-reffer ,(symbolicate "DATA-VECTOR-REF/" type) ,type
+                       vector-data-offset other-pointer-lowtag
+                       ,(remove-if #'(lambda (x) (member x '(null zero))) scs)
+                       ,element-type
+                       data-vector-ref)
+                     (define-full-setter ,(symbolicate "DATA-VECTOR-SET/" type) ,type
+                       vector-data-offset other-pointer-lowtag ,scs ,element-type
+                       data-vector-set)))
            (def-partial-data-vector-frobs (type element-type size signed &rest scs)
-  `(progn
-     (define-partial-reffer ,(symbolicate "DATA-VECTOR-REF/" type) ,type
-       ,size ,signed vector-data-offset other-pointer-lowtag ,scs
-       ,element-type data-vector-ref)
-     (define-partial-setter ,(symbolicate "DATA-VECTOR-SET/" type) ,type
-       ,size vector-data-offset other-pointer-lowtag ,scs
-       ,element-type data-vector-set))))
+             `(progn
+                (define-partial-reffer ,(symbolicate "DATA-VECTOR-REF/" type) ,type
+                  ,size ,signed vector-data-offset other-pointer-lowtag ,scs
+                  ,element-type data-vector-ref)
+                (define-partial-setter ,(symbolicate "DATA-VECTOR-SET/" type) ,type
+                  ,size vector-data-offset other-pointer-lowtag ,scs
+                  ,element-type data-vector-set))))
 
-  (def-full-data-vector-frobs simple-vector *
-    descriptor-reg any-reg null zero)
+  ;; SIMPLE-VECTOR
+  (define-full-reffer data-vector-ref/simple-vector simple-vector
+    vector-data-offset other-pointer-lowtag (descriptor-reg any-reg) * data-vector-ref)
+  (define-vop (data-vector-set/simple-vector)
+    (:translate data-vector-set)
+    (:policy :fast-safe)
+    (:args (object :scs (descriptor-reg)) (index :scs (any-reg))
+           (value :scs (descriptor-reg any-reg null zero)))
+    (:arg-types simple-vector tagged-num *)
+    (:temporary (:scs (non-descriptor-reg)) ea temp)
+    (:vop-var vop)
+    (:generator 6
+      ;; We could potentially eliminate the ADDIU by ensuring that a simple-vector
+      ;; never starts 2 words before the end of a card.
+      ;; However, that's tricky to reason about and I don't care to do it.
+      ;; (and also it's maybe not correct for card-spanning vectors)
+      (inst addu ea object index)
+      (inst addu ea ea (- (ash vector-data-offset word-shift) other-pointer-lowtag))
+      (without-scheduling ()
+        (emit-gengc-barrier object ea temp (vop-nth-arg 2 vop) value)
+        (storew value ea 0 0))))
+  (define-vop (data-vector-set/simple-vector-c)
+    (:translate data-vector-set)
+    (:policy :fast-safe)
+    (:args (object :scs (descriptor-reg)) (value :scs (descriptor-reg any-reg null zero)))
+    (:temporary (:scs (non-descriptor-reg)) ea temp)
+    (:info index)
+    ;; not sure if the load/store-index is off by something now
+    (:arg-types simple-vector (:constant (load/store-index 4 7 2)) *)
+    (:vop-var vop)
+    (:generator 5
+      (inst addu ea object (- (ash (+ vector-data-offset index) word-shift) other-pointer-lowtag))
+      (without-scheduling ()
+        (emit-gengc-barrier object ea temp (vop-nth-arg 1 vop) value)
+        (storew value object (+ vector-data-offset index) other-pointer-lowtag))))
 
   (def-partial-data-vector-frobs simple-base-string character
     :byte nil character-reg)
@@ -478,3 +509,8 @@
   (unsigned-reg) unsigned-num %vector-raw-bits)
 (define-full-setter set-vector-raw-bits * vector-data-offset other-pointer-lowtag
   (unsigned-reg) unsigned-num %set-vector-raw-bits)
+;;; Weak vectors
+(define-full-reffer %weakvec-ref * vector-data-offset other-pointer-lowtag
+  (any-reg descriptor-reg) * %weakvec-ref)
+(define-full-setter %weakvec-set * vector-data-offset other-pointer-lowtag
+  (any-reg descriptor-reg) * %weakvec-set)
