@@ -22,7 +22,7 @@
 
 (defmacro load-asm-rtn-addr (reg name)
   ;; Gencgc has asm code in static space and we can reference it relative to NIL.
-  #+gencgc `(inst addi ,reg null-tn (make-fixup ,name :asm-routine-nil-offset))
+  #+gencgc `(inst addi ,reg null-tn (make-fixup ,name :assembly-routine*))
   ;; Cheneygc has asm code in read-only space which is not within
   ;; a sufficiently small displacement.
   #+cheneygc `(inst lr ,reg (make-fixup ,name :assembly-routine)))
@@ -199,7 +199,10 @@
          (inst stb null-tn thread-base-tn (* n-word-bytes thread-pseudo-atomic-bits-slot))))
      ,@forms
      (unless ,elide-if
-       (when ,sync ; why???
+       (when ,sync
+         ;; sync is generally needed so that stores aren't delayed past the end of P-A,
+         ;; which is important for GC-correctness.
+         ;; Given that, I'm unsure why it's ever not needed.
          (inst sync))
        (without-scheduling ()
          (inst stb thread-base-tn thread-base-tn (* n-word-bytes thread-pseudo-atomic-bits-slot))
