@@ -284,3 +284,18 @@
    ;; Can't convert index to a code-relative index until the boxed header length
    ;; has been determined.
    (inst store-coverage-mark index tmp)))
+
+(define-vop ()
+  (:translate sb-lockless:get-next)
+  (:policy :fast-safe)
+  (:args (node :scs (descriptor-reg)))
+  (:results (next-tagged :scs (descriptor-reg))
+            (next-bits :scs (descriptor-reg)))
+  (:temporary (:sc non-descriptor-reg :offset nl3-offset) pa-flag)
+  (:generator 10
+    ;; Read the first user-data slot and convert to a tagged pointer,
+    ;; also returning the raw value as a secondary result
+    (pseudo-atomic (pa-flag)
+      (loadw next-bits node (+ instance-slots-offset instance-data-start)
+             instance-pointer-lowtag)
+      (inst ori next-tagged next-bits instance-pointer-lowtag))))
