@@ -25,12 +25,12 @@
 ;;;; "The stack pointer (stored in r1) shall maintain quadword alignment."
 ;;;; (quadword = 16 bytes)
 (defconstant +stack-alignment-mask+ 15)
+(defconstant +stack-frame-size+ #+little-endian 12 #+big-endian 14)
 
 (defstruct arg-state
   (gpr-args 0)
   (fpr-args 0)
-  (stack-frame-size #+little-endian 12
-                    #+big-endian 14))
+  (stack-frame-size +stack-frame-size+))
 
 (defun int-arg (state prim-type reg-sc stack-sc)
   (let ((reg-args (arg-state-gpr-args state)))
@@ -140,7 +140,7 @@
       (values (make-wired-tn* 'positive-fixnum any-reg-sc-number nsp-offset)
               (let ((size (arg-state-stack-frame-size arg-state)))
                 (cond #+little-endian
-                      ((= size 12)
+                      ((= size +stack-frame-size+)
                        ;; no stack args
                        0)
                       (t
@@ -172,10 +172,9 @@
   (:info foreign-symbol)
   (:results (res :scs (sap-reg)))
   (:result-types system-area-pointer)
-  (:temporary (:scs (non-descriptor-reg)) addr)
   (:generator 2
-    (inst lr addr (make-fixup foreign-symbol :foreign-dataref))
-    (loadw res addr)))
+    (inst lr res (make-fixup foreign-symbol :foreign-dataref))
+    (loadw res res)))
 
 (define-vop (call-out)
   (:args (function :scs (sap-reg) :target cfunc)
@@ -262,7 +261,9 @@
              (make-random-tn :kind :normal :sc (sc-or-lose
                                                 'double-reg) :offset
                                                 n)))
-      (let* ((segment (make-segment)))
+      (let* ((segment (make-segment))
+             #+big-endian
+             (function-descriptor-size 24))
         (assemble (segment 'nil)
           ;; Copy args from registers or stack to new position
           ;; on stack.
@@ -311,8 +312,7 @@
                    (save-arg (type words)
                      (let ((integerp (not (alien-float-type-p type)))
                            (in-offset (+ (* in-words-processed n-word-bytes)
-                                         (* 12 n-word-bytes
-                                            #+big-endian 2)))
+                                         (* +stack-frame-size+ n-word-bytes)))
                            (out-offset (- (* out-words-processed n-word-bytes)
                                           arg-store-pos)))
                        (cond (integerp
@@ -386,6 +386,11 @@
                               (incf out-words-processed))
                              (t
                               (bug "Unknown alien floating point type: ~S" type))))))
+              ;; Leave a gap for a PPC64ELF ABIv1 function descriptor,
+              ;; to be filled in later relative to the SAP.
+              #+big-endian
+              (dotimes (k (/ function-descriptor-size 4)) ; nop is 4 bytes
+                (inst nop))
               (mapc #'save-arg
                     argument-types
                     (mapcar (lambda (arg)
@@ -411,11 +416,17 @@
               (inst stdu stack-pointer stack-pointer (- frame-size))
 
               ;; And make the call.
+              #+little-endian
               (load-address-into
                r0
-               (foreign-symbol-address
-                #-sb-thread "funcall3"
-                #+sb-thread "callback_wrapper_trampoline"))
+               (foreign-symbol-address "callback_wrapper_trampoline"))
+              #+big-endian
+              (destructuring-bind (r2 r12) (mapcar #'make-gpr '(2 12))
+                (load-address-into
+                 r12
+                 (foreign-symbol-address "callback_wrapper_trampoline"))
+                (inst ld r0 r12 0)
+                (inst ld r2 r12 8))
               (inst mtlr r0)
               (inst blrl)
 
@@ -436,10 +447,10 @@
                  )
                 (t
                  (loop with gprs = (mapcar #'make-gpr '(3 4))
-                       repeat n-return-area-words
                        for gpr = (pop gprs)
                        for offset from (- return-area-pos)
                        by n-word-bytes
+                       repeat n-return-area-words
                        do
                        (unless gpr
                          (bug "Out of return registers in alien-callback trampoline."))
@@ -454,6 +465,13 @@
                                            :element-type '(unsigned-byte 8)
                                            :initial-contents buffer))
                (sap (vector-sap vector)))
+          ;; Fill in the PPC64ELF ABIv1 function descriptor that
+          ;; points just past the end of itself, to the first
+          ;; instruction of the wrapper.  This assembler wrapper only
+          ;; cares about the address, so leave the other descriptor
+          ;; fields filled with no-op instructions.
+          #+big-endian
+          (setf (sap-ref-64 sap 0) (+ (sap-int sap) function-descriptor-size))
           (alien-funcall
            (extern-alien "ppc_flush_icache"
                          (function void
@@ -461,3 +479,18 @@
                                    unsigned-long))
            sap (length buffer))
           vector)))))
+
+;;; For (CAS SAP-REF-{8,16,32})
+(defknown sign-extend ((signed-byte 64) t) fixnum
+    (foldable flushable movable))
+
+(define-vop (sign-extend)
+  (:translate sign-extend)
+  (:policy :fast-safe)
+  (:args (val :scs (signed-reg)))
+  (:arg-types signed-num (:constant fixnum))
+  (:info size)
+  (:results (res :scs (signed-reg)))
+  (:result-types fixnum)
+  (:generator 1
+    (inst* (ecase size (8 'extsb) (16 'extsh) (32 'extsw)) res val)))
