@@ -243,6 +243,7 @@
           #-sb-xc-host
           (let ((type (compiler-specifier-type typespec)))
             (cond ((not type))
+                  ((contains-unknown-type-p type))
                   (ospec
                    (setf (car ospec) (type-union (car ospec) type)))
                   (t
@@ -320,7 +321,12 @@
       (:symbol name "globally declaring ~A ~A" kind)
     (if (eq kind 'always-bound)
         (setf (info :variable :always-bound name) info-value)
-        (setf (info :variable :kind name) info-value))))
+        (setf (info :variable :kind name) info-value)))
+  #-sb-xc-host (unset-symbol-progv-optimize name))
+
+#-sb-xc-host
+(defun unset-symbol-progv-optimize (symbol)
+  (reset-header-bits symbol sb-vm::+symbol-fast-bindable+))
 
 (defun type-proclamation-mismatch-warn (name old new &optional description)
   (warn 'type-proclamation-mismatch-warning
