@@ -10,7 +10,22 @@
 ;;;; files for more information.
 
 (in-package "SB-VM")
-
+
+(defconstant-eqx +fixup-kinds+ #(:absolute :i-type :s-type :u-type :u+i-type) #'equalp)
+
+(defun u-and-i-inst-immediate (value)
+  (let ((hi (ash (+ value (expt 2 11)) -12)))
+    (values hi (- value (ash hi 12)))))
+
+(def!type short-immediate () `(signed-byte 12))
+(def!type short-immediate-fixnum () `(signed-byte ,(- 12 n-fixnum-tag-bits)))
+
+(deftype u+i-immediate ()
+  #-64-bit `(or (signed-byte 32) (unsigned-byte 32))
+  #+64-bit `(or (integer #x-80000800 #x7ffff7ff)
+                (integer ,(+ (ash 1 64) #x-80000800)
+                         ,(1- (ash 1 64)))))
+
 (eval-when (:compile-toplevel :load-toplevel :execute)
   (defvar *register-names* (make-array 32 :initial-element nil)))
 
@@ -30,11 +45,11 @@
                 (defconstant register-arg-count ,(length args)))))
                      ; ABI register mnemonic
   (defreg zero 0)    ; zero
-  (defreg lip 1)     ; ra
+  (defreg ra 1)      ; ra
   (defreg nsp 2)     ; sp
   (defreg global 3)  ; gp
   (defreg tp 4)      ; tp
-  (defreg lra 5)     ; t0, alternate link register
+  (defreg lip 5)     ; t0, alternate link register
   (defreg cfp 6)     ; t1
   (defreg ocfp 7)    ; t2
   (defreg nfp 8)     ; s0, callee-saved
@@ -64,19 +79,19 @@
   (defreg code 30)   ; t5
   (defreg nargs 31)  ; t6
 
-  (defregset non-descriptor-regs nl0 nl1 nl2 nl3 nl4 nl5 nl6 nl7 nargs nfp cfunc)
-  (defregset descriptor-regs a0 a1 a2 a3 a4 a5 l0 l1 #-sb-thread l2 ocfp lra lexenv)
+  (defregset non-descriptor-regs nl0 nl1 nl2 ra nl3 nl4 nl5 nl6 nl7 nargs nfp cfunc)
+  (defregset descriptor-regs a0 a1 a2 a3 a4 a5 l0 l1 #-sb-thread l2 ocfp lexenv)
   (defregset reserve-descriptor-regs lexenv)
   (defregset reserve-non-descriptor-regs cfunc)
   (defregset boxed-regs a0 a1 a2 a3 a4 a5 l0 l1
     #-sb-thread l2 #+sb-thread thread
-    ocfp lra lexenv code)
+    ocfp lexenv code)
 
   (define-argument-register-set a0 a1 a2 a3 a4 a5))
 
 (!define-storage-bases
  (define-storage-base registers :finite :size 32)
- (define-storage-base control-stack :unbounded :size 8)
+ (define-storage-base control-stack :unbounded :size 0)
  (define-storage-base non-descriptor-stack :unbounded :size 0)
 
  (define-storage-base float-registers :finite :size 32)
@@ -86,7 +101,12 @@
  )
 
 (!define-storage-classes
+
+ ;; Non-immediate constants in the constant pool
  (constant constant)
+
+ ;; Immediate constant.
+ (zero immediate-constant)
  (immediate immediate-constant)
 
  (control-stack control-stack)
@@ -95,7 +115,7 @@
           :reserve-locations #.(append reserve-non-descriptor-regs
                                        reserve-descriptor-regs)
           :alternate-scs (control-stack)
-          :constant-scs (immediate constant)
+          :constant-scs (immediate zero constant)
           :save-p t)
 
  ;; Pointer descriptor objects.  Must be seen by GC.
@@ -134,7 +154,7 @@
              :locations #.non-descriptor-regs
              :reserve-locations #.reserve-non-descriptor-regs
              :alternate-scs (signed-stack)
-             :constant-scs (immediate)
+             :constant-scs (zero immediate)
              :save-p t)
  (unsigned-stack non-descriptor-stack)
  (unsigned-reg registers
@@ -185,37 +205,56 @@
                     :sc (sc-or-lose ',sc)
                     :offset ,offset-sym)))))
   (defregtn zero any-reg)
-  (defregtn nargs any-reg)
-
-  (defregtn nfp any-reg)
-  (defregtn ocfp any-reg)
-
+  (defregtn lip interior-reg)
+  (defregtn code descriptor-reg)
   (defregtn null descriptor-reg)
+
+  (defregtn nargs any-reg)
   (defregtn lexenv descriptor-reg)
 
-  (defregtn cfp any-reg)
   (defregtn csp any-reg)
-  (defregtn nsp any-reg)
+  (defregtn cfp any-reg)
 
-  (defregtn code descriptor-reg)
-  (defregtn lip interior-reg))
+  (defregtn nsp any-reg)
+  (defregtn ocfp any-reg)
+  (defregtn nfp any-reg)
+
+  (defregtn ra any-reg))
 
 ;;; If VALUE can be represented as an immediate constant, then return the
 ;;; appropriate SC number, otherwise return NIL.
 (defun immediate-constant-sc (value)
   (typecase value
+    ((integer 0 0)
+     zero-sc-number)
     (null
      (values descriptor-reg-sc-number null-offset))
-    ((or (integer #.sb-xc:most-negative-fixnum #.sb-xc:most-positive-fixnum)
-         character)
-     immediate-sc-number)
     (symbol
      (if (static-symbol-p value)
          immediate-sc-number
-         nil))))
+         nil))
+    ((integer #.most-negative-fixnum #.most-positive-fixnum)
+     ;; KLUDGE: This is a subset on 64-bit because we currently
+     ;; produce untagged intermediates in %LI.
+     (typecase (fixnumize value)
+       #-64-bit
+       ((signed-byte 32)
+        immediate-sc-number)
+       #+64-bit
+       ((integer #x-80000800 #x7ffff7ff)
+        immediate-sc-number)))
+    #-sb-xc-host ; There is no such object type in the host
+    (system-area-pointer
+     immediate-sc-number)
+    (character
+     immediate-sc-number)
+    (structure-object
+     (when (eq value sb-lockless:+tail+)
+       immediate-sc-number))))
 
 (defun boxed-immediate-sc-p (sc)
-  (eql sc immediate-sc-number))
+  (or (eql sc zero-sc-number)
+      (eql sc immediate-sc-number)))
 
 ;;;; Function Call Parameters
 
@@ -226,7 +265,7 @@
 
 ;;; Offsets of special stack frame locations
 (defconstant ocfp-save-offset 0)
-(defconstant lra-save-offset 1)
+(defconstant ra-save-offset 1)
 (defconstant nfp-save-offset 2)
 
 (defparameter *register-arg-tns*
@@ -244,15 +283,17 @@
 ;;; addresses.
 (defconstant single-value-return-byte-offset 0)
 
+;;; This function is called by debug output routines that want a pretty name
+;;; for a TN's location.  It returns a thing that can be printed with PRINC.
 (defun location-print-name (tn)
   (declare (type tn tn))
   (let ((sb (sb-name (sc-sb (tn-sc tn))))
         (offset (tn-offset tn)))
     (ecase sb
       (registers (or (svref *register-names* offset)
-                     (format nil "x~D" offset)))
+                     (format nil "R~D" offset)))
+      (float-registers (format nil "F~D" offset))
       (control-stack (format nil "CS~D" offset))
-      (float-registers (format nil "f~D" offset))
       (non-descriptor-stack (format nil "NS~D" offset))
       (constant (format nil "Const~D" offset))
       (immediate-constant "Immed"))))
@@ -269,6 +310,3 @@
 (progn
   (defconstant pseudo-atomic-flag (ash list-pointer-lowtag 0))
   (defconstant pseudo-atomic-interrupted-flag (ash list-pointer-lowtag 16)))
-
-#-sb-thread
-(defconstant boxed-region (- (+ static-space-start (* 2 n-word-bytes)) nil-value))
