@@ -613,7 +613,7 @@
                    `((args :more t ,@(unless (eq args :fixed)
                                        '(:scs (descriptor-reg control-stack)))))))
 
-               ,@(when (eq return :fixed)
+               ,@(when (memq return '(:fixed :unboxed))
                    '((:results (values :more t))))
 
                (:save-p ,(if (eq return :tail) :compute-only t))
@@ -638,8 +638,10 @@
                    '(fun-type)))
 
                (:ignore
-               ,@(unless (or variable (eq return :tail)) '(arg-locs))
-               ,@(unless variable '(args)))
+                ,@(unless (or variable (eq return :tail)) '(arg-locs))
+                ,@(unless variable '(args))
+                ,@(when (eq return :unboxed)
+                    '(values)))
 
                ;; We pass either the fdefn object (for named call) or
                ;; the actual function object (for unnamed call) in
@@ -730,7 +732,7 @@
                           ;; (without-interrupts ...)
                           ;;
                           ;; dtc; Could be doing a tail call from a
-                          ;; known-local-call etc in which the old-fp
+                          ;; known-call-local etc in which the old-fp
                           ;; or ret-pc are in regs or in non-standard
                           ;; places. If the passing location were
                           ;; wired to the stack in standard locations
@@ -817,7 +819,7 @@
                     '((note-this-location vop :unknown-return)
                       (receive-unknown-values values-start nvals start count
                                               node)))
-                   (:tail))))))
+                   ((:tail :unboxed)))))))
 
   (define-full-call call nil :fixed nil)
   (define-full-call call-named t :fixed nil)
@@ -835,7 +837,10 @@
   (define-full-call call-variable nil :fixed t)
   (define-full-call multiple-call-variable nil :unknown t)
   (define-full-call fixed-call-named t :fixed nil :fixed)
-  (define-full-call fixed-tail-call-named t :tail nil :fixed))
+  (define-full-call fixed-tail-call-named t :tail nil :fixed)
+
+  (define-full-call unboxed-call-named t :unboxed nil)
+  (define-full-call fixed-unboxed-call-named t :unboxed nil :fixed))
 
 ;;; Call NAME "directly" meaning in a single JMP or CALL instruction,
 ;;; if possible (without loading RAX)
@@ -1293,7 +1298,11 @@
           (inst lea :dword rcx (ea nil count (ash 1 shift)))))
     ;; Setup for the CDR of the last cons (or the entire result) being NIL.
     (inst mov result nil-value)
-    (inst jrcxz DONE)
+    (cond ((not (member :allocation-size-histogram sb-xc:*features*))
+           (inst jrcxz DONE))
+          (t ; jumps too far for JRCXZ sometimes
+           (inst test rcx rcx)
+           (inst jmp :z done)))
     (unless (node-stack-allocate-p node)
       (instrument-alloc +cons-primtype+ rcx node (list value dst) thread-tn))
     (pseudo-atomic (:elide-if (node-stack-allocate-p node) :thread-tn thread-tn)
@@ -1394,41 +1403,6 @@
                    ((plusp min)
                     (inst cmp :dword nargs (fixnumize min))
                     (inst jmp :b err-lab))))))))
-
-;; Signal an error about an untagged number.
-;; These are pretty much boilerplate and could be generic except:
-;; - the names of the SCs could differ between backends (or maybe not?)
-;; - in the "/c" case, the older backends don't eval the errcode
-;; And the 6 vops above ought to be generic too...
-;; FIXME: there are still some occurrences of
-;;  note: doing signed word to integer coercion
-;; in regard to SB-C::%TYPE-CHECK-ERROR. Figure out why.
-(define-vop (type-check-error/word)
-  (:policy :fast-safe)
-  (:translate sb-c::%type-check-error)
-  (:args (object :scs (signed-reg unsigned-reg))
-         ;; Types are trees of symbols, so 'any-reg' is not
-         ;; really possible.
-         (type :scs (any-reg descriptor-reg constant)))
-  (:arg-types untagged-num * (:constant t))
-  (:info *location-context*)
-  (:vop-var vop)
-  (:save-p :compute-only)
-  ;; cost is a smidgen less than type-check-error
-  ;; otherwise this does not get selected.
-  (:generator 999
-    (error-call vop 'object-not-type-error object type)))
-(define-vop (type-check-error/word/c)
-  (:policy :fast-safe)
-  (:translate sb-c::%type-check-error/c)
-  (:args (object :scs (signed-reg unsigned-reg)))
-  (:arg-types untagged-num (:constant symbol) (:constant t))
-  (:info errcode *location-context*)
-  (:vop-var vop)
-  (:save-p :compute-only)
-  (:generator 899 ; smidgen less than type-check-error/c
-    (error-call vop errcode object)))
-
 ;;; Single-stepping
 
 (defun emit-single-step-test ()
