@@ -104,40 +104,16 @@
 
 (define-vop ()
   (:translate sap+)
-  (:args (ptr :scs (sap-reg) :target res
-              :load-if (not (location= ptr res)))
-         (offset :scs (signed-reg immediate)))
+  (:args (ptr :scs (sap-reg sap-stack) :target res :load-if nil)
+         (offset :scs (signed-reg signed-stack immediate)))
   (:arg-types system-area-pointer signed-num)
-  (:results (res :scs (sap-reg) :from (:argument 0)
-                 :load-if (not (location= ptr res))))
+  (:results (res :scs (sap-reg sap-stack) :load-if nil))
+  (:vop-var vop)
   (:result-types system-area-pointer)
-  (:temporary (:sc signed-reg) temp)
+  (:temporary (:sc signed-reg) temp) ; TODO: add an :unused-if on this
   (:policy :fast-safe)
   (:generator 1
-    (cond ((and (sc-is ptr sap-reg) (sc-is res sap-reg)
-                (not (location= ptr res)))
-           (sc-case offset
-             (signed-reg
-              (inst lea res (ea ptr offset)))
-             (immediate
-              (let ((value (tn-value offset)))
-                (cond ((typep value '(signed-byte 32))
-                       (inst lea res (ea value ptr)))
-                      (t
-                       (inst mov temp value)
-                       (inst lea res (ea ptr temp))))))))
-          (t
-           (move res ptr)
-           (sc-case offset
-             (signed-reg
-              (inst add res offset))
-             (immediate
-              (let ((value (tn-value offset)))
-                (cond ((typep value '(signed-byte 32))
-                       (inst add res (tn-value offset)))
-                      (t
-                       (inst mov temp value)
-                       (inst add res temp))))))))))
+    (emit-inline-add-sub 'add ptr offset res temp vop 'identity)))
 
 (define-vop ()
   (:translate sap-)
@@ -167,7 +143,19 @@ https://llvm.org/doxygen/MemorySanitizer_8cpp.html
 (defun emit-sap-ref (size insn modifier result ea node vop temp)
   (declare (ignorable node size vop temp))
   (cond
-   #+linux
+   ;; MSAN as implemented can't correctly interact with the C sanitizer because
+   ;; C msan shadow memory indicates bit-for-bit whether the user-visible memory
+   ;; was written, whereas in Lisp we only have byte granularity. When Lisp users
+   ;; need to read bit fields they have to mask off bits explicitly after doing
+   ;; a 1, 2, 4, or 8-byte load. Unfortunately that load could spuriously report
+   ;; that it was uninitialized if only _part_ was uninitialized, but we didn't
+   ;; want to examine the uninitialized bits. The problem is, it doesn't know that.
+   ;; Even though DEFINE-ALIEN-TYPE conveys the sematics of sub-byte fields, the
+   ;; compiler is itself unable to extract partial bytes.
+   ;; (You get "cannot extract 4-bit integers" for example)
+   ;; So unfortunately, because the IR can't convey what we mean, we have to do
+   ;; a larger load than wanted, which may fail.
+   #+nil
    ((and (sb-c:msan-unpoison sb-c:*compilation*) (policy node (> safety 0)))
     ;; Must not clobber TEMP with the load.
     (aver (not (location= temp result)))
@@ -398,6 +386,9 @@ https://llvm.org/doxygen/MemorySanitizer_8cpp.html
   (:result-types system-area-pointer)
   (:generator 2
     (let ((disp (- (* vector-data-offset n-word-bytes) other-pointer-lowtag)))
-      (if (location= sap vector)
-          (inst add sap disp)
-          (inst lea sap (ea disp vector))))))
+      (flet ((2-operand-add (dst src)
+               (cond ((eql src 1) (inst inc dst))
+                     (t (inst add dst src)))))
+        (if (location= sap vector)
+            (2-operand-add sap disp)
+            (inst lea sap (ea disp vector)))))))
