@@ -731,10 +731,8 @@
           (value :scs (any-reg descriptor-reg)))
    (:vop-var vop)
    (:temporary (:sc unsigned-reg) temp)
-   (:node-var node)
    (:generator 1
-     (unless (sb-c::set-slot-old-p node)
-       (emit-gengc-barrier object nil temp (vop-nth-arg 1 vop) value))
+     (emit-gengc-barrier object nil temp (vop-nth-arg 1 vop) value)
      (inst mov :dword (ea (- 4 instance-pointer-lowtag) object) value)))
  (define-vop (%fun-layout %instance-layout)
    (:translate %fun-layout)
@@ -742,8 +740,7 @@
  (define-vop (%set-fun-layout %set-instance-layout)
    (:translate %set-fun-layout)
    (:generator 1
-     (unless (sb-c::set-slot-old-p node)
-       (emit-gengc-barrier object nil temp (vop-nth-arg 1 vop) value))
+     (emit-gengc-barrier object nil temp (vop-nth-arg 1 vop) value)
      (inst mov :dword (ea (- 4 fun-pointer-lowtag) object) value)))
  (define-vop ()
   (:translate sb-c::layout-eq)
