@@ -82,7 +82,7 @@
 (defconstant msan-temp-reg-number 0)
 
 ;;; The encoding anomaly for r12 makes it a perfect choice for the card table base.
-;;; It will never be used with a constant displacement.
+;;; It will seldom be used with a constant displacement.
 (define-symbol-macro card-table-reg 12)
 (define-symbol-macro gc-card-table-reg-tn r12-tn)
 (define-symbol-macro card-index-mask (make-fixup nil :card-table-index-mask))
@@ -619,7 +619,7 @@
                   (power-of-two-p (lvar-value d))
                   (and (csubtypep (sb-c::lvar-type n) (specifier-type 'signed-word))
                        (not (csubtypep (sb-c::lvar-type n) (specifier-type 'word)))))
-             (values :direct nil)
+             (values :maybe nil)
              (values :default nil))))
       (%dpb
        (flet ((validp (type result-type)
@@ -659,3 +659,7 @@
 #+sb-xc-host
 (setq *backend-cross-foldable-predicates*
       '(power-of-two-p))
+
+#+nil
+(define-cond-sc 32-bit-immediate immediate
+  (typep (tn-value tn) '(signed-byte 32)))
