@@ -122,7 +122,7 @@
     ((not and or) `(,(car type) ,@(mapcar #'convert-to-system-type
                                           (cdr type))))
     ((class class-eq) ; class-eq is impossible to do right
-     (wrapper-classoid (class-wrapper (cadr type))))
+     (layout-classoid (class-wrapper (cadr type))))
     (eql type)
     (t (if (null (cdr type))
            (car type)
@@ -190,7 +190,7 @@
                       (/noshow "entering DIRECT-SUPERS" (classoid-name class))
                       (if (typep class 'built-in-classoid)
                           (built-in-classoid-direct-superclasses class)
-                          (let ((inherits (wrapper-inherits (classoid-wrapper class))))
+                          (let ((inherits (layout-inherits (classoid-layout class))))
                             (/noshow inherits)
                             (list (svref inherits (1- (length inherits)))))))
                     (direct-subs (class)
@@ -213,8 +213,8 @@
                                   `(list ',name
                                          ',(mapcar #'classoid-name (direct-supers class))
                                          ',(mapcar #'classoid-name (direct-subs class))
-                                         ',(map 'list #'wrapper-classoid-name
-                                                (reverse (wrapper-inherits (classoid-wrapper class))))
+                                         ',(map 'list #'layout-classoid-name
+                                                (reverse (layout-inherits (classoid-layout class))))
                                          ,(getf (cdr kernel-bic-entry) :prototype-form))))
                               (remove-if (lambda (kernel-bic-entry)
                                            (member (first kernel-bic-entry)
@@ -365,18 +365,10 @@
 (defclass method-combination (metaobject)
   ((%documentation :initform nil :initarg :documentation)))
 
-;;; TODO: we can express MAKE-GF-HASHSET in terms of the robinhood hashset
-;;; using the same stable FSC-INSTANCE-HASH as this.
 (defun make-gf-hashset ()
-  ;; Return what is logically a weak hashset, but physically a weak hash-table
-  ;; because we don't implement hashsets.
-  (make-hash-table :test 'eq
-                   :hash-function #'fsc-instance-hash ; stable hash
-                   :weakness :key
+  (make-hashset 64 #'eq #'fsc-instance-hash
+                   :weakness t
                    :synchronized t))
-(defun add-to-weak-hashset (key set) (setf (gethash key set) t))
-(defun remove-from-weak-hashset (key set) (remhash key set))
-(defun weak-hashset-memberp (key set) (gethash key set))
 
 (defclass standard-method-combination (definition-source-mixin
                                        method-combination)
@@ -566,11 +558,6 @@
                            specializer-with-object)
   ((object :initarg :object :reader specializer-object
            :reader eql-specializer-object)
-   ;; created on demand (if and when needed), the CTYPE is the representation
-   ;; of this metaobject as an internalized type object understood by the
-   ;; kernel's type machinery. The CLOS object is really just a MEMBER-TYPE,
-   ;; but the type system doesn't know that.
-   (sb-kernel:ctype)
    ;; Because EQL specializers are interned, any two putative instances
    ;; of EQL-specializer referring to the same object are in fact EQ to
    ;; each other. Therefore a list of direct methods in the specializer can
