@@ -1641,15 +1641,15 @@ Except see also BREAK-VICIOUS-METACIRCLE.  -- CSR, 2003-05-28
                                             all-applicable-p
                                             (all-sorted-p t)
                                             function-p)
-   (if (null methods)
+  (if (null methods)
       (lambda (method-alist wrappers)
         (declare (ignore method-alist wrappers))
         (lambda (&rest args)
           (call-no-applicable-method gf args)))
       (let* ((key (car methods))
              (cache
-              (if (listp key) ; early method
-                  (sixth key) ; See !EARLY-MAKE-A-METHOD
+              (if (listp key)           ; early method
+                  (sixth key)           ; See !EARLY-MAKE-A-METHOD
                   (or (method-em-cache key)
                       (setf (method-em-cache key) (cons nil nil))))))
         (if (and (null (cdr methods)) all-applicable-p ; the most common case
@@ -1672,28 +1672,39 @@ Except see also BREAK-VICIOUS-METACIRCLE.  -- CSR, 2003-05-28
 (defun get-secondary-dispatch-function2 (gf methods types method-alist-p
                                          wrappers-p all-applicable-p
                                          all-sorted-p function-p)
-  (cond
-    ((not (and all-applicable-p all-sorted-p (not function-p)))
-     (let ((net (generate-discrimination-net
-                 gf methods types all-sorted-p)))
-       (compute-secondary-dispatch-function1 gf net function-p)))
-    ((eq **boot-state** 'complete)
-     (let* ((combin (generic-function-method-combination gf))
-            (effective (compute-effective-method gf combin methods)))
-       (make-effective-method-function1
-        gf effective method-alist-p wrappers-p)))
-    ((eq (generic-function-name gf) 'make-specializer-form-using-class)
-     ;; FIXME: instead of the above form, this should be
-     ;; (eq (generic-function-method-combination gf) *or-method-combination*)
-     ;; but that does not work for reasons I (JM) do not understand.
-     (let* ((combin (generic-function-method-combination gf))
-            (effective (short-compute-effective-method gf combin methods)))
-       (make-effective-method-function1
-        gf effective method-alist-p wrappers-p)))
-    (t
-     (let ((effective (standard-compute-effective-method gf nil methods)))
-       (make-effective-method-function1
-        gf effective method-alist-p wrappers-p)))))
+  (flet ((maybe-wrap (effective)
+           (if (gf-requires-emf-keyword-checks gf)
+               (multiple-value-bind (valid-keys keyargs-start)
+                   (compute-applicable-keywords gf methods)
+                 (wrap-with-applicable-keyword-check effective valid-keys keyargs-start))
+               effective)))
+    (cond
+      ((not (and all-applicable-p all-sorted-p (not function-p)))
+       (let ((net (generate-discrimination-net
+                   gf methods types all-sorted-p)))
+         (compute-secondary-dispatch-function1 gf net function-p)))
+      ((eq **boot-state** 'complete)
+       (let ((combin (generic-function-method-combination gf)))
+         (if (null (compute-primary-methods gf combin methods))
+             (lambda (method-alist wrappers)
+               (declare (ignore method-alist wrappers))
+               (lambda (&rest args)
+                 (call-no-primary-method gf args)))
+             (let ((effective (maybe-wrap (compute-effective-method gf combin methods))))
+               (make-effective-method-function1
+                gf effective method-alist-p wrappers-p)))))
+      ((eq (generic-function-name gf) 'make-specializer-form-using-class)
+       ;; FIXME: instead of the above form, this should be
+       ;; (eq (generic-function-method-combination gf) *or-method-combination*)
+       ;; but that does not work for reasons I (JM) do not understand.
+       (let* ((combin (generic-function-method-combination gf))
+              (effective (maybe-wrap (short-compute-effective-method gf combin methods))))
+         (make-effective-method-function1
+          gf effective method-alist-p wrappers-p)))
+      (t
+       (let ((effective (maybe-wrap (standard-compute-effective-method gf nil methods))))
+         (make-effective-method-function1
+          gf effective method-alist-p wrappers-p))))))
 
 (defun get-effective-method-function (gf methods
                                          &optional method-alist wrappers)
