@@ -386,6 +386,16 @@
 (defmethod gf-fast-method-function-p ((gf standard-generic-function))
   (gf-info-fast-mf-p (slot-value gf 'arg-info)))
 
+(defun add-to-weak-hashset (key set)
+  (with-system-mutex ((hashset-mutex set))
+    (hashset-insert set key)))
+(defun remove-from-weak-hashset (key set)
+  (with-system-mutex ((hashset-mutex set))
+    (hashset-remove set key)))
+(defun weak-hashset-memberp (key set)
+  (with-system-mutex ((hashset-mutex set))
+    (hashset-find set key)))
+
 (defmethod initialize-instance :after ((gf standard-generic-function)
                                        &key (lambda-list nil lambda-list-p)
                                        argument-precedence-order)
@@ -913,9 +923,9 @@
                                    'get-accessor-method-function)))
                     ,optimized-std-fun)))
                 (wrappers
-                 (let ((wrappers (list (wrapper-of class)
+                 (let ((wrappers (list (layout-of class)
                                        (class-wrapper class)
-                                       (wrapper-of slotd))))
+                                       (layout-of slotd))))
                    (if (eq type 'writer)
                        (cons (class-wrapper *the-class-t*) wrappers)
                        wrappers)))
@@ -1678,7 +1688,7 @@
    gf (generic-function-encapsulations gf) (call-next-method)))
 
 (defmethod (setf class-name) (new-value class)
-  (let ((classoid (wrapper-classoid (class-wrapper class))))
+  (let ((classoid (layout-classoid (class-wrapper class))))
     (if (and new-value (symbolp new-value))
         (setf (classoid-name classoid) new-value)
         (setf (classoid-name classoid) nil)))
