@@ -16,13 +16,12 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include "gc.h"
-#include "gc-internal.h"
-#include "gc-private.h"
-#include "gencgc-private.h"
 #include "pseudo-atomic.h"
 #include "genesis/gc-tables.h"
 #include "genesis/vector.h"
 #include "arch.h" // why is this where funcall2 is declared???
+#include "genesis/fdefn.h" // for StaticSymbolFunction
+#include "code.h"
 
 lispobj* atomic_bump_static_space_free_ptr(int nbytes)
 {
@@ -139,6 +138,7 @@ void sync_close_regions(int block_signals, int options,
     for (i=0; i<count; ++i) {
         page_index_t p = find_page_index(a[i].r->start_addr);
         if (p < 0) continue;
+#ifndef LISP_FEATURE_MARK_REGION_GC
         /* Potentially use up all remaining bytes in the TLAB before closing.
          * Pages below the alloc_start for the page type cannot possibly be used,
          * but we didn't properly account for that space, which has a bad effect
@@ -158,6 +158,7 @@ void sync_close_regions(int block_signals, int options,
             deposit_filler(freeptr, new_end);
             a[i].r->free_pointer = new_end;
         }
+#endif
         ensure_region_closed(a[i].r, a[i].type);
     }
     if (options & LOCK_PAGE_TABLE) release_gc_page_table_lock();
@@ -276,7 +277,7 @@ lispobj alloc_code_object(unsigned total_words, unsigned boxed)
         (void*)lisp_alloc(nbytes >= LARGE_OBJECT_SIZE, code_region, nbytes, PAGE_TYPE_CODE, th);
     result = mutex_release(&code_allocator_lock);
     gc_assert(result);
-    THREAD_JIT(0);
+    THREAD_JIT_WP(0);
 
     code->header = ((uword_t)total_words << CODE_HEADER_SIZE_SHIFT) | CODE_HEADER_WIDETAG;
     // 'boxed_size' is an untagged word expressing the number of *bytes* in the boxed section
@@ -290,8 +291,7 @@ lispobj alloc_code_object(unsigned total_words, unsigned boxed)
     for ( ; p < end ; ++p) *p = 0;
     *p = 0; // 'p' now points to the jump table count word which must be 0
     ((lispobj*)code)[total_words-1] = 0; // zeroize the simple-fun table count
-    THREAD_JIT(1);
-
+    THREAD_JIT_WP(1);
     return make_lispobj(code, OTHER_POINTER_LOWTAG);
 }
 
@@ -365,18 +365,6 @@ listify_rest_arg(lispobj* context, sword_t nbytes, int sys) {
         *tail = make_lispobj((void*)c, LIST_POINTER_LOWTAG);
         int ncells = partial_request >> (1+WORD_SHIFT);
         nbytes -= N_WORD_BYTES * 2 * ncells;
-        // Unroll x 4
-        int n_unrolled_iterations = ncells >> 2;
-        struct cons* limit = c + n_unrolled_iterations * 4;
-        while (c < limit) {
-            c[0].car = context[ 0]; c[0].cdr = make_lispobj(c+1, LIST_POINTER_LOWTAG);
-            c[1].car = context[-1]; c[1].cdr = make_lispobj(c+2, LIST_POINTER_LOWTAG);
-            c[2].car = context[-2]; c[2].cdr = make_lispobj(c+3, LIST_POINTER_LOWTAG);
-            c[3].car = context[-3]; c[3].cdr = make_lispobj(c+4, LIST_POINTER_LOWTAG);
-            c += 4;
-            context -= 4;
-        }
-        ncells -= n_unrolled_iterations * 4;
         while (ncells--) {
             c->car = *context--;
             c->cdr = make_lispobj(c+1, LIST_POINTER_LOWTAG);
@@ -406,18 +394,6 @@ NO_SANITIZE_MEMORY lispobj listify_rest_arg(lispobj* context, sword_t context_by
         *tail = make_lispobj((void*)c, LIST_POINTER_LOWTAG);
         int ncells = partial_request >> (1+WORD_SHIFT);
         nbytes -= N_WORD_BYTES * 2 * ncells;
-        // Unroll x 4
-        int n_unrolled_iterations = ncells >> 2;
-        struct cons* limit = c + n_unrolled_iterations * 4;
-        while (c < limit) {
-            c[0].car = context[ 0]; c[0].cdr = make_lispobj(c+1, LIST_POINTER_LOWTAG);
-            c[1].car = context[ 1]; c[1].cdr = make_lispobj(c+2, LIST_POINTER_LOWTAG);
-            c[2].car = context[ 2]; c[2].cdr = make_lispobj(c+3, LIST_POINTER_LOWTAG);
-            c[3].car = context[ 3]; c[3].cdr = make_lispobj(c+4, LIST_POINTER_LOWTAG);
-            c += 4;
-            context += 4;
-        }
-        ncells -= n_unrolled_iterations * 4;
         while (ncells--) {
             c->car = *context++;
             c->cdr = make_lispobj(c+1, LIST_POINTER_LOWTAG);
@@ -441,219 +417,7 @@ double generation_average_age(generation_index_t gen_index)
     return (double)gen->cum_sum_bytes_allocated / (double)gen->bytes_allocated;
 }
 
-static int count_immobile_objects(__attribute__((unused)) int gen, int res[4])
-{
-#ifdef LISP_FEATURE_IMMOBILE_SPACE
-    lispobj* where = (lispobj*)FIXEDOBJ_SPACE_START;
-    lispobj* end = fixedobj_free_pointer;
-    while (where < end) {
-        if (immobile_obj_generation(where) == gen) {
-            switch (widetag_of(where)) {
-            case INSTANCE_WIDETAG: ++res[0]; break;
-            case FDEFN_WIDETAG: ++res[1]; break;
-            case SYMBOL_WIDETAG: ++res[2]; break;
-            }
-        }
-        where += object_size(where);
-    }
-    where = (lispobj*)TEXT_SPACE_START;
-    end = text_space_highwatermark;
-    while (where < end) {
-        if (widetag_of(where) != FILLER_WIDETAG && immobile_obj_generation(where) == gen)
-            ++res[3];
-        where += object_size(where);
-    }
-#endif
-    return (res[0] | res[1] | res[2] | res[3]) != 0;
-}
-
-/* Count the number of pages in the given generation.
- * Additionally, if 'n_write_protected' is non-NULL, then assign
- * into *n_write_protected the count of marked pages.
- */
-page_index_t
-count_generation_pages(generation_index_t generation, page_index_t* n_dirty)
-{
-    page_index_t i, total = 0, dirty = 0;
-    int j;
-
-    for (i = 0; i < next_free_page; i++)
-        if (!page_free_p(i) && (page_table[i].gen == generation)) {
-            total++;
-            long card = page_to_card_index(i);
-            for (j=0; j<CARDS_PER_PAGE; ++j, ++card)
-                if (card_dirtyp(card)) ++dirty;
-        }
-    // divide by cards per page rounding up
-    if (n_dirty) *n_dirty = (dirty + (CARDS_PER_PAGE-1)) / CARDS_PER_PAGE;
-    return total;
-}
-
-// You can call this with 0 and NULL to perform its assertions silently
-void gc_gen_report_to_file(int filedes, FILE *file)
-{
-#ifdef LISP_FEATURE_X86
-    extern void fpu_save(void *), fpu_restore(void *);
-    int fpu_state[27];
-
-    /* Can end up here after calling alloc_tramp which doesn't prepare
-     * the x87 state, and the C ABI uses a different mode */
-    fpu_save(fpu_state);
-#endif
-
-#define OUTPUT(str, len) \
-    {if (file) fwrite(str, 1, len, file); if (filedes>=0) ignore_value(write(filedes, str, len));}
-
-    /* Print the heap stats. */
-    char header1[] =
-            "        Immobile Object Counts\n";
-    OUTPUT(header1, sizeof header1-1);
-    char header2[] =
-            " Gen layout fdefn symbol   code  Boxed   Cons    Raw   Code  SmMix  Mixed  LgRaw LgCode  LgMix"
-            " Waste%       Alloc        Trig   Dirty GCs Mem-age\n";
-    OUTPUT(header2, sizeof header2-1);
-
-    generation_index_t gen_num, begin, end;
-    int immobile_matrix[8][4], have_immobile_obj = 0;
-    int immobile_totals[4];
-    memset(immobile_matrix, 0, sizeof immobile_matrix);
-    memset(immobile_totals, 0, sizeof immobile_totals);
-    for (gen_num = 0; gen_num <= 6; ++gen_num) {
-        if (count_immobile_objects(gen_num, immobile_matrix[gen_num]))
-            have_immobile_obj |= 1 << gen_num;
-        immobile_totals[0] += immobile_matrix[gen_num][0];
-        immobile_totals[1] += immobile_matrix[gen_num][1];
-        immobile_totals[2] += immobile_matrix[gen_num][2];
-        immobile_totals[3] += immobile_matrix[gen_num][3];
-    }
-    // Print from the lowest gen that has any allocated pages.
-    for (begin = 0; begin <= PSEUDO_STATIC_GENERATION; ++begin)
-        if ((have_immobile_obj>>begin)&1 || generations[begin].bytes_allocated) break;
-    // Print up to and including the highest gen that has any allocated pages.
-    for (end = SCRATCH_GENERATION; end >= 0; --end)
-        if (generations[end].bytes_allocated) break;
-
-    char linebuf[180];
-    page_index_t coltot[9];
-    uword_t eden_words_allocated = 0;
-    page_index_t eden_pages = 0;
-    memset(coltot, 0, sizeof coltot);
-    for (gen_num = begin; gen_num <= end; gen_num++) {
-        page_index_t page;
-        page_index_t pagect[9];
-        int *objct = immobile_matrix[gen_num];
-        memset(pagect, 0, sizeof pagect);
-        if (gen_num == 0) { // Count the eden pages
-            for (page = 0; page < next_free_page; page++)
-                if (page_table[page].gen == 0 && page_table[page].type & THREAD_PAGE_FLAG) {
-                    int column;
-                    switch (page_table[page].type & ~THREAD_PAGE_FLAG) {
-                    case PAGE_TYPE_BOXED:   column = 0; break;
-                    case PAGE_TYPE_CONS:    column = 1; break;
-                    case PAGE_TYPE_CODE:    column = 3; break;
-                    case PAGE_TYPE_MIXED:   column = 5; break;
-                    default: lose("Bad eden page subtype: %x\n", page_table[page].type);
-                    }
-                    pagect[column]++;
-                    coltot[column]++;
-                    ++eden_pages;
-                    eden_words_allocated += page_words_used(page);
-                }
-            uword_t waste = npage_bytes(eden_pages) - (eden_words_allocated<<WORD_SHIFT);
-            double pct_waste = eden_pages > 0 ?
-                               (double)waste / (double)npage_bytes(eden_pages) * 100 : 0.0;
-            if (eden_pages) {
-                int linelen = snprintf(linebuf, sizeof linebuf,
-                        "  E %6d %6d %6d %6d %7"PAGE_INDEX_FMT"%7"PAGE_INDEX_FMT"%14"PAGE_INDEX_FMT
-                        "%14"PAGE_INDEX_FMT
-                        "%28.1f %11"OS_VM_SIZE_FMT"\n",
-                        objct[0], objct[1], objct[2], objct[3],
-                        pagect[0], pagect[1], pagect[3], pagect[5],
-                        pct_waste, eden_words_allocated<<WORD_SHIFT);
-                OUTPUT(linebuf, linelen);
-            }
-            memset(pagect, 0, sizeof pagect);
-        }
-        uword_t words_allocated = 0;
-        page_index_t tot_pages = 0;
-        for (page = 0; page < next_free_page; page++)
-            if (!page_free_p(page) && page_table[page].gen == gen_num
-                && !(page_table[page].type & THREAD_PAGE_FLAG)) {
-                int column;
-                switch (page_table[page].type & (SINGLE_OBJECT_FLAG|PAGE_TYPE_MASK)) {
-                case PAGE_TYPE_BOXED:   column = 0; break;
-                case PAGE_TYPE_CONS:    column = 1; break;
-                case PAGE_TYPE_UNBOXED: column = 2; break;
-                case PAGE_TYPE_CODE:    column = 3; break;
-                case PAGE_TYPE_SMALL_MIXED:   column = 4; break;
-                case PAGE_TYPE_MIXED:   column = 5; break;
-                case SINGLE_OBJECT_FLAG|PAGE_TYPE_UNBOXED: column = 6; break;
-                case SINGLE_OBJECT_FLAG|PAGE_TYPE_CODE:    column = 7; break;
-                case SINGLE_OBJECT_FLAG|PAGE_TYPE_MIXED:   column = 8; break;
-                default: lose("Invalid page type %#x (p%"PAGE_INDEX_FMT")", page_table[page].type, page);
-                }
-                pagect[column]++;
-                coltot[column]++;
-                ++tot_pages;
-                words_allocated += page_words_used(page);
-            }
-        struct generation* gen = &generations[gen_num];
-        /* This wouldn't appear to hold for how mark-region GC treats pages.
-         * TODO: Write what should hold. */
-        if (gen_num == 0)
-            gc_assert(gen->bytes_allocated ==
-                      (words_allocated+eden_words_allocated) << WORD_SHIFT);
-        else {
-            gc_assert(gen->bytes_allocated == words_allocated << WORD_SHIFT);
-        }
-        page_index_t n_dirty;
-        count_generation_pages(gen_num, &n_dirty);
-        uword_t waste = npage_bytes(tot_pages) - (words_allocated<<WORD_SHIFT);
-        double pct_waste = tot_pages > 0 ?
-          (double)waste / (double)npage_bytes(tot_pages) * 100 : 0.0;
-        int linelen =
-            snprintf(linebuf, sizeof linebuf,
-                "  %d %6d %6d %6d %6d"
-                "%7"PAGE_INDEX_FMT"%7"PAGE_INDEX_FMT"%7"PAGE_INDEX_FMT"%7"PAGE_INDEX_FMT
-                "%7"PAGE_INDEX_FMT"%7"PAGE_INDEX_FMT"%7"PAGE_INDEX_FMT"%7"PAGE_INDEX_FMT
-                "%7"PAGE_INDEX_FMT" %6.1f %11"OS_VM_SIZE_FMT" %11"OS_VM_SIZE_FMT,
-                gen_num, objct[0], objct[1], objct[2], objct[3],
-                pagect[0], pagect[1], pagect[2], pagect[3], pagect[4], pagect[5],
-                pagect[6], pagect[7], pagect[8],
-                pct_waste, words_allocated<<WORD_SHIFT,
-                (uintptr_t)gen->gc_trigger);
-        // gen0 pages are never WPed
-        linelen += snprintf(linebuf+linelen, sizeof linebuf-linelen,
-                            gen_num==0?"       -" : " %7"PAGE_INDEX_FMT, n_dirty);
-        linelen += snprintf(linebuf+linelen, sizeof linebuf-linelen,
-                            " %3d %7.4f\n", gen->num_gc, generation_average_age(gen_num));
-        OUTPUT(linebuf, linelen);
-    }
-    page_index_t tot_pages = coltot[0] + coltot[1] + coltot[2] + coltot[3] + coltot[4] +
-                             coltot[5] + coltot[6] + coltot[7] + coltot[8];
-    uword_t waste = npage_bytes(tot_pages) - bytes_allocated;
-    double pct_waste = (double)waste / (double)npage_bytes(tot_pages) * 100;
-    double heap_use_frac = 100 * (double)bytes_allocated / (double)dynamic_space_size;
-    int *objct = immobile_totals;
-    int linelen =
-        snprintf(linebuf, sizeof linebuf,
-            "Tot %6d %6d %6d %6d"
-            "%7"PAGE_INDEX_FMT"%7"PAGE_INDEX_FMT"%7"PAGE_INDEX_FMT"%7"PAGE_INDEX_FMT
-            "%7"PAGE_INDEX_FMT"%7"PAGE_INDEX_FMT"%7"PAGE_INDEX_FMT"%7"PAGE_INDEX_FMT
-            "%7"PAGE_INDEX_FMT" %6.1f%12"OS_VM_SIZE_FMT
-            " [%.1f%% of %"OS_VM_SIZE_FMT" max]\n",
-            objct[0], objct[1], objct[2], objct[3],
-            coltot[0], coltot[1], coltot[2], coltot[3], coltot[4], coltot[5], coltot[6],
-            coltot[7], coltot[8], pct_waste,
-            (uintptr_t)bytes_allocated, heap_use_frac, (uintptr_t)dynamic_space_size);
-    OUTPUT(linebuf, linelen);
-#undef OUTPUT
-
-#ifdef LISP_FEATURE_X86
-    fpu_restore(fpu_state);
-#endif
-}
-
+extern void gc_gen_report_to_file(int filedes, FILE *file);
 void write_generation_stats(FILE *file) { gc_gen_report_to_file(-1, file); }
 
 void log_generation_stats(char *logfile, char *header)
@@ -693,7 +457,7 @@ void write_heap_exhaustion_report(FILE *file, long available, long requested,
             read_TLS(STOP_FOR_GC_PENDING,thread)==NIL ? "false" : "true");
 #endif
     fprintf(file, "Collection trigger variables:\n");
-    fprintf(file, "   dynamic_space_size = %ld\n   bytes_allocated = %ld\n   auto_gc_trigger = %ld\n   bytes_consed_between_gcs = %ld\n",
+    fprintf(file, "   dynamic_space_size = %zd\n   bytes_allocated = %zd\n   auto_gc_trigger = %zd\n   bytes_consed_between_gcs = %zd\n",
             dynamic_space_size,
             bytes_allocated,
             auto_gc_trigger,
