@@ -17,8 +17,7 @@
 #include <stdio.h>
 #include <sys/param.h>
 #include <sys/file.h>
-#include "sbcl.h"
-#include "./signal.h"
+#include "genesis/sbcl.h"
 #include "os.h"
 #include "arch.h"
 #include "globals.h"
@@ -68,12 +67,6 @@ os_context_register_addr(os_context_t *context, int offset)
     return (os_context_register_t*)&(&context->uc_mcontext.arm_r0)[offset];
 }
 
-os_context_register_t *
-os_context_pc_addr(os_context_t *context)
-{
-    return os_context_register_addr(context, reg_PC);
-}
-
 os_context_register_t *
 os_context_lr_addr(os_context_t *context)
 {
@@ -103,8 +96,8 @@ os_flush_icache(os_vm_address_t address, os_vm_size_t length)
 static void
 sigtrap_handler(int signal, siginfo_t *siginfo, os_context_t *context)
 {
-    unsigned int code = *((unsigned char *)(4+*os_context_pc_addr(context)));
-    u32 trap_instruction = *((u32 *)*os_context_pc_addr(context));
+    unsigned int code = *((unsigned char *)(4+OS_CONTEXT_PC(context)));
+    uint32_t trap_instruction = *(uint32_t *)OS_CONTEXT_PC(context);
 
     if (trap_instruction != 0xe7f001f0) {
         lose("Unrecognized trap instruction %08lx in sigtrap_handler()",
