@@ -14,7 +14,7 @@
  * files for more information.
  */
 
-#include "sbcl.h"
+#include "genesis/sbcl.h"
 
 #ifndef LISP_FEATURE_WIN32
 #include <sys/mman.h>
@@ -84,7 +84,7 @@ open_binary(char *filename, int mode)
     return open(filename, mode);
 }
 
-#if defined(LISP_FEATURE_ELF) && defined(LISP_FEATURE_IMMOBILE_CODE)
+#if defined LISP_FEATURE_LINUX && defined LISP_FEATURE_X86_64
 #define ELFCORE 1
 #elif !defined(ELFCORE)
 #define ELFCORE 0
@@ -92,10 +92,12 @@ open_binary(char *filename, int mode)
 
 #if !ELFCORE
 int lisp_code_in_elf() { return 0; }
+lispobj* get_alien_linkage_table_initializer() { return 0; }
 #else
 extern __attribute__((weak)) lispobj
  lisp_code_start, lisp_jit_code, lisp_code_end, alien_linkage_values;
 int lisp_code_in_elf() { return &lisp_code_start != 0; }
+lispobj* get_alien_linkage_table_initializer() { return &alien_linkage_values; }
 #endif
 
 /* Search 'filename' for an embedded core.  An SBCL core has, at the
@@ -261,12 +263,22 @@ static inline lispobj adjust_word(struct heap_adjust* adj, lispobj word) {
     return word + calc_adjustment(adj, word);
 }
 
+struct coreparse_space {
+    int id;
+    size_t desired_size; // size wanted, ORed with 1 if addr must be <2GB
+    // Values from the core file:
+    uword_t len; // length in bytes, as an integral multiple of os_vm_page_size
+    uword_t base;
+    lispobj** pfree_pointer; // pointer to x_free_pointer
+} spaces;
+
 static void
-set_adjustment(struct heap_adjust* adj,
-               uword_t actual_addr,
-               uword_t desired_addr,
-               uword_t len)
+set_adjustment(struct cons* pair, int id, uword_t actual_addr)
 {
+    struct coreparse_space* spaces = (void*)pair->car;
+    struct heap_adjust* adj = (void*)pair->cdr;
+    uword_t desired_addr = spaces[id].base;
+    sword_t len = spaces[id].len;
     sword_t delta = len ? actual_addr - desired_addr : 0;
     if (!delta) return;
     int j = adj->n_ranges;
@@ -318,7 +330,7 @@ static void adjust_pointers(lispobj *where, sword_t n_words, struct heap_adjust*
 }
 
 #include "var-io.h"
-#include "unaligned.h"
+#include "align.h"
 static void
 adjust_code_refs(struct heap_adjust __attribute__((unused)) *adj,
                  struct code __attribute__((unused)) *code,
@@ -376,26 +388,6 @@ static inline void fix_fun_header_layout(lispobj __attribute__((unused)) *fun,
 #endif
 }
 
-/* Refer to the figures above CALCULATE-DD-BITMAP in src/code/defstruct
- * for the meaning of these constants.
- * I think this fixes what may have been a latent bug in space relocation.
- * Due to inconsistent use of the bitmap, it could have processed funinstance
- * LAYOUT slots twice.  Layout is supposed to have a 0 bit, meaning do NOT scan
- * in a loop over slots, but instead externally to the loop. By marking it as 1,
- * it could be processed both outside and inside the loop. In the extremely rare
- * case of partial overlap of the desired and actual heap addresses, a pointer
- * somewhere in the middle can not be discerned as having been fixed up or not
- * fixed up based on its value. You have to know based on code flow.
- * Therefore layouts slots always have to be treated as raw in the bitmap,
- * but I did not see where it is enforced in lisp for funinstances. */
-#ifndef LISP_FEATURE_EXECUTABLE_FUNINSTANCES
-# define FUNINSTANCE_BITMAP -4
-#elif defined LISP_FEATURE_COMPACT_INSTANCE_HEADER
-# define FUNINSTANCE_BITMAP -8
-#else
-# define FUNINSTANCE_BITMAP -24
-#endif
-
 static void relocate_space(uword_t start, lispobj* end, struct heap_adjust* adj)
 {
     int widetag;
@@ -417,6 +409,12 @@ static void relocate_space(uword_t start, lispobj* end, struct heap_adjust* adj)
         widetag = header_widetag(word);
         nwords = sizetab[widetag](where);
         switch (widetag) {
+        case FUNCALLABLE_INSTANCE_WIDETAG:
+            /* If non-executable funinstance, then word index 1 points at read-only space,
+             * hence needs no adjustment. Otherwise, the word points within the funinstance.
+             * Either way, adjust_word_at will do the right thing */
+            adjust_word_at(where+1, adj);
+            // FALLTHROUGH_INTENDED
         case INSTANCE_WIDETAG:
             layout = layout_of(where);
             adjusted_layout = adjust_word(adj, layout);
@@ -430,25 +428,6 @@ static void relocate_space(uword_t start, lispobj* end, struct heap_adjust* adj)
                 if (bitmap_logbitp(i, bitmap)) adjust_pointers(slots+i, 1, adj);
             }
             continue;
-        case FUNCALLABLE_INSTANCE_WIDETAG:
-            /* If non-executable funinstance, then word index 1 points at read-only space,
-             * hence needs no adjustment. Otherwise, the word points within the funinstance.
-             * Either way, adjust_word_at will do the right thing */
-            adjust_word_at(where+1, adj);
-            /* We have to adjust the layout, but its bitmap can be disregarded.
-             * There is no variation in bitmap by particular subtype of funinstance */
-            layout = funinstance_layout(where);
-            adjusted_layout = adjust_word(adj, layout);
-            if (adjusted_layout != layout) funinstance_layout(where) = adjusted_layout;
-            const int mask = FUNINSTANCE_BITMAP;
-            gc_assert((mask & (1<<0)) == 0); // bitmap must reflect it as untagged
-            {
-            lispobj* slots = where+1;
-            // slots[0] is where[1] which we just adjusted, so skip it.
-            for (i=1; i<(nwords-1); ++i) // -1 from nwords because 'slots' is +1 from 'where'
-                if ((1<<i) & mask) adjust_pointers(slots+i, 1, adj);
-            }
-            continue;
         case SYMBOL_WIDETAG:
             { // Copied from scav_symbol() in gc-common
             struct symbol* s = (void*)where;
@@ -610,9 +589,9 @@ static void relocate_heap(struct heap_adjust* adj)
 #endif
     relocate_space(DYNAMIC_SPACE_START, (lispobj*)dynamic_space_highwatermark(),
                    adj);
-#ifdef LISP_FEATURE_IMMOBILE_SPACE
+    // FIXME: This fixes pointers in the space but I think it won't work
+    // if the space itself fails to map where requested.
     relocate_space(TEXT_SPACE_START, text_space_highwatermark, adj);
-#endif
 }
 
 #if defined(LISP_FEATURE_ELF) && defined(LISP_FEATURE_IMMOBILE_SPACE)
@@ -633,10 +612,19 @@ void calc_asm_routine_bounds()
     } else {
         lispobj *where = (lispobj*)STATIC_SPACE_OBJECTS_START;
         for (; where < static_space_free_pointer; where += object_size(where))
-            if (widetag_of((lispobj*)where) == CODE_HEADER_WIDETAG) {
+            if (widetag_of(where) == CODE_HEADER_WIDETAG) {
                 asm_routines_start = (uword_t)where;
                 break;
             }
+        /* I thought it would be possible to erase the static-space copy of asm routines
+         * after converting to text space but maybe not. So they'll still be here.
+         * But I want to erase them. This disabled block represents such aspiration */
+        if (0) { // !asm_routines_start && TEXT_SPACE_START != 0) {
+            lispobj *where = (lispobj*)TEXT_SPACE_START;
+            where += object_size(where);
+            gc_assert(widetag_of(where) == CODE_HEADER_WIDETAG);
+            asm_routines_start = (uword_t)where;
+        }
         if (!asm_routines_start) lose("Can't find asm routines");
     }
 #endif
@@ -705,13 +693,14 @@ static os_vm_address_t reserve_space(int space_id, int attr,
     return addr;
 }
 
-struct coreparse_space {
-    size_t desired_size; // size wanted, ORed with 1 if addr must be <2GB
-    // Values from the core file:
-    uword_t len; // length in bytes, as an integral multiple of os_vm_page_size
-    uword_t base;
-    lispobj** pfree_pointer; // pointer to x_free_pointer
-} spaces;
+static __attribute__((unused)) uword_t corespace_checksum(uword_t* base, int nwords)
+{
+    uword_t result  = 0;
+    int i;
+    for (i = 0; i<nwords; ++i)
+        result = ((result << 1) | (result >> (N_WORD_BITS-1))) ^ base[i];
+    return result;
+}
 
 /* TODO: If static + readonly were mapped as desired without disabling ASLR
  * but one of the large spaces couldn't be mapped as desired, start over from
@@ -727,6 +716,9 @@ process_directory(int count, struct ndir_entry *entry,
                   struct coreparse_space *spaces,
                   struct heap_adjust *adj)
 {
+    // If ELF core is supported, then test whether the weak symbol
+    // 'lisp_code_start' exists in this executable. If it does, then parse
+    // the ELF sections containing Lisp code and everything else.
 #if ELFCORE
     if (&lisp_code_start) {
         TEXT_SPACE_START = (uword_t)&lisp_code_start;
@@ -741,54 +733,35 @@ process_directory(int count, struct ndir_entry *entry,
                (uword_t)&lisp_code_start, (uword_t)&lisp_code_end,
                text_space_highwatermark);
 #endif
+        // unprotect the pages
+        os_protect((void*)TEXT_SPACE_START, text_space_size, OS_VM_PROT_ALL);
+
+#ifdef LISP_FEATURE_IMMOBILE_SPACE
+        // ELF core without immobile space has alien linkage space below static space.
         ALIEN_LINKAGE_TABLE_SPACE_START =
             (uword_t)os_alloc_gc_space(ALIEN_LINKAGE_TABLE_CORE_SPACE_ID, 0, 0,
                                        ALIEN_LINKAGE_TABLE_SPACE_SIZE);
-        // Prefill the alien linkage table so that shrinkwrapped executables which link in
-        // all their C library dependencies can avoid linking with -ldl
-        // but extern-alien still works for newly compiled code.
-        lispobj* ptr = &alien_linkage_values;
-        gc_assert(ptr);
-        int entry_index = 0;
-        int count;
-        extern int alien_linkage_table_n_prelinked;
-        count = alien_linkage_table_n_prelinked = *ptr++;
-        for ( ; count-- ; entry_index++ ) {
-            bool datap = *ptr == (lispobj)-1; // -1 can't be a function address
-            if (datap)
-                ++ptr;
-            arch_write_linkage_table_entry(entry_index, (void*)*ptr++, datap);
-        }
-
-        // unprotect the pages
-        os_protect((void*)TEXT_SPACE_START, text_space_size, OS_VM_PROT_ALL);
+#endif
     } else
 #endif
-#ifdef LISP_FEATURE_IMMOBILE_SPACE
+    // NB: The preceding 'else', if it's there, needs at least an empty
+    // statement following it if there is no immobile space.
     {
+#ifdef LISP_FEATURE_IMMOBILE_SPACE
         spaces[IMMOBILE_FIXEDOBJ_CORE_SPACE_ID].desired_size +=
             text_space_size + ALIEN_LINKAGE_TABLE_SPACE_SIZE;
-    }
 #endif
+    }
 
-    for ( ; --count>= 0; ++entry) {
+    for ( ; --count >= 0 ; ++entry) {
         long id = entry->identifier;
         uword_t addr = entry->address;
-#ifdef DEBUG_COREPARSE
-        printf("space %d @ %10lx pg=%4d+%4d nwords=%ld\n",
-               (int)id, addr, (int)entry->data_page, (int)entry->page_count,
-               entry->nwords);
-#endif
         int compressed = id & DEFLATED_CORE_SPACE_ID_FLAG;
         id -= compressed;
         if (id < 1 || id > MAX_CORE_SPACE_ID)
             lose("unknown space ID %ld addr %p", id, (void*)addr);
 
-#ifdef LISP_FEATURE_DARWIN_JIT
-        int enforce_address = (id == READ_ONLY_CORE_SPACE_ID);
-#else
         int enforce_address = 0;
-#endif
 #ifndef LISP_FEATURE_RELOCATABLE_STATIC_SPACE
         enforce_address |= (id == STATIC_CORE_SPACE_ID);
 #endif
@@ -808,7 +781,6 @@ process_directory(int count, struct ndir_entry *entry,
                  (unsigned long)len >> 10,
                  (unsigned long)dynamic_space_size >> 10);
         }
-#ifndef LISP_FEATURE_DARWIN_JIT
 #ifdef LISP_FEATURE_RELOCATABLE_STATIC_SPACE
         if (id == READ_ONLY_CORE_SPACE_ID || id == STATIC_CORE_SPACE_ID) {
 #else
@@ -823,7 +795,6 @@ process_directory(int count, struct ndir_entry *entry,
                 READ_ONLY_SPACE_START = READ_ONLY_SPACE_END = addr;
             }
         }
-#endif
         if (len != 0) {
             spaces[id].len = len;
             size_t request = spaces[id].desired_size;
@@ -841,12 +812,11 @@ process_directory(int count, struct ndir_entry *entry,
                 STATIC_SPACE_START = addr;
                 STATIC_SPACE_END = addr + len;
 #endif
-#ifndef LISP_FEATURE_DARWIN_JIT
+                break;
             case READ_ONLY_CORE_SPACE_ID:
                 READ_ONLY_SPACE_START = addr;
                 READ_ONLY_SPACE_END = addr + len;
                 break;
-#endif
 #ifdef LISP_FEATURE_IMMOBILE_SPACE
             case IMMOBILE_FIXEDOBJ_CORE_SPACE_ID:
             case IMMOBILE_TEXT_CORE_SPACE_ID:
@@ -859,7 +829,12 @@ process_directory(int count, struct ndir_entry *entry,
                 else
                     TEXT_SPACE_START = addr;
                 break;
+#else
+            case IMMOBILE_TEXT_CORE_SPACE_ID:
+                TEXT_SPACE_START = addr;
+                break;
 #endif
+
             case DYNAMIC_CORE_SPACE_ID:
                 {
                 uword_t aligned_start = ALIGN_UP(addr, GENCGC_PAGE_BYTES);
@@ -914,36 +889,30 @@ process_directory(int count, struct ndir_entry *entry,
               / GENCGC_PAGE_BYTES;
             anon_dynamic_space_start = (os_vm_address_t)(addr + len);
         }
+#ifdef DEBUG_COREPARSE
+        printf("space %d @ %10lx pg=%4d+%4d nwords=%9ld checksum=%lx\n",
+               (int)id, addr, (int)entry->data_page, (int)entry->page_count,
+               entry->nwords, corespace_checksum((void*)addr, entry->nwords));
+#endif
     }
 
     calc_asm_routine_bounds();
-#ifndef LISP_FEATURE_DARWIN_JIT
-    set_adjustment(adj, READ_ONLY_SPACE_START, // actual
-                   spaces[READ_ONLY_CORE_SPACE_ID].base, // expected
-                   spaces[READ_ONLY_CORE_SPACE_ID].len);
-#endif
-    set_adjustment(adj, DYNAMIC_SPACE_START, // actual
-                   spaces[DYNAMIC_CORE_SPACE_ID].base, // expected
-                   spaces[DYNAMIC_CORE_SPACE_ID].len);
+    struct cons spaceadj = {(lispobj)spaces, (lispobj)adj};
+    set_adjustment(&spaceadj, READ_ONLY_CORE_SPACE_ID, READ_ONLY_SPACE_START);
+    set_adjustment(&spaceadj, DYNAMIC_CORE_SPACE_ID, DYNAMIC_SPACE_START);
 #ifdef LISP_FEATURE_RELOCATABLE_STATIC_SPACE
-    set_adjustment(adj, STATIC_SPACE_START, // actual
-                   spaces[STATIC_CORE_SPACE_ID].base, // expected
-                   spaces[STATIC_CORE_SPACE_ID].len);
+    set_adjustment(&spaceadj, STATIC_CORE_SPACE_ID, STATIC_SPACE_START);
 #endif
 #ifdef LISP_FEATURE_IMMOBILE_SPACE
     if (lisp_code_in_elf() && TEXT_SPACE_START != spaces[IMMOBILE_TEXT_CORE_SPACE_ID].base) {
         lose("code-in-elf + PIE not supported");
     }
-    set_adjustment(adj, FIXEDOBJ_SPACE_START, // actual
-                   spaces[IMMOBILE_FIXEDOBJ_CORE_SPACE_ID].base, // expected
-                   spaces[IMMOBILE_FIXEDOBJ_CORE_SPACE_ID].len);
+    set_adjustment(&spaceadj, IMMOBILE_FIXEDOBJ_CORE_SPACE_ID, FIXEDOBJ_SPACE_START);
     if (!apply_pie_relocs(TEXT_SPACE_START
                           - spaces[IMMOBILE_TEXT_CORE_SPACE_ID].base,
                           DYNAMIC_SPACE_START - spaces[DYNAMIC_CORE_SPACE_ID].base,
                           fd))
-        set_adjustment(adj, TEXT_SPACE_START, // actual
-                       spaces[IMMOBILE_TEXT_CORE_SPACE_ID].base, // expected
-                       spaces[IMMOBILE_TEXT_CORE_SPACE_ID].len);
+        set_adjustment(&spaceadj, IMMOBILE_TEXT_CORE_SPACE_ID, TEXT_SPACE_START);
 #endif
 }
 
@@ -1000,6 +969,9 @@ bool gc_allocate_ptes()
     ++page_table;
     gc_page_pins = calloc(page_table_pages, 1);
     gc_assert(gc_page_pins);
+#ifdef LISP_FEATURE_DARWIN_JIT
+    page_execp = calloc(page_table_pages, 1);
+#endif
 
     // The card table size is a power of 2 at *least* as large
     // as the number of cards. These are the default values.
@@ -1028,7 +1000,17 @@ bool gc_allocate_ptes()
     num_gc_cards = 1L << gc_card_table_nbits;
 
     gc_card_table_mask =  num_gc_cards - 1;
+#if defined LISP_FEATURE_SB_SAFEPOINT && defined LISP_FEATURE_X86_64
+    /* The card table is hardware-page-aligned. Preceding it and occupying a whole
+     * "backend page" - which by the way is overkill - is the global safepoint trap page.
+     * The dummy TEST instruction for safepoints encodes shorter this way */
+    void* result = os_alloc_gc_space(0, MOVABLE, 0,
+                                     ALIGN_UP(num_gc_cards, BACKEND_PAGE_BYTES) + BACKEND_PAGE_BYTES);
+    gc_card_mark = (unsigned char*)result + BACKEND_PAGE_BYTES;
+#else
     gc_card_mark = successful_malloc(num_gc_cards);
+#endif
+
     /* The mark array used to work "by accident" if the numeric value of CARD_MARKED
      * is 0 - or equivalently the "WP'ed" state - which is the value that calloc()
      * fills with. If using malloc() we have to fill with CARD_MARKED,
@@ -1063,7 +1045,8 @@ bool gc_allocate_ptes()
 }
 
 extern void gcbarrier_patch_code(void*, int);
-#if !(defined LISP_FEATURE_MIPS || defined LISP_FEATURE_PPC64   \
+// Architectures that lack a method for gcbarrier_patch_code get this dummy stub.
+#if !(defined LISP_FEATURE_ARM64 || defined LISP_FEATURE_MIPS || defined LISP_FEATURE_PPC64  \
       || defined LISP_FEATURE_X86 || defined LISP_FEATURE_X86_64)
 void gcbarrier_patch_code(void* __attribute__((unused)) where, int __attribute__((unused)) nbits)
 {
@@ -1096,7 +1079,7 @@ static void gengcbarrier_patch_code_range(uword_t start, lispobj* limit)
  * it's not written to after changing its protection flags.
  * Touch every page... */
 void darwin_jit_code_pages_kludge () {
-    THREAD_JIT(0);
+    THREAD_JIT_WP(0);
     page_index_t page;
     for (page = 0; page  < next_free_page; page++) {
         if(is_code(page_table[page].type)) {
@@ -1107,20 +1090,19 @@ void darwin_jit_code_pages_kludge () {
             }
         }
     }
-    THREAD_JIT(1);
+    THREAD_JIT_WP(1);
 }
 #endif
 
 /* Read corefile ptes from 'fd' which has already been positioned
  * and store into the page table */
 void gc_load_corefile_ptes(int card_table_nbits,
-                           core_entry_elt_t n_ptes, core_entry_elt_t total_bytes,
+                           core_entry_elt_t n_ptes,
+                           __attribute__((unused)) core_entry_elt_t total_bytes,
                            os_vm_offset_t offset, int fd,
                            __attribute__((unused)) struct coreparse_space *spaces,
                            struct heap_adjust *adj)
 {
-    gc_assert(ALIGN_UP(n_ptes * sizeof (struct corefile_pte), N_WORD_BYTES)
-              == (size_t)total_bytes);
     if (next_free_page != n_ptes)
         lose("n_PTEs=%"PAGE_INDEX_FMT" but expected %"PAGE_INDEX_FMT,
              n_ptes, next_free_page);
@@ -1130,6 +1112,15 @@ void gc_load_corefile_ptes(int card_table_nbits,
 
     if (LSEEK(fd, offset, SEEK_SET) != offset) lose("failed seek");
 
+#ifdef LISP_FEATURE_MARK_REGION_GC
+    sword_t bitmapsize = bitmap_size(n_ptes);
+    if (read(fd, allocation_bitmap, bitmapsize) != bitmapsize)
+        lose("failed to read allocation bitmap from core");
+    total_bytes -= bitmapsize;
+#endif
+    gc_assert(ALIGN_UP(n_ptes * sizeof (struct corefile_pte), N_WORD_BYTES)
+              == (size_t)total_bytes);
+
     char data[8192];
     // Process an integral number of ptes on each read.
     // Parentheses around sizeof (type) are necessary to suppress a
@@ -1156,12 +1147,15 @@ void gc_load_corefile_ptes(int card_table_nbits,
             pte.words_used &= ~1;
             /* It is possible, though rare, for the saved page table
              * to contain free pages below alloc_ptr. */
-            if (type != FREE_PAGE_FLAG) {
-                gc_assert(pte.words_used);
-                page_table[page].words_used_ = pte.words_used;
-                set_page_scan_start_offset(page, pte.sso & ~0x07);
-                page_table[page].gen = gen;
-            }
+            if (type == FREE_PAGE_FLAG) continue;
+            gc_assert(pte.words_used);
+            page_table[page].words_used_ = pte.words_used;
+            set_page_scan_start_offset(page, pte.sso & ~0x07);
+            page_table[page].gen = gen;
+#ifdef LISP_FEATURE_MARK_REGION_GC
+            if (!page_single_obj_p(page))
+                for_lines_in_page(l, page) line_bytemap[l] = ENCODE_GEN(gen);
+#endif
             bytes_allocated += pte.words_used << WORD_SHIFT;
         }
     }
@@ -1170,9 +1164,6 @@ void gc_load_corefile_ptes(int card_table_nbits,
 
     // Adjust for discrepancies between actually-allocated space addresses
     // and desired addresses.
-#ifdef LISP_FEATURE_MARK_REGION_GC
-    load_corefile_bitmaps(fd, n_ptes);
-#endif
     if (adj->n_ranges) relocate_heap(adj);
 
 #ifdef LISP_FEATURE_IMMOBILE_SPACE
@@ -1197,41 +1188,54 @@ void gc_load_corefile_ptes(int card_table_nbits,
         gengcbarrier_patch_code_range(READ_ONLY_SPACE_START, read_only_space_free_pointer);
         gengcbarrier_patch_code_range(STATIC_SPACE_START, static_space_free_pointer);
         gengcbarrier_patch_code_range(DYNAMIC_SPACE_START, (lispobj*)dynamic_space_highwatermark());
-#ifdef LISP_FEATURE_IMMOBILE_SPACE
         gengcbarrier_patch_code_range(TEXT_SPACE_START, text_space_highwatermark);
-#endif
     }
 
-    // Apply physical page protection as needed.
-    // The non-soft-card-mark code is disgusting and I do not understand it.
+    // Toggle page protection bits as needed. There are essentially two major cases:
+    // - when soft card marking is used, all cards are "clean" but since that's
+    //   not the default state of the mark bit, we have to set it.
+    //   Additionally, with darwin-jit the mapping type on on pages of code
+    //   needs to change from 'rw-' to 'rwx'
+    // - when MMU-based marking is used, then all pages except code get
+    //   mprotected; but code pages writes are tracked with soft marking.
+
     if (gen != 0 && ENABLE_PAGE_PROTECTION) {
 #ifdef LISP_FEATURE_SOFT_CARD_MARKS
         page_index_t p;
         for (p = 0; p < next_free_page; ++p)
             if (page_words_used(p)) assign_page_card_marks(p, CARD_UNMARKED);
-#else
-        // coreparse can avoid hundreds to thousands of mprotect() calls by
-        // treating the whole range from the corefile as protectable, except
-        // that soft-marked code pages must NOT be subject to mprotect.
-        // So just watch out for empty pages and code.  Unboxed object pages
-        // will get unprotected on demand.
-#define non_protectable_page_p(x) !page_words_used(x) || is_code(page_table[x].type)
-        page_index_t start = 0, end;
-        // cf. write_protect_generation_pages()
-        while (start  < next_free_page) {
+
 #ifdef LISP_FEATURE_DARWIN_JIT
-            if(is_code(page_table[start].type)) {
-              SET_PAGE_PROTECTED(start,1);
+        page_index_t start = 0, end;
+        while (start  < next_free_page) {
+            if (is_code(page_table[start].type)) {
+                set_page_executable(start, 1);
                 for (end = start + 1; end < next_free_page; end++) {
                     if (!page_words_used(end) || !is_code(page_table[end].type))
                         break;
-                    SET_PAGE_PROTECTED(end,1);
+                    set_page_executable(end, 1);
                 }
                 os_protect(page_address(start), npage_bytes(end - start), OS_VM_PROT_ALL);
                 start = end+1;
                 continue;
             }
+            ++start;
+        }
 #endif
+
+#else
+        // coreparse can avoid hundreds to thousands of mprotect() calls by
+        // treating the whole range from the corefile as protectable, except
+        // that code pages must NOT be subject to mprotect.
+        // So just watch out for empty pages and code.
+        // Unboxed pages do not technically need to be mprotected since we don't
+        // need to track writes, but they are lumped in with other pages for simplicity
+        // of this loop. Optimistically assume that doing so won't cause too many
+        // extra page faults if the unboxed pages do get written.
+#define non_protectable_page_p(x) !page_words_used(x) || is_code(page_table[x].type)
+        page_index_t start = 0, end;
+        // cf. write_protect_generation_pages()
+        while (start  < next_free_page) {
             if (non_protectable_page_p(start)) {
                 ++start;
                 continue;
@@ -1242,7 +1246,7 @@ void gc_load_corefile_ptes(int card_table_nbits,
                     break;
                 SET_PAGE_PROTECTED(end,1);
             }
-            os_protect(page_address(start), npage_bytes(end - start), OS_VM_PROT_JIT_READ);
+            os_protect(page_address(start), npage_bytes(end - start), OS_VM_PROT_READ);
             start = end;
         }
 #endif
@@ -1257,6 +1261,21 @@ void gc_load_corefile_ptes(int card_table_nbits,
 #endif
 }
 
+static struct coreparse_space*
+init_coreparse_spaces(int n, struct coreparse_space* input)
+{
+    // Indexing of spaces[] by the space ID should conveniently just work,
+    // so we have to leave an empty row for space ID 0 which doesn't exist.
+    struct coreparse_space* output =
+      successful_malloc(sizeof (struct coreparse_space)*(MAX_CORE_SPACE_ID+1));
+    int i;
+    for (i=0; i<n; ++i) {
+        int id = input[i].id;
+        memcpy(&output[id], &input[i], sizeof (struct coreparse_space));
+    }
+    return output;
+}
+
 /* 'merge_core_pages': Tri-state flag to determine whether we attempt to mark
  * pages as targets for virtual memory deduplication via MADV_MERGEABLE.
  * 1: Yes
@@ -1296,24 +1315,29 @@ load_core_file(char *file, os_vm_offset_t file_offset, int merge_core_pages)
         lose("invalid magic number in core: %"OBJ_FMTX" should have been %x",
              (lispobj)val, CORE_MAGIC);
 
-    struct coreparse_space spaces[MAX_CORE_SPACE_ID+1] = {
-        {0, 0, 0, 0}, // blank for space ID 0
-        {dynamic_space_size, 0, DYNAMIC_SPACE_START, 0},
-        // This order is determined by constants in compiler/generic/genesis
-        {0, 0, STATIC_SPACE_START, &static_space_free_pointer},
-
-        {0, 0, READ_ONLY_SPACE_START, &read_only_space_free_pointer},
-
+    struct coreparse_space defined_spaces[] = {
+        {READ_ONLY_CORE_SPACE_ID, 0, 0, READ_ONLY_SPACE_START, &read_only_space_free_pointer},
+        {STATIC_CORE_SPACE_ID, 0, 0, STATIC_SPACE_START, &static_space_free_pointer},
 #ifdef LISP_FEATURE_DARWIN_JIT
-        {0, 0, STATIC_CODE_SPACE_START, &static_code_space_free_pointer},
+        {STATIC_CODE_CORE_SPACE_ID, 0, 0, STATIC_CODE_SPACE_START, &static_code_space_free_pointer},
 #endif
-
 #ifdef LISP_FEATURE_IMMOBILE_SPACE
-        {FIXEDOBJ_SPACE_SIZE | 1, 0,
-            FIXEDOBJ_SPACE_START, &fixedobj_free_pointer},
-        {1, 0, TEXT_SPACE_START, &text_space_highwatermark}
+        {IMMOBILE_FIXEDOBJ_CORE_SPACE_ID, FIXEDOBJ_SPACE_SIZE | 1, 0,
+            0, &fixedobj_free_pointer},
+        {IMMOBILE_TEXT_CORE_SPACE_ID, 1, 0, 0, &text_space_highwatermark},
+#else
+        // Without the immobile-space feature, immobile-text-space is nonetheless valid.
+        // editcore can put all code there, and then convert the space to an ELF section.
+        // Unlike static-code-core-space, this space _is_ a root for GC.
+        {IMMOBILE_TEXT_CORE_SPACE_ID, 0, 0, 0, &text_space_highwatermark},
 #endif
+        {DYNAMIC_CORE_SPACE_ID, dynamic_space_size, 0, 0, 0}
     };
+    // The initializer ensures that indexing into spaces[] is insensitive
+    // to the space numbering and the order listed in defined_spaces.
+    struct coreparse_space* spaces =
+      init_coreparse_spaces(sizeof defined_spaces/sizeof (struct coreparse_space),
+                            defined_spaces);
 
     for ( ; ; ptr += remaining_len) {
         val = *ptr++;
@@ -1361,6 +1385,7 @@ load_core_file(char *file, os_vm_offset_t file_offset, int merge_core_pages)
                 print_generation_stats();
             }
             sanity_check_loaded_core(initial_function);
+            free(spaces);
             return initial_function;
         case RUNTIME_OPTIONS_MAGIC: break; // already processed
         default:
@@ -1575,11 +1600,11 @@ static void tally(lispobj ptr, struct visitor* v)
  * to it from another object that is definitely in the cold core, via FOP-LOAD-CODE
  * and who know what else. Thus it remains a graph tracing problem in nature,
  * which is best left to GC */
-static uword_t visit(lispobj* where, lispobj* limit, uword_t arg)
+static uword_t visit_range(lispobj* where, lispobj* limit, uword_t arg)
 {
     struct visitor* v = (struct visitor*)arg;
-    lispobj* obj = where;
-    while (obj < limit) {
+    lispobj* obj = next_object(where, 0, limit);
+    while (obj) {
         if (widetag_of(obj) == FILLER_WIDETAG) {
             obj += object_size(obj);
             continue;
@@ -1587,7 +1612,7 @@ static uword_t visit(lispobj* where, lispobj* limit, uword_t arg)
         lispobj ptr = compute_lispobj(obj);
         tally(ptr, v);
         if (!hopscotch_get(v->reached, ptr, 0)) printf("unreachable: %p\n", (void*)ptr);
-        obj += object_size(obj);
+        obj = next_object(obj, object_size(obj), limit);
     }
     return 0;
 }
@@ -1612,7 +1637,7 @@ static void sanity_check_loaded_core(lispobj initial_function)
     free(c);
     // Pass 2: Count all heap objects
     v[1].reached = &reached;
-    walk_generation(visit, -1, (uword_t)&v[1]);
+    walk_generation(visit_range, -1, (uword_t)&v[1]);
 
     // Pass 3: Compare
     // Start with the conses
@@ -1635,7 +1660,7 @@ static void sanity_check_loaded_core(lispobj initial_function)
                    v[0].headers[i].count, v[0].headers[i].words,
                    v[1].headers[i].count, v[1].headers[i].words,
                    i<64 ? (i ? widetag_names[i] : "cons") : "TOTAL",
-                   mismatch ? " <<<<" : "");
+                   mismatch ? " ****" : "");
             if (i == SIMPLE_VECTOR_WIDETAG>>2) {
                 int j;
                 for(j=1; j <= 2; ++j)
