@@ -49,14 +49,9 @@ static void* get_free_page() {
     --free_page;
     if (free_page < next_free_page)
         lose("Needed more space to GC");
-    page_table[free_page].type = PAGE_TYPE_UNBOXED;
+    set_page_type(page_table[free_page], PAGE_TYPE_UNBOXED);
     char* mem = page_address(free_page);
-#ifdef LISP_FEATURE_DARWIN_JIT
-    // Might need to alter MMU-based protection
-    zeroize_pages_if_needed(free_page, free_page, 0);
-#elif defined LISP_FEATURE_WIN32
-    os_commit_memory(page_address(free_page), GENCGC_PAGE_BYTES);
-#endif
+    prepare_pages(1, free_page, free_page, PAGE_TYPE_UNBOXED, -1);
     return mem;
 }
 
@@ -272,6 +267,15 @@ void prepare_for_full_mark_phase()
     fullcgcmarks = (void*)os_allocate(markbits_size);
 }
 
+void scav_static_range(lispobj* where, lispobj* end)
+{
+    while (where < end) {
+        lispobj obj = compute_lispobj(where);
+        gc_enqueue(obj);
+        where += listp(obj) ? 2 : headerobj_size(where);
+    }
+}
+
 void execute_full_mark_phase()
 {
 #ifdef HAVE_GETRUSAGE
@@ -279,13 +283,11 @@ void execute_full_mark_phase()
     getrusage(RUSAGE_SELF, &before);
 #endif
     trace_object((lispobj*)NIL_SYMBOL_SLOTS_START);
-    lispobj* where = (lispobj*)STATIC_SPACE_OBJECTS_START;
-    lispobj* end = static_space_free_pointer;
-    while (where < end) {
-        lispobj obj = compute_lispobj(where);
-        gc_enqueue(obj);
-        where += listp(obj) ? 2 : headerobj_size(where);
-    }
+    scav_static_range((lispobj*)STATIC_SPACE_OBJECTS_START, static_space_free_pointer);
+#ifndef LISP_FEATURE_IMMOBILE_SPACE
+    // if NO immobile-space, then text space is equivalent to static space
+    scav_static_range((lispobj*)TEXT_SPACE_START, text_space_highwatermark);
+#endif
     gc_mark_obj(lisp_package_vector);
     gc_mark_obj(lisp_init_function);
     gc_mark_obj(alloc_profile_data);
@@ -449,7 +451,7 @@ void dispose_markbits() {
         gc_assert((page_table[page].type & PAGE_TYPE_MASK) == PAGE_TYPE_UNBOXED);
         gc_assert(!page_bytes_used(page));
         set_page_need_to_zero(page, 1);
-        page_table[page].type = FREE_PAGE_FLAG;
+        set_page_type(page_table[page], FREE_PAGE_FLAG);
       }
 }
 
