@@ -16,7 +16,7 @@
 #ifndef _GC_H_
 #define _GC_H_
 
-#include "sbcl.h"
+#include "genesis/sbcl.h"
 #include "gc-assert.h"
 #include "gc-typedefs.h"
 #include <stdbool.h>
@@ -129,4 +129,30 @@ extern lispobj *component_ptr_from_pc(char *pc);
 #define MEASURE_STOP_THE_WORLD_PAUSE
 #endif
 
+#ifdef LISP_FEATURE_X86_64
+#define GC_SAFEPOINT_PAGE_ADDR (void*)(gc_card_mark-BACKEND_PAGE_BYTES)
+#define GC_SAFEPOINT_TRAP_ADDR (void*)(gc_card_mark-8)
+#endif
+void remap_free_pages(page_index_t,page_index_t);
+void page_remap_as_type(int,void*,sword_t);
+extern lispobj *dynamic_space_code_from_pc(char *pc);
+
+#ifdef TRACE_MMAP_SYSCALLS
+extern void set_page_type_impl(struct page*, int newval);
+#define set_page_type(pte, newval) set_page_type_impl(&(pte), newval)
+#else
+#define set_page_type(pte, newval) pte.type = newval
+#endif
+extern void prepare_pages(bool commit, page_index_t start, page_index_t end,
+                          int page_type, generation_index_t);
+
+/* One byte per page indicating data vs code, but only if the OS generally
+ * prohibits memory from being writable + executable. There's probably a way to
+ * squeeze a bit into the 'type' field of the page table, but it's clearer to
+ * have this externally so that page type 0 remains as "free" */
+#ifdef LISP_FEATURE_DARWIN_JIT
+_Atomic(char) *page_execp;
+static inline void set_page_executable(page_index_t i, bool val) { page_execp[i] = val; }
+#endif
+
 #endif /* _GC_H_ */
