@@ -20,7 +20,7 @@
 # include "os.h"
 #endif
 
-#include "sbcl.h"
+#include "genesis/sbcl.h"
 
 #ifndef __ASSEMBLER__
 
@@ -55,19 +55,17 @@ extern os_vm_size_t thread_control_stack_size;
 extern uword_t STATIC_SPACE_START, STATIC_SPACE_END;
 #endif
 
-#ifndef LISP_FEATURE_DARWIN_JIT
 extern uword_t READ_ONLY_SPACE_START, READ_ONLY_SPACE_END;
-#endif
 #ifdef LISP_FEATURE_CHENEYGC
 extern uword_t DYNAMIC_0_SPACE_START, DYNAMIC_1_SPACE_START;
 #else
 extern uword_t DYNAMIC_SPACE_START;
 #endif
-#ifdef LISP_FEATURE_IMMOBILE_SPACE
 extern uword_t FIXEDOBJ_SPACE_START, TEXT_SPACE_START;
+extern unsigned int text_space_size;
+#ifdef LISP_FEATURE_IMMOBILE_SPACE
 extern uword_t immobile_space_lower_bound, immobile_space_max_offset;
 extern uword_t immobile_range_1_max_offset, immobile_range_2_min_offset;
-extern unsigned int text_space_size;
 #endif
 extern uword_t asm_routines_start, asm_routines_end;
 extern int gc_card_table_nbits;
@@ -97,27 +95,22 @@ extern lispobj *static_space_free_pointer;
 static inline bool readonly_space_p(lispobj ptr) {
     return ptr >= READ_ONLY_SPACE_START && (lispobj*)ptr < read_only_space_free_pointer;
 }
+static inline bool is_in_static_space(void* ptr) {
+    return (uword_t)ptr >= STATIC_SPACE_START && (lispobj*)ptr < static_space_free_pointer;
+}
 
 #ifdef LISP_FEATURE_DARWIN_JIT
 extern lispobj *static_code_space_free_pointer;
 #endif
 
-#ifdef LISP_FEATURE_IMMOBILE_SPACE
 extern lispobj *text_space_highwatermark;
+#ifdef LISP_FEATURE_IMMOBILE_SPACE
 extern lispobj *fixedobj_free_pointer;
 extern lispobj ALIEN_LINKAGE_TABLE_SPACE_START;
 #endif
 extern os_vm_address_t anon_dynamic_space_start;
 extern lispobj* tlsf_mem_start; // meaningful only if immobile space
 
-# ifndef LISP_FEATURE_GENCGC
-extern lispobj *current_auto_gc_trigger;
-# endif
-
-#ifdef LISP_FEATURE_CHENEYGC
-extern lispobj *current_dynamic_space;
-#endif
-
 extern lispobj lisp_package_vector;
 
 extern void globals_init(void);
