@@ -226,6 +226,8 @@ bool try_allocate_small_after_region(sword_t nbytes, struct alloc_region *region
   return try_allocate_small(nbytes, region, address_line(region->end_addr), end);
 }
 
+extern generation_index_t get_alloc_generation();
+
 /* try_allocate_small_from_pages updates the start pointer to after the
  * claimed page. */
 bool try_allocate_small_from_pages(sword_t nbytes, struct alloc_region *region,
@@ -242,7 +244,12 @@ bool try_allocate_small_from_pages(sword_t nbytes, struct alloc_region *region,
         try_allocate_small(nbytes, region,
                            address_line(page_address(where)),
                            address_line(page_address(where + 1)))) {
-      page_table[where].type = page_type | OPEN_REGION_PAGE_FLAG;
+      // mark-region has a different way of zeroing, so just tell prepare_pages
+      // that the page is unboxed if it's boxed, so that it doesn't try to zero.
+      if (!page_table[where].type)
+          prepare_pages(1, where, where, page_type==PAGE_TYPE_CODE?page_type:0,
+                        get_alloc_generation());
+      set_page_type(page_table[where], page_type | OPEN_REGION_PAGE_FLAG);
       page_table[where].gen = 0;
       set_page_scan_start_offset(where, 0);
       start->page = where + 1;
@@ -281,12 +288,20 @@ page_index_t try_allocate_large(uword_t nbytes,
   while (1) {
     page_index_t chunk_start = find_free_page(where, end);
     if (chunk_start == -1) return -1;
+    /* TODO: this is suboptimal - the full extent of the free space is irrelevant
+     * as long as it's at _least_ pages_needed. So find_used_page is a poor choice
+     * of algorithm for this. It's not wrong, though I've seen it say (via added
+     * printing) that it needed 2 pages but found 40000 pages. So it scanned
+     * the entire page table before deciding yup, we have enough to work with..
+     */
     page_index_t chunk_end = find_used_page(chunk_start, end);
     uword_t hole_size = chunk_end - chunk_start;
     if (hole_size >= pages_needed) {
       page_index_t last_page = chunk_start + pages_needed - 1;
+      prepare_pages(1, chunk_start, last_page, page_type==PAGE_TYPE_CODE?page_type:0,
+                    get_alloc_generation());
       for (page_index_t p = chunk_start; p <= last_page; p++) {
-        page_table[p].type = SINGLE_OBJECT_FLAG | page_type;
+        set_page_type(page_table[p], SINGLE_OBJECT_FLAG | page_type);
         page_table[p].gen = gen;
         set_page_bytes_used(p,
                             (p == last_page && remainder > 0) ? remainder
@@ -863,7 +878,8 @@ static void __attribute__((noinline)) sweep_pages() {
         allocation_bitmap[mark_bitmap_word_index(page_address(p))] = 0;
       /* Why is reset_page_flags(p) much slower here? It does other stuff
        * for gencgc, sure, but not that much more stuff. */
-      page_table[p].type = FREE_PAGE_FLAG;
+      set_page_need_to_zero(p, 1);
+      set_page_type(page_table[p], FREE_PAGE_FLAG);
       page_table[p].scan_start_offset_ = 0;
     } else {
       bytes_allocated += page_bytes_used(p);
@@ -962,12 +978,6 @@ void mr_preserve_leaf(lispobj obj) {
   }
 }
 
-/* Trace through an object outside dynamic space.
- * Used by immobile space. */
-void mr_trace_object(lispobj *obj) {
-  trace_object(compute_lispobj(obj));
-}
-
 /* Preserve an exact pointer, trace and pin it.
  * Used by pin_exact_root. */
 void mr_preserve_object(lispobj obj) {
@@ -1161,14 +1171,6 @@ static void CPU_SPLIT raise_survivors(void) {
 /* Main entrypoints into GC */
 
 void mrgc_init() {
-#ifndef LISP_FEATURE_GCC_TLS
-  int ok = !pthread_key_create(&recycle_list_key, 0) &&
-    !pthread_key_create(&output_block_key, 0) &&
-    !pthread_key_create(&dirty_generation_source_key, 0) &&
-    !pthread_key_create(&dirty_key, 0) &&
-    !pthread_key_create(&source_object_key, 0);
-  gc_assert(ok);
-#endif
   allocate_bitmaps();
   thread_pool_init();
   compactor_init();
@@ -1221,7 +1223,7 @@ void mr_collect_garbage(bool raise) {
           dirty_root_objects, root_objects_checked,
           next_free_page, raise ? ", raised" : "");
 #endif
-  if (gencgc_verbose) mr_print_meters();
+  if (gencgc_verbose>1) mr_print_meters();
   reset_alloc_start_pages(false);
   reset_pinned_pages();
 }
