@@ -16,7 +16,7 @@
 /* Use AVX2 versions of code when we can, since blasting bytes faster
  * is always nice.
  * If used more widely, we should put these in runtime.h */
-#ifdef __linux__
+#if defined(__linux__) && defined(LISP_FEATURE_X86_64)
 #define CPU_SPLIT __attribute__((target_clones("default,avx2")))
 /* clang on linux seems to demand that the prototype of a multi-versioned function
  * have the same attribute as the definition. If omitted, then link errors occur such as
@@ -565,8 +565,6 @@ static inline os_vm_size_t npage_bytes(page_index_t npages)
 }
 extern os_vm_size_t auto_gc_trigger;
 
-void zeroize_pages_if_needed(page_index_t start, page_index_t end, int page_type);
-
 typedef unsigned int page_bytes_t;
 #define page_words_used(index) page_table[index].words_used_
 #define page_bytes_used(index) ((page_bytes_t)page_table[index].words_used_<<WORD_SHIFT)
