@@ -11,44 +11,22 @@
  * files for more information.
  */
 
-/*
- * FIXME:
- *   Some of the code in here is deeply broken, depending on guessing
- *   already out-of-date values instead of getting them from sbcl.h.
- */
-
 #include <stdio.h>
 #include <string.h>
 
-#include "sbcl.h"
+#include "genesis/sbcl.h"
 #include "print.h"
 #include "runtime.h"
 #include "code.h"
-#include "gc-internal.h"
-#include "gc-private.h"
+#include "gc.h"
 #include "genesis/gc-tables.h"
-#include <stdarg.h>
-#include "thread.h"              /* genesis/primitive-objects.h needs this */
+#include "thread.h"
 #include <errno.h>
 #include <stdlib.h>
 #include <inttypes.h>
 #include <setjmp.h>
 
-/* FSHOW and odxprint provide debugging output for low-level information
- * (signal handling, exceptions, safepoints) which is hard to debug by
- * other means.
- *
- * If enabled at all, environment variables control whether calls of the
- * form odxprint(name, ...) are enabled at run-time, e.g. using
- * SBCL_DYNDEBUG="fshow safepoints".
- *
- * In the case of FSHOW, old-style code from runtime.h
- * can also be used to enable or disable these more aggressively.
- */
-
-struct dyndebug_config dyndebug_config = {
-    QSHOW == 2,
-};
+struct dyndebug_config dyndebug_config;
 
 void
 dyndebug_init()
@@ -67,7 +45,6 @@ dyndebug_init()
     char *names[DYNDEBUG_NFLAGS];
     int *ptrs[DYNDEBUG_NFLAGS];
 
-    dyndebug_init1(fshow,          "FSHOW");
     dyndebug_init1(gencgc_verbose, "GENCGC_VERBOSE");
     dyndebug_init1(safepoints,     "SAFEPOINTS");
     dyndebug_init1(seh,            "SEH");
@@ -120,7 +97,7 @@ dyndebug_init()
             }
         }
     }
-#if defined(LISP_FEATURE_GENCGC)
+#if defined(LISP_FEATURE_GENERATIONAL)
     if (dyndebug_config.dyndebug_gencgc_verbose) {
         gencgc_verbose = 1;
     }
@@ -130,87 +107,8 @@ dyndebug_init()
 #undef DYNDEBUG_NFLAGS
 }
 
-/* Temporarily, odxprint merely performs the equivalent of a traditional
- * FSHOW call, i.e. it merely formats to stderr.  Ultimately, it should
- * be restored to its full win32 branch functionality, where output to a
- * file or to the debugger can be selected at runtime. */
-
-void vodxprint_fun(const char *, va_list);
-
-void
-odxprint_fun(const char *fmt, ...)
-{
-    va_list args;
-    va_start(args, fmt);
-    vodxprint_fun(fmt, args);
-    va_end(args);
-}
-
-void
-vodxprint_fun(const char *fmt, va_list args)
-{
-#ifdef LISP_FEATURE_WIN32
-    DWORD lastError = GetLastError();
-#endif
-    int original_errno = errno;
-
-    char buf[1024];
-    int n = 0;
-
-#ifdef LISP_FEATURE_SB_THREAD
-    snprintf(buf, sizeof(buf), "["THREAD_ID_LABEL"] ", THREAD_ID_VALUE);
-    n = strlen(buf);
-#endif
-
-    vsnprintf(buf + n, sizeof(buf) - n - 1, fmt, args);
-    /* buf is now zero-terminated (even in case of overflow).
-     * Our caller took care of the newline (if any) through `fmt'. */
-
-    /* A sufficiently POSIXy implementation of stdio will provide
-     * per-FILE locking, as defined in the spec for flockfile.  At least
-     * glibc complies with this.  Hence we do not need to perform
-     * locking ourselves here.  (Should it turn out, of course, that
-     * other libraries opt for speed rather than safety, we need to
-     * revisit this decision.) */
-    fputs(buf, stderr);
-
-#ifdef LISP_FEATURE_WIN32
-    /* stdio's stderr is line-bufferred, i.e. \n ought to flush it.
-     * Unfortunately, MinGW does not behave the way I would expect it
-     * to.  Let's be safe: */
-    fflush(stderr);
-#endif
-
-#ifdef LISP_FEATURE_WIN32
-    SetLastError(lastError);
-#endif
-    errno = original_errno;
-}
-
-/* Translate the rather awkward syntax
- *   FSHOW((stderr, "xyz"))
- * into the new and cleaner
- *   odxprint("xyz").
- * If we were willing to clean up all existing call sites, we could remove
- * this wrapper function.  (This is a function, because I don't know how to
- * strip the extra parens in a macro.) */
-void
-fshow_fun(void __attribute__((__unused__)) *ignored,
-          const char *fmt,
-          ...)
-{
-    va_list args;
-    va_start(args, fmt);
-    vodxprint_fun(fmt, args);
-    va_end(args);
-}
-
-#include "monitor.h"
 #include "vars.h"
 #include "os.h"
-#ifdef LISP_FEATURE_GENCGC
-#include "gencgc-alloc-region.h" /* genesis/thread.h needs this */
-#endif
 #include "genesis/static-symbols.h"
 #include "genesis/primitive-objects.h"
 #include "genesis/static-symbols.h"
@@ -219,7 +117,7 @@ fshow_fun(void __attribute__((__unused__)) *ignored,
 static int max_lines = 20, cur_lines = 0;
 static int max_depth = 5, brief_depth = 2, cur_depth = 0;
 static int max_length = 5;
-static boolean dont_descend = 0, skip_newline = 0;
+static bool dont_descend = 0, skip_newline = 0;
 static int cur_clock = 0;
 
 static void print_obj(char *prefix, lispobj obj);
@@ -239,7 +137,7 @@ static void indent(int in)
 }
 
 static jmp_buf ldb_print_nlx;
-static boolean continue_p(boolean newline)
+static bool continue_p(bool newline)
 {
     char buffer[256];
 
@@ -452,21 +350,13 @@ static void brief_struct(lispobj obj)
     }
 }
 
-#include "genesis/layout.h"
 #include "genesis/defstruct-description.h"
-#include "genesis/defstruct-slot-description.h"
-static boolean tagged_slot_p(struct layout *layout, int slot_index)
+static bool tagged_slot_p(struct layout *layout, int slot_index)
 {
     // Since we're doing this scan, we could return the name
     // and exact raw type.
-#ifdef LISP_FEATURE_METASPACE
-    struct wrapper *wrapper = (void*)(layout->friend-INSTANCE_POINTER_LOWTAG);
-    if (instancep(wrapper->_info)) {
-        struct defstruct_description* dd = (void*)(wrapper->_info-INSTANCE_POINTER_LOWTAG);
-#else
     if (instancep(layout->_info)) {
         struct defstruct_description* dd = (void*)(layout->_info-INSTANCE_POINTER_LOWTAG);
-#endif
         lispobj slots = dd->slots;
         for ( ; slots != NIL ; slots = CONS(slots)->cdr ) {
             struct defstruct_slot_description* dsd =
@@ -475,7 +365,15 @@ static boolean tagged_slot_p(struct layout *layout, int slot_index)
                 return (fixnum_value(dsd->bits) & DSD_RAW_TYPE_MASK) == 0;
         }
     }
-    return 0;
+    /* Revision 2b783b49 said to prefer LAYOUT-INFO vs BITMAP because the bitmap
+     * can indicate a 0 bit ("raw") for any slot that _may_ be ignored by GC, such as
+     * slots constrained to FIXNUM. Unfortunately that misses that CONDITION instances
+     * have trailing variable-length tagged data. In practice an instance may have raw
+     * words only if it has a DD, which most CONDITION subtypes do not. Therefore this
+     * could almost always return 1. But layout-of-layout is an important use of trailing
+     * raw slots. Attempting to print random words as tagged could be disastrous.
+     * Therefore, test the bitmap if the above loop failed to find slot_index. */
+    return bitmap_logbitp(slot_index, get_layout_bitmap(layout));
 }
 
 static void print_struct(lispobj obj)
@@ -812,7 +710,7 @@ static void print_fun_or_otherptr(lispobj obj)
 
     case FDEFN_WIDETAG:
         print_slots(fdefn_slots, 2, ptr);
-        print_obj("entry: ", fdefn_callee_lispobj((struct fdefn*)(ptr-1)));
+        print_obj("entry: ", decode_fdefn_rawfun((struct fdefn*)(ptr-1)));
         break;
 
     // Make some vectors printable from C, for when all hell breaks lose
@@ -846,7 +744,7 @@ static void print_obj(char *prefix, lispobj obj)
     int type = lowtag_of(obj);
     struct var *var = lookup_by_obj(obj);
     char buffer[256];
-    boolean verbose = cur_depth < brief_depth;
+    bool verbose = cur_depth < brief_depth;
 
     if (!continue_p(verbose))
         return;
@@ -932,8 +830,6 @@ void brief_print(lispobj obj)
 // and return a Lisp string, are designed to be foolproof during GC,
 // hence all the forwarding checks.
 
-#include "forwarding-ptr.h"
-#include "genesis/classoid.h"
 struct vector * symbol_name(struct symbol* sym)
 {
   if (forwarding_pointer_p((lispobj*)sym))
@@ -941,7 +837,7 @@ struct vector * symbol_name(struct symbol* sym)
   lispobj name = sym->name;
   if (lowtag_of(name) != OTHER_POINTER_LOWTAG) return NULL;
   lispobj string = decode_symbol_name(name);
-  return VECTOR(follow_maybe_fp(string));
+  return VECTOR(follow_fp(string)); // can't have a nameless symbol
 }
 struct vector * classoid_name(lispobj * classoid)
 {
@@ -953,17 +849,9 @@ struct vector * classoid_name(lispobj * classoid)
 }
 struct vector * layout_classoid_name(lispobj * layout)
 {
-#ifdef LISP_FEATURE_METASPACE
-  // layout can't be forwarded, but wrapper could be
-  lispobj* wrapper = native_pointer(((struct layout*)layout)->friend);
-  if (forwarding_pointer_p(wrapper))
-      wrapper = native_pointer(forwarding_pointer_value(wrapper));
-  lispobj classoid = ((struct wrapper*)wrapper)->classoid;
-#else
   if (forwarding_pointer_p(layout))
       layout = native_pointer(forwarding_pointer_value(layout));
   lispobj classoid = ((struct layout*)layout)->classoid;
-#endif
   return instancep(classoid) ? classoid_name(native_pointer(classoid)) : NULL;
 }
 struct vector * instance_classoid_name(lispobj * instance)
