@@ -19,7 +19,7 @@
 #include <strings.h>
 #include <errno.h>
 
-#include "sbcl.h"
+#include "genesis/sbcl.h"
 #include "runtime.h"
 #include "os.h"
 #include "globals.h"
@@ -28,24 +28,21 @@
 #include "purify.h"
 #include "interr.h"
 #include "gc.h"
-#include "gc-internal.h"
-#include "gc-private.h"
 #include "thread.h"
 #include "genesis/gc-tables.h"
 #include "genesis/primitive-objects.h"
 #include "genesis/static-symbols.h"
-#include "genesis/layout.h"
 #include "genesis/defstruct-description.h"
 #include "genesis/hash-table.h"
 #include "code.h"
-#include "getallocptr.h"
+#include "pseudo-atomic.h"
 
 /* We don't ever do purification with GENCGC as of 1.0.5.*. There was
  * a lot of hairy and fragile ifdeffage in here to support purify on
  * x86oids, which has now been removed. So this code can't even be
  * compiled with GENCGC any more.  -- JES, 2007-04-30.
  */
-#ifndef LISP_FEATURE_GENCGC
+#ifdef LISP_FEATURE_CHENEYGC
 
 #define PRINTNOISE
 
@@ -75,8 +72,7 @@ later {
 static long later_count = 0;
 
 
-static boolean
-forwarding_pointer_p(lispobj obj)
+static boolean forwarded_p(lispobj obj)
 {
     lispobj *ptr = native_pointer(obj);
 
@@ -132,6 +128,10 @@ pscav_later(lispobj *where, long count)
     ++later_count;
 }
 
+//FILE *xlog;
+//#define XLOG(old,new) fprintf(xlog, "%x > %x\n", old, (uword_t)new)
+#define XLOG(dumm1,dummy2)
+
 static lispobj
 ptrans_boxed(lispobj thing, lispobj header, boolean constant)
 {
@@ -139,6 +139,7 @@ ptrans_boxed(lispobj thing, lispobj header, boolean constant)
     lispobj *old = native_pointer(thing);
     long nwords = sizetab[header_widetag(header)](old);
     lispobj *new = newspace_alloc(nwords,constant);
+    XLOG(thing, new);
 
     /* Copy it. */
     memcpy(new, old, nwords * sizeof(lispobj));
@@ -160,8 +161,8 @@ static lispobj
 ptrans_instance(lispobj thing, lispobj header, boolean /* ignored */ constant)
 {
     constant = 0;
-    lispobj info = LAYOUT(instance_layout(native_pointer(thing)))->info;
-    if (info != NIL) {
+    lispobj info = LAYOUT(instance_layout(native_pointer(thing)))->_info;
+    if (instancep(info)) {
         lispobj pure = ((struct defstruct_description*)native_pointer(info))->pure;
         if (pure != NIL && pure != T) {
             gc_abort();
@@ -179,6 +180,7 @@ ptrans_fdefn(lispobj thing, lispobj header)
     lispobj *old = native_pointer(thing);
     long nwords = sizetab[header_widetag(header)](old);
     lispobj *new = newspace_alloc(nwords, 0);    /* inconstant */
+    XLOG(thing, new);
 
     /* Copy it. */
     memcpy(new, old, nwords * sizeof(lispobj));
@@ -204,6 +206,7 @@ ptrans_unboxed(lispobj thing, lispobj header)
     lispobj *old = native_pointer(thing);
     long nwords = sizetab[header_widetag(header)](old);
     lispobj *new = newspace_alloc(nwords, 1);     /* always constant */
+    XLOG(thing, new);
 
     /* copy it. */
     memcpy(new, old, nwords * sizeof(lispobj));
@@ -222,6 +225,7 @@ ptrans_vector(lispobj thing, boolean boxed, boolean constant)
     long nwords = sizetab[header_widetag(vector->header)]((lispobj*)vector);
 
     lispobj *new = newspace_alloc(nwords, (constant || !boxed));
+    XLOG(thing, new);
     memcpy(new, vector, nwords * sizeof(lispobj));
 
     lispobj result = make_lispobj(new, lowtag_of(thing));
@@ -240,6 +244,7 @@ ptrans_code(lispobj thing)
     long nwords = code_total_nwords(code);
 
     struct code *new = (struct code *)newspace_alloc(nwords,1); /* constant */
+    XLOG(thing, new);
 
     memcpy(new, code, nwords * sizeof(lispobj));
 
@@ -259,7 +264,7 @@ ptrans_code(lispobj thing)
 #endif
     /* Put in forwarding pointers for all the functions. */
     for_each_simple_fun(i, newfunc, new, 1, {
-        lispobj* old = (lispobj*)LOW_WORD((char*)newfunc - displacement);
+        lispobj* old = (lispobj*)((char*)newfunc - displacement);
         *old = make_lispobj(newfunc, FUN_POINTER_LOWTAG);
         pscav(&newfunc->self, 1, 1); // and fix the self-pointer now
     });
@@ -314,6 +319,7 @@ ptrans_func(lispobj thing, lispobj header)
 
         lispobj *new = newspace_alloc
             (nwords,(header_widetag(header)!=FUNCALLABLE_INSTANCE_WIDETAG));
+        XLOG(thing, new);
 
         /* Copy it. */
         memcpy(new, old, nwords * sizeof(lispobj));
@@ -337,7 +343,7 @@ ptrans_returnpc(lispobj thing, lispobj header)
 
     /* Make sure it's been transported. */
     lispobj new = *native_pointer(code);
-    if (!forwarding_pointer_p(new))
+    if (!forwarded_p(new))
         new = ptrans_code(code);
 
     /* Maintain the offset: */
@@ -353,6 +359,7 @@ ptrans_list(lispobj thing, boolean constant)
     long length;
 
     orig = (struct cons *) newspace_alloc(0,constant);
+    //fprintf(xlog, "%x > %x", thing, (uword_t)orig);
     length = 0;
 
     do {
@@ -371,9 +378,10 @@ ptrans_list(lispobj thing, boolean constant)
         length++;
     } while (listp(thing) &&
              dynamic_pointer_p(thing) &&
-             !(forwarding_pointer_p(*native_pointer(thing))));
+             !(forwarded_p(*native_pointer(thing))));
 
     /* Scavenge the list we just copied. */
+    //fprintf(xlog, "*%d\n", (int)length);
     pscav((lispobj *)orig, length * WORDS_PER_CONS, constant);
 
     return make_lispobj(orig, LIST_POINTER_LOWTAG);
@@ -410,7 +418,6 @@ ptrans_otherptr(lispobj thing, lispobj header, boolean constant)
     case COMPLEX_CHARACTER_STRING_WIDETAG:
 #endif
       case COMPLEX_BIT_VECTOR_WIDETAG:
-      case COMPLEX_VECTOR_NIL_WIDETAG:
       case COMPLEX_VECTOR_WIDETAG:
       case COMPLEX_ARRAY_WIDETAG:
         return ptrans_boxed(thing, header, constant);
@@ -473,7 +480,7 @@ pscav(lispobj *addr, long nwords, boolean constant)
                 /* Maybe. Have we already moved it? */
                 thingp = native_pointer(thing);
                 header = *thingp;
-                if (is_lisp_pointer(header) && forwarding_pointer_p(header))
+                if (is_lisp_pointer(header) && forwarded_p(header))
                     /* Yep, so just copy the forwarding pointer. */
                     thing = header;
                 else {
@@ -529,7 +536,7 @@ pscav(lispobj *addr, long nwords, boolean constant)
                 //     [1] : vector length
                 //     [2] : element[0] = high-water mark
                 //     [3] : element[1] = rehash bit
-                if (is_vector_subtype(thing, VectorAddrHashing))
+                if (vector_flagp(thing, VectorAddrHashing))
                     addr[3] = make_fixnum(1); // just flag it for rehash
                 count = 2;
                 break;
@@ -549,7 +556,7 @@ pscav(lispobj *addr, long nwords, boolean constant)
                 /* Weak pointers get preserved during purify, 'cause I
                  * don't feel like figuring out how to break them. */
                 pscav(addr+1, 2, constant);
-                count = WEAK_POINTER_NWORDS;
+                count = ALIGN_UP(WEAK_POINTER_SIZE,2);
                 break;
 
               case FDEFN_WIDETAG:
@@ -560,23 +567,22 @@ pscav(lispobj *addr, long nwords, boolean constant)
 
               case INSTANCE_WIDETAG:
                 {
-                    lispobj lbitmap = LAYOUT(instance_layout(addr))->bitmap;
-                    lispobj* slots = addr + 1;
-                    long nslots = instance_length(*addr) | 1;
-                    int index;
-                    if (fixnump(lbitmap)) {
-                      sword_t bitmap = fixnum_value(lbitmap);
-                      for (index = 0; index < nslots ; index++, bitmap >>= 1)
-                        if (bitmap & 1)
-                          pscav(slots + index, 1, constant);
-                    } else {
-                      struct bignum * bitmap;
-                      bitmap = (struct bignum*)native_pointer(lbitmap);
-                      for (index = 0; index < nslots ; index++)
-                        if (positive_bignum_logbitp(index, bitmap))
-                          pscav(slots + index, 1, constant);
-                    }
-                    count = 1 + nslots;
+                lispobj layout = instance_layout(addr);
+                lispobj layout_header = *native_pointer(layout);
+                if (is_lisp_pointer(layout_header)) {
+                    gc_assert(forwarded_p(layout_header));
+                    layout = layout_header;
+                }
+                if (!layoutp(layout)) lose("Bad layout in instance: %p %x", addr, layout);
+                struct bitmap bitmap = get_layout_bitmap(LAYOUT(layout));
+                gc_assert(bitmap.nwords >= 1);
+                long nslots = instance_length(*addr);
+                int index;
+                for (index = 0; index < nslots ; index++)
+                    // logically treat index 0 (layout) as a tagged slot
+                    if (index == 0 || bitmap_logbitp(index, bitmap))
+                        pscav((addr+1) + index, 1, constant);
+                count = 1 + (nslots | 1);
                 }
                 break;
 
@@ -601,25 +607,7 @@ pscav(lispobj *addr, long nwords, boolean constant)
     return addr;
 }
 
-static void
-verify_range(lispobj *base, lispobj *end)
-{
-    lispobj* where = base;
-    while (where < end) {
-        if (widetag_of(where) == CODE_HEADER_WIDETAG) {
-            int n_boxed_words = code_header_words((struct code*)where);
-            int i;
-            for (i = 0; i < n_boxed_words; ++i) {
-                lispobj word = where[i];
-                if (is_lisp_pointer(word) && dynamic_pointer_p(word))
-                    lose("purify failed, obj=%p, where=%p, val=%"OBJ_FMTX,
-                         where, where+i, word);
-            }
-        }
-        where += OBJECT_SIZE(*where, where);
-    }
-}
-
+extern void dump_space_to_file(lispobj* where, lispobj* limit, char* pathname);
 int
 purify(lispobj static_roots, lispobj read_only_roots)
 {
@@ -634,6 +622,7 @@ purify(lispobj static_roots, lispobj read_only_roots)
         fflush(stderr);
         return 0;
     }
+    // verify_heap(0); fprintf(stderr, "pre-verify passed\n");
 
 #ifdef PRINTNOISE
     printf("[doing purification:");
@@ -667,9 +656,8 @@ purify(lispobj static_roots, lispobj read_only_roots)
     printf(" handlers");
     fflush(stdout);
 #endif
-    pscav((lispobj *) interrupt_handlers,
-          sizeof(interrupt_handlers) / sizeof(lispobj),
-          0);
+    pscav(lisp_sig_handlers, NSIG, 0);
+    pscav(&lisp_package_vector, 1, 0);
 
 #ifdef PRINTNOISE
     printf(" stack");
@@ -690,25 +678,6 @@ purify(lispobj static_roots, lispobj read_only_roots)
            all_threads->binding_stack_start,
           0);
 
-    /* The original CMU CL code had scavenge-read-only-space code
-     * controlled by the Lisp-level variable
-     * *SCAVENGE-READ-ONLY-SPACE*. It was disabled by default, and it
-     * wasn't documented under what circumstances it was useful or
-     * safe to turn it on, so it's been turned off in SBCL. If you
-     * want/need this functionality, and can test and document it,
-     * please submit a patch. */
-#if 0
-    if (SymbolValue(SCAVENGE_READ_ONLY_SPACE) != UNBOUND_MARKER_WIDETAG
-        && SymbolValue(SCAVENGE_READ_ONLY_SPACE) != NIL) {
-      unsigned  read_only_space_size =
-          read_only_space_free_pointer - (lispobj *)READ_ONLY_SPACE_START;
-      fprintf(stderr,
-              "scavenging read only space: %d bytes\n",
-              read_only_space_size * sizeof(lispobj));
-      pscav( (lispobj *)READ_ONLY_SPACE_START, read_only_space_size, 0);
-    }
-#endif
-
 #ifdef PRINTNOISE
     printf(" static");
     fflush(stdout);
@@ -735,9 +704,6 @@ purify(lispobj static_roots, lispobj read_only_roots)
     printf(" cleanup");
     fflush(stdout);
 #endif
-#ifdef LISP_FEATURE_HPUX
-    clear_auto_gc_trigger(); /* restore mmap as it was given by os */
-#endif
 
     os_zero((os_vm_address_t) current_dynamic_space, dynamic_space_size);
 
@@ -760,17 +726,18 @@ purify(lispobj static_roots, lispobj read_only_roots)
     os_flush_icache((os_vm_address_t)STATIC_SPACE_START, STATIC_SPACE_SIZE);
 
 #ifdef PRINTNOISE
-    verify_range((lispobj*)READ_ONLY_SPACE_START, read_only_space_free_pointer);
+    // verify_heap(1);
     printf(" done]\n");
     fflush(stdout);
 #endif
+    //fclose(xlog);
     return 0;
 }
-#else /* LISP_FEATURE_GENCGC */
+#else // dummy stub
 int
 purify(lispobj __attribute__((unused)) static_roots,
        lispobj __attribute__((unused)) read_only_roots)
 {
     lose("purify called for GENCGC. This should not happen.");
 }
-#endif /* LISP_FEATURE_GENCGC */
+#endif /* LISP_FEATURE_GENERATIONAL */
