@@ -229,10 +229,9 @@ open_core_for_saving(char *filename)
     return fopen(filename, "wb");
 }
 
-void unwind_binding_stack()
+static void unwind_binding_stack(struct thread* th)
 {
     bool verbose = !lisp_startup_options.noinform;
-    struct thread *th = all_threads;
 
     /* Smash the enclosing state. (Once we do this, there's no good
      * way to go back, which is a sufficient reason that this ends up
@@ -561,34 +560,6 @@ prepare_to_save(char *filename, bool prepend_runtime, void **runtime_bytes,
     return file;
 }
 
-#ifdef LISP_FEATURE_CHENEYGC
-bool save(char *filename, lispobj init_function, bool prepend_runtime,
-          bool save_runtime_options, bool compressed, int compression_level,
-          int application_type)
-{
-    FILE *file;
-    void *runtime_bytes = NULL;
-    size_t runtime_size;
-
-    file = prepare_to_save(filename, prepend_runtime, &runtime_bytes, &runtime_size);
-    if (file == NULL)
-        return 1;
-
-    if (prepend_runtime)
-        save_runtime_to_filehandle(file, runtime_bytes, runtime_size, application_type);
-
-    /* This unwinding is necessary for proper restoration of the
-     * symbol-value slots to their toplevel values, but it occurs
-     * too late to remove old references from the binding stack.
-     * There's probably no safe way to do that from Lisp */
-    unwind_binding_stack();
-    os_unlink_runtime();
-    return save_to_filehandle(file, filename, init_function, prepend_runtime,
-                              save_runtime_options,
-                              compressed ? compressed : COMPRESSION_LEVEL_NONE);
-}
-#endif
-
 #include "incremental-compact.h"
 
 /* Things to do before doing a final GC before saving a core.
@@ -600,7 +571,7 @@ bool save(char *filename, lispobj init_function, bool prepend_runtime,
  * + Instances on unboxed pages need to have their layout pointer visited,
  *   so all pages have to be turned to boxed.
  */
-static void prepare_dynamic_space_for_final_gc()
+static void prepare_dynamic_space_for_final_gc(struct thread* thread)
 {
     page_index_t i;
 
@@ -645,7 +616,6 @@ static void prepare_dynamic_space_for_final_gc()
 #ifdef LISP_FEATURE_SB_THREAD
     // Avoid tenuring of otherwise-dead objects referenced by
     // dynamic bindings which disappear on image restart.
-    struct thread *thread = get_sb_vm_thread();
     char *start = (char*)&thread->lisp_thread;
     char *end = (char*)thread + dynamic_values_bytes;
     memset(start, 0, end-start);
@@ -748,6 +718,7 @@ gc_and_save(char *filename, bool prepend_runtime, bool purify,
     struct thread *thread = get_sb_vm_thread();
     gc_close_thread_regions(thread, 0);
     gc_close_collector_regions(0);
+    pre_verify_gen_0 = 1;
 #ifdef LISP_FEATURE_MARK_REGION_GC
     /* Do a minor GC to instate allocation bitmap for new objects.
      * This is needed to make heap walking in move_rospace_to_dynamic
@@ -755,10 +726,9 @@ gc_and_save(char *filename, bool prepend_runtime, bool purify,
     collect_garbage(0);
 #endif
     move_rospace_to_dynamic(0);
-    pre_verify_gen_0 = 1;
     prepare_immobile_space_for_final_gc(); // once is enough
-    prepare_dynamic_space_for_final_gc();
-    unwind_binding_stack();
+    prepare_dynamic_space_for_final_gc(thread);
+    unwind_binding_stack(thread);
     // Avoid tenuring of otherwise-dead objects referenced by bindings which
     // disappear on image restart. This must occcur *after* unwind_binding_stack()
     // because unwinding moves values from the binding stack into TLS.
@@ -775,7 +745,7 @@ gc_and_save(char *filename, bool prepend_runtime, bool purify,
     collect_garbage(0);
     verify_heap(0, VERIFY_POST_GC);
 
-    THREAD_JIT(0);
+    THREAD_JIT_WP(0);
 
     // We always coalesce copyable numbers. Additional coalescing is done
     // only on request, in which case a message is shown (unless verbose=0).
@@ -793,7 +763,7 @@ gc_and_save(char *filename, bool prepend_runtime, bool purify,
     collect_garbage(1+PSEUDO_STATIC_GENERATION);
     prepare_readonly_space(purify, 0);
     if (verbose) { printf("[performing final GC..."); fflush(stdout); }
-    prepare_dynamic_space_for_final_gc();
+    prepare_dynamic_space_for_final_gc(thread);
     save_lisp_gc_iteration = 2;
     gencgc_alloc_start_page = 0;
     collect_garbage(0);
@@ -804,7 +774,7 @@ gc_and_save(char *filename, bool prepend_runtime, bool purify,
     if (verbose)
         printf(" done]\n");
 
-    THREAD_JIT(0);
+    THREAD_JIT_WP(0);
     // Scrub remaining garbage
     extern void zero_all_free_ranges(void);
     zero_all_free_ranges();
