@@ -13,8 +13,7 @@
 #include <ctype.h>
 #include <stdint.h>
 
-#include "genesis/config.h"
-#include "genesis/constants.h"
+#include "genesis/sbcl.h"
 #include "interr.h"
 #include "lispobj.h"
 #include "os.h"
@@ -48,12 +47,31 @@ search_static_space(void *pointer)
     return gc_search_space(start, pointer);
 }
 
+#ifndef LISP_FEATURE_IMMOBILE_SPACE
+lispobj *search_codeblob_offsets(void* pointer) {
+    if ((uword_t)pointer < TEXT_SPACE_START ||
+        (uword_t)pointer >= (uword_t)text_space_highwatermark) return 0;
+    struct vector* v = (void*)TEXT_SPACE_START;
+    uint32_t* data = (void*)v->data;
+    int index = bsearch_lesseql_uint32((char*)pointer - (char*)TEXT_SPACE_START,
+                                       data, vector_len(v));
+    if (index >= 0) {
+        lispobj* base = (lispobj*)(TEXT_SPACE_START + data[index]);
+        gc_assert(widetag_of(base) == CODE_HEADER_WIDETAG);
+        return base;
+    }
+    return 0;
+}
+#endif
+
 lispobj *search_all_gc_spaces(void *pointer)
 {
     lispobj *start;
     if (((start = search_dynamic_space(pointer)) != NULL) ||
 #ifdef LISP_FEATURE_IMMOBILE_SPACE
         ((start = search_immobile_space(pointer)) != NULL) ||
+#else
+        ((start = search_codeblob_offsets(pointer)) != NULL) ||
 #endif
         ((start = search_static_space(pointer)) != NULL) ||
         ((start = search_read_only_space(pointer)) != NULL))
@@ -348,6 +366,14 @@ int bsearch_greatereql_uint32(uint32_t item, uint32_t* array, int nelements)
     if (low < nelements) return low;
     return -1;
 }
+#else
+// these passthru stubs could probably just be linker symbol aliases
+int bsearch_lesseql_uint32(uint32_t item, uint32_t* array, int nelements) {
+    return bsearch_lesseql_uword(item, array, nelements);
+}
+int bsearch_greatereql_uint32(uint32_t item, uint32_t* array, int nelements) {
+    return bsearch_greatereql_uword(item, array, nelements);
+}
 #endif
 
 /* Find in an address-based split-ordered list
