@@ -1,23 +1,20 @@
-#include "sbcl.h"
+#include "genesis/sbcl.h"
 #include "arch.h"
 #include "runtime.h"
 #include "lispregs.h"
-#include "gc-internal.h"
-#include "gc-private.h"
+#include "gc.h"
 #include "code.h"
 #include "genesis/closure.h"
 #include "genesis/cons.h"
-#include "genesis/constants.h"
+#include "genesis/fdefn.h"
 #include "genesis/gc-tables.h"
 #include "genesis/hash-table.h"
 #include "genesis/instance.h"
-#include "genesis/layout.h"
 #include "genesis/package.h"
 #include "genesis/vector.h"
 #include "search.h"
 #include "genesis/avlnode.h"
 #include "genesis/sap.h"
-#include "genesis/thread-instance.h"
 #include "print.h"
 
 #include <stdlib.h>
@@ -53,8 +50,6 @@ int heap_trace_verbose = 0;
 
 typedef uintptr_t traceroot_pointer;
 
-extern generation_index_t gencgc_oldest_gen_to_gc;
-
 /// Each "layer" is a set of objects reachable by tracing one reverse pointer
 /// from any object in the previously built layer.
 /// An object will not appear in more than one layer.
@@ -748,7 +743,7 @@ static uword_t build_refs(lispobj* where, lispobj* end,
             n_immediates = 0, n_pointers = 0;
 
     bool count_only = !ss->record_ptrs;
-    for ( ; where < end ; where += nwords ) {
+    for (where = next_object(where, 0, end) ; where ; where = next_object(where, nwords, end)) {
         if (ss->ignored_objects && ignorep(where, ss->ignored_objects)) {
             nwords = object_size(where);
             continue;
@@ -1022,7 +1017,7 @@ gc_pathfind_aux(lispobj* stackptr, lispobj input, lispobj results,
     for_each_thread(th) {
         lispobj* stackptr = get_stackptr(th);
         os_context_t* context = get_register_context(th);
-        fprintf(stderr, "thread %p stack range %p..%p\n", th, stackptr, th->control_stack_end);
+        /* fprintf(stderr, "thread %p stack range %p..%p\n", th, stackptr, th->control_stack_end); */
         if (context) visit_context_registers(add_to_roots, context, &roots);
         for ( ; stackptr < th->control_stack_end ; ++stackptr ) {
             lispobj word = *stackptr;
