@@ -32,7 +32,7 @@
 #include <sys/param.h>
 #include <sys/file.h>
 #include <io.h>
-#include "sbcl.h"
+#include "genesis/sbcl.h"
 #include "os.h"
 #include "arch.h"
 #include "globals.h"
@@ -57,13 +57,13 @@
 #include "validate.h"
 #include "thread.h"
 #include "align.h"
-#include "unaligned.h"
 
 #include "gc.h"
 #include <wincrypt.h>
 #include <stdarg.h>
 #include <string.h>
 #include "print.h"
+#include <Shlobj.h>
 
 /* missing definitions for modern mingws */
 #ifndef EH_UNWINDING
@@ -114,8 +114,10 @@ static void set_seh_frame(void *frame)
 
 void alloc_gc_page()
 {
+#ifndef LISP_FEATURE_64_BIT // 64-bit uses the page below the card mark table
     gc_assert(VirtualAlloc(GC_SAFEPOINT_PAGE_ADDR, BACKEND_PAGE_BYTES,
                            MEM_RESERVE|MEM_COMMIT, PAGE_READWRITE));
+#endif
 }
 
 /* Permit loads from GC_SAFEPOINT_PAGE_ADDR (NB page state change is
@@ -841,11 +843,6 @@ void os_commit_memory(os_vm_address_t addr, os_vm_size_t len)
     if (len) gc_assert(VirtualAlloc(addr, len, MEM_COMMIT, PAGE_EXECUTE_READWRITE));
 }
 
-void os_decommit_mem(os_vm_address_t addr,  os_vm_size_t len) {
-    gc_assert(gc_active_p);
-    gc_assert(VirtualFree(addr, len, MEM_DECOMMIT));
-}
-
 /*
  * load_core_bytes() is called to load a chunk of the core file into memory.
  *
